/*
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

Copyright 2005 TU-Berlin, Tilman Knebel tk@ni.cs.tu-berlin.de
*/
#include <stdio.h>
#include <string.h>
#include "mex.h"
#include "psvm.h"

/* mexFunction is the gateway routine for the MEX-file. */ 
void
mexFunction( int nlhs, mxArray *plhs[],
             int nrhs, const mxArray *prhs[] )
{
	//nlhs - nr of left hand side args (output)
	//plhs - left hand side args (output)
	//nrhs - nr of right hand side args (input)
	//prhs - right hand side args (input)

	//[np]=psvmgetnp(K)
	//check type of input args
	if(nrhs!=1 || nlhs>1)
		mexErrMsgTxt("usage: np=psvmgetnp(K)");
	if(!mxIsNumeric(prhs[0]))
		mexErrMsgTxt("K must be numeric");
	if(mxIsComplex(prhs[0]))
		mexErrMsgTxt("complex data not supported");
	if(!mxIsDouble(prhs[0]))
		mexErrMsgTxt("K must be double");

	//check dimension of input args
	int d1;
	const int *pd1;
	int L,P;
	double* pK;
	d1=mxGetNumberOfDimensions(prhs[0]);
	pd1=mxGetDimensions(prhs[0]);
	if(d1!=2) mexErrMsgTxt("K must be a two dimensional array");
	pK=mxGetPr(prhs[0]);
	L=pd1[0];
	P=pd1[1];
	//create destination matrix for matlab
	plhs[0]=mxCreateDoubleMatrix(4,P,mxREAL);
	double* pKK;
	pKK=mxGetPr(plhs[0]);
	int j,i;
	double dmin,dmax,dmean,dvar,x;

	for (j=0;j<P;j++)
    {
		//compute mean and variance
		dmin= pK[0+L*j];
		dmax= dmin;
		dmean=dmin;
		for (i=1;i<L;i++)
		{
			x=pK[i+L*j];
			dmin=min(x,dmin);
			dmax=max(x,dmax);
			dmean+= x;
		}
		dmean/=(double)L;
		dvar=0;
		for (i=0;i<L;i++)
			dvar+= sqr(pK[i+L*j]-dmean);
		dvar=max(0.0000000001,dvar);
		dvar=1/sqrt(dvar);
		//(max,min,mean,variance)
		pKK[4*j+1]=dmin;
		pKK[4*j+0]=dmax;
		pKK[4*j+2]=dmean;
		pKK[4*j+3]=dvar;
	}
}

