/*
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

Copyright 2005 TU-Berlin, Tilman Knebel tk@ni.cs.tu-berlin.de
*/
#include <stdio.h>
#include <string.h>
#include "mex.h"
#include "psvm.h"
/*
bool rcomp(const Rankingi &a,const Rankingi &b)
{//global comparison operator for sorting to get sparse support vector matrix
	return a.dimRef<b.dimRef;
}
*/


/* mexFunction is the gateway routine for the MEX-file. */ 
void
mexFunction( int nlhs, mxArray *plhs[],
             int nrhs, const mxArray *prhs[] )
{
	verbose=false;
	//nlhs - nr of left hand side args (output)
	//plhs - left hand side args (output)
	//nrhs - nr of right hand side args (input)
	//prhs - right hand side args (input)

	//check type of input args
	if((nrhs!=2&&nrhs!=3)|| nlhs>1)
		mexErrMsgTxt("usage: [Y]=psvmpredict(X,model,usenormlimit=0)\n"
		"[Y]=psvmpredict(K,rmodel,usenormlimit=0)\n"
		"   X = [x1;x2;x3...xL] row vectors\n"
		"   K = matrix with relational data as L row objects with P coloumns\n"
		"   Y = vector with predictions for L input objects\n"
		"   model = {[alpha],[b],[normpar],[supportvectors],[kernelpar]}\n"
		"   rmodel = {[alpha],[b],[normpar]}\n"
		"   use normlimit = 0(no min-max), 1(project vectors into training data rect)\n");
	if(!mxIsNumeric(prhs[0]))
		mexErrMsgTxt("first arg: X must be a numeric matrix");
	if(!mxIsCell(prhs[1]))
		mexErrMsgTxt("second arg: model must be a cell matrix");
	int ncell;
    int limit=0;
	if(nrhs==3) limit=(int)mxGetScalar(prhs[2]);
	ncell=mxGetNumberOfElements(prhs[1]);
	if(ncell!=3 && ncell!=5)
		mexErrMsgTxt("wrong model:\n"
		"   model = {[alpha],[b],[normpar],[supportvectors],[kernelpar]}\n"
		"   rmodel = {[alpha],[b],[normpar]}\n");

	const mxArray *mxa,*mxb,*mxnp;
	mxa=mxGetCell(prhs[1],0);
	mxb=mxGetCell(prhs[1],1);
	mxnp=mxGetCell(prhs[1],2);
	//check types
	if(!mxIsNumeric(mxa) || mxIsComplex(mxa) ||!mxIsNumeric(mxb) || !mxIsNumeric(mxnp) || mxIsComplex(mxb) || mxIsComplex(mxnp))
		mexErrMsgTxt("All model cells must be real and numeric!");
	if(!mxIsSparse(mxa))
		mexErrMsgTxt("first model arg: a must be sparse");
	if(!mxIsDouble(mxb) ||!mxIsDouble(mxnp))
		mexErrMsgTxt("All model cells must be double!");
	if(!mxIsDouble(prhs[0]))
		mexErrMsgTxt("first input arg must be a double matrix!");


	//check dimension of input args
	int dimX;
	const int *pdimX;
	int dima,dimb,dimnp;
	const int *pdima,*pdimb,*pdimnp;
	int psvm_dim;//model dimensionality
	int features;//number of model features
	double* apr;	//alpha ranking
	int* air,*ajc;	//alpha ranking
	double psvm_b;
	double* npr;
	double* xpr;
	dimX=mxGetNumberOfDimensions(prhs[0]);
	pdimX=mxGetDimensions(prhs[0]);
	dima=mxGetNumberOfDimensions(mxa);
	dimb=mxGetNumberOfDimensions(mxb);
	dimnp=mxGetNumberOfDimensions(mxnp);
	pdima=mxGetDimensions(mxa);
	pdimb=mxGetDimensions(mxb);
	pdimnp=mxGetDimensions(mxnp);
	psvm_dim=pdima[0];//alpha rows = model dimensionality
	apr=mxGetPr(mxa);
	air=mxGetIr(mxa);
	ajc=mxGetJc(mxa);
	psvm_b=mxGetScalar(mxb);	//b level
	npr=mxGetPr(mxnp);	//norm
	xpr=mxGetPr(prhs[0]);//Xinput
	features=ajc[1];//alpha elements - number of features 
//	mexPrintf("a b np %d %d %d %d %d %d\n ",pd1[0],pd1[1],pd2[0],pd2[1],pd3[0],pd3[1]);
	if(dimX!=2)
		mexErrMsgTxt("first input arg must be a two dimensional matrix!");
	if(dima!=2 || pdima[1]!=1)
		mexErrMsgTxt("first model arg: a must be a sparse coloumn vector");
	if(dimb!=2 || pdimb[1]!=1 && pdimb[0]!=1)
		mexErrMsgTxt("second model arg: b must be a scalar\n");
	if(dimnp!=2 || pdimnp[0]!=features || pdimnp[1]!=4)
		mexErrMsgTxt("third model arg: NormParams must be a (features * 4) array\nwith one row per feature of alpha!");

	if(ncell==3)
	{//x is relation matrix
		int L,P;
		L=pdimX[0];//rows
		P=pdimX[1];//cols
		if(P!=psvm_dim)
			mexErrMsgTxt("Datamatrix must have same number of colums as model dimension!");		//create destination matrix
		plhs[0]=mxCreateDoubleMatrix(L,1,mxREAL);
		double* yout;
		yout=mxGetPr(plhs[0]);
		//get input data
		//and norm
		//and build K*a+b
		double temp_x;
		for(int i=0;i<L;i++)
		{
			yout[i]=psvm_b;
			for(int j=0;j<features;j++)
			{
				temp_x=xpr[i+air[j]*L];//get selected dimension of input matrix
				if(limit!=0){
					temp_x=max(temp_x,npr[j+1*features]);//limit
					temp_x=min(temp_x,npr[j+0*features]);//limit
				}
				yout[i]+=apr[j]*((temp_x-npr[j+2*features])*(npr[j+3*features]));//scale shift *a
			}
		}
	}else if(ncell==5)
	{//use kernel
		int L,N;
		L=pdimX[0];//rows
		N=pdimX[1];//cols
		const mxArray *mxsv,*mxkp;
		mxsv=mxGetCell(prhs[1],3);
		mxkp=mxGetCell(prhs[1],4);
		double* svr,*kpr;
		svr=mxGetPr(mxsv);
		kpr=mxGetPr(mxkp);
		int dimsv,dimkp;
		const int *pdimsv,*pdimkp;
		int kps;//number of kernelparameters (missing amounts will be overwritten by defaults)
		dimsv=mxGetNumberOfDimensions(mxsv);
		dimkp=mxGetNumberOfDimensions(mxkp);
		pdimsv=mxGetDimensions(mxsv);
		pdimkp=mxGetDimensions(mxkp);
//		mexPrintf("x sv kp %d %d %d %d %d %d\n ",pdimX[0],pdimX[1],pdimsv[0],pdimsv[1],pdimkp[0],pdimkp[1]);
//		mexPrintf("aP afeat %d %d \n ",psvm_dim,features);
		if(dimsv!=2 || pdimsv[1]!=N || pdimsv[0]!=features)
			mexErrMsgTxt("fourth model arg: nr. of cols in sv must match with cols in X!\nnumber of rows in sv must match with nr. of alpha elements!");
		if(dimkp!=2 || (pdimkp[0]!=1 && pdimkp[1]!=1))
			mexErrMsgTxt("fifth model arg: Kernelparameters must be a vector with at least one element!");
		if(!mxIsNumeric(mxsv) || !mxIsNumeric(mxkp) || mxIsComplex(mxsv) || mxIsComplex(mxkp))
			mexErrMsgTxt("All model cells must be real and numeric!");
		if(!mxIsDouble(mxsv) || !mxIsDouble(mxkp))
			mexErrMsgTxt("All model cells must be double!");
		kps=max(pdimkp[0],pdimkp[1]);

		//load data matrix
		Dataset xset;
		xset.reserve(L,N,false);
		xset.mDim=N;
		xset.mN=L;
		//get X into psvm structure 'xset'
		//get X into psvm structure 'xset'
		int i,j;
		for(i=0;i<L;i++)for(j=0;j<N;j++)
			xset.mData[i*N+j]=(ftype)xpr[i+j*L];
		xset.mTrans=false;
		
		//load support vector matrix
		Dataset zset;
		zset.reserve(features,N,false);
		zset.mDim=N;
		zset.mN=features;
        //get Z into psvm structure 'zset'
		for(i=0;i<features;i++)for(j=0;j<N;j++)
			zset.mData[i*N+j]=(ftype)svr[i+j*features];
		zset.mTrans=false;

		//kparams
		PSVMmodel psvm;
		if(kps>=1) psvm.par.kernel=(int)kpr[0];
		if(kps>=2) psvm.par.k1=kpr[1];
		if(kps>=3) psvm.par.k2=kpr[2];
		if(kps>=4) psvm.par.k3=kpr[3];

		psvm.kernel(&xset,&zset);
		//create destination matrix
		plhs[0]=mxCreateDoubleMatrix(L,1,mxREAL);
		double* yout;
		yout=mxGetPr(plhs[0]);
		//get kernel data
		//and norm
		//and build K*a+b
		double temp_x;
		for(i=0;i<L;i++)
		{
			yout[i]=psvm_b;
//			mexPrintf("Starting l=%d with y=%f\n",i,psvm_b);
			for(j=0;j<features;j++)
			{
				temp_x=xset.mData[j*L+i];//get selected dimension of input matrix
//				mexPrintf("loading kernel l=%d feat=%d  :%f\n",i,j,temp_x);
				if(limit!=0){
					temp_x=max(temp_x,npr[j+1*features]);//limit
					temp_x=min(temp_x,npr[j+0*features]);//limit
				}
//				mexPrintf("limiting min=%f max=%f\n",npr[j+1*features],npr[j+0*features]);
				yout[i]+=apr[j]*((temp_x-npr[j+2*features])*(npr[j+3*features]));//scale shift *a
//				mexPrintf("alpha*(k-m)*sc=%f*(%f-%f)*%f :: sum=%f\n",apr[j],temp_x,npr[j+2*features],npr[j+3*features]);
			}
		}
	}
}

