/*
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

Copyright 2005 TU-Berlin, Tilman Knebel tk@ni.cs.tu-berlin.de
*/
#include <stdio.h>
#include <string.h>
#include "mex.h"
#include "psvm.h"

/* mexFunction is the gateway routine for the MEX-file. */ 
void
mexFunction( int nlhs, mxArray *plhs[],
             int nrhs, const mxArray *prhs[] )
{
	//nlhs - nr of left hand side args (output)
	//plhs - left hand side args (output)
	//nrhs - nr of right hand side args (input)
	//prhs - right hand side args (input)

	//[normedK]=psvmputnorm(K,np)
	//check type of input args
	if((nrhs!=2 && nrhs!=3) || nlhs>1)
		mexErrMsgTxt("usage: normedK = psvmputnp(K,np,usenormlimit=0)");
	if(!mxIsNumeric(prhs[0]) || !mxIsDouble(prhs[0]))
		mexErrMsgTxt("first arg: K must be numeric double");
	if(!mxIsNumeric(prhs[1]) || !mxIsDouble(prhs[1]))
		mexErrMsgTxt("second arg: np must be numeric double");
	if(mxIsComplex(prhs[0]) || mxIsComplex(prhs[0]))
		mexErrMsgTxt("complex data not supported");

	//check dimension of input args
	int d1,d2;
	int i,j;
	const int *pd1,*pd2;
	int L,P;
	d1=mxGetNumberOfDimensions(prhs[0]);
	pd1=mxGetDimensions(prhs[0]);
	d2=mxGetNumberOfDimensions(prhs[1]);
	pd2=mxGetDimensions(prhs[1]);
    int limit=0;
	if(nrhs==3) limit=(int)mxGetScalar(prhs[2]);

	if(d1!=2)
		mexErrMsgTxt("firat arg: K must be a two dimensional array");
	if(d2!=2)
		mexErrMsgTxt("second arg: np must be a two dimensional array");
	L=pd1[0];
	P=pd1[1];
	if(pd2[0]!=4 || pd2[1]!=P)
		mexErrMsgTxt("size of np does not match matrix K");
	double *pK,*pNN;
	double *pKK;
	pK=mxGetPr(prhs[0]);
	pNN=mxGetPr(prhs[1]);

	//create destination K-matrix
	plhs[0]=mxCreateDoubleMatrix(L,P,mxREAL);
	pKK=mxGetPr(plhs[0]);
	double x;
	//fill MATLAB destination matrix with normalized K-data
	for(i=0;i<L;i++)
		for(j=0;j<P;j++) 
		{
			x=pK[j*L+i];//get data
			if(limit!=0){
				x=max(x,pNN[1+j*4]);//must be limited to the range of selected vectors, 
				x=min(x,pNN[j*4]);//the svm learns/classifies only in the range of the learnset (min..max)
			}
			pKK[j*L+i]=(x-pNN[2+j*4])*pNN[3+j*4];//save normalized data to "this"
		}
}

