/*
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

Copyright 2005 TU-Berlin, Tilman Knebel tk@cs.tu-berlin.de
*/
#include <stdio.h>
#include <string.h>
#include "mex.h"
#include "psvm.h"
#include <malloc.h>

/*
void heapdump( void )
{
   _HEAPINFO hinfo;
   int heapstatus;
   hinfo._pentry = NULL;
   while( ( heapstatus = _heapwalk( &hinfo ) ) == _HEAPOK )
   { mexPrintf( "%6s block at %Fp of size %4.4X\n",
        ( hinfo._useflag == _USEDENTRY ? "USED" : "FREE" ),
          hinfo._pentry, hinfo._size );
   }

   switch( heapstatus )
   {
   case _HEAPEMPTY:
      mexPrintf( "OK - empty heap\n" );
      break;
   case _HEAPEND:
      mexPrintf( "OK - end of heap\n" );
      break;
   case _HEAPBADPTR:
      mexPrintf( "ERROR - bad pointer to heap\n" );
      break;
   case _HEAPBADBEGIN:
      mexPrintf( "ERROR - bad start of heap\n" );
      break;
   case _HEAPBADNODE:
      mexPrintf( "ERROR - bad node in heap\n" );
      break;
   }
}
*/
/* mexFunction is the gateway routine for the MEX-file. */ 
void
mexFunction( int nlhs, mxArray *plhs[],
             int nrhs, const mxArray *prhs[] )
{
	verbose=false;
	//nlhs - nr of left hand side args (output)
	//plhs - left hand side args (output)
	//nrhs - nr of right hand side args (input)
	//prhs - right hand side args (input)

	//check type of input args
	if((nrhs!=3 && nrhs!=4 && nrhs!=5)|| nlhs>3)
		mexErrMsgTxt("usage: [model,error]=psvmtrain(X,Y,kernelpar,psvmpar)\n"
		"[model,error]=psvmtrain(X,Z,Y,kernelpar,psvmpar)\n"
		"[rmodel,error,diagnose]=psvmtrain(K,Y,psvmpar)\n"
		"   X = [x1;x2;x3;...;xL] data matrix as row vectors\n"
		"   Y = [y1 y2 y3 ... yL] labels\n"
		"   Z = [z1;z2;z3;...;zP] complex features as row vectors\n"
		"   K = kernel matrix with L row vectors and P coloumns\n"
		"   kernelpar = [kerneltype,k1(factor),k2(level),k3(degree)] -- depends on kerneltype\n"
		"   psvmpar = [C,epsilon,epsitol,timeout\n"
		"      doreduction,redsize,redfact,domaxselection,selectionfact\n"
		"      doblock,blockNmax,blockCmax,blockOver\n"
		"      doannealing,estart,edec,eterm\n"
		"   kerneltype = 0(LINEAR) 1(POLYNOM) 2(RBF) 3(SIGMOID) 4(PLUMMER) 5(SIN)\n"
		"   model = {[alpha],[b],[normpar],[supportvectors],[kernelpar]}\n"
		"   rmodel = {[alpha],[b],[normpar]} for relational data\n"
		"   error = [MSE ; classification errore rate]\n"
		"   diagnose = [epsi,feat,Cfeat,P,#it,#block,#Q,time,Qtime,Btime,Btime_cl,Btime_smax,WSS1Time,WSS2Time,U1Time,U2Time,#BlockS,#BlockC\n"
		"               diagnostics require a compilation with flag 'timelog=1'\n"
		);

//   heapdump();
	if(nrhs==3)
	{
		const mxArray *mxY,*mxK,*mxppar;
		mxK=prhs[0];
		mxY=prhs[1];
		mxppar=prhs[2];

		if(!mxIsNumeric(mxK))
			mexErrMsgTxt("first arg: K must be a numeric matrix");
		if(!mxIsNumeric(mxY))
			mexErrMsgTxt("second arg: Y must be a numeric matrix");
		if(!mxIsNumeric(mxppar))
			mexErrMsgTxt("third arg: Parameters must be a numeric vector");
		if(mxIsComplex(mxK) || mxIsComplex(mxY) || mxIsComplex(mxppar))
			mexErrMsgTxt("complex data not supported");
		if(!mxIsDouble(mxK) || !mxIsDouble(mxY) || !mxIsDouble(mxppar))
			mexErrMsgTxt("all data must be of type double");

		//check dimension of input args
		int dimK,dimY,dimppar;
		const int *pdimK,*pdimY,*pdimppar;
		double* aK,*aY,*appar;
		int L,P,ppars,Ys;//input
		int features;//output
		dimK=mxGetNumberOfDimensions(mxK);
		dimY=mxGetNumberOfDimensions(mxY);
		dimppar=mxGetNumberOfDimensions(mxppar);
		pdimK=mxGetDimensions(mxK);
		pdimY=mxGetDimensions(mxY);
		pdimppar=mxGetDimensions(mxppar);
		if(dimK!=2)
			mexErrMsgTxt("first arg: K must be a two dimensional array");
		if(dimY!=2 || (pdimY[1]!=1 && pdimY[0]!=1))
			mexErrMsgTxt("second arg: Y must be a vector\n");
		if(dimppar!=2 || (pdimppar[1]!=1 && pdimppar[0]!=1))
			mexErrMsgTxt("third arg: Parameters must be a vector\n");

		L=pdimK[0];//rows in K-matrix
		P=pdimK[1];//cols in K-matrix
		ppars=max(pdimppar[0],pdimppar[1]);//parameter vector
		Ys=max(pdimY[0],pdimY[1]);//number of Y elements

		//check size of input args
		if(Ys!=L)
			mexErrMsgTxt("Y must have same number of elements as rows in K\n");
		aK=mxGetPr(mxK);
		aY=mxGetPr(mxY);
		appar=mxGetPr(mxppar);

		PSVMmodel psvm;//create psvm
		Trainres res;

		//read psvmpars
		if(ppars>=1)psvm.par.C=(ftype)appar[0];
		if(ppars>=2)psvm.par.epsi=(ftype)appar[1];
		if(ppars>=3)psvm.par.epsiTol=(ftype)appar[2];
		if(ppars>=4)psvm.par.timeout=(int)appar[3];
		if(ppars>=5)psvm.par.doreduction=(int)appar[4];
		if(ppars>=6)psvm.par.redsize=(int)appar[5];
		if(ppars>=7)psvm.par.redfact=(double)appar[6];
		if(ppars>=8)psvm.par.domaxselection=(int)appar[7];
		if(ppars>=9)psvm.par.selectionfact=(double)appar[8];
		if(ppars>=10)psvm.par.doblock=(int)appar[9];
		if(ppars>=11)psvm.par.blockNmax=(int)appar[10];
		if(ppars>=12)psvm.par.blockCmax=(int)appar[11];
		if(ppars>=13)psvm.par.blockOver=(double)appar[12];
		if(ppars>=14)psvm.par.doannealing=(int)appar[13];
		if(ppars>=15)psvm.par.estart=(ftype)appar[14];
		if(ppars>=16)psvm.par.edec=(ftype)appar[15];
		if(ppars>=17)psvm.par.eterm=(ftype)appar[16];

		//reserve space for input K(ernel)-matrix in psvm structure
//   heapdump();
		Dataset set;
		set.reserve(L,P,true);
		set.mDim=P;
		set.mN=L;

		//get K into psvm structure 'set'
		int all=L*P;
		for(int i=0;i<all;i++)
			set.mData[i]=(ftype)aK[i];
		set.mTrans=true;

		//put input Y-data in psvm structure
		for(int i=0;i<L;i++)
			set.mY[i]=(ftype)aY[i];

//   heapdump();
		if(!psvm.getModel(&set,&res))
			mexPrintf("SMO skipped after timeout\n");
//   heapdump();
		features=psvm.rank.mFeat.size();
		

		//create destination matrix for alpha,normalizationparameters
		plhs[0]=mxCreateCellMatrix(1,3);
		mxArray *a,*b,*np;
		a=mxCreateSparse(P,1,features,mxREAL);//coloumn vector with 'features' used elements of P
		b=mxCreateDoubleMatrix(1,1,mxREAL);//scalar is not compatible with older matlab versions
		np=mxCreateDoubleMatrix(features,4,mxREAL);//four normparams per feature
		mxSetCell(plhs[0],0,a);
		mxSetCell(plhs[0],1,b);
		mxSetCell(plhs[0],2,np);
		double* aa;
		int* rr;
		double* npa;
		aa=mxGetPr(a);//get pointer to vector data
		rr=mxGetIr(a);//get pointer to references
		npa=mxGetPr(np);//get pointer to norm vector data
		mxGetPr(b)[0]=psvm.b;//fill b
		int ref;
		for(int i=0;i<features;i++)
		{
			aa[i]=psvm.rank.mFeat[i].a;
			ref=psvm.rank.mFeat[i].dimRef;
			rr[i]=ref;
			npa[i]=psvm.np.mX[ref].max;
			npa[i+features]=psvm.np.mX[ref].min;
			npa[i+2*features]=psvm.np.mX[ref].mean;
			npa[i+3*features]=psvm.np.mX[ref].scale;
		}
		mxGetJc(a)[0]=0;//index of first nonzero in first coloumn
		mxGetJc(a)[1]=features;//last index
//		mexPrintf("epsi:%.5f feat:%d/%d/%d #pair:%d #blk:%d #Q:%d time:%.4f\n",psvm.par.epsi,-psvm.smo.mShz,psvm.smo.mShc,P,psvm.smo.mACount,psvm.smo.mBCount,psvm.smo.mQCount,psvm.smo.mTCount);
//		mexPrintf(" Qupdate:%.5f block total:%.5f b-sum:%.5f b-max:%.5f WSS1:%.5f WSS2:%.5f\n",(double)((double)psvm.smo.mTQCount/(double)CLOCKS_PER_SEC),(double)((double)psvm.smo.mTBCount/(double)CLOCKS_PER_SEC),(double)((double)psvm.smo.mTBCounts/(double)CLOCKS_PER_SEC),psvm.smo.mTBCountsmax,(double)((double)psvm.smo.mTWSS1Count/(double)CLOCKS_PER_SEC),(double)((double)psvm.smo.mTWSS2Count/(double)CLOCKS_PER_SEC));
		if(nlhs==2||nlhs==3)
		{
			plhs[1]=mxCreateDoubleMatrix(2,1,mxREAL);
			mxGetPr(plhs[1])[0]=res.reg_TError;
			mxGetPr(plhs[1])[1]=res.cla_TError;
			if(nlhs==3)
			{
				plhs[2]=mxCreateDoubleMatrix(18,1,mxREAL);
				mxGetPr(plhs[2])[0]=psvm.par.epsi;
				mxGetPr(plhs[2])[1]=-psvm.smo.mShz;
				mxGetPr(plhs[2])[2]=psvm.smo.mShc;
				mxGetPr(plhs[2])[3]=P;
				mxGetPr(plhs[2])[4]=psvm.smo.mACount;
				mxGetPr(plhs[2])[5]=psvm.smo.mBCount;
				mxGetPr(plhs[2])[6]=psvm.smo.mQCount;
				mxGetPr(plhs[2])[7]=psvm.smo.mTCount;
				mxGetPr(plhs[2])[8]=(double)psvm.smo.mTQCount/(double)CLOCKS_PER_SEC;
				mxGetPr(plhs[2])[9]=(double)psvm.smo.mTBCount/(double)CLOCKS_PER_SEC;
				mxGetPr(plhs[2])[10]=(double)psvm.smo.mTBCounts/(double)CLOCKS_PER_SEC;
				mxGetPr(plhs[2])[11]=psvm.smo.mTBCountsmax;//reference implicite by mean calculction
				mxGetPr(plhs[2])[12]=(double)psvm.smo.mTWSS1Count/(double)CLOCKS_PER_SEC;
				mxGetPr(plhs[2])[13]=(double)psvm.smo.mTWSS2Count/(double)CLOCKS_PER_SEC;
				mxGetPr(plhs[2])[14]=(double)psvm.smo.mTU1Count/(double)CLOCKS_PER_SEC;
				mxGetPr(plhs[2])[15]=(double)psvm.smo.mTU2Count/(double)CLOCKS_PER_SEC;
				mxGetPr(plhs[2])[16]=psvm.smo.mBCCount;
				mxGetPr(plhs[2])[17]=psvm.smo.mBSCount;
				//epsi, feat, Cfeat, P=maxfeat, #it, #block, #Q, time, Qtime, blocktime, blocktime wthout overhead, Block max single, WSS1, WSS2, U1,U2, blocksingluar, blocktoocomplexC
			}
		}
//   heapdump();
//   mexPrintf("set\n");
//		set.free();
//   heapdump();
//   mexPrintf("rank\n");
//		psvm.rank.mFeat.clear();
//   heapdump();
//   mexPrintf("np\n");
//		psvm.np.free();
//   heapdump();
//   mexPrintf("destruction\n");
	}else if(nrhs==4)//#################################################################################
	{//the same for kernels without Z
		const mxArray *mxX,*mxY,*mxppar,*mxkpar;
		mxX=prhs[0];
		mxY=prhs[1];
		mxppar=prhs[3];
		mxkpar=prhs[2];

		if(!mxIsNumeric(mxX))
			mexErrMsgTxt("first arg: X must be a numeric matrix");
		if(!mxIsNumeric(mxY))
			mexErrMsgTxt("second arg: Y must be a numeric matrix");
		if(!mxIsNumeric(mxppar))
			mexErrMsgTxt("third arg: Parameters must be a numeric vector");
		if(!mxIsNumeric(mxkpar))
			mexErrMsgTxt("fourth arg: Parameters must be a numeric vector");
		if(mxIsComplex(mxX) || mxIsComplex(mxY) || mxIsComplex(mxppar) || mxIsComplex(mxkpar))
			mexErrMsgTxt("complex data not supported");
		if(!mxIsDouble(mxX) || !mxIsDouble(mxY) || !mxIsDouble(mxppar) || !mxIsDouble(mxkpar))
			mexErrMsgTxt("all data must be of type double");

		//check dimension of input args
		int dimX,dimY,dimppar,dimkpar;
		const int *pdimX,*pdimY,*pdimppar,*pdimkpar;
		int L,N,Ys,ppars,kpars;
		double* aX,*aY,*appar,*akpar;//double data
		int features;

		dimX=mxGetNumberOfDimensions(mxX);
		dimY=mxGetNumberOfDimensions(mxY);
		dimppar=mxGetNumberOfDimensions(mxppar);
		dimkpar=mxGetNumberOfDimensions(mxkpar);
		pdimX=mxGetDimensions(mxX);
		pdimY=mxGetDimensions(mxY);
		pdimppar=mxGetDimensions(mxppar);
		pdimkpar=mxGetDimensions(mxkpar);
		if(dimX!=2)
			mexErrMsgTxt("first arg: X must be a two dimensional array");
		if(dimY!=2 || (pdimY[1]!=1 && pdimY[0]!=1))
			mexErrMsgTxt("second arg: Y must be a vector\n");
		if(dimppar!=2 || (pdimppar[1]!=1 && pdimppar[0]!=1))
			mexErrMsgTxt("third arg: Parameters must be a vector\n");
		if(dimkpar!=2 || (pdimkpar[1]!=1 && pdimkpar[0]!=1))
			mexErrMsgTxt("fourth arg: Parameters must be a vector\n");

		L=pdimX[0];//rows in X, x-vectors
		N=pdimX[1];//cols in X, x-dim
		Ys=max(pdimY[0],pdimY[1]);
		ppars=max(pdimppar[0],pdimppar[1]);//parameter vector
		kpars=max(pdimkpar[0],pdimkpar[1]);//parameter vector

		//check size of input args
		if(Ys!=L)
			mexErrMsgTxt("Y must have same number of elements as rows in X\n");
		aX=mxGetPr(mxX);
		aY=mxGetPr(mxY);
		appar=mxGetPr(mxppar);
		akpar=mxGetPr(mxkpar);
        
		PSVMmodel psvm;//
		Trainres res;

		//put input args in psvm structure
		//read psvmpars
		if(ppars>=1)psvm.par.C=(ftype)appar[0];
		if(ppars>=2)psvm.par.epsi=(ftype)appar[1];
		if(ppars>=3)psvm.par.epsiTol=(ftype)appar[2];
		if(ppars>=4)psvm.par.timeout=(int)appar[3];
		if(ppars>=5)psvm.par.doreduction=(int)appar[4];
		if(ppars>=6)psvm.par.redsize=(int)appar[5];
		if(ppars>=7)psvm.par.redfact=(double)appar[6];
		if(ppars>=8)psvm.par.domaxselection=(int)appar[7];
		if(ppars>=9)psvm.par.selectionfact=(double)appar[8];
		if(ppars>=10)psvm.par.doblock=(int)appar[9];
		if(ppars>=11)psvm.par.blockNmax=(int)appar[10];
		if(ppars>=12)psvm.par.blockCmax=(int)appar[11];
		if(ppars>=13)psvm.par.blockOver=(double)appar[12];
		if(ppars>=14)psvm.par.doannealing=(int)appar[13];
		if(ppars>=15)psvm.par.estart=(ftype)appar[14];
		if(ppars>=16)psvm.par.edec=(ftype)appar[15];
		if(ppars>=17)psvm.par.eterm=(ftype)appar[16];

		//read kernel pars
		if(kpars>=1) psvm.par.kernel=(int)akpar[0];
		if(kpars>=2) psvm.par.k1=(double)akpar[1];
		if(kpars>=3) psvm.par.k2=(double)akpar[2];
		if(kpars>=4) psvm.par.k3=(double)akpar[3];

		//reserve space for first input arg X
		Dataset xset;
		int j,i;
		xset.reserve(L,N,true);
		xset.mN=L;
		xset.mDim=N;

		//get X into psvm structure 'xset'
		for(i=0;i<L;i++)for(j=0;j<N;j++)
			xset.mData[i*N+j]=(ftype)aX[i+j*L];
		xset.mTrans=false;

		//put input Y-data in psvm structure
		for(i=0;i<L;i++)
			xset.mY[i]=(ftype)aY[i];

		//compute Kernel matrix
		psvm.kernel(&xset);//replaces xset!!!

		//run the psvm with normalization
		if(!psvm.getModel(&xset,&res))
			mexPrintf("SMO skipped after timeout\n");
		features=psvm.rank.mFeat.size();
		//create destination matrix for model
		plhs[0]=mxCreateCellMatrix(1,5);
		mxArray *a,*b,*np,*sv,*kp;
		a=mxCreateSparse(L,1,features,mxREAL);
		b=mxCreateDoubleMatrix(1,1,mxREAL);
		np=mxCreateDoubleMatrix(features,4,mxREAL);
		sv=mxCreateDoubleMatrix(features,N,mxREAL);
		kp=mxCreateDoubleMatrix(1,4,mxREAL);
		mxSetCell(plhs[0],0,a);
		mxSetCell(plhs[0],1,b);
		mxSetCell(plhs[0],2,np);
		mxSetCell(plhs[0],3,sv);
		mxSetCell(plhs[0],4,kp);
		double* aa;
		int* rr;
		double* npa;
		double* sva;
		aa=mxGetPr(a);//get pointer to vector data
		rr=mxGetIr(a);//get pointer to references
		npa=mxGetPr(np);//get pointer to vector data
		mxGetPr(b)[0]=psvm.b;
		sva=mxGetPr(sv);//get pointer to vector data
		int ref;
		for(i=0;i<features;i++)
		{
			aa[i]=psvm.rank.mFeat[i].a;
			ref=psvm.rank.mFeat[i].dimRef;
			rr[i]=ref;
			npa[i]=psvm.np.mX[ref].max;
			npa[i+features]=psvm.np.mX[ref].min;
			npa[i+2*features]=psvm.np.mX[ref].mean;
			npa[i+3*features]=psvm.np.mX[ref].scale;
			for(j=0;j<N;j++)
				sva[i+j*features]=aX[ref+j*L];//xset.mData[ref][j];
		}
		mxGetJc(a)[0]=0;//index of first nonzero in first coloumn
		mxGetJc(a)[1]=features;//last index
		mxGetPr(kp)[0]=psvm.par.kernel;
		mxGetPr(kp)[1]=psvm.par.k1;
		mxGetPr(kp)[2]=psvm.par.k2;
		mxGetPr(kp)[3]=psvm.par.k3;
		if(nlhs==2)
		{
			plhs[1]=mxCreateDoubleMatrix(2,1,mxREAL);
			mxGetPr(plhs[1])[0]=res.reg_TError;
			mxGetPr(plhs[1])[1]=res.cla_TError;
		}
	}else if(nrhs==5)//#################################################################################
	{//the same for kernels
		const mxArray *mxX,*mxY,*mxZ,*mxppar,*mxkpar;
		mxX=prhs[0];
		mxZ=prhs[1];
		mxY=prhs[2];
		mxppar=prhs[4];
		mxkpar=prhs[3];

		if(!mxIsNumeric(mxX) || !mxIsNumeric(mxZ) || !mxIsNumeric(mxY) || !mxIsNumeric(mxppar) || !mxIsNumeric(mxkpar))
			mexErrMsgTxt("all parameters must be numeric");
		if(mxIsComplex(mxX) || mxIsComplex(mxZ) || mxIsComplex(mxY) || mxIsComplex(mxppar) || mxIsComplex(mxkpar))
			mexErrMsgTxt("complex data not supported");
		if(!mxIsDouble(mxX) || !mxIsDouble(mxZ) || !mxIsDouble(mxY) || !mxIsDouble(mxppar) || !mxIsDouble(mxkpar))
			mexErrMsgTxt("all data must be of type double");

		//check dimension of input args
		int dimX,dimZ,dimY,dimppar,dimkpar;
		const int *pdimX,*pdimZ,*pdimY,*pdimppar,*pdimkpar;
		int L,P,Nx,Nz,Ys,ppars,kpars;
		double* aX,*aZ,*aY,*appar,*akpar;//double data
		int features;

		dimX=mxGetNumberOfDimensions(mxX);
		dimY=mxGetNumberOfDimensions(mxY);
		dimZ=mxGetNumberOfDimensions(mxZ);
		dimppar=mxGetNumberOfDimensions(mxppar);
		dimkpar=mxGetNumberOfDimensions(mxkpar);
		pdimX=mxGetDimensions(mxX);
		pdimZ=mxGetDimensions(mxZ);
		pdimY=mxGetDimensions(mxY);
		pdimppar=mxGetDimensions(mxppar);
		pdimkpar=mxGetDimensions(mxkpar);
		if(dimX!=2)
			mexErrMsgTxt("first arg: X must be a two dimensional array");
		if(dimZ!=2)
			mexErrMsgTxt("second arg: Z must be a two dimensional array");
		if(dimY!=2 || (pdimY[1]!=1 && pdimY[0]!=1))
			mexErrMsgTxt("third arg: Y must be a vector\n");
		if(dimppar!=2 || (pdimppar[1]!=1 && pdimppar[0]!=1))
			mexErrMsgTxt("fourth arg: Parameters must be a vector\n");
		if(dimkpar!=2 || (pdimkpar[1]!=1 && pdimkpar[0]!=1))
			mexErrMsgTxt("fifth arg: Parameters must be a vector\n");

		L=pdimX[0];//rows in X, x-vectors
		P=pdimZ[0];//rows in Z
		Nx=pdimX[1];//cols in X, x-dim
		Nz=pdimZ[1];//cols in Z
		Ys=max(pdimY[0],pdimY[1]);
		ppars=max(pdimppar[0],pdimppar[1]);//parameter vector
		kpars=max(pdimkpar[0],pdimkpar[1]);//parameter vector

		//check size of input args
		if(Ys!=L)
			mexErrMsgTxt("Y must have same number of elements as rows in X\n");
		if(Nx!=Nz)
			mexErrMsgTxt("X and Z must have same number of cols\n");
		aX=mxGetPr(mxX);
		aZ=mxGetPr(mxZ);
		aY=mxGetPr(mxY);
		appar=mxGetPr(mxppar);
		akpar=mxGetPr(mxkpar);
        
		PSVMmodel psvm;//
		Trainres res;

		//put input args in psvm structure
		//read psvmpars
		if(ppars>=1)psvm.par.C=(ftype)appar[0];
		if(ppars>=2)psvm.par.epsi=(ftype)appar[1];
		if(ppars>=3)psvm.par.epsiTol=(ftype)appar[2];
		if(ppars>=4)psvm.par.timeout=(int)appar[3];
		if(ppars>=5)psvm.par.doreduction=(int)appar[4];
		if(ppars>=6)psvm.par.redsize=(int)appar[5];
		if(ppars>=7)psvm.par.redfact=(double)appar[6];
		if(ppars>=8)psvm.par.domaxselection=(int)appar[7];
		if(ppars>=9)psvm.par.selectionfact=(double)appar[8];
		if(ppars>=10)psvm.par.doblock=(int)appar[9];
		if(ppars>=11)psvm.par.blockNmax=(int)appar[10];
		if(ppars>=12)psvm.par.blockCmax=(int)appar[11];
		if(ppars>=13)psvm.par.blockOver=(double)appar[12];
		if(ppars>=14)psvm.par.doannealing=(int)appar[13];
		if(ppars>=15)psvm.par.estart=(ftype)appar[14];
		if(ppars>=16)psvm.par.edec=(ftype)appar[15];
		if(ppars>=17)psvm.par.eterm=(ftype)appar[16];
		//read kernel pars
		if(kpars>=1) psvm.par.kernel=(int)akpar[0];
		if(kpars>=2) psvm.par.k1=(double)akpar[1];
		if(kpars>=3) psvm.par.k2=(double)akpar[2];
		if(kpars>=4) psvm.par.k3=(double)akpar[3];

		//reserve space for first input arg X
		Dataset xset;
		int j,i;
		xset.reserve(L,Nx,true);
		xset.mN=L;
		xset.mDim=Nx;

		//get X into psvm structure 'xset'
		for(i=0;i<L;i++)for(j=0;j<Nx;j++)
			xset.mData[i*Nx+j]=(ftype)aX[i+j*L];
		xset.mTrans=false;

		//reserve space for Z
		Dataset zset;
		zset.reserve(P,Nz,true);
		zset.mN=P;
		zset.mDim=Nz;

		//get X into psvm structure 'xset'
		for(i=0;i<P;i++)for(j=0;j<Nz;j++)
			zset.mData[i*Nz+j]=(ftype)aZ[i+j*P];
		zset.mTrans=false;

		//put input Y-data in psvm structure
		for(i=0;i<L;i++)
			xset.mY[i]=(ftype)aY[i];

		//compute Kernel matrix
		psvm.kernel(&xset,&zset);//replaces xset!!!

		//run the psvm with normalization
		if(!psvm.getModel(&xset,&res))
			mexPrintf("SMO skipped after timeout\n");
		features=psvm.rank.mFeat.size();
		//create destination matrix for model
		plhs[0]=mxCreateCellMatrix(1,5);
		mxArray *a,*b,*np,*sv,*kp;
		a=mxCreateSparse(P,1,features,mxREAL);
		b=mxCreateDoubleMatrix(1,1,mxREAL);
		np=mxCreateDoubleMatrix(features,4,mxREAL);
		sv=mxCreateDoubleMatrix(features,Nx,mxREAL);
		kp=mxCreateDoubleMatrix(1,4,mxREAL);
		mxSetCell(plhs[0],0,a);
		mxSetCell(plhs[0],1,b);
		mxSetCell(plhs[0],2,np);
		mxSetCell(plhs[0],3,sv);
		mxSetCell(plhs[0],4,kp);
		double* aa;
		int* rr;
		double* npa;
		double* sva;
		aa=mxGetPr(a);//get pointer to vector data
		rr=mxGetIr(a);//get pointer to references
		npa=mxGetPr(np);//get pointer to vector data
		mxGetPr(b)[0]=psvm.b;
		sva=mxGetPr(sv);//get pointer to vector data
		int ref;
		for(i=0;i<features;i++)
		{
			aa[i]=psvm.rank.mFeat[i].a;
			ref=psvm.rank.mFeat[i].dimRef;
			rr[i]=ref;
			npa[i]=psvm.np.mX[ref].max;
			npa[i+features]=psvm.np.mX[ref].min;
			npa[i+2*features]=psvm.np.mX[ref].mean;
			npa[i+3*features]=psvm.np.mX[ref].scale;
			for(j=0;j<Nz;j++)
				sva[i+j*features]=aZ[ref+j*P];//zset.mData[ref][j];
		}
		mxGetJc(a)[0]=0;//index of first nonzero in first coloumn
		mxGetJc(a)[1]=features;//last index
		mxGetPr(kp)[0]=psvm.par.kernel;
		mxGetPr(kp)[1]=psvm.par.k1;
		mxGetPr(kp)[2]=psvm.par.k2;
		mxGetPr(kp)[3]=psvm.par.k3;
		if(nlhs==2)
		{
			plhs[1]=mxCreateDoubleMatrix(2,1,mxREAL);
			mxGetPr(plhs[1])[0]=res.reg_TError;
			mxGetPr(plhs[1])[1]=res.cla_TError;
		}
	}
//  heapdump();
}

