%usage: 
%[model,error] = psvmtrain(X,Y,kernelparameters, psvmparameters)
%[model,error] = psvmtrain(X,Z,Y,kernelparameters, psvmparameters)
%[rmodel,error] = psvmtrain(K,Y,psvmparameters)
%where 
%X = [x1;x2;x3;...;xL] data matrix as row vectors
%Y = [y1 y2 y3 ... yL] label vector
%Z = [z1;z2;z3;...;zP] complex features as row vectors
%K = relational data with L rows and P coloumns
%kernelparameters = [kerneltype,p1,p2,p3]
%kerneltypes: 0(LINEAR) 1(POLYNOMIAL) 2(RBF) 3(SIGMOID) 4(PLUMMER) 5(SIN)
%Standard case:
%	0: k(l,p)=<x(l),x(p)>
%	1: k(l,p)=(p1 * <x(l),x(p)> + p2)^p3
%	2: k(l,p)=exp(-p1 * |x(l)-x(p)|^2)
%	3: k(l,p)=tanh(p1 * <x(l),x(p)> + p2)
%	4: k(l,p)=(p1 * |x(l)-x(p)|^2 + p2)^-p3
%	5: k(l,p)=sin(p1 * |x(l)-x(p)| + p2)*exp(-p3 * |x(l)-x(p)|)
%   6: user defined
%With use of Z-data:
%	0: k(l,p)=<x(l),z(p)>
%	1: k(l,p)=(p1 * <x(l),z(p)> + p2)^p3
%	2: k(l,p)=exp(-p1 * |x(l)-z(p)|^2)
%	3: k(l,p)=tanh(p1 * <x(l),z(p)> + p2)
%	4: k(l,p)=(p1 * |x(l)-z(p)|^2 + p2)^-p3
%	5: k(l,p)=sin(p1 * |x(l)-z(p)| + p2)*exp(-p3 * |x(l)-z(p)|)
%   6: user defined
%Directly for relational data:
%	0: k(l,p)=x(l,p)
%	1: k(l,p)=(p1 * x(l,p) + p2)^p3
%	2: k(l,p)=exp(-p1 * x(l,p))
%	3: k(l,p)=tanh(p1 * x(l,p) + p2)
%	4: k(l,p)=(p1 * x(l,p) + p2)^-p3
%	5: k(l,p)=sin(p1 * x(l,p + p2)*exp(-p3 * x(l,p))
%   6: user defined
%psvmparameters = [C,epsilon,epsitol,timeout
%   doreduction,redsize,redfact,domaxselection,selectionfact
%   doblock,blockNmax,blockCmax,blockOver
%   doannealing,estart,edec,eterm
%C: SMO parameter for contraint dominance
%epsilon: SMO parameter for assumed noise strength 
%epsitol: SMO termination, tolerance for epsilon, should be below 0.05
%timeout: amount of seconds for SMO, -1 means no timeout
%model = {[alpha],[b],[normpar],[supportvectors],[kernelpar]}
%rmodel = {[alpha],[b],[normpar]} for relational data
%error = [MSE ; classification errore rate]


%Copyright TU-Berlin 2005, Tilman Knebel

