/*
P-SVM Command Line Application
Copyright 2005 TU-Berlin, Tilman Knebel

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

If you use the software please cite 
"Sepp Hochreiter, Klaus Obermayer;
 Support Vector Machines for Dyadic Data, Neural Computation, 2005"
in your publications.
*/

//#define new DEBUG_NEW
//#include <afx.h>
#include <cstring>
#include <cmath>
#include <cstdio>
#include <ctime>
#include <cstdlib>
#include <list>
#include <algorithm>
#include <float.h>
#include <ctype.h>
using namespace std;

#include "psvm.h"
#include "psvmparse.h"
//#define CRTDBG_MAP_ALLOC
//#include <stdlib.h>
//#include <crtdbg.h>
//C_ENDING is either "" or ".txt"(for win users) -- see header file

void hinfo(char *fmt,...)
{//print progress messages to screen and logfile
    va_list ap;
	va_start(ap,fmt);
	vprintf(fmt,ap);//show on screen
	if(flog) if(vfprintf(flog,fmt,ap)<0) {printf("error writing to disc!\n"); exit(-1);} //show in logfile
	va_end(ap);
}
void check_f(FILE* h)
{//check headers and print error messages
	if(!h) {hinfo("error writing to disc!\n");exit(-1);}
}
void finfo(FILE* h,char *fmt,...)
{//fileoutput with error messages - no screen message
    va_list ap;
	va_start(ap,fmt);
	if(vfprintf(h,fmt,ap)<0) {printf("error writing to disc!\n");exit(-1);}
}
void relogg()
{//insert program start marker into logfile
	FILE* f;
//	f=fopen("logfile"C_ENDING,"w"); //delete old logfile??
	f=fopen("logfile"C_ENDING,"a");
	check_f(f);
	finfo(f,"\n\n##########################################################\n\n\n");
	fclose(f);
}
double konfilow(double alpha,double pos,double total)
{//with probability alpha the real probability of pos is below konfilow or above konfihigh
	double x,y,z;
	x=(2.*alpha*pos+1.)/(2.*total*alpha+2.);
    y=(4.*alpha*pos*(1.-pos/total)+1);
	z=(2.*total*alpha+2.);
	return x-sqrt(y)/z;
}
double konfihigh(double alpha,double pos,double total)
{//with probability alpha the real probability of pos is below konfilow or above konfihigh
	double x,y,z;
	x=(2.*alpha*pos+1.)/(2.*total*alpha+2.);
    y=(4.*alpha*pos*(1.-pos/total)+1);
	z=(2.*total*alpha+2.);
	return x+sqrt(y)/z;
}
int rndrng(int limit)
{//returns random number between 0 and limit-1
	unsigned int maxi=RAND_MAX;
	int f;//number of necessary rand()
	int k,r;
	int fact[9];
	if(maxi==INT_MAX) maxi=INT_MAX; else maxi++;

	f=0;
	k=1;
	fact[f]=k;
	while(k<limit)
	{
		f=f+1;
		k=k*maxi;
		fact[f]=k;
	};
	// rand= 0..255(=randmax)
	//fact= [1 256(=randmax+1) 65536 ...]
	//r=0..255/256
	//if randmax=intmax, a little problem occurs, but 'around-worked'

	r=0;
	for(k=0;k<f;k++)r=r+rand()*fact[k];
	r=(int)((double)r*(double)limit/(double)fact[f]);
	if(r>=limit) r=limit-1;//work-around...
	return r;
}
int main(int argc,char* argv[])
{
	PSVMmodel psvm;
	char *lname,*cname,*rname,*tname,*model,*finame,*fvname,*frname,*sname;
	bool ll,cc,rr,tt,mm,ffi,ffv,ffr,ss,yrr;//filename given?
//	int nfold=0;//nr of crossvalidation parts
	int sigcycles=0;
	bool cv,modelsel;//crossvalidation active? modelselection active?
	bool ocv;
	bool mpset;//warning, if options will be overwritten
	bool makeroc;
	bool balanceY;//enforce b=0=mean(y), maybe helpful for unbalanced datasets
	ocv=false;
	cv=false;modelsel=false;
	ll=false;cc=false;rr=false;tt=false;mm=false;ffi=false;ffv=false;ffr=false;ss=false;
	yrr=false;
	makeroc=false;
	mpset=false;
	balanceY=false;
	Dataset lset,tset,cset,rset,sset;
	lname=new char[200];//filename of trainingdata
	cname=new char[200];//filename of dataset to get class label predictions
	rname=new char[200];//filename of dataset to get real valued predictions
	tname=new char[200];//filename of dataset to test
	model=new char[200];//filename of trained or loaded model
	finame=new char[200];//filename of selected feature indices
	fvname=new char[200];//filename of selected feature vectors
	frname=new char[200];//filename of selected feature indices as ranking
	sname=new char[200];//filename of dataset to get statistics
/*
*/
	welcome();
//log
	relogg();
	flog=fopen("logfile"C_ENDING,"a");
	check_f(flog);
//parse
	int i;
	{//print datetime into logfile
		char* fttime=new char[200];
		time_t fttm;
		struct tm *newtime;
		time(&fttm);
		newtime=localtime(&fttm);
		strftime(fttime,200,"%#c",newtime);
		finfo(flog,"%s\n",fttime);
		delete[] fttime;
	}
	for(i=0;i<argc;i++)
		finfo(flog,"%s ",argv[i]);
	finfo(flog,"\n::\n");
	for(i=1;i<argc;)
	{//argv[0] is program name
		if(i>=argc)
			exit_with_help(&psvm);
		i++;
		if(strcmp(argv[i-1],"-train")==0 && i<argc){//training (regression and classification error output, retrieving regression model)
			strcpy(lname,argv[i++]);//learnset
			ll=true;
			psvm.par.doclass=true;
			psvm.par.doregress=true;
		}else if(strcmp(argv[i-1],"-trainc")==0 && i<argc){//training classification
			strcpy(lname,argv[i++]);//learnset
			ll=true;
			psvm.par.doclass=true;
			psvm.par.doregress=false;
		}else if(strcmp(argv[i-1],"-trainr")==0 && i<argc){//training regression
			strcpy(lname,argv[i++]);//learnset
			psvm.par.doclass=false;
			psvm.par.doregress=true;
			ll=true;
		}else if(strcmp(argv[i-1],"-predictc")==0 && i<argc){
			strcpy(cname,argv[i++]);//prediction of class labels
			cc=true;
		}else if(strcmp(argv[i-1],"-predict")==0 && i<argc){
			strcpy(rname,argv[i++]);//prediction of real values
			rr=true;
		}else if(strcmp(argv[i-1],"-predictr")==0 && i<argc){
			strcpy(rname,argv[i++]);//prediction of real values
			rr=true;
		}else if(strcmp(argv[i-1],"-b")==0){
			//enforce b=0
			psvm.par.dobalanceY=true;
			balanceY=true;
		}else if(strcmp(argv[i-1],"-r")==0 && i<argc){
			psvm.par.doretrain = true;
			psvm.par.retrain_fe = atof(argv[i++]);
		}else if(strcmp(argv[i-1],"-test")==0 && i<argc){//testing given dataset with current model (retrieve ROC)
			strcpy(tname,argv[i++]);//testset
			tt=true;
		}else if(strcmp(argv[i-1],"-stats")==0 && i<argc){
			strcpy(sname,argv[i++]);//get statistics for sname
			ss=true;
		}else if(strcmp(argv[i-1],"-model")==0 && i<argc){//saves learned model to disc or loads model from disc
			strcpy(model,argv[i++]);//model filename
			mm=true;
		}else if(strcmp(argv[i-1],"-fi")==0 && i<argc){
			strcpy(finame,argv[i++]);//save feature indices
			ffi=true;
		}else if(strcmp(argv[i-1],"-fv")==0 && i<argc){
			strcpy(fvname,argv[i++]);//save feature vectors
			ffv=true;
		}else if(strcmp(argv[i-1],"-frank")==0 && i<argc){
			strcpy(frname,argv[i++]);//save feature ranking
			ffr=true;
//		}else if(strcmp(argv[i-1],"-roc")==0 && i<argc){//ROC is default in testing
//			strcpy(frname,argv[i++]);//roc curve
//			makeroc=true;
		}else if(strcmp(argv[i-1],"-crossval")==0 && i<argc){//n-fold crossvalidation
			if(i<argc) if(argv[i][0]!='-' || (argv[i][1]==0 || !isalpha(argv[i][1])))//negative amounts must be followed by [a-z]
				psvm.par.infold = atoi(argv[i++]);
			cv=!(psvm.par.infold==0 || psvm.par.infold==1);
			if(i<argc) if(argv[i][0]!='-' || (argv[i][1]==0 || !isalpha(argv[i][1])))//negative amounts must be followed by [a-z]
				psvm.par.outfold = atoi(argv[i++]);//ranking-mode currently not used
			ocv=!(psvm.par.outfold==0 || psvm.par.outfold==1);
		}else if(strcmp(argv[i-1],"-sig")==0 && i<argc){
			sigcycles = atoi(argv[i++]);
			yrr=true;//significance testing with randomly permutated label vector y 
		}else if(strcmp(argv[i-1],"-c")==0 && i<argc){//set hyperparameter c, possibility to give a range for grid search
			psvm.par.C= (ftype)atof(argv[i++]);//C
			psvm.par.ms_c1=psvm.par.C;
			psvm.par.ms_c2=psvm.par.C;
			psvm.par.ms_cd=1;
			if(i<argc) if(argv[i][0]!='-' || (argv[i][1]==0 || !isalpha(argv[i][1])))//negative amounts must be followed by [a-z]
			{
				psvm.par.ms_c2=(ftype)atof(argv[i++]);
				modelsel=true;
			}
			if(i<argc) if(argv[i][0]!='-' || (argv[i][1]==0 || !isalpha(argv[i][1])))//negative amounts must be followed by [a-z]
				psvm.par.ms_cd=(ftype)atof(argv[i++]);
		}else if(strcmp(argv[i-1],"-e")==0 && i<argc){//set hyperparameter e, possibility to give a range for grid search
			psvm.par.epsi= (ftype)atof(argv[i++]);//epsilon
			psvm.par.ms_e1=psvm.par.epsi;
			psvm.par.ms_e2=psvm.par.epsi;
			psvm.par.ms_ed=1;
			if(i<argc) if(argv[i][0]!='-' || (argv[i][1]==0 || !isalpha(argv[i][1])))//negative amounts must be followed by [a-z]
			{
				psvm.par.ms_e2=(ftype)atof(argv[i++]);
				modelsel=true;
			}
			if(i<argc) if(argv[i][0]!='-' || (argv[i][1]==0 || !isalpha(argv[i][1])))//negative amounts must be followed by [a-z]
				psvm.par.ms_ed=(ftype)atof(argv[i++]);
		}else if(strcmp(argv[i-1],"-g")==0 && i<argc){
			psvm.par.epsiTol= (ftype)atof(argv[i++]);//epsilon tolerance for setting termination criterion
		}else if(strcmp(argv[i-1],"-flimit")==0 && i<argc){
			psvm.par.flimit= atoi(argv[i++]);// limit features
			if(i<argc) if(argv[i][0]!='-' || (argv[i][1]==0 || !isalpha(argv[i][1])))//negative amounts must be followed by [a-z]
				psvm.par.allowCcut=atoi(argv[i++]);
		}else if(strcmp(argv[i-1],"-shuffle")==0 && i<argc){
			psvm.par.shuffle= atoi(argv[i++]);// shuffle mode
		}else if(strcmp(argv[i-1],"-anneal")==0 && i<argc){
			psvm.par.doannealing= atoi(argv[i++]);// epsilon annealing mode
			if(i<argc) if(argv[i][0]!='-' || (argv[i][1]==0 || !isalpha(argv[i][1])))//negative amounts must be followed by [a-z]
				psvm.par.estart=(ftype)atof(argv[i++]);
			if(i<argc) if(argv[i][0]!='-' || (argv[i][1]==0 || !isalpha(argv[i][1])))//negative amounts must be followed by [a-z]
				psvm.par.edec=(ftype)atof(argv[i++]);
			if(i<argc) if(argv[i][0]!='-' || (argv[i][1]==0 || !isalpha(argv[i][1])))
				psvm.par.eterm=(ftype)atof(argv[i++]);
		}else if(strcmp(argv[i-1],"-block")==0 && i<argc){
			psvm.par.doblock= atoi(argv[i++]);// multiple alpha opt mode
			if(i<argc) if(argv[i][0]!='-' || (argv[i][1]==0 || !isalpha(argv[i][1])))//negative amounts must be followed by [a-z]
				psvm.par.blockNmax=atoi(argv[i++]);
			if(i<argc) if(argv[i][0]!='-' || (argv[i][1]==0 || !isalpha(argv[i][1])))//negative amounts must be followed by [a-z]
				psvm.par.blockCmax=atoi(argv[i++]);
			if(i<argc) if(argv[i][0]!='-' || (argv[i][1]==0 || !isalpha(argv[i][1])))//negative amounts must be followed by [a-z]
				psvm.par.blockOver=atoi(argv[i++]);
		}else if(strcmp(argv[i-1],"-wss")==0 && i<argc){
			psvm.par.domaxselection= atoi(argv[i++]);// wss-selection
			if(i<argc) if(argv[i][0]!='-' || (argv[i][1]==0 || !isalpha(argv[i][1])))
				psvm.par.selectionfact=atof(argv[i++]);
		}else if(strcmp(argv[i-1],"-reduce")==0 && i<argc){
			psvm.par.doreduction= atoi(argv[i++]);// reduction mode
			if(i<argc) if(argv[i][0]!='-' || (argv[i][1]==0 || !isalpha(argv[i][1])))//negative amounts must be followed by [a-z]
				psvm.par.redsize=atoi(argv[i++]);
			if(i<argc) if(argv[i][0]!='-' || (argv[i][1]==0 || !isalpha(argv[i][1])))
				psvm.par.redfact=atof(argv[i++]);
		}else if(strcmp(argv[i-1],"-timeout")==0 && i<argc){
			psvm.par.timeout= atoi(argv[i++]);// timeout
		}else if(strcmp(argv[i-1],"-kernel")==0 && i<argc){
			psvm.par.kernel= atoi(argv[i++]);// kernel aquired,
			if(i<argc) if(argv[i][0]!='-' || (argv[i][1]==0 || !isalpha(argv[i][1])))//negative amounts must be followed by [a-z]
				psvm.par.k1=atof(argv[i++]);
			if(i<argc) if(argv[i][0]!='-' || (argv[i][1]==0 || !isalpha(argv[i][1])))
				psvm.par.k2=atof(argv[i++]);
			if(i<argc) if(argv[i][0]!='-' || (argv[i][1]==0 || !isalpha(argv[i][1])))
				psvm.par.k3=atof(argv[i++]);
			mpset=true;//give warning if option overwritten by loaded model
		}else if(strcmp(argv[i-1],"-isrelation")==0){
			psvm.par.isrelation=true;
			mpset=true;//give warning if option overwritten by loaded model
		}else if(strcmp(argv[i-1],"-uselimit")==0){
			psvm.par.useNormLimit=true;
		}else if(strcmp(argv[i-1],"-be")==0){
			psvm.par.useBalancedErrors=true;//use balanced errors!!
		}else if(strcmp(argv[i-1],"-verbose")==0){
			verbose=true; verb_ms=true;
			int vvv;
			if(i<argc) if(argv[i][0]!='-' || (argv[i][1]==0 || !isalpha(argv[i][1])))
			{	vvv=atoi(argv[i++]);if(vvv==0){verbose=false;verb_ms=false;}else if(vvv==1){verbose=true;verb_ms=true;}else if(vvv==2){verbose=false;verb_ms=true;}}
		}else
			exit_with_help(&psvm);
	}
	if(argc<=1) 
	{
		exit_with_help(&psvm);
	}
	verbimp=true;//print important messages from psvm-library
	//verbose is controlled by user command, default is off
	if(ss)//-stats
	{//show statistics
		char* sx,*sy,*sz;
		sx=new char[200];
		sy=new char[200];
		sz=new char[200];
		strcpy(sx,sname);
		strcpy(sy,sname);
		strcpy(sz,sname);
		strcat(sx,".data"C_ENDING);
		strcat(sy,".y"C_ENDING);
		strcat(sz,".zdata"C_ENDING);
		Dataset zset;
		if(!sset.load(sx,sy,psvm.par.isrelation?1:0))
		{
			hinfo("data files %s and %s not found!\n",sx,sy);
			exit(0);
		}
		if(balanceY) sset.balance();
		//build kernel
		if(psvm.par.isrelation)
		{//X is relational(dyadic matrix) data -> determine mean of X
			//X is transposed!!
			int i;
			double sca_mean,sca_min,sca_max,sca_std;
			int nnn;
			double sca;
			sca_mean=0;
			sca_std=0;
			{
				sca=(double)sset.mData[0];
			}
			sca_min=sca_max=sca;
			nnn=sset.mN*sset.mDim;
			for(i=0;i<nnn;i++)
			{
				sca=(double)sset.mData[i];
				sca_min=min(sca,sca_min);
				sca_max=max(sca,sca_max);
				sca_mean+=sca;
				sca_std+=sqr(sca);
			}
			sca_mean/=nnn;
			sca_std/=nnn;
			sca_std=sqrt(sca_std-sqr(sca_mean));
			hinfo("   relational data: mean=%g\n"
					"                    stddev=%g\n"
				"                    min=%g\n"
				"                    max=%g\n",sca_mean,sca_std,sca_min,sca_max);
			hinfo("-> relational kernel factors should be about %g \n",1/sca_mean);
		}else{//load feature vectors from z or x, not transposed!!
			if(zset.load(sz))
			{//try loading complex feature vectors Z, otherwise use Z=X
				if(zset.mDim!=sset.mDim)
				{//Z compatible with X?
					hinfo("zdata dimensionality does not match with main dataset!\n");
					exit(-1);
				}
				{// determine the mean of <Xi,Zj> and (Xi-Zj)^2 for optimum kernel parameters
					int p,l;
					double euk_mean,euk_min,euk_max,euk_std;
					double sca_mean,sca_min,sca_max,sca_std;
					int nnn;
					double sca,euk;
					euk_mean=0;
					sca_mean=0;
					euk_std=0;
					sca_std=0;
					{
						sca=0;
						for(i=0;i<sset.mDim;i++) sca+=(double)sset.mData[i]*(double)zset.mData[i];
						euk=0;
						for(i=0;i<sset.mDim;i++) euk+=sqr((double)sset.mData[i]-(double)zset.mData[i]);
					}
					euk_min=euk_max=euk;
					sca_min=sca_max=sca;
					nnn=sset.mN*zset.mN;
					for(l=0;l<sset.mN;l++)for(p=0;p<zset.mN;p++)
					{
						ftype*ldata,*pdata;
						ldata=&sset.mData[l*sset.mDim];
						pdata=&zset.mData[p*sset.mDim];
						sca=0;
						for(i=0;i<sset.mDim;i++) sca+=(double)ldata[i]*(double)pdata[i];
						euk=0;
						for(i=0;i<sset.mDim;i++) euk+=sqr((double)ldata[i]-(double)pdata[i]);
						euk_min=min(euk,euk_min);
						euk_max=max(euk,euk_max);
						sca_min=min(sca,sca_min);
						sca_max=max(sca,sca_max);
						euk_mean+=euk;
						sca_mean+=sca;
						euk_std+=sqr(euk);
						sca_std+=sqr(sca);
					}
					euk_mean/=nnn;
					sca_mean/=nnn;
					euk_std=sqrt(euk_std-sqr(euk_mean));
					sca_std=sqrt(sca_std-sqr(sca_mean));
					hinfo("scalar product kernel: mean=%g\n"
						"                       stddev=%g\n"
						"                       min=%g\n"
						"                       max=%g\n",sca_mean,sca_std,sca_min,sca_max);
					hinfo("-> scalarproduct kernel factors should be about %g \n",1/sca_mean);
					hinfo("euklidian distance kernel: mean=%g\n"
						"                      stddev=%g\n"
						"                      min=%g\n"
						"                      max=%g\n",euk_mean,euk_std,euk_min,euk_max);
					hinfo("-> euklidian dist kernel factors should be about %g \n",1/euk_mean);
				}//nothing
				psvm.kernel(&sset,&zset);
			}else//no complex feature vectors available (Z)
			{// determine the mean of <Xi,Xj> and (Xi-Xj)^2 for optimum kernel parameters
				int p,l;
				double euk_mean,euk_min,euk_max,euk_std;
				double sca_mean,sca_min,sca_max,sca_std;
				int nnn;
				double euk,sca;
				euk_mean=0;
				sca_mean=0;
				euk_std=0;
				sca_std=0;
				{
					sca=0;
					for(i=0;i<sset.mDim;i++) sca+=(double)sset.mData[i]*(double)sset.mData[i];
					euk=0;
					for(i=0;i<sset.mDim;i++) euk+=sqr((double)sset.mData[i]-(double)sset.mData[i]);
				}
				euk_min=euk_max=euk;
				sca_min=sca_max=sca;
				nnn=sset.mN*sset.mN;
				for(l=0;l<sset.mN;l++)for(p=l;p<sset.mN;p++)
				{
					ftype*ldata,*pdata;
					ldata=&sset.mData[l*sset.mDim];
					pdata=&sset.mData[p*sset.mDim];
					sca=0;
					for(i=0;i<sset.mDim;i++) sca+=(double)ldata[i]*(double)pdata[i];
					euk=0;
					for(i=0;i<sset.mDim;i++) euk+=sqr((double)ldata[i]-(double)pdata[i]);
					euk_min=min(euk,euk_min);
					euk_max=max(euk,euk_max);
					sca_min=min(sca,sca_min);
					sca_max=max(sca,sca_max);
					euk_mean+=euk;
					sca_mean+=sca;
					euk_std+=sqr(euk);
					sca_std+=sqr(sca);
					if(p!=l)
					{
						euk_mean+=euk;
						sca_mean+=sca;
						euk_std+=sqr(euk);
						sca_std+=sqr(sca);
					}
				}
				euk_mean/=nnn;
				sca_mean/=nnn;
				euk_std/=nnn;
				sca_std/=nnn;
				euk_std=sqrt(euk_std-sqr(euk_mean));
				sca_std=sqrt(sca_std-sqr(sca_mean));
				hinfo("scalar product kernel: mean=%g\n"
					"                       stddev=%g\n"
					"                       min=%g\n"
					"                       max=%g\n",sca_mean,sca_std,sca_min,sca_max);
				hinfo("-> scalarproduct kernel factors should be about %g \n",1/sca_mean);
				hinfo("euklidian distance kernel: mean=%g\n"
					"                      stddev=%g\n"
					"                      min=%g\n"
					"                      max=%g\n",euk_mean,euk_std,euk_min,euk_max);
				hinfo("-> euklidian dist kernel factors should be about %g \n",1/euk_mean);
			}//relational or not
			psvm.kernel(&sset);
		}
		//normalize data after computing kernel function
		sset.normalize();
		psvm.getB(&sset);
		{//compute K*Y
			int p,l;
			double maxepsi,lsum;
			maxepsi=0;
			for (p=0;p<sset.mDim;p++)//row size of kernel matrix
			{
				//compute F for alpha=0
				ftype *sdata;
				sdata=&sset.mData[p*sset.mN];
				lsum=0;
				for (l=0;l<sset.mN;l++)//col size of kernel matrix and vector y
					lsum+=sset.mY[l] * sdata[l];
				maxepsi=max(fabs(lsum),maxepsi);
			}
			hinfo("maximum epsilon to find at least one feature: %g \n",maxepsi);
		}
		{//compute largest alpha after training with max-epsilon to get an upper limit for C affecting the predictor
			double maxc;
			psvm.par.C=1e15;
			psvm.par.epsi=-1;
			psvm.getRankFromSet(&sset);
			maxc=fabs((double)psvm.rank.mFeat.front().a);
			hinfo("maximum C which affects the predictor if epsilon=%g : %g\n",psvm.par.epsi,maxc);
		}
		{//calculate errors for trivial models
			double rvar,cvar;
			int pn;
			rvar=0;
			pn=0;
			for(i=0;i<sset.mN;i++)
			{
				rvar+=sqr(psvm.b-sset.mY[i]);
				if(sset.mY[i]>0) pn++;
			}
			rvar=(rvar/sset.mN);
			pn=min(pn,sset.mN-pn);
			cvar=(double)pn/sset.mN;
			hinfo("MSE for trivial estimator = variance(y): %g\n",rvar);
			hinfo("Error-rate for trivial classificator : %g\n",cvar);
		}
		delete[]sx;
		delete[]sy;
		delete[]sz;
	}//get statistics ready
	if(ll)//-train
	{//learn model
		Trainres trainres;//for retrieving training results in psvm.getModel
		char* lx,*ly,*lz;
		bool getmodel;
		lx=new char[200];
		ly=new char[200];
		lz=new char[200];
		strcpy(lx,lname);
		strcpy(ly,lname);
		strcpy(lz,lname);
		strcat(lx,".data"C_ENDING);
		strcat(ly,".y"C_ENDING);
		strcat(lz,".zdata"C_ENDING);
		Dataset zset;//support vectorset
		Dataset svsave;//saves support vectors
		if(!lset.load(lx,ly,psvm.par.isrelation?1:0))// transpose relational kernel-like data
		{//error loading dataset
			hinfo("data files %s and %s not found!\n",lx,ly);
			exit(0);
		}
		if(balanceY) lset.balance();//rescale y vector to enforce mean(y)=0, maintain the signs(classes)!!
		//build kernel
		clock_t t0,t1;
		t0=clock();
		if(psvm.par.isrelation)
		{//lset is relational data, no support feature vectors
			psvm.kernel(&lset);//lset is transposed
		}else{
			if(zset.load(lz))
			{//complex feature vectors Z available??
				svsave.copyFrom(&zset);//for retrieving support vectors
				psvm.kernel(&lset,&zset);//both not transposed ->kernel in lset is transposed
			}else{//Z=X
				svsave.copyFrom(&lset);//for retrieving support vectors
				psvm.kernel(&lset);//not transposed -> transposed kernel
				//this case uses the stdkernel flag in lset, 
				//that is for correct tiling in lset.tileTo (crossvalidation)
			}
		}
		hinfo("Kernel time: %f \n",timeDiff(t0,clock()));
		//lset - transposed relational kernel matrix
		//svsave - possible set of support vectors (only non-relational mode)
		t1=clock();
		lset.shuffle(psvm.par.shuffle);//permutate lines for crossvalidation with big testset
		hinfo("Shuffle time: %f \n",timeDiff(t1,clock()));
		//note that only the data objects become permutated, the references to features or SVs remain valid.
		//lset is kernel matrix with shuffled lines and is transposed:
		//lset=[(n=1,dim=1)...(n=1,dim=Dim)
		//       |         ...   |
		//       v         ...   v
		//      (n=N,dim=1)...(n=N,dim=Dim)]
		//svsave is for non-relational mode and holds non-shuffled column vectors
	//	psvm.np.getFromSet(&lset);//before all routines normalize lset
		if(modelsel) 
		{	//prepare hyperparameterselection-plot script files
			char* lms;
			FILE* fplot;
			lms=new char[200];
			//write gnuplotscript modelselection
			strcpy(lms,lname);
			strcat(lms,".msclass.plt");
			fplot=fopen(lms,"w");
			check_f(fplot);
			//calculate real crossvalidation cycles
			int nfold;
			nfold=psvm.par.infold;
			if(nfold>lset.mN||nfold<0) nfold=lset.mN;
			if(nfold==0) nfold=1;
			//print file
			if(cv)
			{//toDo: outfold??????????????????????????????
				finfo(fplot,"set cntrparam levels auto\n set contour base\n set clabel 'contour line %%8.3g'\n set hidden3d offset 2\n"
					"set ylabel 'epsilon'\n set xlabel 'C'\n set zlabel 'error'\n set view 60,100,1,1\n"
					"splot '%s.msdata"C_ENDING"' using 2:1:3 title '%d-fold cv-MSE classification dataset <%s>' with lines\n"
					"set term postscript eps enhanced color solid\n set output '%s.msclass.eps'\n replot\n"
					"pause -1\n",lname,nfold,lname,lname);
			}else{
				finfo(fplot,"set cntrparam levels auto\n set contour base\n set clabel 'contour line %%8.3g'\n set hidden3d offset 2\n"
					"set ylabel 'epsilon'\n set xlabel 'C'\n set zlabel 'error'\n set view 60,100,1,1\n"
					"splot '%s.msdata"C_ENDING"' using 2:1:3 title 'training-error rate classification dataset <%s>' with lines\n"
					"set term postscript eps enhanced color solid\n set output '%s.msclass.eps'\n replot\n"
					"pause -1\n",lname,lname,lname);
			}
			fclose(fplot);
			strcpy(lms,lname);
			strcat(lms,".msregress.plt");
			fplot=fopen(lms,"w");
			check_f(fplot);
			if(cv)
			{
				finfo(fplot,"set cntrparam levels auto\n set contour base\n set clabel 'contour line %%8.3g'\n set hidden3d offset 2\n"
					"set ylabel 'epsilon'\n set xlabel 'C'\n set zlabel 'error'\n set view 60,100,1,1\n"
					"splot '%s.msdata"C_ENDING"' using 2:1:4 title '%d-fold cv-MSE regression dataset <%s>' with lines\n"
					"set term postscript eps enhanced color solid\n set output '%s.msregress.eps'\n replot\n"
					"pause -1\n",lname,nfold,lname,lname);
			}else{
				finfo(fplot,"set cntrparam levels auto\n set contour base\n set clabel 'contour line %%8.3g'\n set hidden3d offset 2\n"
					"set ylabel 'epsilon'\n set xlabel 'C'\n set zlabel 'error'\n set view 60,100,1,1\n"
					"splot '%s.msdata"C_ENDING"' using 2:1:4 title 'training-error rate regression dataset <%s>' with lines\n"
					"set term postscript eps enhanced color solid\n set output '%s.msregress.eps'\n replot\n"
					"pause -1\n",lname,lname,lname);
			}
			fclose(fplot);
			delete[] lms;
		}
		trainres.ms=NULL;
		t1=clock();
		if(mm || ffi || ffr || ffv || cc || rr || tt) getmodel=true;else getmodel=false;
		//this software is for hyperparameter estimation, generalization errors - if a model is required, additional calculations are required.
		if(!psvm.getModelWithLoops(&lset,&trainres,getmodel))//calculates all required model components (alpha, b, np)
			hinfo("one or more of the training phases were stopped after timeout!\n");
		hinfo("Training time: %f \n",timeDiff(t1,clock()));
		hinfo("Total time: %f \n",timeDiff(t0,clock()));
		hinfo("Maximum Feature Intensity: %f \n",trainres.maxFeat);
		/*results for:
		ocv ms iCV getmodel
		-   -   -   -   nonsense (---1)
		-   -   -   1   T-error, alpha
		-   -   1   -   n*(T'-error,alpha',Ptest) Gi-error
		-   -   1   1   n*(T'-error,alpha',Ptest) Gi-error, retraining: T-error, alpha
		-   1   -	-   nonsense (-1-1)
		-   1   -   1   m*(T-error,alpha), best(T-error,C,E,alpha)
		-   1   1   -   m*(n*(T'-error,alpha',Ptest) Gi-error), best(Gi-error,C,E)
		-   1   1   1   m*(n*(T'-error,alpha',Ptest) Gi-error), best(Gi-error,C,E), retraining(--11): T-error, alpha
		1   -   -   -   nonsense (--1-)
		1   -   -   1   nonsense (--11)
		1   -   1   -   nonsense n1*(n*(T-error,alpha,Ptest),G-error, retraining Ptest) G-error --- zur Bestimmung der Abweichung der G-error zur Gre ???
		1   -   1   1   nonsense n1*(n*(T-error,alpha,Ptest),G-error, retraining Ptest) G-error --- zur Bestimmung der Abweichung der G-error zur Gre ???
		1   1   -   -   n1*(m*(T'-error,alpha'), best(T'-error,C,E,alpha'), Ptest), Go-error
		1   1   -   1   n1*(m*(T'-error,alpha'), best(T'-error,C,E,alpha'), Ptest), Go-error, retraining(-1-1)
		1   1   1   -   n1*(m*(n*(T''-error,alpha'',Ptest) Gi'-error), best(Gi'-error,C,E), retraining(CE)' Ptest) Goi-error 
		1   1   1   1   n1*(m*(n*(T''-error,alpha'',Ptest) Gi'-error), best(Gi'-error,C,E), retraining(CE)' Ptest) Goi-error, retraining(-111)
		possible results: T-error(reg,cla,fp,tp,fn,tn), alpha, C,E, G-error
		additional infos:
			best hyperparameters: min max, mean
			best models nr. of features: min max mean
			n1,m,n loop 
			stat. ranking of all cv-models alpha'
			stat result Ptest: samples vs. error stat.
		opt.
			all cv-models
		//modelsel+CV: trainres:C,epsilon,G-error, psvm: undefined
		//CV: trainres:G-error, psvm: undefined
		//modelsel: trainres: C,epsilon,T-error,rank psvm: b,np,rank
		//simple training: trainres: T-error,rank psvm: b,np,rank
		*/

		//print results:
//		hinfo("maximum alpha (for checking influence of C): %g\n",trainres.maxFeat);
		if(psvm.par.useBalancedErrors)
			hinfo("All classification errors are based on balanced errors!\n");
		else
			hinfo("All classification errors are based on unbalanced errors!\n");
		if(ocv)
		{
            if(modelsel)
			{
				if(cv)
				{
					hinfo("Hyperparameter selection done with respect to crossvalidation error:\n");
					hinfo("Generalization measurement includes hyperparameter selection\n");
					if(psvm.par.doregress)hinfo("reg: mean epsi: %g [%g, %g], mean C: %g [%g, %g], mean feat: %d, G-MSE %g\n",trainres.reg_Ge,trainres.e_min,trainres.e_max,trainres.reg_Gc,trainres.c_min,trainres.c_max,trainres.reg_Gnf,trainres.reg_GError);
					if(psvm.par.doclass)hinfo("cls: mean epsi: %g [%g, %g], mean C: %g [%g, %g], mean feat: %d, G-err %g\n",trainres.cla_Ge,trainres.e_min,trainres.e_max,trainres.cla_Gc,trainres.c_min,trainres.c_max,trainres.cla_Gnf,trainres.cla_GError);
//					if(psvm.par.doclass)hinfo("cls: mean accuracy: %g %g \n",trainres.cla_Gacc,trainres.cla_Gerrb);
					if(psvm.par.doclass)hinfo("cls: mean true pos: %d, mean true neg: %d, mean false pos: %d, mean false neg: %d, total %d\n",trainres.cla_Gtp,trainres.cla_Gtn,trainres.cla_Gfp,trainres.cla_Gfn);
					if(psvm.par.doclass)hinfo("cls: mean accuracy: %g, mean number of missclassified objects: %d of %d\n",1-trainres.cla_GError,(trainres.cla_Gfp+trainres.cla_Gfn),(trainres.cla_Gtp+trainres.cla_Gtn+trainres.cla_Gfp+trainres.cla_Gfn));
					if(getmodel==false)
						hinfo("No more model calculations aquired!\n");
					else{
						hinfo("Using this hyperparameter selection method leads to:\n");
						if(psvm.par.doregress)hinfo("reg: epsi: %g [%g, %g], C: %g [%g, %g], feat: %d, T-MSE %g\n",trainres.reg_Te,trainres.e_min,trainres.e_max,trainres.reg_Tc,trainres.c_min,trainres.c_max,trainres.reg_Tnf,trainres.reg_TError);
						if(psvm.par.doclass)hinfo("cls: epsi: %g [%g, %g], C: %g [%g, %g], feat: %d, T-err %g\n",trainres.cla_Te,trainres.e_min,trainres.e_max,trainres.cla_Tc,trainres.c_min,trainres.c_max,trainres.cla_Tnf,trainres.cla_TError);
//						if(psvm.par.doclass)hinfo("cls: accuracy: %g, balanced: %g, tp: %d, tn: %d, fp: %d, fn: %d\n",trainres.cla_Tacc,trainres.cla_Taccb,trainres.cla_Ttp,trainres.cla_Ttn,trainres.cla_Tfp,trainres.cla_Tfn);
//						if(psvm.par.doclass)hinfo("cls: accuracy: %g, balanced accuracy: %g, balanced error: %g\n",trainres.cla_Tacc,trainres.cla_Taccb,trainres.cla_Terrb);
						if(psvm.par.doclass)hinfo("cls: true pos: %d, true neg: %d, false pos: %d, false neg: %d\n",trainres.cla_Ttp,trainres.cla_Ttn,trainres.cla_Tfp,trainres.cla_Tfn);
						if(psvm.par.doclass)hinfo("cls: mean accuracy: %g, mean number of missclassified objects: %d of %d\n",1-trainres.cla_TError,(trainres.cla_Tfp+trainres.cla_Tfn),(trainres.cla_Ttp+trainres.cla_Ttn+trainres.cla_Tfp+trainres.cla_Tfn));
	//					if(psvm.par.doclass)hinfo("cls: mean accuracy: %g, mean number of missclassified objects: %d of %d\n",1-trainres.cla_GError,trainres.cla_misses,lset.mN);
					}
				}else
				{
					hinfo("Hyperparameter selection done with respect to training error with flimit=%d:\n",psvm.par.flimit);
					hinfo("Generalization measurement includes hyperparameterselection\n");
					if(psvm.par.doregress)hinfo("reg: mean epsi: %g [%g, %g], mean C: %g [%g, %g], mean feat: %d, G-MSE %g\n",trainres.reg_Ge,trainres.e_min,trainres.e_max,trainres.reg_Gc,trainres.c_min,trainres.c_max,trainres.reg_Gnf,trainres.reg_GError);
					if(psvm.par.doclass)hinfo("cls: mean epsi: %g [%g, %g], mean C: %g [%g, %g], mean feat: %d, G-err %g\n",trainres.cla_Ge,trainres.e_min,trainres.e_max,trainres.cla_Gc,trainres.c_min,trainres.c_max,trainres.cla_Gnf,trainres.cla_GError);
//					if(psvm.par.doclass)hinfo("cls: mean accuracy: %g, balanced: %g, tp: %d, tn: %d, fp: %d, fn: %d\n",trainres.cla_Gacc,trainres.cla_Gaccb,trainres.cla_Gtp,trainres.cla_Gtn,trainres.cla_Gfp,trainres.cla_Gfn);
//					if(psvm.par.doclass)hinfo("cls: mean accuracy: %g, mean balanced accuracy: %g, mean balanced error: %g\n",trainres.cla_Gacc,trainres.cla_Gaccb,trainres.cla_Gerrb);
					if(psvm.par.doclass)hinfo("cls: mean true pos: %d, mean true neg: %d, mean false pos: %d, mean false neg: %d\n",trainres.cla_Gtp,trainres.cla_Gtn,trainres.cla_Gfp,trainres.cla_Gfn);
					if(psvm.par.doclass)hinfo("cls: mean accuracy: %g, mean number of missclassified objects: %d of %d\n",1-trainres.cla_GError,(trainres.cla_Gfp+trainres.cla_Gfn),(trainres.cla_Gtp+trainres.cla_Gtn+trainres.cla_Gfp+trainres.cla_Gfn));
//					if(psvm.par.doclass)hinfo("cls: mean accuracy: %g, mean number of missclassified objects: %d of %d\n",1-trainres.cla_GError,trainres.cla_misses,lset.mN);
					if(getmodel==false)
						hinfo("No more model calculations aquired!\n");
					else{
						hinfo("Using this hyperparameterselection method leads to:\n");
						if(psvm.par.doregress)hinfo("reg: epsi: %g [%g, %g], C: %g [%g, %g], feat: %d, T-MSE %g\n",trainres.reg_Te,trainres.e_min,trainres.e_max,trainres.reg_Tc,trainres.c_min,trainres.c_max,trainres.reg_Tnf,trainres.reg_TError);
						if(psvm.par.doclass)hinfo("cls: epsi: %g [%g, %g], C: %g [%g, %g], feat: %d, T-err %g\n",trainres.cla_Te,trainres.e_min,trainres.e_max,trainres.cla_Tc,trainres.c_min,trainres.c_max,trainres.cla_Tnf,trainres.cla_TError);
//						if(psvm.par.doclass)hinfo("cls: accuracy: %g, balanced: %g, tp: %d, tn: %d, fp: %d, fn: %d\n",trainres.cla_Tacc,trainres.cla_Taccb,trainres.cla_Ttp,trainres.cla_Ttn,trainres.cla_Tfp,trainres.cla_Tfn);
//						if(psvm.par.doclass)hinfo("cls: accuracy: %g, balanced accuracy: %g, balanced error: %g\n",trainres.cla_Tacc,trainres.cla_Taccb,trainres.cla_Terrb);
						if(psvm.par.doclass)hinfo("cls: true pos: %d, true neg: %d, false pos: %d, false neg: %d\n",trainres.cla_Ttp,trainres.cla_Ttn,trainres.cla_Tfp,trainres.cla_Tfn);
						if(psvm.par.doclass)hinfo("cls: mean accuracy: %g, mean number of missclassified objects: %d of %d\n",1-trainres.cla_TError,(trainres.cla_Tfp+trainres.cla_Tfn),(trainres.cla_Ttp+trainres.cla_Ttn+trainres.cla_Tfp+trainres.cla_Tfn));
	//					if(psvm.par.doclass)hinfo("cls: mean accuracy: %g, mean number of missclassified objects: %d of %d\n",1-trainres.cla_GError,trainres.cla_misses,lset.mN);
					}
				}
			}else{
				hinfo("Using both crossvalidation methods need a hyperparameter selection!!\n");
				if(cv)
				{
					hinfo("Hyperparameter selection done with respect to crossvalidation error:\n");
					hinfo("Generalization measurement includes hyperparameter selection\n");
					if(psvm.par.doregress)hinfo("reg: mean epsi: %g [%g, %g], mean C: %g [%g, %g], mean feat: %d, G-MSE %g\n",trainres.reg_Ge,trainres.e_min,trainres.e_max,trainres.reg_Gc,trainres.c_min,trainres.c_max,trainres.reg_Gnf,trainres.reg_GError);
					if(psvm.par.doclass)hinfo("cls: mean epsi: %g [%g, %g], mean C: %g [%g, %g], mean feat: %d, G-err %g\n",trainres.cla_Ge,trainres.e_min,trainres.e_max,trainres.cla_Gc,trainres.c_min,trainres.c_max,trainres.cla_Gnf,trainres.cla_GError);
//					if(psvm.par.doclass)hinfo("cls: mean accuracy: %g, balanced: %g, tp: %d, tn: %d, fp: %d, fn: %d\n",trainres.cla_Gacc,trainres.cla_Gaccb,trainres.cla_Gtp,trainres.cla_Gtn,trainres.cla_Gfp,trainres.cla_Gfn);
//					if(psvm.par.doclass)hinfo("cls: mean accuracy: %g, mean balanced accuracy: %g, mean balanced error: %g\n",trainres.cla_Gacc,trainres.cla_Gaccb,trainres.cla_Gerrb);
					if(psvm.par.doclass)hinfo("cls: mean true pos: %d, mean true neg: %d, mean false pos: %d, mean false neg: %d\n",trainres.cla_Gtp,trainres.cla_Gtn,trainres.cla_Gfp,trainres.cla_Gfn);
					if(psvm.par.doclass)hinfo("cls: mean accuracy: %g, mean number of missclassified objects: %d of %d\n",1-trainres.cla_GError,(trainres.cla_Gfp+trainres.cla_Gfn),(trainres.cla_Gtp+trainres.cla_Gtn+trainres.cla_Gfp+trainres.cla_Gfn));
//					if(psvm.par.doclass)hinfo("cls: mean accuracy: %g, mean number of missclassified objects: %d of %d\n",1-trainres.cla_GError,trainres.cla_misses,lset.mN);
					if(getmodel==false)
						hinfo("No more model calculations aquired!\n");
					else{
						hinfo("Using this hyperparameter selection method leads to:\n");
						if(psvm.par.doregress)hinfo("reg: epsi: %g [%g, %g], C: %g [%g, %g], feat: %d, T-MSE %g\n",trainres.reg_Te,trainres.e_min,trainres.e_max,trainres.reg_Tc,trainres.c_min,trainres.c_max,trainres.reg_Tnf,trainres.reg_TError);
						if(psvm.par.doclass)hinfo("cls: epsi: %g [%g, %g], C: %g [%g, %g], feat: %d, T-err %g\n",trainres.cla_Te,trainres.e_min,trainres.e_max,trainres.cla_Tc,trainres.c_min,trainres.c_max,trainres.cla_Tnf,trainres.cla_TError);
//						if(psvm.par.doclass)hinfo("cls: accuracy: %g, balanced: %g, tp: %d, tn: %d, fp: %d, fn: %d\n",trainres.cla_Tacc,trainres.cla_Taccb,trainres.cla_Ttp,trainres.cla_Ttn,trainres.cla_Tfp,trainres.cla_Tfn);
//						if(psvm.par.doclass)hinfo("cls: accuracy: %g, balanced accuracy: %g, balanced error: %g\n",trainres.cla_Tacc,trainres.cla_Taccb,trainres.cla_Terrb);
						if(psvm.par.doclass)hinfo("cls: true pos: %d, true neg: %d, false pos: %d, false neg: %d\n",trainres.cla_Ttp,trainres.cla_Ttn,trainres.cla_Tfp,trainres.cla_Tfn);
						if(psvm.par.doclass)hinfo("cls: mean accuracy: %g, mean number of missclassified objects: %d of %d\n",1-trainres.cla_TError,(trainres.cla_Tfp+trainres.cla_Tfn),(trainres.cla_Ttp+trainres.cla_Ttn+trainres.cla_Tfp+trainres.cla_Tfn));
	//					if(psvm.par.doclass)hinfo("cls: mean accuracy: %g, mean number of missclassified objects: %d of %d\n",1-trainres.cla_GError,trainres.cla_misses,lset.mN);
					}
				}else
				{
					hinfo("Hyperparameter selection done with respect to training error with flimit=%d:\n",psvm.par.flimit);
					hinfo("Generalization measurement includes hyperparameterselection\n");
					if(psvm.par.doregress)hinfo("reg: mean epsi: %g [%g, %g], mean C: %g [%g, %g], mean feat: %d, G-MSE %g\n",trainres.reg_Ge,trainres.e_min,trainres.e_max,trainres.reg_Gc,trainres.c_min,trainres.c_max,trainres.reg_Gnf,trainres.reg_GError);
					if(psvm.par.doclass)hinfo("cls: mean epsi: %g [%g, %g], mean C: %g [%g, %g], mean feat: %d, G-err %g\n",trainres.cla_Ge,trainres.e_min,trainres.e_max,trainres.cla_Gc,trainres.c_min,trainres.c_max,trainres.cla_Gnf,trainres.cla_GError);
//					if(psvm.par.doclass)hinfo("cls: mean accuracy: %g, balanced: %g, tp: %d, tn: %d, fp: %d, fn: %d\n",trainres.cla_Gacc,trainres.cla_Gaccb,trainres.cla_Gtp,trainres.cla_Gtn,trainres.cla_Gfp,trainres.cla_Gfn);
//					if(psvm.par.doclass)hinfo("cls: mean accuracy: %g, mean balanced accuracy: %g, mean balanced error: %g\n",trainres.cla_Gacc,trainres.cla_Gaccb,trainres.cla_Gerrb);
					if(psvm.par.doclass)hinfo("cls: mean true pos: %d, mean true neg: %d, mean false pos: %d, mean false neg: %d\n",trainres.cla_Gtp,trainres.cla_Gtn,trainres.cla_Gfp,trainres.cla_Gfn);
					if(psvm.par.doclass)hinfo("cls: mean accuracy: %g, mean number of missclassified objects: %d of %d\n",1-trainres.cla_GError,(trainres.cla_Gfp+trainres.cla_Gfn),(trainres.cla_Gtp+trainres.cla_Gtn+trainres.cla_Gfp+trainres.cla_Gfn));
//					if(psvm.par.doclass)hinfo("cls: mean accuracy: %g, mean number of missclassified objects: %d of %d\n",1-trainres.cla_GError,trainres.cla_misses,lset.mN);
					if(getmodel==false)
						hinfo("No more model calculations aquired!\n");
					else{
						hinfo("Using this hyperparameterselection method leads to:\n");
						if(psvm.par.doregress)hinfo("reg: epsi: %g [%g, %g], C: %g [%g, %g], feat: %d, T-MSE %g\n",trainres.reg_Te,trainres.e_min,trainres.e_max,trainres.reg_Tc,trainres.c_min,trainres.c_max,trainres.reg_Tnf,trainres.reg_TError);
						if(psvm.par.doclass)hinfo("cls: epsi: %g [%g, %g], C: %g [%g, %g], feat: %d, T-err %g\n",trainres.cla_Te,trainres.e_min,trainres.e_max,trainres.cla_Tc,trainres.c_min,trainres.c_max,trainres.cla_Tnf,trainres.cla_TError);
//						if(psvm.par.doclass)hinfo("cls: accuracy: %g, balanced: %g, tp: %d, tn: %d, fp: %d, fn: %d\n",trainres.cla_Tacc,trainres.cla_Taccb,trainres.cla_Ttp,trainres.cla_Ttn,trainres.cla_Tfp,trainres.cla_Tfn);
//						if(psvm.par.doclass)hinfo("cls: accuracy: %g, balanced accuracy: %g, balanced error: %g\n",trainres.cla_Tacc,trainres.cla_Taccb,trainres.cla_Terrb);
						if(psvm.par.doclass)hinfo("cls: true pos: %d, true neg: %d, false pos: %d, false neg: %d\n",trainres.cla_Ttp,trainres.cla_Ttn,trainres.cla_Tfp,trainres.cla_Tfn);
						if(psvm.par.doclass)hinfo("cls: mean accuracy: %g, mean number of missclassified objects: %d of %d\n",1-trainres.cla_TError,(trainres.cla_Tfp+trainres.cla_Tfn),(trainres.cla_Ttp+trainres.cla_Ttn+trainres.cla_Tfp+trainres.cla_Tfn));
	//					if(psvm.par.doclass)hinfo("cls: mean accuracy: %g, mean number of missclassified objects: %d of %d\n",1-trainres.cla_GError,trainres.cla_misses,lset.mN);
					}
				}
			}
		}else{
            if(modelsel)
			{
				if(cv)
				{
					hinfo("Hyperparameter selection done with respect to crossvalidation error:\n");
					hinfo("Generalization measurement does not include hyperparameterselection!!\n");
					if(psvm.par.doregress)hinfo("reg: epsi: %g [%g, %g], C: %g [%g, %g], feat: %d, G-MSE %g\n",trainres.reg_Ge,trainres.e_min,trainres.e_max,trainres.reg_Gc,trainres.c_min,trainres.c_max,trainres.reg_Gnf,trainres.reg_GError);
					if(psvm.par.doclass)hinfo("cls: epsi: %g [%g, %g], C: %g [%g, %g], feat: %d, G-err %g\n",trainres.cla_Ge,trainres.e_min,trainres.e_max,trainres.cla_Gc,trainres.c_min,trainres.c_max,trainres.cla_Gnf,trainres.cla_GError);
//					if(psvm.par.doclass)hinfo("cls: mean accuracy: %g, balanced: %g, tp: %d, tn: %d, fp: %d, fn: %d\n",trainres.cla_Gacc,trainres.cla_Gaccb,trainres.cla_Gtp,trainres.cla_Gtn,trainres.cla_Gfp,trainres.cla_Gfn);
//					if(psvm.par.doclass)hinfo("cls: mean accuracy: %g, mean balanced accuracy: %g, mean balanced error: %g\n",trainres.cla_Gacc,trainres.cla_Gaccb,trainres.cla_Gerrb);
					if(psvm.par.doclass)hinfo("cls: mean accuracy: %g, mean number of missclassified objects: %d of %d\n",1-trainres.cla_GError,(trainres.cla_Gfp+trainres.cla_Gfn),(trainres.cla_Gtp+trainres.cla_Gtn+trainres.cla_Gfp+trainres.cla_Gfn));
					if(psvm.par.doclass)hinfo("cls: mean true pos: %d, mean true neg: %d, mean false pos: %d, mean false neg: %d\n",trainres.cla_Gtp,trainres.cla_Gtn,trainres.cla_Gfp,trainres.cla_Gfn);
					if(getmodel==false)
						hinfo("No more model calculations aquired!\n");
					else{
						hinfo("Using these hyperparameters leads to:\n");
						if(psvm.par.doregress)hinfo("reg: epsi: %g, C: %g, feat: %d, T-MSE %g\n",trainres.reg_Te,trainres.reg_Tc,trainres.reg_Tnf,trainres.reg_TError);
						if(psvm.par.doclass)hinfo("cls: epsi: %g, C: %g, feat: %d, T-err %g\n",trainres.cla_Te,trainres.cla_Tc,trainres.cla_Tnf,trainres.cla_TError);
//						if(psvm.par.doclass)hinfo("cls: accuracy: %g balanced: %g, tp: %d, tn: %d, fp: %d, fn: %d\n",trainres.cla_Tacc,trainres.cla_Taccb,trainres.cla_Ttp,trainres.cla_Ttn,trainres.cla_Tfp,trainres.cla_Tfn);
//						if(psvm.par.doclass)hinfo("cls: accuracy: %g, balanced accuracy: %g, balanced error: %g\n",trainres.cla_Tacc,trainres.cla_Taccb,trainres.cla_Terrb);
						if(psvm.par.doclass)hinfo("cls: true pos: %d, true neg: %d, false pos: %d, false neg: %d\n",trainres.cla_Ttp,trainres.cla_Ttn,trainres.cla_Tfp,trainres.cla_Tfn);
						if(psvm.par.doclass)hinfo("cls: mean accuracy: %g, mean number of missclassified objects: %d of %d\n",1-trainres.cla_TError,(trainres.cla_Tfp+trainres.cla_Tfn),(trainres.cla_Ttp+trainres.cla_Ttn+trainres.cla_Tfp+trainres.cla_Tfn));
					}
				}else
				{
					hinfo("Hyperparameter selection done with respect to training error with flimit=%d:\n",psvm.par.flimit);
					if(psvm.par.doregress)hinfo("reg: epsi: %g [%g, %g], C: %g [%g, %g], feat: %d, T-MSE %g\n",trainres.reg_Te,trainres.e_min,trainres.e_max,trainres.reg_Tc,trainres.c_min,trainres.c_max,trainres.reg_Tnf,trainres.reg_TError);
					if(psvm.par.doclass)hinfo("cls: epsi: %g [%g, %g], C: %g [%g, %g], feat: %d, T-err %g\n",trainres.cla_Te,trainres.e_min,trainres.e_max,trainres.cla_Tc,trainres.c_min,trainres.c_max,trainres.cla_Tnf,trainres.cla_TError);
//					if(psvm.par.doclass)hinfo("cls: accuracy: %g, balanced: %g, tp: %d, tn: %d, fp: %d, fn: %d\n",trainres.cla_Tacc,trainres.cla_Taccb,trainres.cla_Ttp,trainres.cla_Ttn,trainres.cla_Tfp,trainres.cla_Tfn);
//					if(psvm.par.doclass)hinfo("cls: accuracy: %g, balanced accuracy: %g, balanced error: %g\n",trainres.cla_Tacc,trainres.cla_Taccb,trainres.cla_Terrb);
					if(psvm.par.doclass)hinfo("cls: true pos: %d, true neg: %d, false pos: %d, false neg: %d\n",trainres.cla_Ttp,trainres.cla_Ttn,trainres.cla_Tfp,trainres.cla_Tfn);
					if(psvm.par.doclass)hinfo("cls: mean accuracy: %g, mean number of missclassified objects: %d of %d\n",1-trainres.cla_TError,(trainres.cla_Tfp+trainres.cla_Tfn),(trainres.cla_Ttp+trainres.cla_Ttn+trainres.cla_Tfp+trainres.cla_Tfn));
				}
			}else{
				if(cv)
				{
					hinfo("crossvalidation result for constant hyperparameters:\n");
					if(psvm.par.doregress)hinfo("reg: epsi: %g, C: %g, feat: %d, G-MSE %g\n",trainres.reg_Ge,trainres.reg_Gc,trainres.reg_Gnf,trainres.reg_GError);
					if(psvm.par.doclass)hinfo("cls: epsi: %g, C: %g, feat: %d, G-err %g\n",trainres.cla_Ge,trainres.cla_Gc,trainres.cla_Gnf,trainres.cla_GError);
//					if(psvm.par.doclass)hinfo("cls: mean accuracy: %g, balanced: %g, tp: %d, tn: %d, fp: %d, fn: %d\n",trainres.cla_Gacc,trainres.cla_Gaccb,trainres.cla_Gtp,trainres.cla_Gtn,trainres.cla_Gfp,trainres.cla_Gfn);
//					if(psvm.par.doclass)hinfo("cls: mean accuracy: %g, mean balanced accuracy: %g, mean balanced error: %g\n",trainres.cla_Gacc,trainres.cla_Gaccb,trainres.cla_Gerrb);
					if(psvm.par.doclass)hinfo("cls: mean true pos: %d, mean true neg: %d, mean false pos: %d, mean false neg: %d\n",trainres.cla_Gtp,trainres.cla_Gtn,trainres.cla_Gfp,trainres.cla_Gfn);
					if(psvm.par.doclass)hinfo("cls: mean accuracy: %g, mean number of missclassified objects: %d of %d\n",1-trainres.cla_GError,(trainres.cla_Gfp+trainres.cla_Gfn),(trainres.cla_Gtp+trainres.cla_Gtn+trainres.cla_Gfp+trainres.cla_Gfn));
					if(getmodel==false)
						hinfo("No more model calculations aquired!\n");
					else{
						hinfo("corresponding Training error:\n");
						if(psvm.par.doregress)hinfo("reg: epsi: %g, C: %g, feat: %d, T-MSE %g\n",trainres.reg_Te,trainres.reg_Tc,trainres.reg_Tnf,trainres.reg_TError);
						if(psvm.par.doclass)hinfo("cls: epsi: %g, C: %g, feat: %d, T-err %g\n",trainres.cla_Te,trainres.cla_Tc,trainres.cla_Tnf,trainres.cla_TError);
//						if(psvm.par.doclass)hinfo("cls: accuracy: %g, balanced: %g, tp: %d, tn: %d, fp: %d, fn: %d\n",trainres.cla_Tacc,trainres.cla_Taccb,trainres.cla_Ttp,trainres.cla_Ttn,trainres.cla_Tfp,trainres.cla_Tfn);
//						if(psvm.par.doclass)hinfo("cls: accuracy: %g, balanced accuracy: %g, balanced error: %g\n",trainres.cla_Tacc,trainres.cla_Taccb,trainres.cla_Terrb);
						if(psvm.par.doclass)hinfo("cls: true pos: %d, true neg: %d, false pos: %d, false neg: %d\n",trainres.cla_Ttp,trainres.cla_Ttn,trainres.cla_Tfp,trainres.cla_Tfn);
						if(psvm.par.doclass)hinfo("cls: mean accuracy: %g, mean number of missclassified objects: %d of %d\n",1-trainres.cla_TError,(trainres.cla_Tfp+trainres.cla_Tfn),(trainres.cla_Ttp+trainres.cla_Ttn+trainres.cla_Tfp+trainres.cla_Tfn));
	//					if(psvm.par.doclass)hinfo("cls: mean accuracy: %g, mean number of missclassified objects: %d of %d\n",1-trainres.cla_GError,trainres.cla_misses,lset.mN);
					}
				}else
				{
					hinfo("Training error for constant hyperparameters:\n");
					if(psvm.par.doregress)hinfo("reg: epsi: %g, C: %g, feat: %d, T-MSE %g\n",trainres.reg_Te,trainres.reg_Tc,trainres.reg_Tnf,trainres.reg_TError);
					if(psvm.par.doclass)hinfo("cls: epsi: %g, C: %g, feat: %d, T-err %g\n",trainres.cla_Te,trainres.cla_Tc,trainres.cla_Tnf,trainres.cla_TError);
//					if(psvm.par.doclass)hinfo("cls: accuracy: %g, balanced: %g, tp: %d, tn: %d, fp: %d, fn: %d\n",trainres.cla_Tacc,trainres.cla_Taccb,trainres.cla_Ttp,trainres.cla_Ttn,trainres.cla_Tfp,trainres.cla_Tfn);
//					if(psvm.par.doclass)hinfo("cls: accuracy: %g, balanced accuracy: %g, balanced error: %g\n",trainres.cla_Tacc,trainres.cla_Taccb,trainres.cla_Terrb);
					if(psvm.par.doclass)hinfo("cls: true pos: %d, true neg: %d, false pos: %d, false neg: %d\n",trainres.cla_Ttp,trainres.cla_Ttn,trainres.cla_Tfp,trainres.cla_Tfn);
					if(psvm.par.doclass)hinfo("cls: mean accuracy: %g, mean number of missclassified objects: %d of %d\n",1-trainres.cla_TError,(trainres.cla_Tfp+trainres.cla_Tfn),(trainres.cla_Ttp+trainres.cla_Ttn+trainres.cla_Tfp+trainres.cla_Tfn));
//					if(psvm.par.doclass)hinfo("cls: mean accuracy: %g, mean number of missclassified objects: %d of %d\n",1-trainres.cla_GError,trainres.cla_misses,lset.mN);
				}
			}
		}
		if(modelsel)
		{
			//write ms to disk
			//prepare hyperparameterselection-plot
			char* lms;
			int i,j,k;
			FILE* fplot;
			lms=new char[200];
			strcpy(lms,lname);
			strcat(lms,".msdata"C_ENDING);
			fplot=fopen(lms,"w");
			check_f(fplot);
			i=0;
			for(j=0;j<trainres.msnc;j++)
			{
				for(k=0;k<trainres.msne;k++)
				{
				   finfo(fplot,"%g %g %g %g\n",trainres.ms[i].e,trainres.ms[i].c,trainres.ms[i].eec,trainres.ms[i].eer);
				   i++;
				}
                finfo(fplot,"\n");
			}
			delete[] lms;
			//trainres.ms has memory, it will be deleted after significance testing
			//that prevents time consuming additional allocations, see getModelWithLoops
		}
		//in hyperparameterselection-mode the model is currently not ready!!
		//only C,epsilon and corresponding expected Generalization error 
		if(mm || ffi || ffr || ffv || cc || rr || tt)
		{//only those requests need to compute the final model
			if(psvm.par.doclass && psvm.par.doregress) hinfo("Using regression model for further processings\n");
			//last message relys on default behavior in getModelFromRank and the following four lines
			if(modelsel)
			{//get the best hyperparameters into psvmmodel
				if(psvm.par.doregress)
				{
					psvm.par.epsi=(ftype)trainres.reg_Te;
					psvm.par.C=(ftype)trainres.reg_Tc;
				}else
				{
					psvm.par.epsi=(ftype)trainres.cla_Te;
					psvm.par.C=(ftype)trainres.cla_Tc;
				}
			}
			/*
			if(cv)
			{//in crossval-mode, rank is not in final status
				hinfo("Retraining the whole set: \n");
				psvm.getModel(&lset,&trainres);//retrieve normalization, featureset, b and get trainerror
				hinfo("Training errors of retrained model over complete set:\n");
				if(psvm.par.doregress)hinfo("reg: C: %g, epsilon: %g, training regression MSE: %g \n",psvm.par.C,psvm.par.epsi,trainres.reg_TError);
				if(psvm.par.doclass)hinfo("cls: C: %g, epsilon: %g, training classificn err: %g \n",psvm.par.C,psvm.par.epsi,trainres.cla_TError);
				if(psvm.par.doclass)hinfo("cls: accuracy: %g, number of missclassified objects: %d of %d\n",1-trainres.cla_TError,trainres.cla_misses,lset.mN);
			}else{//np,b ready, rank is in trainres
				//this is already done
//				if(psvm.par.doregress)
//					trainres.reg_mrank.putback(&psvm.rank);//model already in trainres
//				else
//					trainres.cla_mrank.putback(&psvm.rank);//model already in trainres
			}
			*/
			if(!psvm.par.isrelation) psvm.getSVZ(&svsave);//saves the support vectors from svsave with indices rank into model
			//relational prediction works with out = norm(in)  alpha + b, no support vector needed
			//kernel prediction works with out = norm(kernel(in,Z)  alpha + b, support vector Z needed
			//model is complete
			if(mm)//options -train -model
			{//save model
				strcat(model,".model"C_ENDING);
				hinfo("writing model into %s\n",model);
				if(!psvm.save(model))//needs prepared support vectors in svz
				{
					hinfo("error saving model file %s\n",model);
				}
			}
			if(ffi)
			{//save feature indices
				strcat(finame,".fi"C_ENDING);
				FILE* fl;
				hinfo("writing feature indices beginning with 1 into %s\n",finame);
				fl=fopen(finame,"w");
				check_f(fl);
				int sz;
				sz=(int)psvm.rank.mFeat.size();
				if(psvm.par.flimit>=0) sz=min(psvm.par.flimit,sz);
				for(i=0;i<sz;i++)
					finfo(fl,"%d\n",psvm.rank.mFeat[i].dimRef+1);
				fclose(fl);
			}
			if(ffr)
			{//save feature ranking
				strcat(frname,".fr"C_ENDING);
				FILE* fl;
				hinfo("writing feature ranking beginning with 1 into %s\n",frname);
				fl=fopen(frname,"w");
				check_f(fl);
				int sz;
				sz=(int)psvm.rank.mFeat.size();
				if(psvm.par.flimit>=0) sz=min(psvm.par.flimit,sz);
				for(i=0;i<sz;i++)
					finfo(fl,"%d %.9e \n",psvm.rank.mFeat[i].dimRef+1,psvm.rank.mFeat[i].a);
				fclose(fl);
			}
			if(ffv)
			{//save feature vectors
				if(psvm.par.isrelation)
				{//in dyadic mode, the descriptor objects itself are unknown
					hinfo("error: No support feature vectors available in relational mode !!\n");
					exit_with_help(&psvm);
				}
				strcat(fvname,".fv"C_ENDING);
				FILE* fl;
				hinfo("writing feature vectors into %s\n",fvname);
				fl=fopen(fvname,"w");
				check_f(fl);
				int sz;
				sz=(int)psvm.rank.mFeat.size();
				if(psvm.par.flimit>=0) sz=min(psvm.par.flimit,sz);
				for(i=0;i<sz;i++)
				{
					ftype* sdata;
					sdata=&psvm.svz->mData[psvm.svz->mDim*i];
					for(int k=0;k<psvm.svz->mDim;k++) finfo(fl,"%.9e ",sdata[k]);
					finfo(fl,"\n");
				}
				fclose(fl);
			}
		}
		//relational prediction works with out = norm(in)  alpha + b, no support vector needed
		//kernel prediction works with out = norm(kernel(in,Z))  alpha + b, support vector Z needed
		//model is complete

		if(yrr)//try random y-vectors
		{//test significance test
			double originalmse,originalrate;
			double originalrege,originalregc;
			double originalclae,originalclac;
			double p_mse,p_rate,p_rege,p_regc,p_clae,p_clac;
			FILE* fcplt;//classification gnuplot script filehandle
			FILE* frplt;//regression gnuplot script filehandle
			FILE* fcplt1;//classification gnuplot script filehandle
			FILE* frplt1;//regression gnuplot script filehandle
			FILE* fsigdata;//significance data filehandle
			FILE* fsigdata1;//significance data filehandle
			char *fnsig;
			fnsig=new char[200];
			char *fnsigdata1;
			fnsigdata1=new char[200];
			char *fncplt1;
			fncplt1=new char[200];
			char *fnrplt1;
			fnrplt1=new char[200];
			strcpy(fnsig,lname);
			strcat(fnsig,".sigdata"C_ENDING);
			fsigdata=fopen(fnsig,"w");//this file will be built up iteratively and then closed
			check_f(fsigdata);
			strcpy(fnsigdata1,lname);
			strcat(fnsigdata1,".sigdata1"C_ENDING);
//			fsigdata1=fopen(signame1,"w");//this file we be completely rewritten at every cycle
//			check_f(fsigdata1);
			strcpy(fnsig,lname);
			strcat(fnsig,".sigclass.plt");
			fcplt=fopen(fnsig,"w");//this file will be written at once and then closed
			check_f(fcplt);
			strcpy(fncplt1,lname);
			strcat(fncplt1,".sigclass1.plt");
//			fcplt1=fopen(signame,"w");//this file we be completely rewritten at every cycle
//			check_f(fcplt1);
			strcpy(fnsig,lname);
			strcat(fnsig,".sigregress.plt");
			frplt=fopen(fnsig,"w");//this file will be written at once and then closed
			check_f(frplt);
			strcpy(fnrplt1,lname);
			strcat(fnrplt1,".sigregress1.plt");
//			frplt1=fopen(signame,"w");//this file we be completely rewritten at every cycle
//			check_f(frplt1);
			delete[] fnsig;

			ftype *ysave;
			int bitdistance,bigloop;
			vector<double> siglistc;
			vector<double> siglistr;

			hinfo("Now testing Significance using %d permutations of the original labels:\n",sigcycles);
			if(cv||ocv){//retrieved errors are generalization approximations
				originalmse=trainres.reg_GError;
				originalrate=trainres.cla_GError;
				originalrege=trainres.reg_Ge;
				originalregc=trainres.reg_Gc;
				originalclae=trainres.cla_Ge;
				originalclac=trainres.cla_Gc;
			}else{
				originalmse=trainres.reg_TError;
				originalrate=trainres.cla_TError;
				originalrege=trainres.reg_Te;
				originalregc=trainres.reg_Tc;
				originalclae=trainres.cla_Te;
				originalclac=trainres.cla_Tc;
			}

			int nfold;
			nfold=psvm.par.infold;
			if(nfold>lset.mN||nfold<0) nfold=lset.mN;
			if(nfold==0) nfold=1;
#if 0
			{//write gnuplot script for viewing error distribution for random labels
				fcplt1=fopen(fncplt1,"w");//this file we be completely rewritten at every cycle
				finfo(fcplt1,"set ylabel 'error'\n set xlabel 'rank'\n"
					"plot '%s.sigdata1"C_ENDING"' using 1:2 title '%d-fold cv-errors of rnd. shuffled labels from dataset <%s>' with linespoints"
					",%f title '%d-fold cv-error of dataset <%s> with original labels'\n"
					"set term postscript eps enhanced color solid\n set output '%s.sigclass1.eps'\n replot\n"
					"pause -1\n",lname,nfold,lname,originalrate,nfold,lname,lname);
				fclose(fcplt1);
				frplt1=fopen(fnrplt1,"w");//this file we be completely rewritten at every cycle
				finfo(frplt1,"set ylabel 'error'\n set xlabel 'rank'\n"
					"plot '%s.sigdata1"C_ENDING"' using 1:3 title '%d-fold cv-MSE of rnd. shuffled labels from dataset <%s>' with linespoints"
					",%f title '%d-fold cv-MSE of dataset <%s> with original labels'\n"
					"set term postscript eps enhanced color solid\n set output '%s.sigregress1.eps'\n replot\n"
					"pause -1\n",lname,nfold,lname,originalmse,nfold,lname,lname);
				fclose(frplt1);
			}
#endif
			{//write gnuplot script for viewing error over label distance
				finfo(fcplt,"set ylabel 'error'\n set xlabel 'distance'\n"
					"plot '%s.sigdata"C_ENDING"' using 1:2 title 'bit distance of shuffled labels with dataset <%s> classification' with points\n"
		            "set term postscript eps enhanced color solid\n set output '%s.sigclass.eps'\n replot\n"
					"pause -1\n",lname,lname,lname);
				fclose(fcplt);
				finfo(frplt,"set ylabel 'error'\n set xlabel 'distance'\n"
					"plot '%s.sigdata"C_ENDING"' using 1:3 title 'bit distance of shuffled labels with dataset <%s> classification' with points\n"
		            "set term postscript eps enhanced color solid\n set output '%s.sigregress.eps'\n replot\n"
					"pause -1\n",lname,lname,lname);
				fclose(frplt);
			}
			//remember true labels
			ysave=new ftype[lset.mN];
			for(i=0;i<lset.mN;i++) ysave[i]=lset.mY[i];
			//collect result in gnuplot-datafile 
			//first row in logfile is for original training result
			finfo(fsigdata,"0 %f %f %f %f %f %f\n",originalrate,originalmse,originalclac,originalregc,originalclae,originalrege);
			fflush(fsigdata);
			srand( (unsigned)time( NULL ) );//time seed for random shuffling
			for(bigloop=0;bigloop<sigcycles;bigloop++)
			{
#if 0
				bitdistance=0;
				for(i=0;i<lset.mN;i++)
				{//flip all classes with probability=0.5
					int r;
					r=rand();
					if(r>RAND_MAX/2) 
					{
						bitdistance++;
						lset.mY[i]=ysave[i]*-1;
					}
					else lset.mY[i]=ysave[i];
				}
				bitdistance=min(bitdistance,lset.mN-bitdistance);
#endif
#if 0
				//shuffle y from ysave
				{
					int i;
					int k1;
					int maxi=RAND_MAX+1;
					int f=lset.mN/maxi+1;
					double s;
					int r,k;
					int fact[5];
					for(k=0;k<=f;k++)fact[k]=1;
					for(k=0;k<=f;k++)for(i=0;i<k;i++)fact[k]*=maxi;
					s=fact[f];
					for(i=0;i<lset.mN;i++) lset.mY[i]=ysave[i];
					for(i=0;i<lset.mN;i++)
					{//too much shuffeling, but simpler
						r=0;
						for(k=0;k<f;k++)r+=rand()*fact[k];
						k1=(int)((double)lset.mN*(double)r/s);
						swap(lset.mY[i],lset.mY[k1]);
					}
					bitdistance=0;
					for(i=0;i<lset.mN;i++) if(lset.mY[i]*ysave[i]<0) 
						bitdistance++;
					bitdistance=min(bitdistance,lset.mN-bitdistance);
				}
#endif
#if 1
				//maintain class relation
				//change half of the bigger class if possible
				{
					int *listA,*listB;
					int nrA,nrB;
					int Ai,Bi;
					int changes;
					int i,j;
					nrA=0;
					nrB=0;
					for(i=0;i<lset.mN;i++)
					{
						if(lset.mY[i]>=0) nrA++;
						else nrB++;
					}
					listA=new int[nrA];
					listB=new int[nrB];
					Ai=0;
					Bi=0;
					for(i=0;i<lset.mN;i++) lset.mY[i]=ysave[i];
					for(i=0;i<lset.mN;i++)
					{
						if(lset.mY[i]>=0) listA[Ai++]=i;
						else listB[Bi++]=i;
					}
					changes=max(nrA,nrB)/2;
					if(changes>min(nrA,nrB)) changes=min(nrA,nrB);
					for(i=0;i<changes;i++)
					{
						//get random indices below nrA and nrB
						Ai=rndrng(nrA);
						Bi=rndrng(nrB);
						swap(lset.mY[listA[Ai]],lset.mY[listB[Bi]]);
						for(j=Ai+1;j<nrA;j++) listA[j-1]=listA[j]; nrA--;
						for(j=Bi+1;j<nrB;j++) listB[j-1]=listB[j]; nrB--;
					}
					bitdistance=0;
					for(i=0;i<lset.mN;i++) if(lset.mY[i]*ysave[i]<0) bitdistance++;
					bitdistance=bitdistance;
					delete[] listA;
					delete[] listB;
				}

#endif
				//do modelselection with new randomly changed class vector
				psvm.getModelWithLoops(&lset,&trainres,false);
				//print results:
				if(cv||ocv){//retrieved errors are generalization approximations
					p_mse=trainres.reg_GError;
					p_rate=trainres.cla_GError;
					p_rege=trainres.reg_Ge;
					p_regc=trainres.reg_Gc;
					p_clae=trainres.cla_Ge;
					p_clac=trainres.cla_Gc;
					if(psvm.par.doregress)hinfo("reg: epsi: %g [%g, %g], C: %g [%g, %g], G-MSE %g\n",p_rege,trainres.e_min,trainres.e_max,p_regc,trainres.c_min,trainres.c_max,p_mse);
					if(psvm.par.doclass)hinfo("cls: epsi: %g [%g, %g], C: %g [%g, %g], G-err %g\n",p_clae,trainres.e_min,trainres.e_max,p_clac,trainres.c_min,trainres.c_max,p_rate);
//					if(psvm.par.doclass)hinfo("cls: mean accuracy: %g, mean number of missclassified objects: %d of %d\n",1-trainres.cla_GError,trainres.cla_misses,lset.mN);
					finfo(fsigdata,"%d %f %f %f %f %f %f\n",bitdistance,p_rate,p_mse,p_clac,p_regc,p_clae,p_rege);
					siglistc.push_back(p_rate);
					siglistr.push_back(p_mse);
				}else{
					p_mse=trainres.reg_TError;
					p_rate=trainres.cla_TError;
					p_rege=trainres.reg_Te;
					p_regc=trainres.reg_Tc;
					p_clae=trainres.cla_Te;
					p_clac=trainres.cla_Tc;
					if(psvm.par.doregress)hinfo("reg: epsi: %g [%g, %g], C: %g [%g, %g], T-MSE %g\n",p_rege,trainres.e_min,trainres.e_max,p_regc,trainres.c_min,trainres.c_max,p_mse);
					if(psvm.par.doclass)hinfo("cls: epsi: %g [%g, %g], C: %g [%g, %g], T-err %g\n",p_clae,trainres.e_min,trainres.e_max,p_clac,trainres.c_min,trainres.c_max,p_rate);
					finfo(fsigdata,"%d %f %f %f %f %f %f\n",bitdistance,p_rate,p_mse,p_clac,p_regc,p_clae,p_rege);
					siglistc.push_back(p_rate);
					siglistr.push_back(p_mse);
				}
/*				{
					if(modelsel&&cv)
					{
						hinfo("Result after hyperparameterselection with crossvalidation:\n");
						if(psvm.par.doregress)hinfo("reg: C: %g [%g, %g], epsi %g [%g, %g], mean#f: %d, G-MSE %g\n",trainres.reg_c,trainres.reg_c1,trainres.reg_c2,trainres.reg_e,trainres.reg_e1,trainres.reg_e2,trainres.reg_nf,trainres.reg_GError);
						if(psvm.par.doclass)hinfo("cls: C: %g [%g, %g], epsi %g [%g, %g], mean#f: %d, G-err %g\n",trainres.cla_c,trainres.cla_c1,trainres.cla_c2,trainres.cla_e,trainres.cla_e1,trainres.cla_e2,trainres.cla_nf,trainres.cla_GError);
						if(psvm.par.doclass)hinfo("cls: mean accuracy: %g, mean number of missclassified objects: %d of %d\n",1-trainres.cla_GError,trainres.cla_misses,lset.mN);
					}else if(cv)
					{
						hinfo("Result after crossvalidation:\n");
						if(psvm.par.doregress)hinfo("reg: C: %g, epsi %g, mean features: %d, G-MSE %g\n",trainres.reg_c,trainres.reg_e,trainres.reg_nf,trainres.reg_GError);
						if(psvm.par.doclass)hinfo("cls: C: %g, epsi %g, mean features: %d, G-err %g\n",trainres.cla_c,trainres.cla_e,trainres.cla_nf,trainres.cla_GError);
						if(psvm.par.doclass)hinfo("cls: mean accuracy: %g, mean number of missclassified objects: %d of %d\n",1-trainres.cla_GError,trainres.cla_misses,lset.mN);
					}else if(modelsel)
					{
						hinfo("Result after hyperparameterselection over training error:\n");
						if(psvm.par.doregress)hinfo("reg: C: %g [%g, %g], epsi %g [%g, %g], #f: %d, T-MSE %g\n",trainres.reg_c,trainres.reg_c1,trainres.reg_c2,trainres.reg_e,trainres.reg_e1,trainres.reg_e2,trainres.reg_nf,trainres.reg_TError);
						if(psvm.par.doclass)hinfo("cls: C: %g [%g, %g], epsi %g [%g, %g], #f: %d, T-err %g\n",trainres.cla_c,trainres.cla_c1,trainres.cla_c2,trainres.cla_e,trainres.cla_e1,trainres.cla_e2,trainres.cla_nf,trainres.cla_TError);
						if(psvm.par.doclass)hinfo("cls: accuracy: %g, number of missclassified objects: %d of %d\n",1-trainres.cla_TError,trainres.cla_misses,lset.mN);
					}else
					{
						hinfo("Result after simple training:\n");
						if(psvm.par.doregress)hinfo("reg: C: %g, epsi %g, features: %d, T-MSE %g\n",trainres.reg_c,trainres.reg_e,trainres.reg_nf,trainres.reg_TError);
						if(psvm.par.doclass)hinfo("cls: C: %g, epsi %g, features: %d, T-err %g\n",trainres.cla_c,trainres.cla_e,trainres.cla_nf,trainres.cla_TError);
						if(psvm.par.doclass)hinfo("cls: accuracy: %g, number of missclassified objects: %d of %d\n",1-trainres.cla_TError,trainres.cla_misses,lset.mN);
					}
				}
				if(cv){//log generalization capability
					finfo(fsigdata,"%d %f %f %f %f %f %f\n",bitdistance,trainres.cla_GError,trainres.reg_GError,trainres.cla_c,trainres.reg_c,trainres.cla_e,trainres.reg_e);
					siglistc.push_back(trainres.cla_GError);
					siglistr.push_back(trainres.reg_GError);
				}else{
					finfo(fsigdata,"%d %f %f %f %f %f %f\n",bitdistance,trainres.cla_TError,trainres.reg_TError,trainres.cla_c,trainres.reg_c,trainres.cla_e,trainres.reg_e);
					siglistc.push_back(trainres.cla_TError);
					siglistr.push_back(trainres.reg_TError);
				}
*/				fflush(fsigdata);
				if(bigloop%1==0)
				{//save error distribution online, (saves data produced before ctrl-c pressed)
					//rebuild complete file with error v. ranking position
					int ntotal,nreg_beq,ncla_beq;
					double pe_c_mean,pe_c_var;
					double pe_r_mean,pe_r_var;
					vector<double>::iterator sigitc; 
					vector<double>::iterator sigitr; 
					sort(siglistc.begin(),siglistc.end());
					sort(siglistr.begin(),siglistr.end());
//					sort(siglistc.begin(),siglistc.end(),less<double>( ));
//					sort(siglistr.begin(),siglistr.end(),less<double>( ));

					//write data for gnuplot scripts to file
					//and calculate statistics for console output
					{
						i=0;
						ntotal=0;
						nreg_beq=0;
						ncla_beq=0;
						pe_c_mean=0;
						pe_c_var=0;
						pe_r_mean=0;
						pe_r_var=0;
						sigitr=siglistr.begin();
						fsigdata1=fopen(fnsigdata1,"w");
						for(sigitc=siglistc.begin();sigitc!=siglistc.end();sigitc++)
						{//descending error looks better in plot
							finfo(fsigdata1,"%d %f %f\n",i,*sigitc,*sigitr);
							if(*sigitc<=originalrate*1.000001) ncla_beq++;
							if(*sigitr<=originalmse*1.000001) nreg_beq++;
							pe_c_mean+=*sigitc;
							pe_c_var+=sqr(*sigitc);
							pe_r_mean+=*sigitr;
							pe_r_var+=sqr(*sigitr);
							ntotal++;
							i++;
							sigitr++;
						}
						fclose(fsigdata1);
					}
					{//print current significance level to console
						double signif_c,signif_r;//,upperc,upperr,lowerc,lowerr;
//						double alpha;
						signif_c=(double)ncla_beq/(double)ntotal;
						signif_r=(double)nreg_beq/(double)ntotal;
						pe_c_mean/=ntotal;
						pe_c_var=pe_c_var/ntotal-sqr(pe_c_mean);
						pe_r_mean/=ntotal;
						pe_r_var=pe_r_var/ntotal-sqr(pe_r_mean);
						/*
						alpha=0.05;
						upperc=konfihigh(alpha,ncla_beq,ntotal);
						lowerc=konfilow(alpha,ncla_beq,ntotal);
						upperr=konfihigh(alpha,nreg_beq,ntotal);
						lowerr=konfilow(alpha,nreg_beq,ntotal);
						if(psvm.par.doregress)hinfo("reg:current significance: %g in range [%g to %g] with prob %g\n",signif_r,lowerr,upperr,alpha);
						if(psvm.par.doclass)hinfo("cls:current significance: %g in range [%g to %g] with prob %g\n",signif_c,lowerc,upperc,alpha);
						*/
						if(psvm.par.doregress)hinfo("reg: shuffled labels outperform original in %d of %d cases, rate %g\n",nreg_beq,ntotal,signif_r);
						if(psvm.par.doclass)hinfo("cls: shuffled labels outperform original in %d of %d cases, rate %g\n",ncla_beq,ntotal,signif_c);
					}
					{//write gnuplot script for viewing error distribution for random labels with current mean and variance of randomly shuffled errors
						fcplt1=fopen(fncplt1,"w");//this file we be completely rewritten at every cycle
						finfo(fcplt1,"set ylabel 'error'\n set xlabel 'rank'\n"
							"plot '%s.sigdata1"C_ENDING"' using 1:2 title '%d-fold cv-errors of rnd. permutated labels from dataset <%s>' with linespoints"
							",%f title '%d-fold cv-error of dataset <%s> with original labels'"
							",%f title 'mean of cv-error with rnd. permutated labels'"
//							",%f title 'mean minus std deviation of cv-error with rnd. permutated labels'"
//							",%f title 'mean plus std deviation of cv-error with rnd. permutated labels'"
							"\n"
							"set term postscript eps enhanced color solid\n set output '%s.sigclass1.eps'\n replot\n"
							"pause -1\n",lname,nfold,lname,originalrate,nfold,lname,pe_c_mean,/*pe_c_mean+sqrt(pe_c_var),pe_c_mean-sqrt(pe_c_var),*/lname);
						fclose(fcplt1);
						frplt1=fopen(fnrplt1,"w");//this file we be completely rewritten at every cycle
						finfo(frplt1,"set ylabel 'error'\n set xlabel 'rank'\n"
							"plot '%s.sigdata1"C_ENDING"' using 1:3 title '%d-fold cv-MSE of rnd. permutated labels from dataset <%s>' with linespoints"
							",%f title '%d-fold cv-MSE of dataset <%s> with original labels'"
							",%f title 'mean of cv-error with rnd. permutated labels'"
//							",%f title 'mean minus std deviation of cv-error with rnd. permutated labels'"
//							",%f title 'mean plus std deviation of cv-error with rnd. permutated labels'"
							"\n"
							"set term postscript eps enhanced color solid\n set output '%s.sigregress1.eps'\n replot\n"
							"pause -1\n",lname,nfold,lname,originalmse,nfold,lname,pe_r_mean,/*pe_r_mean+sqrt(pe_r_var),pe_r_mean-sqrt(pe_r_var),*/lname);
						fclose(frplt1);
					}
				}
			}//bigloop
			delete[]ysave;
			{//save error distribution
				int ntotal,nreg_beq,ncla_beq;
				vector<double>::iterator sigitc; 
				vector<double>::iterator sigitr; 
//              sort(siglistc.begin(),siglistc.end(),less<double>());
//				sort(siglistr.begin(),siglistr.end(),less<double>());
				sort(siglistc.begin(),siglistc.end());
				sort(siglistr.begin(),siglistr.end());
				i=0;
				sigitr=siglistr.begin();
				ntotal=0;
				nreg_beq=0;
				ncla_beq=0;
				fsigdata1=fopen(fnsigdata1,"w");
				for(sigitc=siglistc.begin();sigitc!=siglistc.end();sigitc++)
				{
					finfo(fsigdata1,"%d %f %f\n",i,*sigitc,*sigitr);
					ntotal++;
					if(*sigitc<=originalrate*1.000001) ncla_beq++;
					if(*sigitr<=originalmse*1.000001) nreg_beq++;
					i++;
					sigitr++;
				}
				fclose(fsigdata1);
				{//print current significance level
					double signif_c,signif_r;//,upperc,upperr,lowerc,lowerr;
//					double alpha;
					signif_c=(double)ncla_beq/(double)ntotal;
					signif_r=(double)nreg_beq/(double)ntotal;
/*					alpha=0.05;
					upperc=konfihigh(alpha,ncla_beq,ntotal);
					lowerc=konfilow(alpha,ncla_beq,ntotal);
					upperr=konfihigh(alpha,nreg_beq,ntotal);
					lowerr=konfilow(alpha,nreg_beq,ntotal);
					if(psvm.par.doregress)hinfo("reg:final significance: %g in range [%g to %g] with prob %g\n",signif_r,lowerr,upperr,alpha);
					if(psvm.par.doclass)hinfo("cls:final significance: %g in range [%g to %g] with prob %g\n",signif_c,lowerc,upperc,alpha);
					*/
					if(psvm.par.doregress)hinfo("reg finished: shuffled labels outperform original in %d of %d cases, rate %g\n",nreg_beq,ntotal,signif_r);
					if(psvm.par.doclass)hinfo("cls finished: shuffled labels outperform original in %d of %d cases, rate %g\n",ncla_beq,ntotal,signif_c);
				}
			}
			delete[]fnsigdata1;
			delete[]fncplt1;
			delete[]fnrplt1;
			fclose(fsigdata);
		}
		delete[]trainres.ms;
		delete[]lx;
		delete[]ly;
		delete[]lz;
	}else{
		if(mm){//-model
			if(mpset)
			{
				hinfo("warning: kernel and relation options overwritten by loaded model!\n");
			}
			strcat(model,".model"C_ENDING);
			if(!psvm.load(model))
			{
				hinfo("error: model file %s not found!\n",model);
				exit(-1);
			}
		}

		if(ffi)
		{//save feature indices
			strcat(finame,".fi"C_ENDING);
			FILE* fl;
			hinfo("writing feature indices beginning with 1 into %s\n",finame);
			fl=fopen(finame,"w");
			check_f(fl);
			int sz;
			sz=(int)psvm.rank.mFeat.size();
			if(psvm.par.flimit>=0) sz=min(psvm.par.flimit,sz);
			for(i=0;i<sz;i++)
			{
				finfo(fl,"%d\n",psvm.rank.mFeat[i].dimRef+1);
			}
			fclose(fl);
		}
		if(ffr)
		{//save feature ranking
			strcat(frname,".fr"C_ENDING);
			FILE* fl;
			hinfo("writing feature indices beginning with 1 into %s\n",frname);
			fl=fopen(frname,"w");
			check_f(fl);
			int sz;
			sz=(int)psvm.rank.mFeat.size();
			if(psvm.par.flimit>=0) sz=min(psvm.par.flimit,sz);
			for(i=0;i<sz;i++)
			{
				finfo(fl,"%d %.9e \n",psvm.rank.mFeat[i].dimRef+1,psvm.rank.mFeat[i].a);
			}
			fclose(fl);
		}
		if(ffv)
		{//save feature vectors
			if(psvm.par.isrelation)
			{//in dyadic mode, descriptor objects are unknown
				hinfo("error: No support feature vectors available in relational mode !!\n");
				exit_with_help(&psvm);
			}
			strcat(fvname,".fv"C_ENDING);
			FILE* fl;
			hinfo("writing feature vectors into %s\n",fvname);
			fl=fopen(fvname,"w");
			check_f(fl);
			int sz;
			sz=(int)psvm.rank.mFeat.size();
			if(psvm.par.flimit>=0) sz=min(psvm.par.flimit,sz);
			for(i=0;i<sz;i++)
			{
				ftype* sdata;
				sdata=&psvm.svz->mData[psvm.svz->mDim*i];
				for(int k=0;k<psvm.svz->mDim;k++) finfo(fl,"%.9e ",sdata[k]);
				finfo(fl,"\n");
			}
			fclose(fl);
		}
	}
	if(cc)//-predictc
	{//get classes Y for given data
		char *cx,*cy;//filenames
		ftype *y;//generated discrete labels
		cx=new char[200];cy=new char[200];
		strcpy(cx,cname);
		strcpy(cy,cname);
		strcat(cx,".data"C_ENDING);
		strcat(cy,".yout"C_ENDING);
		if(!cset.load(cx,NULL,psvm.par.isrelation?1:0))//transpose relational data
		{
			hinfo("data file %s not found!\n",cx);
			exit(0);
		}
		y=new ftype[cset.mN];
		clock_t t0;
		t0=clock();
		psvm.getY(y,&cset,psvm.par.flimit,true);//normalize and use support vectors, if not relational
		printf("Whole prediction time: %f \n",timeDiff(t0,clock()));
		FILE* fl;
		fl=fopen(cy,"w");
		check_f(fl);
		for(i=0;i<cset.mN;i++)
			finfo(fl,"%d \n",(int)signc(y[i]));
		fclose(fl);
		delete[] y;
		hinfo("   classes for file %s saved in file %s \n",cx,cy);
		delete[] cx;
		delete[] cy;
	}
	if(rr)//-predict(regression)
	{//get Y for given data
//		if(!psvm.rank)
//		{//-classify -predict -test
//			printf("error: A model must either be loaded or learned!!\n");
//			fprintf(log,"error: A model must either be loaded or learned!!\n");
//			exit_with_help(&psvm);
//		}
		char *rx,*ry;
		ftype *y;
		rx=new char[200];ry=new char[200];
		strcpy(rx,rname);
		strcpy(ry,rname);
		strcat(rx,".data"C_ENDING);
		strcat(ry,".yout"C_ENDING);
		if(!rset.load(rx,NULL,psvm.par.isrelation?1:0))
		{
			hinfo("data file %s not found!\n",rx);
			exit(0);
		}
		y=new ftype[rset.mN];
		clock_t t0;
		t0=clock();
		psvm.getY(y,&rset,psvm.par.flimit,true);//normalize and use support vectors, if not relational
		printf("Whole prediction time: %f \n",timeDiff(t0,clock()));
		FILE* fl;
		fl=fopen(ry,"w");
		check_f(fl);
		for(i=0;i<rset.mN;i++)
			finfo(fl,"%f \n",y[i]);
		fclose(fl);
		delete[] y;
		hinfo("   real valued predictions for file %s saved in file %s \n",rx,ry);
		delete[] rx;
		delete[] ry;
	}
	if(tt)//-test
	{//get Y for given data and compare with given Y from dataset
//		if(!psvm.rank)
//		{//-classify -predict -test
//			printf("error: A model must either be loaded or learned!!\n");
//			fprintf(log,"error: A model must either be loaded or learned!!\n");
//			exit_with_help(&psvm);
//		}
		char *tx,*ty;
		tx=new char[200];ty=new char[200];
		strcpy(tx,tname);
		strcpy(ty,tname);
		strcat(tx,".data"C_ENDING);
		strcat(ty,".y"C_ENDING);
		hinfo("Testing with dataset %s:\n",tname);
		if(!tset.load(tx,ty,psvm.par.isrelation?1:0))
		{
			hinfo("data file %s and %s not found!\n",tx,ty);
			exit(0);
		}
		if(balanceY) tset.balance();
		{//test data
			ftype *yy;
			double mse;//mean square error for regression
			double rate;//success rate for classification
			yy=new ftype[tset.mN];
			psvm.getY(yy,&tset,psvm.par.flimit,true);//normalize and use support vectors, if not relational
			int ktp,ktn,kfp,kfn;//class statistics
			psvm.getErrors(yy,&tset,&mse,&rate,&ktp,&ktn,&kfp,&kfn);
			if(psvm.par.doregress)hinfo("mean squared error:%g \n",mse);
			if(psvm.par.doclass){
				hinfo("classification error rate:%g \n",rate);
				hinfo("           |prediction\n");
				hinfo("           |      pos       neg       sum\n");
				hinfo("        pos|%9d %9d %9d\n",ktp,kfn,ktp+kfn);
				hinfo("dataset:neg|%9d %9d %9d\n",kfp,ktn,kfp+ktn);
				hinfo("        sum|%9d %9d %9d\n",ktp+kfp,kfn+ktn,ktp+ktn+kfp+kfn);
			}
			{//build roc-curve
				//needs tset, yy
				vector<rs> d;//collects yy,tset.mY
				vector<rs>::iterator di;
				vector<rs> roc;//collects roc-points
				rs dd;
				int np,nn;
				np=0;nn=0;
				for(i=0;i<tset.mN;i++)
				{
					dd.yo=tset.mY[i];
					dd.yt=yy[i];
					d.push_back(dd);
					if(dd.yo>=0) np++;else nn++;
				}
				sort(d.begin(),d.end());
				di=d.begin();
				ftype yl;
				int fp,tp;
				yl=-1e30;
				fp=0;tp=0;
				for(i=0;i<tset.mN;i++)
				{
					if(yy[i]!=yl)
					{
						dd.yo=(ftype)fp/(ftype)nn;
						dd.yt=(ftype)tp/(ftype)np;
						roc.push_back(dd);
						yl=yy[i];
					}
					if((*di).yo>=0)
						tp++;
					else fp++;
					di++;
				}
				FILE* rocf;
				char* fnrd,*fnrs;
				fnrd=new char[200];
				fnrs=new char[200];
				strcpy(fnrd,tname);
				strcpy(fnrs,tname);
				strcat(fnrd,".rocdata"C_ENDING);
				strcat(fnrs,".roctest.plt");
				rocf=fopen(fnrd,"w");
				check_f(rocf);
				for(di=roc.begin();di!=roc.end();di++)
				{
					finfo(rocf,"%f %f\n",(*di).yo,(*di).yt);
				}
				fclose(rocf);
				rocf=fopen(fnrs,"w");
				check_f(rocf);
				finfo(rocf,"set ylabel 'true positive'\n set xlabel 'false positive'\n"
					"plot '%s.rocdata"C_ENDING"' using 1:2 title 'ROC of dataset <%s>' with linespoints\n"
					"set term postscript eps enhanced color solid\n set output '%s.roc.eps'\n replot\n"
					"pause -1\n",tname,tname,tname);
				fclose(rocf);
				hinfo("ROC saved into %s and %s\n",fnrd,fnrs);
				delete[] fnrd;
				delete[] fnrs;
			}
			delete[]yy;
		}
		delete[] tx;
		delete[] ty;
	}
//	printf("\n \n...thanks for using this software, visit www.ni.cs.tu-berlin.de\n");
	fclose(flog);
	delete[] lname;
	delete[] cname;
	delete[] rname;
	delete[] tname;
	delete[] model;
	delete[] finame;
	delete[] fvname;
	delete[] frname;
	delete[] sname;
//	psvm.~PSVMmodel();
	i=i;
}
//###################################################################################################
//###################################################################################################
//###################################################################################################
//###################################################################################################
//###################################################################################################
//###################################################################################################
//###################################################################################################
//###################################################################################################
//###################################################################################################
//###################################################################################################
//###################################################################################################
//###################################################################################################
//###################################################################################################
//###################################################################################################
//###################################################################################################
//###################################################################################################
//###################################################################################################
void welcome()
{
	printf(
		"============================================================================\n"
		"PSVM Release 1.31 (25.05.2008) for statistical data analysis\n"
		"using the Potential Support vector machine. See more on www.ni.cs.tu-berlin.de\n"
		"Sepp Hochreiter, Klaus Obermayer; Support Vector Machines for Dyadic Data\n"
        "                                  Neural Computation, 2005.\n"
//        "Sepp Hochreiter, Klaus Obermayer; Classification, Regression and Feature\n"
//        "                             Selection on Matrix Data; TU-Report 2004/02\n"
		"Copyright TU-Berlin 2006, Sepp Hochreiter, Tilman Knebel\n"
		"published under GNU General Public License, see www.gnu.org/licenses\n"
		"============================================================================\n"
		"\n"
		);
}
void exit_with_help(PSVMmodel *p)
{
	printf(
	"Usage: psvm [options]\n"
	"options:\n"
	"-stats [filename] : show statistics for dataset\n"
	"-train [filename] : training with dataset, regression and classification \n"
	"-trainr [filename] : training with dataset, regression \n"
	"-trainc [filename] : training with dataset, classification \n"
	"-predictc [filename] : save binary predictions for dataset (+1,-1)\n"
	"-predict [filename] : save real valued predictions for dataset\n"
	"-test [filename] : get model error for dataset\n"
	"-model [filename] : load/save model\n"
	"-fi [filename] : save indices [1..n] to support feature vectors \n"
	"-fv [filename] : save support feature vectors\n"
	"-frank [filename] : save sorted model features with indices [1..n]\n"
//	"-roc [filename] : save roc-curve\n"
	"-crossval [n] : n-fold crossvalidation mode -1=leave-one-out\n"
//	"      optional: outer crossvalidation mode, -1=leave-one-out\n"
	"-sig [n] : try n random label vectors for significance testing \n"
	"-c [c][c2 dc] : set the parameter C (%g)\n"
	"        optional: hyperparameterselection from c to c2 with stepsize dc\n"
	"-e [epsilon][e2 de] : set the epsilon in loss function (%g)\n"
	"        optional: hyperparameterselection from e to e2 with stepsize ec\n"
	"-g [epsitol] : set relative tolerance of epsilon(%g)\n"
	"-shuffle [0|1|2] : 0-no shuffle 1-random 2-random with timeseed (%d)\n"
	"-anneal [0|1 estart edec eterm] : 0-no annealing 1-use annealing (%d)\n"
	"    estart:(%g), edec: (%g), eterm: (%g)\n"
	"-block [0|1 nmax cmax cycmax] : 0-no blockopt 1-use blockopt (%d)\n"
	"    nmax: max blocksize(%d), cmax: max clamped(%d), cycmax: max cycles(%g)\n"
	"-wss [0|1|2 fact] : 0-first violation, 1-max violation, 2-relative: (%d)\n"
	"    select first violation > max violation * fact(%g)\n"
	"-reduce [0|1 size fact]:0-no reduction 1-reduction(%d) size(%d) fact(%g)\n"
	"-flimit [f][cut] : limit maximum number of features (%d)\n"
	"    optional [cut]: 0 - if #boundings>f delete model, 1 - opp. (%d) \n"
	"-timeout [t] : stops smo after t seconds\n"
	"-kernel [type p1 p2 p3] : apply kernel of type\n"
	"    0 LINEAR, 1 POLY, 2 RBF, 3 SIGMOID, 4 PLUMMER, 5 SIN, 6 USER - (%d)\n"
	"    and set Kernel parameters (kernel dependent)\n"
	"    p1=factor(%g) p2=offset(%g) p3=exponent(%g)\n"
	"-b : scale labels to enforce zero bias\n"
	"-be : use balanced error(tp/2p + tn/2n) instead of rate(tp+tn)/(p+n)\n"
	"-r [fe] : if positive, retrain feature expression levels with epsilon=e*fe (%g)\n"
	"-normlimit : use max and min during normalization\n"
	"-isrelation : input data is dyadic (matrix) data\n"
	"-verbose [0|1] : 1 - give progress info; 0 - give results only(default)\n"
	"\n"
	"extensions were added to given filenames as follows:\n"
	"   *.data"C_ENDING"  for dyadic (matrix) or vectorial data \n"
	"   *.y"C_ENDING"     for classes (+1,-1) or real valued target (same #rows as in *.data) \n"
	"   *.yout"C_ENDING"  for a vector Y calculated by model \n"
	"   *.zdata"C_ENDING" for complex feature vector matrix \n"
	"   *.model"C_ENDING" for model data in plain text\n"
	"   *.fi"C_ENDING" for feature indices \n"
	"   *.fv"C_ENDING" for support feature vectors\n"
	"   *.fr"C_ENDING" for support feature ranking\n"
	"   *.msclass.plt, *.msregress.plt, *.msdata"C_ENDING" for modelselection\n"
	"   *.sigclass.plt, *.sigregress.plt, *.sigdata"C_ENDING" for random classes\n"
	"   *.roctest.plt, *.rocdata"C_ENDING" for roc visualization\n"
	,p->par.C,p->par.epsi,p->par.epsiTol,p->par.shuffle,p->par.doannealing,p->par.estart,p->par.edec,p->par.eterm,p->par.doblock,p->par.blockNmax,p->par.blockCmax,p->par.blockOver,p->par.domaxselection,p->par.selectionfact,p->par.doreduction,p->par.redsize,p->par.redfact,p->par.flimit,p->par.allowCcut,p->par.kernel,p->par.k1,p->par.k2,p->par.k3,p->par.retrain_fe);
	exit(1);
}
