/*
Software library implementing the P-SVM according to

"Sepp Hochreiter, Klaus Obermayer;
 Support Vector Machines for Dyadic Data, Neural Computation, 2005"

Copyright 2005 TU-Berlin, Tilman Knebel
release 1.31 25.5.2008 (bugfix in getCError)

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

If you use the software please cite 
"Sepp Hochreiter, Klaus Obermayer;
 Support Vector Machines for Dyadic Data, Neural Computation, 2005"
in your publications.
*/

//to do:
//if all labels equal zero, and no c,e -> error
//feature rank
//block with brunn nielson algorithm
//include std svm from libsvm for better comparison
//timestamps into logfile
//significance: flag controlling shuffle algorithm (hamming distance minimum etc.)
//significance: inverse cumulated beta function for confidence
//flag for using (un)balanced errors
//logarithmic parameter search
//dll into package
//new chart: # of features vs. (G)error
//catch out of memory errors
//significance plot as 3d with: hamming dist - occurences - error
//matlab library with crossvalidation loops
//if remove bias is activated, it is applied before doing crossvalidation. Better: do bias removal for each crossval step
//flag for using (un)normalized feature intensities

//#define new DEBUG_NEW
//#define delete DEBUG_DELETE
//#include <afx.h>
#include <cstring>
#include <cmath>
#include <cstdio>
#include <ctime>
#include <cstdlib>
#include <list>
#include <algorithm>
#include <vector>
#include <float.h>
#include <stdarg.h>
using namespace std;
#include "psvm.h"

//Compiler definitions: SingleSMO, smofloat(see header file)
//WSS = working set selection

//for checking the validity of the algorithm
#define logi 0//def=0 print additional debugging information - fast
#define blocklogi 0//def=0 print debugging information for block optimization - fast
#define check 0//def=0 check consistency after each optimization step - very slow - makes sense with logi
#define laughter 0 //def=0 print sthg if sub problems are singular

//export interna to file for matlab processing (obsolete since matlab interface)
#define dynamics 0//def=0 log dynamics of convergence to files dyna, dynktt, dyndi, dynt - slow

//for timing
#define matlablog 0 //def=0 log alpha for matlab analysis - very slow

//WSS1
#define useRndWss 0//def=0// 0=use last found index, 1=use random with timeseed, 2=use random with const seed, //wrks only with domaxselection==0, random seed changed by shuffle option
//#define WSS1 1//def=1, 1=maxKKT, 2=KKT>maxselecfact, 0=KKT>0, 3=max delta alpha, 4=max delta alpha exact signum
//WSS1 steht in domaxselection


#if defined S1
#define WSS2 5
#define DualSMO 0
#define SingleUpdate 1
#define timelog 1
#elif defined S2
#define WSS2 5
#define DualSMO 0
#define SingleUpdate 2
#define timelog 1

#elif defined D11
#define WSS2 1
#define DualSMO 1
#define DualUpdate 1
#define timelog 1
#elif defined D12
#define WSS2 1
#define DualSMO 1
#define DualUpdate 2
#define timelog 1
#elif defined D21
#define WSS2 2
#define DualSMO 1
#define DualUpdate 1
#define timelog 1
#elif defined D22
#define WSS2 2
#define DualSMO 1
#define DualUpdate 2
#define timelog 1
#elif defined D41
#define WSS2 4
#define DualSMO 1
#define DualUpdate 1
#define timelog 1
#elif defined D42
#define WSS2 4
#define DualSMO 1
#define DualUpdate 2
#define timelog 1
#elif defined D51
#define WSS2 5
#define DualSMO 1
#define DualUpdate 1
#define timelog 1
#elif defined D52
#define WSS2 5
#define DualSMO 1
#define DualUpdate 2
#define timelog 1
#elif defined D61
#define WSS2 6
#define DualSMO 1
#define DualUpdate 1
#define timelog 1
#elif defined D62
#define WSS2 6
#define DualSMO 1
#define DualUpdate 2
#define timelog 1
#else
#define timelog 1 //def=0 log timing for WSS1,WSS2,block,updateQ,update1,update2 - med speed
//WSS2
#define WSS2 1// def=5, 1=maxKKT, 2=KKT>0, 3=SINGLEmaxdelaalpha, 4=SINGLEmaxdeltaalphasignum, 5=DUALmaxdelta, 6=DUALmaxdeltasignum

//Update1
#define SingleUpdate 1//def=1, 1=leave signum unchanged, 2=calculate exact signum
//Update2
#define DualUpdate 1//def=1, 1=leave signum unchanged, 2=calculate exact signum

//Update
#if defined SingleSMO //compiler option - def=0 (sometimes single is faster)
#define DualSMO 0//0 - single alpha optimization, 1 - optimize alpha pairs
#else 
#define DualSMO 1//0 - single alpha optimization, 1 - optimize alpha pairs
#endif
#endif


//Block optimization
#define boxIgnoreSingularity 1 //def=1// 1=if subproblem is singular, ignore divides by zero, 0=stop optimization if singular

//Annealing heuristic
#define termAnn 0 //def=0 annealing in termination crit instead of epsilon annealing: with space dataset this has (exactly) no advantage as block optimization only

//nonsense
#define testepsi 0//def=0// not yet ready: calculate epsilon from demanded number of features
#define useBothConst 1//def=1// (only with useBox==1) 0: ignore negative primal constraint -> use only positive alphas (a bit stupid)
#define useClass 1//not used

bool verbose=false;//true:print all progress messages to stdout and logfile, can be overwritten in main()
bool verbimp=false;//true:print important messages to stdout and logfile, can be overwritten in main()
bool verb_ms=false;//true:print important messages to stdout and logfile, can be overwritten in main()
FILE* flog=NULL;//handle to global logfile

//conventions:
//class members are labeled with leading "m"
//class names begin with capitals
//classes which reserve memory are responsible to delete it, at least in destructor

void pinfo(char *fmt,...)
{//function for printing messages to stdout and logfile
	if(verbose){
        va_list ap;
		va_start(ap,fmt);
		vprintf(fmt,ap);
		if(flog) if(vfprintf(flog,fmt,ap)<0) {if(verbimp)printf("error writing to disc!\n");exit(-1);}
		va_end(ap);
	}
}
void minfo(char *fmt,...)
{//function for printing messages to stdout and logfile
	if(verb_ms){
        va_list ap;
		va_start(ap,fmt);
		vprintf(fmt,ap);
		if(flog) if(vfprintf(flog,fmt,ap)<0) {if(verbimp)printf("error writing to disc!\n");exit(-1);}
		va_end(ap);
	}
}
void xinfo(char *fmt,...)
{//function for printing important messages to stdout and logfile
	if(verbimp){
        va_list ap;
		va_start(ap,fmt);
		vprintf(fmt,ap);
		if(flog) if(vfprintf(flog,fmt,ap)<0) {printf("error writing to disc!\n");exit(-1);}
		va_end(ap);
	}
}
double timeDiff(clock_t t0,clock_t tk)
{//calculates difference between two times
	return ((double)tk-(double)t0)/(double)CLOCKS_PER_SEC;
}


//###################################################################################
//###################################################################################
//class dataset: stores dyadic matrix data or vectorial data and/or corresponding labels
//###################################################################################
//###################################################################################
Dataset::Dataset()
{//zero initialization of all members, no memory reserved
	mTrans=false;//false: vectorial data, stored vector by vector ([obj1, obj2, ...] where obj=[dim1,dim2,...])
				//true: dyadic or kernel data, stored description by description (transposed) [x1(1) x2(1) x3(1)...x1(2) x2(2)...]
	mStdKernel=false;//true: data is pairwise from a kernel - false: data is not from a kernel
	mN=0;//currently _used_ number of vectors (data objects)
	mDim=0;//currently _used_ number of components per data objects (if dataset is kernelmatrix, this is the number of complex features)
	mDatamem=0;//total number of data elements reserved in memory
	mYmem=0;//number of y-elements reserved in memory
	mY=NULL;//pointer to y-data
	mData=NULL;//pointer to x-data
}
Dataset::~Dataset()
{//free all reserved memory
	free();//release memory
}
void Dataset::free()
{//free all reserved memory, zero initialization
	if(mData) delete[] mData;
	mData=NULL;
	if(mY) delete[] mY;
	mY=NULL;
	mDatamem=0;
	mYmem=0;
	mDim=0;
	mN=0;
	mStdKernel=false;
	mTrans=false;
}
void Dataset::reserve(int n,int dim, bool y)
{//allocates memory for a dataset of size(n,dim)
//y=true: care about y-vector, y=false: delete y-vector
//mN and mDim remains unchanged, because the data memory will not be initialized
	//new memory needed?
	if(n*dim>mDatamem)
	{//new x-memory is needed: reallocate memory
		delete[] mData;
	    mData = new ftype[n*dim];//allocation
		if(!mData) xinfo("not enough memory - sorry PSVM needs VERY much memory\n");
		mDatamem=n*dim;//size of reserved memory, needed to save time if new data is loaded
	}
	if(y)
	{
		if(n>mYmem)
		{//new y-memory is needed: reallocate memory
			if(mY)delete[]mY;
			mY=new ftype[n];
			if(!mY) xinfo("not enough memory - sorry PSVM needs VERY much memory\n");
			mYmem=n;
		}
	}else
	{//y is not required: delete y
		if(mY)delete[]mY;
		mYmem=0;
		mY=NULL;
	}
	//mN and mDim has to be initialized by caller, when the data becomes valid
//	mN=n;
//	mDim=dim;
}
void Dataset::copyFrom(Dataset *src)
{//copies a complete dataset object from 'src' into 'this'
	reserve(src->mN,src->mDim,src->mY!=NULL);
	int sze,n;
	sze=src->mN*src->mDim;
	for(n=0;n<sze;n++) mData[n]=src->mData[n];
	if(src->mY) for(n=0;n<src->mN;n++) mY[n]=src->mY[n];
	mN=src->mN;mDim=src->mDim;
	mTrans=src->mTrans;
	mStdKernel=src->mStdKernel;
}
bool Dataset::load(char* fnData, char* fnY,int transpose)
{	//allocates mem
	//loads data matrix and y-vector from external file with filenames fnData and fnClass
	//updates mN,mDim
	//returns false, if file not exists
	//for data without labels, set second param fnY=NULL
/*
	Fileformat fnData (all values are of type ftype) in plain text: 
	k(1,1) k(1,2) k(1,3) ... k(1,dim) \n
	k(2,1) k(2,2) k(2,3) ... k(2,dim) \n
	...
	k(N,1) k(N,2) k(N,3) ... k(N,dim) \n
    
	Fileformat fnClass (all values are float types):
	y(1) y(2) y(3) ... y(N) [\n]

	where
	k is either dyadic or kernel martix K or vectorial data (X or Z)
	dim = nr of cols in file, math symbol is P or N
	N = nr of rows(objects) in file , math symbol is L

	file content is interpreted as row vectors
	if transpose==0:
		access to element k(row,col) with mData[row*mDim+col]
	if transpose==1:
		access to element k(row,col) with mData[col*mN+row]
	watch swapping operations after random shuffling
*/
	unsigned char* raw;//data store
	int l;//length of raw data in bytes

	//Read file into memory
	{
		FILE* file;
		file = fopen(fnData, "r");
		if(!file) {
			free();
			return false;
		}
		fseek(file,0,SEEK_END);
		l=ftell(file);
		fseek(file,0,SEEK_SET);
		raw=new unsigned char[l+1];
		l=fread(raw,1,l,file);
		fclose(file);
		raw[l]=0;
	}
	//raw[0..l-1] = loaded data
	//raw[l]=NULL for zerotermination if file ends directly after last character

	//Get size of Data by parsing all numbers and terminate all element strings with zero
	{
		int ldim,ln;
		int i,k;
		unsigned char c,c1;

		ldim=0;
		ln=0;
		k=0;
		c=0;
		for(i=0;i<l;i++)
		{//i=bytepos, k=element pos, ln=current line
			if(raw[i]>32 && c<=32) k++;//new element begins
			if((raw[i]==13|| raw[i]==10) && !(c==13 || c==10)) ln++;//new line command
			c1=c;
			c=raw[i];//save nl-commands before zero termination
			if(raw[i]<=32 && c1>32) raw[i]=0;//zero termination of items
		}
		if(k!=ln*(int)(k/ln))
		{
			free();
			xinfo("error in Dataset::load - lines in datafile have different numbers of elements\n");
			return false;
		}
		ldim=k/ln;
		//ldim = nr of cols
		//ln = nr of rows
		xinfo("   loading dataset with labels dim(%d:%d)\n",ln,ldim);

		//new memory needed?
		reserve(ln,ldim,fnY!=NULL);//reserve space for y and data
		mN=ln;mDim=ldim;//content is valid
	}

	//load data
	int k,i,j;
	double x;
	k=0;
	if(transpose==0)
	{
		l=mN*mDim;
		for(i = 0; i <l; i++)
		{
				while(raw[k]<=32) k++;
				x=atof((char*)&raw[k]);
				while(raw[k]!=0) k++;
				mData[i]=(ftype)x;
		}
		mTrans=false;
	}else{
		for(i = 0; i <mN; i++)
		{
			for (j=0;j<mDim;j++)
			{
				while(raw[k]<=32) k++;
				x=atof((char*)&raw[k]);
				while(raw[k]!=0) k++;
				mData[j*mN+i]=(ftype)x;
			}
		}
		mTrans=true;
	}
	delete[] raw;

	if(fnY)
	{
        //load class labels directly from file
		FILE* file;
		int i;
		file = fopen(fnY, "r");
		if(!file) {
			free();
			return false;
		}
		for(i = 0; i <mN; i++)
		{
			if(fscanf(file,"%le\n",&x)==EOF)//read integer, ignore white spaces 
			{
				free();
				xinfo("error in Dataset::load - size of label-vector does not match to data rows\n");
				return false;
			}
            mY[i]=(ftype)x;
		}
		fclose(file);
	}
	return true;
}
void Dataset::shuffle(int shufflemode)
{//random shuffleling of the data objects and labels removes dependence of crossvalidation results with sequence of data objects
//shufflemode=0 - no shuffling
//shufflemode=1 - constant seed
//shufflemode=2 - time seed
	unsigned int maxi=RAND_MAX;
	int f;//number of necessary rand()
	int k,r;
	int i,j;
	int fact[9];
	if(shufflemode==0)return;
	if(maxi==INT_MAX) maxi=INT_MAX; else maxi++;

	f=0;
	k=1;
	fact[f]=k;
	while(k<mN)
	{
		f=f+1;
		k=k*maxi;
		fact[f]=k;
	};
	// rand= 0..255(=randmax)
	//fact= [1 256(=randmax+1) 65536 ...]
	//r=0..255/256
	//if randmax=intmax, a little problem occurs, but around worked

	if(shufflemode==2) srand( (unsigned)time( NULL ) );
	if(mTrans)
	{//objects are stored column wise
		for(i=0;i<mN;i++)
		{//too much shuffeling, but simpler
			r=0;
			for(k=0;k<f;k++)r=r+rand()*fact[k];
			r=(int)((double)r*(double)mN/(double)fact[f]);
			if(r>=mN) r=mN-1;//work around...
			for(j=0;j<mDim;j++) swap(mData[j*mN+i],mData[j*mN+r]);
			if(mY) swap(mY[i],mY[r]);
		}
	}else{//objects are stored vector by vector
		for(i=0;i<mN;i++)
		{//too much shuffeling, but simpler
			r=0;
			for(k=0;k<f;k++)r=r+rand()*fact[k];
			r=(int)((double)r*(double)mN/(double)fact[f]);
			if(r>=mN) r=mN-1;//work around...
			ftype* datai,*datak;
			datai=&mData[i*mDim];
			datak=&mData[r*mDim];
			for(j=0;j<mDim;j++) swap(datai[j],datak[j]);
			if(mY) swap(mY[i],mY[r]);
		}
	}
}
void Dataset::tileTo(Dataset* learn,Dataset* test,int testpart,int parts)
{
	//Split Dataset "this" into "learn" and "test"
	//by dividing "this" into "parts" parts and copying part nr. "testpart" into "test",
	//vectors from the other parts are copied into "learn"
	//for data matrices obtained by a kernel function, the tiling is applied on columns and rows!
    if(parts>mN) parts=mN;
	if(testpart>=parts || !mData || parts<=1 || testpart<0) {xinfo("error in Dataset::tileTo - unexpected parameters\n");exit(-1);}
	if(mStdKernel && mDim!=mN) {xinfo("error in Dataset::tileTo - unexpected parameters\n");exit(-1);}//StdKernels are quadratic

	//compute the start and end position of testset
	int nl,nt;
	int start, end;//testset
	{
		int b,m;
		//the number of samples per part is not always the same:
		//example: 38 samples, 5 parts
		b=mN/parts;//Minimum samples per part: 7
		m=mN%parts;//parts with more samples:  3
		//samples per part: 7 7 8 8 8 = 0-6 7-13 14-21 22-29 30-37
		if(testpart>=parts-m) {//parts with 8 samples: 2 3 4
			start=(parts-m)*b+(testpart-parts+m)*(b+1);
			end=start+b;
		}else {//parts with 7 samples: 0 1
			start=testpart*b;
			end=start+b-1;
		}
		nt=end+1-start;//size of testset
		nl=mN-nt;//size of learnset
	}
	if(!mStdKernel)
	{
		learn->reserve(nl,mDim,true);
		test->reserve(nt,mDim,true);
	}else
	{//with standard kernels, the tiling must be over rows and cols, cols depend from rows by kernelfunction
		learn->reserve(nl,nl,true);
		test->reserve(nt,nl,true);
	}
	pinfo("   CV: extraction of part %d/%d from data, testsize %d learnsize %d\n",testpart+1,parts,nt,nl);

	//copy data into learn and test
	if(!mStdKernel)
	{
		int s,i,j;
		if(mTrans)
		{
			s=0;
			for(i=0;i<mN;i++)
			{
				if(i==start) i=end+1;//jump over testset
				if(i<mN){
					learn->mY[s]=mY[i];
					for(j=0;j<mDim;j++) learn->mData[s+j*nl]=mData[i+j*mN];
					s++;
				}
			}
			s=0;
			for(i=start;i<=end;i++)
			{
				test->mY[s]=mY[i];
				for(j=0;j<mDim;j++) test->mData[s+j*nt]=mData[i+j*mN];
				s++;
			}
		}else
		{
			s=0;
			for(i=0;i<mN;i++)
			{
				if(i==start) i=end+1;//jump over testset
				if(i<mN){
					learn->mY[s]=mY[i];
					for(j=0;j<mDim;j++) learn->mData[s*mDim+j]=mData[i*mDim+j];
					s++;
				}
			}
			s=0;
			for(i=start;i<=end;i++)
			{
				test->mY[s]=mY[i];
				for(j=0;j<mDim;j++) test->mData[s*mDim+j]=mData[i*mDim+j];
				s++;
			}
		}
		learn->mN=nl;
		learn->mDim=mDim;
		learn->mTrans=mTrans;
		test->mN=nt;
		test->mDim=mDim;
		test->mTrans=mTrans;
		return;
	}else
	{//StdKernel:Tiling must be in rows and cols, rows depend from cols
		int i,j;//source
		int s;//destination in N-direction
		int t;//destination in Dim-direction
		if(mTrans)
		{
			s=0;
			for(i=0;i<mN;i++)
			{
				if(i==start) i=end+1;//jump over testset
				if(i>=mN) break;
				//dont continue with learnsetif testset is at the upper end 
				learn->mY[s]=mY[i];
				t=0;
				for(j=0;j<mDim;j++) {
					if(j==start) j=end+1;
					if(j>=mDim) break;
					learn->mData[s+t*nl]=mData[i+j*mN];
					t++;
				}
				s++;
			}
			s=0;
			for(i=start;i<=end;i++)
			{
				test->mY[s]=mY[i];
				t=0;
				for(j=0;j<=mDim;j++) {
					if(j==start) j=end+1;
					if(j>=mDim) break;
					test->mData[s+t*nt]=mData[i+j*mN];
					t++;
				}
				s++;
			}
		}else
		{//StdKernels are always transposed, but nevertheless:
			s=0;
			for(i=0;i<mN;i++)
			{
				if(i==start) i=end+1;//jump over testset
				if(i>=mN) break;
				learn->mY[s]=mY[i];
				t=0;
				for(j=0;j<mDim;j++) {
					if(j==start) j=end+1;
					if(j>=mDim) break;
					learn->mData[s*nl+t]=mData[i*mDim+j];
					t++;
				}
				s++;
			}
			s=0;
			for(i=start;i<=end;i++)
			{
				test->mY[s]=mY[i];
				t=0;
				for(j=0;j<=mDim;j++) {
					if(j==start) j=end+1;
					if(j>=mDim) break;
					test->mData[s*nl+t]=mData[i*mDim+j];
					t++;
				}
				s++;
			}
		}
		learn->mN=nl;
		learn->mDim=nl;
		learn->mTrans=mTrans;
		learn->mStdKernel=mStdKernel;
		test->mN=nt;
		test->mDim=nl;
		test->mTrans=mTrans;
		test->mStdKernel=mStdKernel;
	}
}
void Dataset::normalize()
{//normalizes set 'this' without saving normalization parameters
	double dmean,dvar;
	double x;
	int i,j;

	pinfo("   normalizing dataset dim(%d,%d)\n",mN,mDim);
	for (j=0;j<mDim;j++)
    {
		ftype* data;
		data=&mData[j*mN];
		//compute mean and variance
		dmean= data[0];
		for (i=1;i<mN;i++)
		{
			x=data[i];
			dmean+= x;
		}
		dmean/=(double)mN;
		dvar=0;
		for (i=0;i<mN;i++)
			dvar+= sqr(data[i]-(ftype)dmean);
		//leaving out following scaling factor saves computational costs in smo
//		if(set->n>1) dvar/=(double)(set->n-1);
//		else dvar=1.;
//		x=x/sqrt(dvar)
//		x=x/sqrt(set->n-1) 
		dvar=max(0.0000000001,dvar);
		dvar=1/sqrt(dvar);
	
		//normalize all data in this dataset to mean=0, variance=1/(sqrt(N-1))
		for (i=0;i<mN;i++)
		{
			x=data[i];//get data
			data[i]=(ftype)((x-dmean)*(dvar));//save normalized data to "this"
		}
    }//next j=0..this.dim-1
}
void Dataset::normalize(Dataset* set,bool limit)
{//normalizes 'this' without saving normalization parameters and 
//normalize and limit "set" in the same manner like "this"
	double dmin,dmax,dmean,dvar;
	double x;
	int i,j;

	if(!set) {normalize();return;}
	pinfo("   normalizing dataset dim(%d,%d) and dim(%d,%d)\n",mN,mDim,set->mN,set->mDim);
	for (j=0;j<mDim;j++)
    {
		ftype* data;
		ftype* data1;
		data=&mData[j*mN];
		data1=&set->mData[j*set->mN];
		//compute mean and variance
		dmin= data[0];
		dmax= dmin;
		dmean=dmin;
		for (i=1;i<mN;i++)
		{
			x=data[i];
			dmin=min(x,dmin);
			dmax=max(x,dmax);
			dmean+= x;
		}
		dmean/=(double)mN;
		dvar=0;
		for (i=0;i<mN;i++)
			dvar+= sqr(data[i]-(ftype)dmean);
		//leaving out following factor saves computational costs in smo
//		if(set->n>1) dvar/=(double)(set->n-1);
//		else dvar=1.;
//		x=x/sqrt(dvar)
//		x=x/sqrt(set->n-1) 
		dvar=max(0.0000000001,dvar);
		dvar=1/sqrt(dvar);
		//(max,min,mean,variance)
	
		//normalize all data in this dataset
		for (i=0;i<mN;i++)
		{
			x=data[i];//get data
			data[i]=(ftype)((x-dmean)*(dvar));//save normalized data to "this"
		}
		if(limit)
			for (i=0;i<set->mN;i++)
			{
				x=data1[i];//get data
				x=max(x,dmin);//must be limited to the range of selected vectors, 
				x=min(x,dmax);//the svm learns/classifies only in the range of the learnset (min..max)
				data1[i]=(ftype)((x-dmean)*(dvar));//save normalized data to "this"
			}
		else
			for (i=0;i<set->mN;i++)
				data1[i]=(ftype)((data1[i]-dmean)*(dvar));//save normalized data to "this"
    }//next j=0..this.dim-1
}
void Dataset::balance()
{
	double psum,nsum;
	int i;
	int pn,nn;
	if(!mY || mYmem<=0) return;
	psum=0;
	nsum=0;
	pn=0;
	nn=0;
	for(i=0;i<mN;i++)
	{
		if(mY[i]<0) {nsum+=mY[i];nn++;}
		if(mY[i]>0) {psum+=mY[i];pn++;}
	}
	nsum=(-2.)*(double)mN/nsum;
	psum=(2.)*(double)mN/psum;
	for(i=0;i<mN;i++)
	{
		if(mY[i]<0) mY[i]*=nsum;
		if(mY[i]>0) mY[i]*=psum;
	}
}

//###################################################################################################
//###################################################################################################
//###################################################################################################
//###################################################################################################
//###################################################################################################
//###################################################################################################
//###################################################################################################
//###################################################################################################
//###################################################################################################
//###################################################################################################
//###################################################################################################
//###################################################################################################
//###################################################################################################
//###################################################################################################
//###################################################################################################
//###################################################################################################
//class NormParam offers functions to apply and remove normalization of kernel matrices or dyadic data
NormParam::NormParam()
{//zero initialization of all members
	mDim=0;//number of dimensions or features which obtain a normalization 
	mDimMem=0;//number of dimensions or features reserved in memory
	mX=NULL;//pointer to array of normalization parameters, mDim valid elements, mDimMem allocated elements
}
NormParam::~NormParam()
{//destructor: delete memory
	free();
}
void NormParam::free()
{//delete memory and zero initialization
	if(mX) delete[]mX;
	mX=NULL;
	mDimMem=0;
	mDim=0;
}
void NormParam::reserve(int dim)
{//reserves memory for 'dim' normalization parameter records, if nessessary
	if(mX)
	{
		if(dim>mDimMem) {
			delete[]mX;
			mX=new NpEntry[dim];
		}
	}else mX=new NpEntry[dim];
	mDimMem=dim;
}
#if 0 //currently not needed
void NormParam::clear(int dim)
{
	reserve(dim);
	int i;
	for(i=0;i<dim;i++)
	{
		mX[i].max=0;
		mX[i].min=0;
		mX[i].scale=0;
		mX[i].mean=0;
	}
}
#endif
void NormParam::getFromSet(Dataset* set)
{//fills array mX with normalization parameters obtained from set
	double dmin,dmax,dmean,dvar;
	double x;
	int i,j;

	if(!set || !set->mTrans)
	{
		xinfo("error in NormParam::getFromSet - Normalization is only for Kernel which must be transposed\n");
		exit(-1);
	}
	pinfo("   getting %d normalization parameters\n",set->mDim);
	reserve(set->mDim);
//get normalization parameters for every data vector component or complex feature
	for (j=0;j<set->mDim;j++)
    {
		//compute mean and variance
		ftype* data;
		data=&set->mData[j*set->mN];
		dmin= data[0];
		dmax= dmin;
		dmean=dmin;
		for (i=1;i<set->mN;i++)
		{
			x=data[i];
			dmin=min(x,dmin);
			dmax=max(x,dmax);
			dmean+= x;
		}
		dmean/=(double)set->mN;
		dvar=0;
		for (i=0;i<set->mN;i++)
			dvar+= sqr(data[i]-(ftype)dmean);
		//leaving out following factor saves computational costs in smo
//		if(set->n>1) dvar/=(double)(set->n-1);
//		else dvar=1.;
//		x=x/sqrt(dvar)
//		x=x/sqrt(set->n-1) 
		dvar=max(0.0000000001,dvar);
		dvar=1/sqrt(dvar);
		//(max,min,mean,variance)
		mX[j].min=(ftype)dmin;
		mX[j].max=(ftype)dmax;
		mX[j].mean=(ftype)dmean;
		mX[j].scale=(ftype)dvar;//this scale leads to std deviation 1/sqrt(n-1)
	}
	mDim=set->mDim;//so much entries in mX are valid, while mDimMem are available in memory
}
void NormParam::putOnSet(Dataset* set,bool limit)
{//normalizes "set" with stored normalization parameters mX
	int j,i;
	ftype x;
	if(!set || set->mDim!=mDim) 
	{
		xinfo("error in NormParam::putOnSet - Dataset does not match to normalization parameters\n");
		exit(-1);
	}
	pinfo("   normalizing dataset dim(%d,%d)\n",set->mN,set->mDim);
	if(limit)
		for (j=0;j<set->mDim;j++)
		{
			ftype* data;
			data=&set->mData[j*set->mN];
			//normalize all data in this dataset
			for (i=0;i<set->mN;i++)
			{
				x=data[i];//get data
				x=max(x,mX[j].min);//must be limited to the range of selected vectors, 
				x=min(x,mX[j].max);//the svm learns/classifies only in the range of the learnset (min..max)
				data[i]=(ftype)((x-mX[j].mean)*(mX[j].scale));//save normalized data to "this"
			}
	    }//next j=0..this.dim-1
	else
	{
		for (j=0;j<set->mDim;j++)
		{
			ftype* data;
			data=&set->mData[j*set->mN];
			for (i=0;i<set->mN;i++)
				data[i]=(ftype)((data[i]-mX[j].mean)*(mX[j].scale));//save normalized data to "this"
	    }//next j=0..this.dim-1
	}
}

//###################################################################################################
//###################################################################################################
//###################################################################################################
//###################################################################################################
//###################################################################################################
//###################################################################################################
//###################################################################################################
//###################################################################################################
//###################################################################################################
//###################################################################################################
//###################################################################################################
//###################################################################################################
//###################################################################################################
//###################################################################################################
//###################################################################################################
//###################################################################################################
//###################################################################################################
// class SMO
//##########################################################################
//##########################################################################
//##########################################################################
//##########################################################################
SMO::SMO()
{//constructor: zero initialization of all memory relevant variables
	mAlpha=NULL;
	mSwp=NULL;
	mQ=NULL;
	mF=NULL;
	mmXY=NULL;
	mSubQ=NULL;
	mAsgn=NULL;
	mMask=NULL;
	mMat=new double[10000];//limited to submatrices with 100 dimensions
	mLine=new list<sline>;
	mAlphaMem=0;
	mSwpMem=0;
	mQMem=0;
	mmXYMem=0;
	mSubQMem=0;
	mAsgnMem=0;
	mMaskMem=0;
	mFMem=0;
}
SMO::~SMO()
{//destructor: release all used memory, result vector mAlpha will also be deleted!
	delete[] mMat;
	delete mLine;
	free();
}
void SMO::free()
{//release all used memory, result vector mAlpha will also be deleted!
	int i;
	if(mQ) for(i=0;i<mQMem;i++) if(mQ[i]){delete[]mQ[i];mQ[i]=NULL;}
	if(mQ) {delete[]mQ;mQ=NULL;}
	if(mSubQ) for(i=0;i<mSubQMem;i++) if(mSubQ[i]){delete[]mSubQ[i];mSubQ[i]=NULL;}
	if(mSubQ)delete[]mSubQ;
	if(mAlpha) delete[]mAlpha;
	if(mSwp) delete[]mSwp;
	if(mF) delete[]mF;
	if(mmXY) delete[]mmXY;
	if(mAsgn)delete[]mAsgn;
	if(mMask) delete[]mMask;
	mQMem=0;
	mSubQMem=0;
	mAlphaMem=0;
	mSwpMem=0;
	mFMem=0;
	mmXYMem=0;
	mAsgnMem=0;
	mMaskMem=0;
}
#if 0
void SMO::init()//private:
{//Initializes SMO-data mF,mAlpha,mQ
	int l,p;
	double lsum;
	mMaxepsi=0;
	for (p=0;p<mDim;p++)//row size of kernel matrix
    {
		//compute mF for mAlpha=0
		ftype* data;
		ftype* y;

		lsum=0;
		data=&mSet->mData[p*mSet->mN];
		y=mSet->mY;
		for (l=0;l<mSet->mN;l++)//col size of kernel matrix and vector y
			lsum+=(y[l]-mB) * data[l];// mB(mean) is for reducing rounding errors
		mmXY[p]=-(ftype)lsum;
		mF[p]=mmXY[p];//|mAlpha=0
		mMaxepsi=max((ftype)fabs(lsum),mMaxepsi);
		//mF=mQ*mAlpha-y*data    
		mAlpha[p]=(ftype)0.;//lagrangian zero initialized 
		mAsgn[p]=0;
	}
	mMaxepsi/=mPar->epsiTol+1;
	mNs=mDim;//all indices are active
	mLastp=0;//remember last chosen a1-index, for alternative working mSet selection
	//statistics
	mQCount=0;//number of updates in mQ
	mShc=0;//number of bounded mAlpha indices
	mShz=0;//number of zero mAlpha indices
}
#endif

inline bool SMO::getAlphaByKT(int &aInd)
{/*	run over mAlpha-indices and check KT-conditions
	if it is fulfilled, return false, 
	otherwise select the alpha with strongest violation of KKT-conditions
	return that index by 'aInd' and true
	
	modes:
	mPar->domaxselection==0: 	find first violation
	mPar->domaxselection==1: 	find max violation
	mPar->domaxselection==2: 	find first violation > selectionfact*maximum(violation)
	mPar->domaxselection==3: 	find index leading to maximum change of alpha while keeping the sign
	mPar->domaxselection==4: 	find index leading to maximum change of alpha using exact evaluation of the sign

	useRndWss==0: begin at the last finding pos
	useRndWss==1: begin at random index
	
	termAnn==0: if annealing is applied, use annealing of epsilon
	termAnn==1: if annealing is applied, use annealing of the termination parameter
	*/

	ftype a,fpz;
	signed char al;//0 - free, +-1 = +-mC
	int p;//runs over all alphas
	ftype error;

	//scan positive and negative violations for each mAlpha index
	//remember the largest absolute

	if(mPar->domaxselection==1 || mPar->domaxselection==2)
	{//1:search max violation 2: search violations above selectionfact*maxviolation
		int i;
		aInd=-9;//save index of most violating mAlpha
		error=(ftype)-1e20;//searching for a maximum, begin with -inf
#if useRndWss==0
		p=mLastp;//start searching with last finding
#else
		p=rand()%mNs;//experiment, if mNs>MAX_RND, this doesnt work optimal!!!!!!!!!!!!!!!!!!!!!!!!
#endif
		for(i=0;i<mNs;i++)
		{//search indices of non optimal mAlpha
			p++;
			if(p>=mNs)p=0;
			//fpz<fmc fmz<fpc
			a=mAlpha[p];
			al=mAsgn[p];
			if(al==0)
			{
#if useBothConst==1
				fpz=max(-mF[p]-mEpsilon,mF[p]-mEpsilon);
#else
				fpz=-mF[p]-mEpsilon;
#endif
				if(error<fpz)
				{
					error=fpz;
					aInd=p;
				}
				continue;
			}
			else if(a>0)
			{
				if(al!=1)//same as a<mC
				{//a=  ]0..mC[
					fpz=max(-mF[p]-mEpsilon,mF[p]+mEpsilon);
					if(error<fpz)
					{
						error=fpz;
						aInd=p;
					}
					continue;
				}
				//a= mC
				fpz=mF[p]+mEpsilon;
				if(error<fpz)
				{
					error=fpz;
					aInd=p;
				}
				continue;
			}
			else//if(a<0)
			{
				if(al!=-1)//same as a>-mC
				{
					fpz=max(mF[p]-mEpsilon,-mF[p]+mEpsilon);
					if(error<fpz)
					{
						error=fpz;
						aInd=p;
					}
					continue;
				}
				//a=-mC
				fpz=-mF[p]+mEpsilon;
				if(error<fpz)
				{
					error=fpz;
					aInd=p;
				}
				continue;
			}
		}
	}
	if(mPar->domaxselection==2)
	{//search next violation above selectionfact*maxviolation beginning from mLastp 
		int i;
		aInd=-9;
#if useRndWss==0
		p=mLastp;//start searching with last finding
#else
		p=rand()%mNs;//experiment, if mNs>MAX_RND, this doesnt work optimal!!!!!!!!!!!!!!!!!!!!!!!!
#endif
		for(i=0;i<mNs;i++)
		{//search indices of non optimal mAlpha
			p++;
			if(p>=mNs)p=0;
			a=mAlpha[p];
			al=mAsgn[p];
			if(al==0)
			{
#if useBothConst==1
				fpz=max(-mF[p]-mEpsilon,mF[p]-mEpsilon);
#else
				fpz=-mF[p]-mEpsilon;
#endif
				if(mPar->selectionfact*error<fpz)
				{
					aInd=p;
					break;
				}
				continue;
			}
			else if(a>0)
			{
				if(al!=1)//same as a<mC
				{
					fpz=max(-mF[p]-mEpsilon,mF[p]+mEpsilon);
					if(mPar->selectionfact*error<fpz)
					{
						aInd=p;
						break;
					}
					continue;
				}
				fpz=mF[p]+mEpsilon;
				if(mPar->selectionfact*error<fpz)
				{
					aInd=p;
					break;
				}
				continue;
			}
			else//if(a<0)
			{
				if(al!=-1)//same as a>-mC
				{
					fpz=max(mF[p]-mEpsilon,-mF[p]+mEpsilon);
					if(mPar->selectionfact*error<fpz)
					{
						aInd=p;
						break;
					}
					continue;
				}
				fpz=-mF[p]+mEpsilon;
				if(mPar->selectionfact*error<fpz)
				{
					aInd=p;
					break;
				}
				continue;
			}
		}
	}
	else if(mPar->domaxselection==0)
	{//search first violation, watch termination  - same as 2 with selfact=0
#if termAnn==0//standard case epsilon annealing
	if(!mLastepsi) error=mEpsilon*mPar->epsiTol*mPar->eterm;
	else error=mEpsilon*mPar->epsiTol;//KKT must be greater
#else
		error=mEpsilon*mTerm;
#endif
		aInd=-9;
		int i;
#if useRndWss==0
		p=mLastp;//start searching with last finding
#else
		p=rand()%mNs;//experiment, if mNs>MAX_RND, this doesnt work optimal!!!!!!!!!!!!!!!!!!!!!!!!
#endif
		for(i=0;i<mNs;i++)
		{//search indices of non optimal mAlpha
			p++;
			if(p>=mNs)p=0;
			a=mAlpha[p];
			al=mAsgn[p];
			if(al==0)
			{
#if useBothConst==1
				fpz=max(-mF[p]-mEpsilon,mF[p]-mEpsilon);
#else
				fpz=-mF[p]-mEpsilon;
#endif
				if(error<fpz)
				{
					error=fpz;
					aInd=p;
					break;
				}
				continue;
			}
			else if(a>0)
			{
				if(al!=1)//same as a<mC
				{
					fpz=max(-mF[p]-mEpsilon,mF[p]+mEpsilon);
					if(error<fpz)
					{
						error=fpz;
						aInd=p;
						break;
					}
					continue;
				}
				fpz=mF[p]+mEpsilon;
				if(error<fpz)
				{
					error=fpz;
					aInd=p;
					break;
				}
				continue;
			}
			else//if(a<0)
			{
				if(al!=-1)//same as a>-mC
				{
					fpz=max(mF[p]-mEpsilon,-mF[p]+mEpsilon);
					if(error<fpz)
					{
						error=fpz;
						aInd=p;
						break;
					}
					continue;
				}
				fpz=-mF[p]+mEpsilon;
				if(error<fpz)
				{
					error=fpz;
					aInd=p;
					break;
				}
				continue;
			}
		}
		//terminate only if no index found, so 
		//avoid termination caused by reaching low error by doubling error
		//this selection routine may break before all indices are tested, 
		//then the content of error is irrelevant
//		if(aInd>=0) error*=2;else error=0;
	}
	else if(mPar->domaxselection==3)
	{//search index leading to a maximum step
		int i;
		ftype maxdelta=(ftype)0;
		int maxind2;
		ftype d;
		maxind2=-8;
#if useRndWss==0
		p=mLastp;//start searching with last finding
#else
		p=rand()%mNs;//experiment, if mNs>MAX_RND, this doesnt work optimal!!!!!!!!!!!!!!!!!!!!!!!!
#endif
		for(i=0;i<mNs;i++)
		{//search indices of non optimal mAlpha
			p++;
			if(p>=mNs)p=0;
			//fpz<fmc fmz<fpc
			a=mAlpha[p];
			al=mAsgn[p];
	//>0:
	// -f-epsi limit -a1 .. C-a1
	//<0:
	// -f+epsi limit -C-a1 .. -a1
			if(al==0)
			{
				d=-mF[p]-mEpsilon;
				if(d<0) d=0;
				if(d>mC) d=mC;
				d=fabs(d);
				if(d>maxdelta){maxdelta=d;maxind2=p;}
				d=-mF[p]+mEpsilon;
				if(d<-mC) d=-mC;
				if(d>0) d=0;
				d=fabs(d);
				if(d>maxdelta){maxdelta=d;maxind2=p;}
				if(maxdelta==mC) break;
			}else{
				if(a>0)
				{
					d=-mF[p]-mEpsilon;
					if(d<-a) d=-a;
					if(d>mC-a) d=mC-a;
					d=fabs(d);
					if(d>maxdelta){maxdelta=d;maxind2=p;}
				}
				if(a<0)
				{
					d=-mF[p]+mEpsilon;
					if(d<-mC-a) d=-mC-a;
					if(d>-a) d=-a;
					d=fabs(d);
					if(d>maxdelta){maxdelta=d;maxind2=p;}
				}
			}
			//toDo:
			//stop if maximum delta is reached
		}
		aInd=maxind2;
		error=maxdelta;
	}
	else if(mPar->domaxselection==4)
	{//search index leading to a maximum step with exact evaluation of sign(alpha)
		int i;
		ftype maxdelta=(ftype)0;
		int maxind2;
		ftype d;
		maxind2=-8;
#if useRndWss==0
		p=mLastp;//start searching with last finding
#else
		p=rand()%mNs;//experiment, if mNs>MAX_RND, this doesnt work optimal!!!!!!!!!!!!!!!!!!!!!!!!
#endif
		for(i=0;i<mNs;i++)
		{//search indices of non optimal mAlpha
			p++;
			if(p>=mNs)p=0;
			//fpz<fmc fmz<fpc
			a=mAlpha[p];
//			al=mAsgn[p];
	//>0:
	// -f-epsi limit -a1 .. C-a1
	//<0:
	// -f+epsi limit -C-a1 .. -a1
			d=a-mF[p]-mEpsilon;
			if(d>0){//positive update
				if(d>mC) d=mC;
				d=fabs(d-a);
				if(d>maxdelta){maxdelta=d;maxind2=p;}
			}else//neg update
			{
				d=-mF[p]+mEpsilon;
				if(d+a<-mC) d=-mC-a;
				if(d+a>0) d=-a;
				d=fabs(d);
				if(d>maxdelta){maxdelta=d;maxind2=p;}
			}
		}
		aInd=maxind2;
		error=maxdelta;
	}
	//aInd=index with most violation
	//error=most violating amount
	if(mFirstepsi){//epsilon was just decreased
		mVio=error;//restart attenuation of mVio-oscillation
		mFirstepsi=false;
	}
	mVio=min(mVio,error);//no influence on the algorithm - only for printing the best KKT status ever
	mLastp=aInd;//for the pseudo randomizer in WSS

#if logi
	pinfo("step %d error %f mVio %f end %f WSSa1 %d \n",mACount,error,mVio,mEpsilon*mPar->epsiTol,aInd);
#endif

	//terminate if no KKT-violation found or violation below termination crit
#if termAnn==0//std
	if(aInd<0 || mVio<mEpsilon*mPar->epsiTol) return false;
	if(!mLastepsi && mVio<mEpsilon*mPar->epsiTol*mPar->eterm) return false;
#else
	if(aInd<0 || mVio<mEpsilon*mTerm) return false;
	//if(!mLastepsi && mVio<mEpsilon*mTerm*mPar->eterm) return false;
#endif
	return true;
}
inline void SMO::updateQ(int a1Ind)
{//matrix mQ(1..P,1..P) should only be computed on demand, now update row 'a1Ind'
	int l,p;
	double isum;
	ftype* dataa1,*datap;
	dataa1=&mSet->mData[(a1Ind+mRb)*mSet->mN];
#if timelog==1
	clock_t tk0,tk;
#endif
	if (!mQ[a1Ind])
	{
#if timelog==1//prevent time measure overhead
		tk0=clock();
#endif
		mQ[a1Ind]=new ftype[mNs];
		mQ[a1Ind][a1Ind]=(ftype)1.;
		for (p=0;p<a1Ind;p++)
		{
			datap=&mSet->mData[(p+mRb)*mSet->mN];
			if(mQ[p])
				isum=mQ[a1Ind][p]=mQ[p][a1Ind];
			else
			{
				isum=0;
				for (l=0;l<mSet->mN;l++)	isum+=dataa1[l] * datap[l];
#if laughter==1
				if(isum==1 || isum==-1)		xinfo("hahaha: %d equals %d, hahaha\n",p,a1Ind);
#endif
				isum=min(isum,mPar->singular);//isum=correlation(a1Ind,p)<1 (mData is scaled by 1/sqrt(mSet->mN))
				isum=max(isum,-mPar->singular);//isum=correlation(a1Ind,p)>-1 (mData is scaled by 1/sqrt(mSet->mN))
				mQ[a1Ind][p]=(ftype)isum;
			}
		}//this splitting is faster
		for (p=a1Ind+1;p<mNs;p++)
		{
			datap=&mSet->mData[(p+mRb)*mSet->mN];
			if(mQ[p])
				isum=mQ[a1Ind][p]=mQ[p][a1Ind];
			else
			{
				isum=0;
				for (l=0;l<mSet->mN;l++)	isum+=dataa1[l] * datap[l];
#if laughter==1
				if(isum==1 || isum==-1)		xinfo("hahaha: %d equals %d, hahaha\n",p,a1Ind);
#endif
				isum=min(isum,mPar->singular);//isum=correlation(a1Ind,p)<1 (mData is scaled by 1/sqrt(mSet->mN))
				isum=max(isum,-mPar->singular);//isum=correlation(a1Ind,p)>-1 (mData is scaled by 1/sqrt(mSet->mN))
				mQ[a1Ind][p]=(ftype)isum;
			}
		}
		mQCount++;
#if timelog==1
		tk=clock();
		mTQCount+=tk-tk0;
#endif
	}
}
inline void SMO::getAlphaUpdates(double &da1,double &da2, signed char &al1,signed char &al2,int a1Ind,int a2Ind,ftype a1,ftype f1,signed char l1,signed char u1)
{//returns update amounts da1,da2 for optimized sup-problem a1Ind,a2Ind
/*
input:
	al1,al2 - element of (-1 0 1 5), current status of alpha 
	a1Ind, a2Ind - indices of selected pair of alphas
	a1 - value of alpha 1
	f1 - value of f +- epsilon, depends on status of alpha1, for status 0, the other case has to be tested too
	l1,u1 - bounds for a1, depends on status of alpha1, for status 0 opposite bounds has to be tested too
output: 
	da1,da2 - calculated changes of alpha
*/
//update depends on sign of mAlpha, if mAlpha=0 check both directions
	double temp1,temp;
	ftype a2;
	temp1=mQ[a1Ind][a2Ind];
	temp=1.0/(1.0-temp1*temp1);
	a2=mAlpha[a2Ind];

	ftype f2;
	signed char u2,l2;
	bool ag1,ag2;//if mAlpha==0, try both possible signs of mAlpha
	double again1,again2;//saves best result
	int all1,all2;//saves best result

#if useBothConst==1
	ag2=false;ag1=(a1==0.);

	//get a2,f2,l2,u2
	if (a2>=0)
	{
		f2=mF[a2Ind]+mEpsilon;
		l2=0;
		u2=1;
		if(a2==0) ag2=true;
	}else// if(a2<0)
	{
		f2=mF[a2Ind]-mEpsilon;
		l2=-1;
		u2=0;
	}
	boxCondition(da1,da2,al1,al2,temp1,temp,f1,f2,a1,a2,l1,l2,u1,u2);
	if(ag1 && !ag2)
	{//alpha1 is zero, so we have to check both directions alpha>0(already done), alpha<0
		again1=da1;
		again2=da2;
		all1=al1;
		all2=al2;
		f1=mF[a1Ind]-mEpsilon;
		l1=-1;
		u1=0;
		boxCondition(da1,da2,al1,al2,temp1,temp,f1,f2,a1,a2,l1,l2,u1,u2);
		if(max(fabs(da1),fabs(da2))<=max(fabs(again1),fabs(again2)))
		{
			da1=again1;
			da2=again2;
			al1=all1;
			al2=all2;
		}
	}
	else if(ag2 && !ag1)
	{//alpha2 is zero, so we have to check both directions alpha>0(already done), alpha<0
		again1=da1;
		again2=da2;
		all1=al1;
		all2=al2;
		f2=mF[a2Ind]-mEpsilon;
		l2=-1;
		u2=0;
		boxCondition(da1,da2,al1,al2,temp1,temp,f1,f2,a1,a2,l1,l2,u1,u2);
		if(max(fabs(da1),fabs(da2))<=max(fabs(again1),fabs(again2)))
		{
			da1=again1;
			da2=again2;
			al1=all1;
			al2=all2;
		}
	}
	else if(ag1 && ag2)
	{//alpha1 and alpha2 is zero, so we have to check both directions alpha>0(already done), alpha<0
		again1=da1;
		again2=da2;
		all1=al1;
		all2=al2;
		f1=mF[a1Ind]-mEpsilon;
		l1=-1;
		u1=0;
		f2=mF[a2Ind]-mEpsilon;
		l2=-1;
		u2=0;
		boxCondition(da1,da2,al1,al2,temp1,temp,f1,f2,a1,a2,l1,l2,u1,u2);
		if(max(fabs(da1),fabs(da2))<=max(fabs(again1),fabs(again2)))
		{
			da1=again1;
			da2=again2;
			al1=all1;
			al2=all2;
		}
		again1=da1;
		again2=da2;
		all1=al1;
		all2=al2;
		f1=mF[a1Ind]+mEpsilon;
		l1=0;
		u1=1;
		f2=mF[a2Ind]-mEpsilon;
		l2=-1;
		u2=0;
		boxCondition(da1,da2,al1,al2,temp1,temp,f1,f2,a1,a2,l1,l2,u1,u2);
		if(max(fabs(da1),fabs(da2))<=max(fabs(again1),fabs(again2)))
		{
			da1=again1;
			da2=again2;
			al1=all1;
			al2=all2;
		}
		again1=da1;
		again2=da2;
		all1=al1;
		all2=al2;
		f1=mF[a1Ind]-mEpsilon;
		l1=-1;
		u1=0;
		f2=mF[a2Ind]+mEpsilon;
		l2=0;
		u2=1;
		boxCondition(da1,da2,al1,al2,temp1,temp,f1,f2,a1,a2,l1,l2,u1,u2);
		if(max(fabs(da1),fabs(da2))<=max(fabs(again1),fabs(again2)))
		{
			da1=again1;
			da2=again2;
			al1=all1;
			al2=all2;
		}
	}
#else

	//get a2,f2,l2,u2
	f2=mF[a2Ind]+mEpsilon;
	l2=0;
	u2=1;
	boxCondition(da1,da2,al1,al2,temp1,temp,f1,f2,a1,a2,l1,l2,u1,u2);
#endif
}
inline void SMO::boxCondition(double &da1,double &da2,signed char &al1,signed char &al2,double temp1,double temp,ftype f1,ftype f2,ftype a1,ftype a2,signed char l1,signed char l2,signed char u1,signed char u2)
{//calculation of optimal two mAlpha changes with limiting box constraints
	//todo: optimize like getalphaupdatesnew, but only for one square
	ftype a1new,a2new;
	double g2,g1;

	da1=(((double)f2*temp1-(double)f1)*temp);//mQ is normalized
	da2=(((double)f1*temp1-(double)f2)*temp);
	a1new=a1+(ftype)da1;
	a2new=a2+(ftype)da2;
	al1=5;al2=5;
	ftype uc1,lc1,lc2,uc2;
	uc1=mC*u1;
	uc2=mC*u2;
	lc1=mC*l1;
	lc2=mC*l2;
	/*
	a2
	|
	|  A|B|C
	|  -----
	|  D|E|F
	|  -----
	|  G|H|I
	|
	----------------a1
	//E is allowed area
	*/
	//check bounding conditions
	if (a1new<lc1)
	{//ADG a1 too low
		da1=lc1-a1;
		if (a2new<lc2)
		{//G a2 too low
			da2=lc2-a2;
			g2=-f2-da1*temp1;//optimal da2 for new da1
			if (da2<g2)
			{//destination is on the border D|E
				da2=g2;
				al1=l1;
				if(da2+a2>uc2) {da2=uc2-a2;al2=u2;}
				return;
			}else{//destination is on the border E|H
				g1=-f1-da2*temp1;//optimal da1 for new da2
				al2=l2;
				if (da1<g1)
				{
					da1=g1;
					if (da1+a1>uc1) {da1=uc1-a1;al1=u1;}
					return;
				}else al1=l1;//destination is in the corner
				return;
			}
		}else if (a2new>uc2)
		{//A a2 too high
			da2=uc2-a2;
			g2=-f2-da1*temp1;//optimal da2 for new da1
			if (da2>g2)
			{
				da2=g2;
				al1=l1;
				if(da2+a2<lc2) {da2=lc2-a2;al2=l2;}
				return;
			}else{
				g1=-f1-da2*temp1;//optimal da1 for new da2
				al2=u2;
				if (da1<g1)
				{
					da1=g1;
					if (da1+a1>uc1) {da1=uc1-a1;al1=u1;}
					return;
				}else al1=l1;
				return;
			}
		}else{//D
			al1=l1;
			da2=-f2-temp1*da1;//f2 will change by Q12*da1
			a2new=a2+(ftype)da2;
			if (a2new<lc2) {da2=lc2-a2;al2=l2;return;}
			else if (a2new>uc2) {da2=uc2-a2;al2=u2;return;}
			return;
		}
	}else if (a1new>uc1)
	{//CFI a1 too high
		da1=uc1-a1;
		if (a2new<lc2)
		{//I a2 too low
			da2=lc2-a2;
			g2=-f2-da1*temp1;//optimal da2 for new da1
			if (da2<g2)
			{
				da2=g2;
				al1=u1;
				if(da2+a2>uc2) {da2=uc2-a2;al2=u2;}
				return;
			}else{
				g1=-f1-da2*temp1;//optimal da1 for new da2
				al2=l2;
				if (da1>g1)
				{
					da1=g1;
					if (da1+a1<lc1) {da1=lc1-a1;al1=l1;}
					return;
				}else al1=u1;
				return;
			}
		}else if (a2new>uc2)
		{//mC a2 too high
			da2=uc2-a2;
			g2=-f2-da1*temp1;//optimal da2 for new da1
			if (da2>g2)
			{
				da2=g2;
				al1=u1;
				if(da2+a2<lc2) {da2=lc2-a2;al2=l2;}
				return;
			}else{
				g1=-f1-da2*temp1;//optimal da1 for new da2
				al2=u2;
				if (da1>g1)
				{
					da1=g1;
					if (da1+a1<lc1) {da1=lc1-a1;al1=l1;}
					return;
				}else al1=u1;
				return;
			}
		}else{//mF
			da2=-f2-temp1*da1;//f2 will change by Q12*da1
			al1=u1;
			a2new=a2+(ftype)da2;
			if (a2new<lc2) {da2=lc2-a2;al2=l2;return;}
			else if (a2new>uc2) {da2=uc2-a2;al2=u2;return;}
			return;
		}
	}else if(a2new<lc2)
	{//H
		da2=lc2-a2;
		al2=l2;
		da1=-f1-temp1*da2;//f1 will change by Q12*a2
		a1new=a1+(ftype)da1;
		if (a1new<lc1) {da1=lc1-a1;al1=l1;return;}
		else if (a1new>uc1) {da1=uc1-a1;al1=u1;return;}
		return;
	}else if(a2new>uc2)
	{//B
		da2=uc2-a2;
		al2=u2;
		da1=-f1-temp1*da2;//f1 will change by Q12*a2
		a1new=a1+(ftype)da1;
		if (a1new<lc1) {da1=lc1-a1;al1=l1;return;}
		else if (a1new>uc1) {da1=uc1-a1;al1=u1;return;}
		return;
	}
}
inline void SMO::getAlphaUpdatesnew(double &da1,double &da2, signed char &al1,signed char &al2,int a1Ind,int a2Ind,ftype a1,ftype f1,signed char l1,signed char u1)
{//alternative version:returns update amounts da1,da2 for optimized sup-problem a1Ind,a2Ind, might be faster than getAlphaUpdates
/*
input:
	al1,al2 - element of (-1 0 1 5), current status of alpha 
	a1Ind, a2Ind - indices of selected pair of alphas
	a1 - value of alpha 1
	f1 - value of f +- epsilon, depends on status of alpha1, for status 0, the other case has to be tested too
	l1,u1 - bounds for a1, depends on status of alpha1, for status 0 opposite bounds has to be tested too
output: 
	da1,da2 - calculated changes of alpha
*/
//update depends on sign of mAlpha, if mAlpha=0 check both directions
	double temp1,temp;
	ftype a2;
	temp1=mQ[a1Ind][a2Ind];
	temp=1.0/(1.0-temp1*temp1);
	a2=mAlpha[a2Ind];
	int qpp,qpn,qnp,qnn;
	ftype f2,f1p,f1n,f2p,f2n;
	double da1pp,da2pp,da1pn,da2pn,da1np,da2np,da1nn,da2nn;
	double a1newpp,a2newpp,a1newpn,a2newpn,a1newnp,a2newnp,a1newnn,a2newnn;
	double line_a,line_b,line_c,line_d,line_e,line_f,line_g,line_h,line_i,line_j,line_k,line_l;

	qpp=0;qpn=0;qnp=0;qnn=0;

	//first check for zero zero
	f1=mF[a1Ind];
	f2=mF[a2Ind];
/*	if(fabs(f1)>=mEpsilon && fabs(f2)>=mEpsilon) 
	{
		//0 - 0
		da1=-a1;
		da2=-a2;
		al1=0;
		al2=0;
		return;
	}*/
	//at first check current setting:
//	if(a1<0 && a2<0) goto c3;
//	if(a1<0 && a2>0) goto c2;
//	if(a1>0 && a2<0) goto c1;

/* check upper right square A

names of squares
	   a2
	    |
	* - * - *
	| D | A |
	* - 0 - *  ->a1
	| C | B |
	* - * - *

names of edges:
	   a2
	    |
	* h * a *
	g   i   b
	* l 0 j *  ->a1
	f   k   c
	* e * d *

*/
	f1p=f1+mEpsilon;
	f2p=f2+mEpsilon;
	da1pp=(((double)f2p*temp1-(double)f1p)*temp);//mQ is normalized
	da2pp=(((double)f1p*temp1-(double)f2p)*temp);
	a1newpp=a1+(ftype)da1pp;
	a2newpp=a2+(ftype)da2pp;
	//check if solution is inside square A:
	if(a1newpp>=0 && a1newpp<=mC && a2newpp>=0 && a2newpp<=mC)
	{
		da1=da1pp;
		da2=da2pp;
		if(a1newpp==0) al1=0;
		else if(a1newpp==mC) al1=1;
		else al1=5;
		if(a2newpp==0) al2=0;
		else if(a2newpp==mC) al2=1;
		else al2=5;
		return;
	}
	//check if solution from A is outside of all four squares (edges a,b,c,h)
	if(a2newpp>=-a1newpp+mC)
	{
		//check if solution is on the top (edges a,h)
		if(a2newpp>=a1newpp)
		{
			//search da1 while da2=C-a2 (edge a)
			line_a=-temp1*(mC-a2)-f1p;//desired da1
			if(line_a>-a1)
			{//solution is on edge a
				da2=mC-a2;
				al2=1;
				if(line_a>=mC-a1) {da1=mC-a1;al1=1;} else {da1=line_a;al1=5;}
				return;
			}else//solution is on edge h
			{//search da1<=-a1 while da2=C-a2 on upper left square
				da2=mC-a2;
				al2=1;
				f1n=f1-mEpsilon;
				line_h=-temp1*(mC-a2)-f1n;//desired da1
				if(line_h>=-a1) {da1=-a1;al1=0;} 
				else if(line_h<=-mC-a1) {da1=-mC-a1;al1=-1;}
				else {da1=line_h;al1=5;}
				return;
			}
		}else//check if solution is on the right (edges (b,c)
		{
			//search da2 while da1=C-a1 (edge b)
			line_b=-temp1*(mC-a1)-f2p;//desired da2
			if(line_b>-a2)
			{//solution is on edge b
				da1=mC-a1;
				al1=1;
				if(line_b>=mC-a2) {da2=mC-a2;al2=1;} else {da2=line_b;al2=5;}
				return;
			}else//solution is on edge c
			{//search da2<=a2 while da1=C-a1 (edge c)
				da1=mC-a1;
				al1=1;
				f2n=f2-mEpsilon;
				line_c=-temp1*(mC-a1)-f2n;//desired da2
				if(line_c>=-a2) {da2=-a2;al2=0;} 
				else if(line_c<=-mC-a2) {da2=-mC-a2;al2=-1;}
				else {da2=line_c;al2=5;}
				return;
			}
		}
	}else//solution of A directs into other squares B or D
	{
		if(a2newpp>=a1newpp)//A directs into square D
		{
			//test square D
			f1n=f1-mEpsilon;
			da1np=(((double)f2p*temp1-(double)f1n)*temp);//mQ is normalized
			da2np=(((double)f1n*temp1-(double)f2p)*temp);
			a1newnp=a1+(ftype)da1np;
			a2newnp=a2+(ftype)da2np;
			//check if solution is inside square D:
			if(a1newnp>=-mC && a1newnp<=0 && a2newnp>=0 && a2newnp<=mC)
			{
				da1=da1np;
				da2=da2np;
				if(a1newnp==0) al1=0;
				else if(a1newnp==mC) al1=1;
				else al1=5;
				if(a2newnp==0) al2=0;
				else if(a2newnp==mC) al2=1;
				else al2=5;
				return;
			}
			if(a2newnp>=a1newnp+mC)
			{//solution of D directs out of all four squares (edge f,g,h)
				if(a2newnp>=-a1newnp)
				{//solution of D is on edge h
					//search da1 while da2=C-a2 on upper left square
					da2=mC-a2;
					al2=1;
					line_h=-temp1*(mC-a2)-f1n;//desired da1
					if(line_h>=-a1) {da1=-a1;al1=0;} 
					else if(line_h<=-mC-a1) {da1=-mC-a1;al1=-1;}
					else {da1=line_h;al1=5;}
					return;
				}else
				{//solution of D is on edges f,g
					//search da2 while da1=-C-a1 (edge g)
					line_g=-temp1*(-mC-a1)-f2p;//desired da2
					if(line_g>-a2)
					{//solution is on edge g
						da1=-mC-a1;
						al1=-1;
						if(line_g>=mC-a2) {da2=mC-a2;al2=1;} else {da2=line_g;al2=5;}
						return;
					}else//solution is on edge f
					{//search da2 while da1=-C-a1 (edge f)
						da1=-mC-a1;
						al1=-1;
						f2n=f2-mEpsilon;
						line_f=-temp1*(-mC-a1)-f2n;//desired da2
						if(line_f>=-a2) {da2=-a2;al2=0;} 
						else if(line_f<=-mC-a2) {da2=-mC-a2;al2=-1;}
						else {da2=line_f;al2=5;}
						return;
					}
				}
			}else
			{//solution of D directs to A or C (edges i,l)
				if(a2newnp>=-a1newnp)
				{//solution of D directs to A, is on edge i
					//search da2 while da1=-a1
					line_i=-temp1*(-a1)-f2p;//desired da2
					if(line_i>-a2)
					{//solution found on i
						da1=-a1;
						al1=0;
						if(line_i>mC-a2) {da2=mC-a2;al2=1;}
						else da2=line_i;al2=5;
						return;
					}else
					{//solution of A and D is 0|0, check C, tricky: only edge l or on C is possible!!!
						f2n=f2-mEpsilon;
						da1nn=(((double)f2n*temp1-(double)f1n)*temp);//mQ is normalized
						da2nn=(((double)f1n*temp1-(double)f2n)*temp);
						a1newnn=a1+(ftype)da1nn;
						a2newnn=a2+(ftype)da2nn;
						//check if solution is inside square C:
						if(a1newnn>=-mC && a1newnn<=0 && a2newnn>=-mC && a2newnn<=0)
						{
							da1=da1nn;
							da2=da2nn;
							if(a1newnn==-mC) al1=-1;
							else if(a1newnn==0) al1=0;
							else al1=5;
							if(a2newnn==-mC) al2=-1;
							else if(a2newnn==0) al2=0;
							else al2=5;
							return;
						}
						//check edge l: da1<=-a1, da2=-a2
						line_l=-temp1*(-a2)-f1n;//desired da1
						da2=-a2;
						al2=0;
						if(line_l<=-mC-a1) {da1=-mC-a1;al1=-1;}
						else if(line_l>=-a1){da1=-a1;al1=0;}
						else {da1=line_l;al1=5;}
						return;
					}
				}else
				{//solution of D is on edge l, check C, tricky: only edges l,f or on C is possible!!!
					f2n=f2-mEpsilon;
					da1nn=(((double)f2n*temp1-(double)f1n)*temp);//mQ is normalized
					da2nn=(((double)f1n*temp1-(double)f2n)*temp);
					a1newnn=a1+(ftype)da1nn;
					a2newnn=a2+(ftype)da2nn;
					//check if solution is inside square C:
					if(a1newnn>=-mC && a1newnn<=0 && a2newnn>=-mC && a2newnn<=0)
					{
						da1=da1nn;
						da2=da2nn;
						if(a1newnn==-mC) al1=-1;
						else if(a1newnn==0) al1=0;
						else al1=5;
						if(a2newnn==-mC) al2=-1;
						else if(a2newnn==0) al2=0;
						else al2=5;
						return;
					}
					if(a2newnn>=-a1newnn-mC)
					{//solution between D,C on edge l
						//check edge l: da1<=-a1, da2=-a2
						line_l=-temp1*(-a2)-f1n;//desired da1
						da2=-a2;
						al2=0;
						if(line_l<=-mC-a1) {da1=-mC-a1;al1=-1;}
						else if(line_l>=-a1){da1=-a1;al1=0;}
						else {da1=line_l;al1=5;}
						return;
					}else{//solution on edge f
						//check edge f: da2<=-a2, da1=-C-a1
						da1=-mC-a1;
						al1=-1;
						line_f=-temp1*(-mC-a1)-f2n;//desired da2
						if(line_f>=-a2) {da2=-a2;al2=0;} 
						else if(line_f<=-mC-a2) {da2=-mC-a2;al2=-1;}
						else {da2=line_f;al2=5;}
						return;
					}
				}
			}
		}else{//solution of A directs into B (same as 'directs into C, but mirrored)
			//############################################
			//############################################
			//############################################
			//############################################
			//try square B
			f2n=f2-mEpsilon;
			da1pn=(((double)f2n*temp1-(double)f1p)*temp);//mQ is normalized
			da2pn=(((double)f1p*temp1-(double)f2n)*temp);
			a1newpn=a1+(ftype)da1pn;
			a2newpn=a2+(ftype)da2pn;
			//check if solution is inside square B:
			if(a1newpn>=0 && a1newpn<=mC && a2newpn>=-mC && a2newpn<=0)
			{
				da1=da1pn;
				da2=da2pn;
				if(a1newpn==0) al1=0;
				else if(a1newpn==mC) al1=1;
				else al1=5;
				if(a2newpn==-mC) al2=-1;
				else if(a2newpn==0) al2=0;
				else al2=5;
				return;
			}
			if(a2newpn>=a1newpn-mC)
			{//solution of B directs to A or C (edges j,k)
				if(a2newpn>=-a1newpn)
				{//solution of B is on edge j
					//search da1>=-a1 while da2=-a2
					line_j=-temp1*(-a2)-f1p;//desired da1
					if(line_j>-a1)
					{//solution found on j
						da2=-a2;
						al2=0;
						if(line_j>mC-a1) {da1=mC-a1;al1=1;}
						else da1=line_j;al1=5;
						return;
					}else
					{//solution of A and B is 0|0, check C, tricky: only edge k or on C is possible!!!
						f1n=f1-mEpsilon;
						da1nn=(((double)f2n*temp1-(double)f1n)*temp);//mQ is normalized
						da2nn=(((double)f1n*temp1-(double)f2n)*temp);
						a1newnn=a1+(ftype)da1nn;
						a2newnn=a2+(ftype)da2nn;
						//check if solution is inside square C:
						if(a1newnn>=-mC && a1newnn<=0 && a2newnn>=-mC && a2newnn<=0)
						{
							da1=da1nn;
							da2=da2nn;
							if(a1newnn==-mC) al1=-1;
							else if(a1newnn==0) al1=0;
							else al1=5;
							if(a2newnn==-mC) al2=-1;
							else if(a2newnn==0) al2=0;
							else al2=5;
							return;
						}
						//check edge k: da2<=-a2, da1=-a1
						line_k=-temp1*(-a1)-f2n;//desired da2
						da1=-a1;
						al1=0;
						if(line_k<=-mC-a2) {da2=-mC-a2;al2=-1;}
						else if(line_k>=-a2){da2=-a2;al2=0;}
						else {da2=line_k;al2=5;}
						return;
					}
				}else
				{//solution of B is on edge k, check C, tricky: only edges k,e or on C is possible!!!
					f1n=f1-mEpsilon;
					da1nn=(((double)f2n*temp1-(double)f1n)*temp);//mQ is normalized
					da2nn=(((double)f1n*temp1-(double)f2n)*temp);
					a1newnn=a1+(ftype)da1nn;
					a2newnn=a2+(ftype)da2nn;
					//check if solution is inside square C:
					if(a1newnn>=-mC && a1newnn<=0 && a2newnn>=-mC && a2newnn<=0)
					{
						da1=da1nn;
						da2=da2nn;
						if(a1newnn==-mC) al1=-1;
						else if(a1newnn==0) al1=0;
						else al1=5;
						if(a2newnn==-mC) al2=-1;
						else if(a2newnn==0) al2=0;
						else al2=5;
						return;
					}
					if(a2newnn>=-a1newnn-mC)
					{//solution on edge k
						//check edge k: da2<=-a2, da1=-a1
						line_k=-temp1*(-a1)-f2n;//desired da2
						da1=-a1;
						al1=0;
						if(line_k<=-mC-a2) {da2=-mC-a2;al2=-1;}
						else if(line_k>=-a2){da2=-a2;al2=0;}
						else {da2=line_k;al2=5;}
						return;
					}else{//solution on edge e
						//check edge f: da1<=-a1, da2=-C-a2
						da2=-mC-a2;
						al2=-1;
						line_e=-temp1*(-mC-a2)-f1n;//desired da1
						if(line_e>=-a1) {da1=-a1;al1=0;} 
						else if(line_e<=-mC-a1) {da1=-mC-a1;al1=-1;}
						else {da1=line_e;al1=5;}
						return;
					}
				}
			}else
			{//solution of B directs out of all four squares (edge c,d,e)
				if(a2newpn>=-a1newpn)
				{//solution of B is on edge c
					//search da2<=-a2 while da1=C-a1 
					da1=mC-a1;
					al1=1;
					line_c=-temp1*(mC-a1)-f2n;//desired da2
					if(line_c>=-a2) {da2=-a2;al2=0;} 
					else if(line_c<=-mC-a2) {da2=-mC-a2;al2=-1;}
					else {da2=line_c;al2=5;}
					return;
				}else
				{//solution of B is on edges d,e
					//search da1>=-a1 while da2=-C-a2 (edge d)
					line_d=-temp1*(-mC-a2)-f1p;//desired da1
					if(line_d>-a1)
					{//solution is on edge d
						da2=-mC-a2;
						al2=-1;
						if(line_d>=mC-a1) {da1=mC-a1;al1=1;} else {da1=line_d;al1=5;}
						return;
					}else//solution is on edge e
					{//search da1<=-a1 while da2=-C-a2 (edge e)
						da2=-mC-a2;
						al2=-1;
						f1n=f1-mEpsilon;
						line_e=-temp1*(-mC-a2)-f1n;//desired da1
						if(line_e>=-a1) {da1=-a1;al1=0;} 
						else if(line_e<=-mC-a1) {da1=-mC-a1;al1=-1;}
						else {da1=line_e;al1=5;}
						return;
					}
				}
			}
		}
	}
}

int SMO::getdalist(list<sline> *&mLine)
{//calculates optimal values for a set of alphas, given in mLine
/*
in: 
	mLine - f, bounds of delta mAlpha(high, low)
	mQ(mLine.ind,mLine.ind)
out:
	mLine.daopt, mLine.asgnopt, minimizes da*mQ*da+da*f, if convex
	watch, that a and its bounds are transformed into deltas(da) 
*/
	//toDo: see BoundConstrainedQuadraticProgrammingViaPiecewiseQuadraticFunctions tecnicalreport IMM-REP1997-05 TU Denmark
	//toDo: see papers of Rosset(NY): path.pdf  piecewise-revised.pdf
	list<sline>::iterator linei;
	list<sline>::iterator linei1;
	list<int> block;
	int i;
	int back;//return value forwarding
	dd1++;
	if(!solvecholesky(mLine))//get solution da for mQ*mLine.daopt=mLine.f
	{//not convex by rounding errors or singularity(!), minimization of non-convex matrix with border constraints is too expensive
		//leave without changes
		for(linei=mLine->begin();linei!=mLine->end();linei++) 
		{
			(*linei).daopt=0;
			(*linei).asgnopt=mAsgn[(*linei).ind];
		}
		return -2;
	}
	//solveLR(mLine);//get solution da for mQ*mLine.daopt=mLine.f
	//solution is in mLine.daopt
	//check daopt for bounding:
	//limit daopt, update asgnopt, remember boundings in block
	int bs=0;//number of boundings
	i=0;
	for(linei=mLine->begin();linei!=mLine->end();linei++)
	{
		if((*linei).daopt+(*linei).a>=(*linei).high)
		{
			(*linei).daopt=(*linei).high-(*linei).a;
			(*linei).asgnopt=(*linei).highsgn;
			bs++;
			block.push_back(i);
		}else
		if((*linei).daopt+(*linei).a<=(*linei).low)
		{
			(*linei).daopt=(*linei).low-(*linei).a;
			(*linei).asgnopt=(*linei).lowsgn;
			block.push_back(i);
			bs++;
		}
		else (*linei).asgnopt=5;
		i++;
	}
	//solve with boundings
	if(bs==0 || (int)mLine->size()==1) return 0;
	if(bs==1)
	{//reoptimize after limitation of one single mAlpha
		int i,j;
		sline liner;
		ftype* qind;

		linei=mLine->begin();
		i=(*block.begin());//get the element, which changed after limiting
		for(j=0;j<i;j++) linei++;
		liner=(*linei);//save it into liner
		//this element will be clamped and excluded in next try, 
		mLine->erase(linei);
		//simulate a correct f (f=-mF-+mEpsilon=-mmXY-mQ*a-+mEpsilon)
		qind=mQ[liner.ind];
		for(linei1=mLine->begin();linei1!=mLine->end();linei1++)
			(*linei1).f-=liner.daopt*qind[(*linei1).ind];
		back=getdalist(mLine);//get new exactly optimized daopt
		//rebuild f, is needed if caller is another getdalist
		for(linei1=mLine->begin();linei1!=mLine->end();linei1++)
			(*linei1).f+=liner.daopt*qind[(*linei1).ind];
		//reinsert erased clamped index
		linei=mLine->begin();
		for(j=0;j<i;j++) linei++;
		linei=mLine->insert(linei,liner);
		return back;
	}
	if(bs>=2 && bs<=mPar->blockCmax)
	{//reoptimize while excluding all possible combinations of bounded mAlpha indices
		list<sline> *cc;
		cc=new list<sline>;
		//give mLine as reference, get cc as solution
		back=reopt(mLine,cc,block,*block.begin(),0);//begin with first bounded mAlpha
		delete mLine;
		mLine=cc;//return solution into mLine, (is reference)
		return back;
	}
	//toDo: the matrix is too big, split and reoptimize:
	{//return default (zero change) solution 
		for(linei=mLine->begin();linei!=mLine->end();linei++) 
		{
			(*linei).daopt=0;
			(*linei).asgnopt=mAsgn[(*linei).ind];
		}
		return -1;
	}

}
int SMO::reopt(list<sline> *mLine,list<sline> *&cc,list<int> &blocks,int lk,int mk)
{//part of the block update, enforces testing all edges of the hyperbox
	list<sline>::iterator it;
	list<sline>::iterator it1;
	list<sline>::iterator itk;
	list<int>::iterator bit;
	double dhold;
	ftype* qhold;
	int i,bk;
	int back;//return value forwarding

	dd2++;
	//exclude one bounding with index lk from block (and mk from mLine)
	/*sample: (mB=bounded, f=free)
	1: +mB  f  f +mB  f -mB -mB 
	2: hold first mB, reoptimize  all other
	3: +mB [f  f  f  f -mB +mB]
	4:         ->*        *  changed
	5: reopt     *
	6:[+mB  f  f]+mB [f -mB -mB]  
	7: 
	...
	its (not?) so easy...
	*/
/*
mLine - list of alphas to reoptimize
cc - new reoptimized list of alphas (returned)
blocks - subset of mline, bounded indices
lk - element of mLine which is to be freezed
mk - same element as lk, but directs to blocks
*/
	//find iterator to mLine
	itk=mLine->begin();
	for(i=0;i<lk;i++) itk++;
	//find iterator to blocks
	bit=blocks.begin();
	for(i=0;i<mk;i++) bit++;
	//remove from blocks
	blocks.erase(bit);
	//create a new mLine-structure cc and insert all but the one with index mk
	dhold=(*itk).daopt;
	qhold=mQ[(*itk).ind];
	i=0;
	for(it=mLine->begin();it!=mLine->end();it++)
	{
		if(i!=lk) {
			cc->push_back(*it);
			it1=cc->end();
			it1--;
			(*it1).f-=(ftype)dhold*qhold[(*it1).ind];//simulate correct f in cc by updating with removed index
		}
		i++;
	}
	//get new exact solution after freezing element lk(=mk)
	back=getdalist(cc);
	//search position for reinsertion of element *itk(=lk=mk)
	it=cc->begin();
	for(i=0;i<lk;i++) it++;
	cc->insert(it,*itk);
	//rebuild f in cc from mLine
	it1=mLine->begin();
	for(it=cc->begin();it!=cc->end();it++)
	{
		(*it).f=(*it1).f;
		it1++;
	}

	//for every index which initially was limited (but the first) and changed after exact optimization
	//do a reoptimization
	//1. solution outside of the hyperquader?
	//2. projection of the solution onto a hyperedge
	//3. search for the optimal solution iteratively along the edges
	//4. ignore edges, where mAlpha didn't change after reoptimization
	bk=0;
	for(bit=blocks.begin();bit!=blocks.end();bit++)
	{
		it=mLine->begin();
		it1=cc->begin();
		for(i=0;i<(*bit);i++){it++;it1++;}
		//it, it1 direct on bounded element bit
		if((*it).asgnopt==(*it1).asgnopt)
		{// the previously limited dimenstion *bit did not change after reoptimization, so 
			//reoptimize while clamping this edge
			cc->clear();
			int back1;
			back1=reopt(mLine,cc,blocks,(*bit),bk);
			back=min(back,back1);
			break;
		}
		bk++;
	}
	return back;
}
void SMO::solveLR(list<sline> *mLine)
{	//currently not used
	//solves mQ*mLine.daopt=mLine.f
	//works also for non convex problems, but then it finds sattle points or maxima instead of minima
	list<sline>::iterator linek;
	list<sline>::iterator linej;
	int n;
	int d;
	int i,k,j,m;
	double sm;
	n=mLine->size();
	d=n*n;
    for(i=0;i<d;i++) mMat[i]=0;
	linej=mLine->begin();
	for(j=0;j<n;j++)
	{
		linek=linej;
		for(k=j;k<n;k++)
		{
			sm=mQ[(*linej).ind][(*linek).ind];
			for(m=0;m<j;m++)
			{
                sm-=mMat[j*n+m]*mMat[m*n+k];
			}
			mMat[j*n+k]=sm;
			linek++;
		}
		linek=linej;
		linek++;
		for(k=j+1;k<n;k++)
		{
			sm=mQ[(*linek).ind][(*linej).ind];
			for(m=0;m<j;m++)
			{
                sm-=mMat[k*n+m]*mMat[m*n+j];
			}
			mMat[k*n+j]=sm/mMat[j*n+j];
			linek++;
		}
		linej++;
	}
    linek=mLine->begin();
	for(k=0;k<n;k++)
	{
		sm=(*linek).f;
		linej=mLine->begin();
		for(j=0;j<k;j++){
			sm-=(*linej).daopt*mMat[k*n+j];
			linej++;
		}
        (*linek).daopt=(ftype)sm;
		linek++;
	}
    linek=mLine->end();
	linek--;
	for(k=n-1;k>=0;k--)
	{
		sm=(*linek).daopt;
		linej=linek;
		linej++;
		for(j=k+1;j<n;j++){
			sm-=(*linej).daopt*mMat[k*n+j];
			linej++;
		}
        (*linek).daopt=(ftype)(sm/mMat[k*n+k]);
		linek--;
	}
}
bool SMO::solvecholesky(list<sline> *mLine)
{	//solves mQ*mLine->daopt=mLine->f
	//use cholesky for splitting mQ
	//then use substitution
	//if mQ is not positive definite, function breaks without result.
	list<sline>::iterator lineik;
	list<sline>::iterator lineij;
	int n;
	int d;
	int i,k,j,l;
	double sm,sm1;
	int np1;
	n=mLine->size();
	np1=n+1;
//	d=n*(n+1)/2;
	d=n*n;

    for(i=0;i<d;i++) mMat[i]=0;
	lineik=mLine->begin();
	for(k=0;k<n;k++)
	{
		sm=0;
		for(j=0;j<k;j++) sm+=sqr(mMat[k*n+j]);
		if(sm>=1) {//illconditioned problem, caused by rounding errors or singularity(!)
#if boxIgnoreSingularity==0
#if laughter==1
			printf("ei");
#endif
			return false;
#else
			sm1=1e-20;//float range +-1e38, double 1e300
#endif
		}else
			sm1=sqrt(1-sm);
		mMat[k*np1]=sm1;
		lineij=lineik;
		lineij++;
		for(j=k+1;j<n;j++)
		{
			sm=0;
			for(l=0;l<k;l++) sm+=mMat[j*n+l]*mMat[k*n+l];
			mMat[j*n+k]=(mQ[(*lineij).ind][(*lineik).ind]-sm)/sm1;
			lineij++;
		}
		lineik++;
	}
    lineik=mLine->begin();
	for(k=0;k<n;k++)
	{
		sm=0;
		lineij=mLine->begin();
		for(j=0;j<k;j++){
			sm+=(*lineij).daopt*mMat[k*n+j];
			lineij++;
		}
        (*lineik).daopt=(ftype)(((*lineik).f-sm)/mMat[k*np1]);
		lineik++;
	}
    lineik=mLine->end();
	lineik--;
	for(k=n-1;k>=0;k--)
	{
		sm=0;
		lineij=lineik;
		lineij++;
		for(j=k+1;j<n;j++){
			sm+=(*lineij).daopt*mMat[j*n+k];
			lineij++;
		}
        (*lineik).daopt=(ftype)(((*lineik).daopt-sm)/mMat[k*np1]);
		lineik--;
	}
	return true;
}

void SMO::run(Dataset *pset,ftype bb,bool Qreset,bool Areset)
{//starts P-SVM feature selection
/*	input:
		pset - normalized training dataset
		this.mPar - learning parameter
	output:
		optimized mAlpha
	if Qreset==false -> all used rows in Q will be reused, assure they are correct
	if Areset==false -> reuse mAlpha,mmXY,mF,mAsgn,mMaxepsi!! assure they are correct - useful in modelselection for small changes of hyperparameters
	reusing configurations from last run requires, that mDim (the descriptors) of pset remains unchanged!!!!
*/
	int i,j;
	bool goon=true;//mSet to false after mTimeout 
    clock_t t0,tk;
    t0=clock();
	if(!pset || !mPar || !pset->mTrans)
	{
		xinfo("error in SMO::run - unallocated parameter or Kernel matrix input not transposed\n");
		exit(-1);
	}
	//initialize some members
	mB=bb;//this is not nessessary, but improves precision while computing mmXY in init()
	mSet=pset;
	mDim=mSet->mDim;


	//reduction variables, only used in reduction mode, default is: no reduction
	int istart,iend,istep,isteps,isize,iswp,minirun;
	ftype **originalQ;
	ftype *originalXY;
	ftype originalEpsi;
	int reduce;
#if dynamics==1
	FILE* fdyna,*fdynkkt,*fdyndi,*fdynt;
	fdyna=fopen("dyna","w");
	fdynkkt=fopen("dynkkt","w");
	fdyndi=fopen("dyndi","w");
	fdynt=fopen("dynt","w");
#endif
#if useRndWss==1
	srand( (unsigned)time( NULL ) );
#endif


    iswp=0;//number of valid swapping entries is zero
	reduce=mPar->doreduction;
	if(reduce==1 && mDim<2*mPar->redsize) reduce=0;//no reduction for small sets
	if(reduce==1) {Qreset=true;Areset=true;}//dont reuse entries for reduction
	if(Qreset)
		if(mQ) for(i=0;i<mQMem;i++) if(mQ[i]){
			delete[]mQ[i];
			mQ[i]=NULL;}
	if(mSubQ) for(i=0;i<mSubQMem;i++) if(mSubQ[i]){delete[]mSubQ[i];mSubQ[i]=NULL;}
	if(reduce==1)
	{//memory allocations in reduction mode
		if(mMaskMem<mDim)
		{//holds for each index a pair of swapped indices
			if(mMask) delete[] mMask;
			mMask=new char[mDim];mMaskMem=mDim;
		}
		if(mSubQMem<mDim)
		{
			if(mSubQ){
				for(i=0;i<mSubQMem;i++) if(mSubQ[i])delete[]mSubQ[i];
				delete[] mSubQ;
			}
            mSubQ=new ftype*[mDim];mSubQMem=mDim;
			for(i=0;i<mSubQMem;i++) mSubQ[i]=NULL;

		}
		if(mmXYMem<mDim)
		{
			if(mmXY) delete[] mmXY;
			mmXY=new ftype[mDim];mmXYMem=mDim;
		}
		if(mAlphaMem<mDim)
		{
			if(mAlpha) delete[] mAlpha;
			mAlpha=new ftype[mDim];mAlphaMem=mDim;
		}
		originalXY=mmXY;//mmXY will be overwritten with subsets
		originalEpsi=mPar->epsi;//epsi will be overwritten with modifications
		originalQ=mQ;//mQ will be overwritten withsubsets of SubQ

		isize=mPar->redsize;
		istep=0;//number of current sub-problem
		isteps=(mDim/isize);//number of sub-problems
		istart=0;//current subproblem range
		iend=isize;
		minirun=1;//in the last run set to 0

		if(mFMem<isize*2)
		{
			if(mF) delete[] mF;
			mF=new ftype[isize*2];mFMem=isize*2;
		}
		if(mAsgnMem<isize*2)
		{
			if(mAsgn) delete[] mAsgn;
			mAsgn=new signed char[isize*2];mAsgnMem=isize*2;
		}

		mPar->epsi*=(ftype)mPar->redfact;//epsilon modification

		{//Initializes SMO-data mF,mAlpha,mQ
			int l,p;
			double lsum;
			mMaxepsi=0;
			for (p=0;p<mDim;p++)//row size of kernel matrix
			{
				//compute mF for mAlpha=0
				lsum=0;
				ftype* datap,*yd;
				datap=&mSet->mData[p*mSet->mN];
				yd=mSet->mY;
				for (l=0;l<mSet->mN;l++)//col size of kernel matrix and vector y
					lsum+=(yd[l]-mB) * datap[l];// mB(mean) is for reducing rounding errors
				mmXY[p]=-(ftype)lsum;
				mMaxepsi=max((ftype)fabs(lsum),mMaxepsi);
				mMask[p]=0;//1 - feature is selected for last run
			}
			mMaxepsi/=mPar->epsiTol+1;
		}

//		mTCount=timeDiff(t0,clock());
//		pinfo("          pre running time: %.4f seconds\n",mTCount);
//		t0=clock();
	}else{//conservative solver, no reduction (default)
		//reserve memory:
		if(mAlphaMem<mDim)
		{
			if(mAlpha) delete[] mAlpha;
			mAlpha=new ftype[mDim];mAlphaMem=mDim;
		}
		if(mQMem<mDim)
		{
			if(mQ){
				for(i=0;i<mQMem;i++) if(mQ[i])delete[]mQ[i];
				delete[] mQ;
			}
            mQ=new ftype*[mDim];mQMem=mDim;
			for(i=0;i<mQMem;i++) mQ[i]=NULL;
		}
		if(mFMem<mDim)
		{
			if(mF) delete[] mF;
			mF=new ftype[mDim];mFMem=mDim;
		}
		if(mAsgnMem<mDim)
		{
			if(mAsgn) delete[] mAsgn;
			mAsgn=new signed char[mDim];mAsgnMem=mDim;
		}
		if(mmXYMem<mDim)
		{
			if(mmXY) delete[] mmXY;
			mmXY=new ftype[mDim];mmXYMem=mDim;
		}
		//initialize mF, mQ, mAlpha, mAsgn, mmXY, mMaxepsi, 
		//initialize mNs, mLastp, mQCount, mShc,mShz
		if(Areset)//sometimes, values from last smo-run can be reused (then Areset=false)
		{//Initializes SMO-data mF,mAlpha,mQ
			int l,p;
			double lsum;
			mMaxepsi=0;
			for (p=0;p<mDim;p++)//row size of kernel matrix
			{
				//compute mF for mAlpha=0
				ftype* data;
				ftype* y;
				lsum=0;
				data=&mSet->mData[p*mSet->mN];
				y=mSet->mY;
				for (l=0;l<mSet->mN;l++)//col size of kernel matrix and vector y
					lsum+=(y[l]-mB) * data[l];// mB(mean) is for reducing rounding errors
				mmXY[p]=-(ftype)lsum;
				mF[p]=mmXY[p];//mF=mQ*mAlpha-y*data    |mAlpha=0
				mMaxepsi=max((ftype)fabs(lsum),mMaxepsi);
				mAlpha[p]=(ftype)0.;//lagrangian zero initialized 
				mAsgn[p]=0;
			}
			mMaxepsi/=mPar->epsiTol+1;
		}
/*		else{
			int p;
			mShc=0;//number of bounded mAlpha indices
			mShz=0;//number of zero mAlpha indices
			for(p=0;p<mDim;p++)
			{
				if(mAsgn[p]!=0) mShz--;
				if(mAsgn[p]==-1 || mAsgn[p]==1) mShc++;
			}
		}
*/		mNs=mDim;//all indices are active
		mRb=0;//no shifting of Q
		minirun=0;//no reduction phase
		originalEpsi=mPar->epsi;//only for consistency, only used in reduction mode
	}//if reduction==true
	//if user entered negative(impossible) epsilon, suggest an epsilon which leads to find one support vector(feature)
	if(mPar->epsi<=0) mPar->epsi=(ftype)0.9*mMaxepsi;
	if(originalEpsi<=0) originalEpsi=(ftype)0.9*mMaxepsi;//only used in reduction mode

	runsubset:
	if(reduce==1)
	{//get mFeatures from smaller featuresubsets
		if(minirun==1)//all but the final step
		{//featuresubset
			mNs=iend-istart;
			mRb=istart;
			mQ=&(mSubQ[istart]);
			mmXY=&(originalXY[istart]);
			for(i=0;i<mNs;i++)
			{//mNs is reduced size 
				mAlpha[i]=0;
				mF[i]=mmXY[i];
				mAsgn[i]=0;
			}
		}else
		//last run over all collected mFeatures?
		{//mMask[0..mDim-1] is filled with useful index mask

			mmXY=originalXY;//xy is back at its base
			mPar->epsi=originalEpsi;//epsi is back
			mQ=originalQ;//Q is back
			//swap the not important features to the upper (mask=0)
			int las=mDim-1;
			while(mMask[las]==0 && las>=0) las--;//las points to the highest relevant index, -1=no relevant found
			//allocate memory for swapping scheme
			if(mSwpMem<mDim*2)
			{//holds for each index a pair of swapped indices
				if(mSwp) delete[] mSwp;
				mSwp=new int[mDim*2];mSwpMem=mDim*2;
			}
			iswp=0;//swp will now collect swapping operations
			for(i=0;i<=las;i++)
			{
				if(mMask[i]==0)
				{//index i is not useful, swap to the upper
					int l;
					ftype *datai,*datalas;
					datai=&mSet->mData[i*mSet->mN];
					datalas=&mSet->mData[las*mSet->mN];
					for (l=0;l<mSet->mN;l++)	swap(datai[l],datalas[l]);
					swap(mmXY[i],mmXY[las]);
					mSwp[iswp++]=i;
					mSwp[iswp++]=las;
					las--;
					while(mMask[las]==0 && las>i) las--;
				}
			}
			las++;
			//mmXY[0..las-1] and mData[0..las-1] are for important mFeatures
			//mSwp contains used swapping scheme, for rebuilding original sequence
            istart=0;
            iend=las;
			istep++;
			mNs=las;//size of final reduced set
			mRb=0;//no more Q-shifting

			for(i=0;i<mDim;i++) mAlpha[i]=0;//for a clean result vector after backswapping

			if(las==0)
			{//all mFeatures are not important!! no mAlpha found
				mShc=0;//number of bounded mAlpha indices
				mShz=0;//number of zero mAlpha indices
				mQCount=0;//nr of mQ-updates, only for user information
				mACount=0;//mAlpha steps, only for user information
				mTimeout=false;
				mFeatures=0;
				goto ready;
			}
			if(mFMem<mNs)
			{
				if(mF) delete[] mF;
				mF=new ftype[mNs];mFMem=mNs;
			}
			if(mAsgnMem<mNs)
			{
				if(mAsgn) delete[] mAsgn;
				mAsgn=new signed char[mNs];mAsgnMem=mNs;
			}
			if(mQMem<mNs)
			{
				if(mQ){
					for(i=0;i<mQMem;i++) if(mQ[i])delete[]mQ[i];
					delete[] mQ;
				}
				mQ=new ftype*[mNs];mQMem=mNs;
				for(i=0;i<mNs;i++) mQ[i]=NULL;
			}
			for(i=0;i<mNs;i++)
			{
				mF[i]=mmXY[i];
				mAsgn[i]=0;
			}
		}
	}//if reduce==true

	mLastp=0;//remember last chosen a1-index, for alternative working mSet selection
	if(Areset)
	{
		mShc=0;//number of bounded mAlpha indices
		mShz=0;//number of zero mAlpha indices
	}
	mQCount=0;//nr of mQ-updates, only for user information
	mACount=0;//mAlpha steps, only for user information
	mBCount=0;//nubmer of Block optimizations
	mBSCount=0;//number of singular block updates
	mBCCount=0;//number of Block update counts with too many Cs on bound
	mTimeout=false;
	mVio=0;//current smoothed strongest violation of KT-conditions
#if timelog==1
	mTQCount=0;//Q-time
	mTBCounts=0;//total blocktime without overhead
	mTBCountsmax=0;//max single blocktime
	mTWSS2Count=0;//total WSS time (second alpha)
	mTWSS1Count=0;//total wss first alpha
	mTU1Count=0;//total update time single update mode
	mTU2Count=0;//total update time double update mode
	mTF1Count=0;//total update time single F update mode
	mTF2Count=0;//total update time double F update mode
	mTBCount=0;//total block update time with overhead
#endif
#if termAnn==0
	if(mPar->doannealing==1)
	{
        //mEpsilon starts with mMaxepsi*estart and decreases by factor mPar->edec to mPar->epsi
		//mEpsilon=mPar->estart;
		//if(mEpsilon>mMaxepsi) mEpsilon=mMaxepsi*estart;
		mEpsilon=mMaxepsi*mPar->estart;
		mEpsilon/=mPar->edec;//one step backward
	}else mEpsilon=0;//leads to mEpsilon=mPar->epsi
	mTerm=0;
#else
	mEpsilon=mPar->epsi;
	if(mPar->doannealing==1)
	{
        //mEpsilon starts with mMaxepsi*estart and decreases by factor mPar->edec to mPar->epsi
		//mEpsilon=mPar->estart;
		//if(mEpsilon>mMaxepsi) mEpsilon=mMaxepsi*estart;
		mTerm=mMaxepsi/mPar->epsi;
		mTerm/=mPar->edec;//one step backward
	}else mTerm=0;//leads to mEpsilon=mPar->epsi
#endif
	mC=mPar->C;//only a copy from mPar->mC, will not change
	mLastepsi=false;//if mEpsilon is decreased to ->epsi, mLastepsi=true, termination
#if testepsi==1
	int veryfirst=1;
#endif
	do
	{//optimizes mAlpha for different mEpsilon, mEpsilon is to be decreased now
#if testepsi==0 //standard
	#if termAnn==0 //standard
		if(mEpsilon*mPar->edec<=mPar->epsi) mLastepsi=true;//next step is the last
		mEpsilon*=mPar->edec;//one step forward
		if(mEpsilon<mPar->epsi) mEpsilon=mPar->epsi;//the last step could be smaller than the others, is even faster
		#if matlablog==1
		ann_e.push_back(mEpsilon);
		#endif
	#else //special
		if(mTerm*mPar->edec<=mPar->epsiTol) mLastepsi=true;//next step is the last
		mTerm*=mPar->edec;//one step forward
		if(mTerm<mPar->epsiTol) mTerm=mPar->epsiTol;//the last step could be smaller than the others, is even faster
	#endif

#else //this part is not yet ready and for determining hyperparameter epsilon from given number of features
	//if(veryfirst==0)
		{
            mEpsilon=0;
			int fff;
			fff=-1;
			for(i=0;i<mNs;i++)
			{
				if(mAsgn[i]==0){
					mEpsilon=max(mEpsilon,fabs(mF[i])/(mPar->eterm*mPar->epsiTol+1));
/*				}else if(mAsgn[i]==1){
					mEpsilon=max(mEpsilon,mF[i]/(mPar->eterm*mPar->epsiTol-1));
				}else if(mAsgn[i]==-1){
					mEpsilon=max(mEpsilon,-mF[i]/(mPar->eterm*mPar->epsiTol-1));
				}else if(mAlpha[i]>0){
					mEpsilon=max(mEpsilon,mF[i]/(mPar->eterm*mPar->epsiTol-1));
					mEpsilon=max(mEpsilon,-mF[i]/(mPar->eterm*mPar->epsiTol+1));
				}else{// if(mAlpha[i]<0){
					mEpsilon=max(mEpsilon,mF[i]/(mPar->eterm*mPar->epsiTol+1));
					mEpsilon=max(mEpsilon,-mF[i]/(mPar->eterm*mPar->epsiTol-1));
*/				}
			}
			mEpsilon*=mPar->edec;
		}//else veryfirst=0;
		if(mEpsilon<=mPar->epsi) mLastepsi=true;//next step is the last
//		mEpsilon*=mPar->edec;//one step forward
		if(mEpsilon<mPar->epsi) mEpsilon=mPar->epsi;//the last step could be smaller than the others, is even faster
#endif
		//mEpsilon=mPar->epsi;
		mFirstepsi=true; //leads to initialization in 'getAlphaByKT'
		mLine->clear();
		mLinecs=mShc;
		mLineit=mACount;
		do{//while goon
			//every step generates local optimal values for a pair of selected alphas
			int a1Ind,a2Ind;
			ftype f1,a1;
			signed char lc1,uc1;
			ftype l1,u1;
			double da,da1,da2,max_da,max_da1,max_da2;
			signed char al1,al2,max_al1,max_al2;
			int max_a2Ind;
			ftype f11;
			ftype l11,u11;
			bool found;// status of searching best mAlpha to optimize

		//select alpha1
#if timelog==1 //wss1
			clock_t tk0,tk;
			tk0=clock();
#endif
			found=getAlphaByKT(a1Ind);//returns next a1Ind which violates KT
#if timelog==1
			tk=clock();
			mTWSS1Count+=tk-tk0;
#endif
			if (!found) 
			{
				break;//no violation above tolerance -> mEpsilon is reached!
			}
#if dynamics==1
			if(minirun==0)
			{
				int p;
				ftype a,fpz;
				signed char al;
				for(p=0;p<mNs;p++)
				{
					a=mAlpha[p];
					fprintf(fdyna,"%g ",a);
					al=mAsgn[p];
					if(al==0)
					{
						fpz=max(-mF[p]-mEpsilon,mF[p]-mEpsilon);
						fprintf(fdynkkt,"%g ",fpz);
						continue;
					}
					else if(a>0)
					{
						if(al!=1)//same as a<mC
						{//a=  ]0..mC[
							fpz=max(-mF[p]-mEpsilon,mF[p]+mEpsilon);
							fprintf(fdynkkt,"%g ",fpz);
							continue;
						}
						//a= mC
						fpz=mF[p]+mEpsilon;
						fprintf(fdynkkt,"%g ",fpz);
						continue;
					}
					else//if(a<0)
					{
						if(al!=-1)//same as a>-mC
						{
							fpz=max(mF[p]-mEpsilon,-mF[p]+mEpsilon);
							fprintf(fdynkkt,"%g ",fpz);
							continue;
						}
						//a=-mC
						fpz=-mF[p]+mEpsilon;
						fprintf(fdynkkt,"%g ",fpz);
						continue;
					}
				}
				fprintf(fdynkkt,"\n");
				fprintf(fdyna,"\n");
			}
#endif

			//update mQ once for all needed indices
			updateQ(a1Ind);
			//compute f1,a1,l1,u1
			a1=mAlpha[a1Ind];
			if (a1>=0)
			{
				f1=mF[a1Ind]+mEpsilon;//try mAlpha positive
				l1=0;//low limit of a1
				lc1=0;//low limit of a1 as mAsgn-code(0=0,1=mC,-1=-mC 5=free)
				u1=mC;//upper limit of a1
				uc1=1;//upper limit of a1 as mAsgn code
				if(a1==0)
				{//try mAlpha to change positive and negative
					f11=mF[a1Ind]-mEpsilon;
					l11=-mC;u11=0;
				}
			}else{//try mAlpha negative
				f1=mF[a1Ind]-mEpsilon;
				l1=-mC;u1=0;//limits of a1
				lc1=-1;uc1=0;
			}
			//check which second mAlpha leads to maximum changes
#if timelog==1
			tk0=clock();//wss2
#endif
			max_da=0.0;//holds max change
			max_a2Ind=-1;//holds index of max change
#if DualSMO==1//use DUAL SMO instead of SINGLE
#if useRndWss==0
			a2Ind=mLastp;//start searching with last finding
#else
			a2Ind=rand()%mNs;//experiment, if mNs>MAX_RND, this doesnt work optimal!!!!!!!!!!!!!!!!!!!!!!!!
#endif
			for (i=0;i<mNs;i++) 
			{//compare change of mAlpha when optimizing for all pairs with alpha1
				a2Ind++;
				if(a2Ind>=mNs)a2Ind=0;
				if(a2Ind==a1Ind) continue;
//-----------------------------------------------------------------------------
#if WSS2==1 //use WSS1
				{//1:search max violation
					ftype a2,fpz;
					signed char a2s;
					a2=mAlpha[a2Ind];
					a2s=mAsgn[a2Ind];
					if(a2s==0)
					{
#if useBothConst==1
						fpz=max(-mF[a2Ind]-mEpsilon,mF[a2Ind]-mEpsilon);
#else
						fpz=-mF[a2Ind]-mEpsilon;
#endif
						if(max_da<fpz){
							max_da=fpz;
							max_a2Ind=a2Ind;
						}
						continue;
					}else if(a2>0){
						if(a2s!=1)//same as a<mC
						{//a=  ]0..mC[
							fpz=max(-mF[a2Ind]-mEpsilon,mF[a2Ind]+mEpsilon);
							if(max_da<fpz){
								max_da=fpz;
								max_a2Ind=a2Ind;
							}
							continue;
						}
						//a= mC
						fpz=mF[a2Ind]+mEpsilon;
						if(max_da<fpz){
							max_da=fpz;
							max_a2Ind=a2Ind;
						}
						continue;
					}else{
						if(a2s!=-1)//same as a>-mC
						{
							fpz=max(mF[a2Ind]-mEpsilon,-mF[a2Ind]+mEpsilon);
							if(max_da<fpz){
								max_da=fpz;
								max_a2Ind=a2Ind;
							}
							continue;
						}
						//a=-mC
						fpz=-mF[a2Ind]+mEpsilon;
						if(max_da<fpz){
							max_da=fpz;
							max_a2Ind=a2Ind;
						}
						continue;
					}
				}
//-----------------------------------------------------------------
#elif WSS2==2
				{//search first violation, watch termination 
					ftype a2,fpz;
					signed char a2s;
					a2=mAlpha[a2Ind];
					a2s=mAsgn[a2Ind];
					ftype error;
#if termAnn==0//standard case epsilon annealing
					if(!mLastepsi) error=mEpsilon*mPar->epsiTol*mPar->eterm;
					else error=mEpsilon*mPar->epsiTol;//KKT must be greater
//					error=mEpsilon*mPar->epsiTol;//KKT must be greater
#else
					error=mEpsilon*mTerm;
#endif
					if(a2s==0)
					{
#if useBothConst==1
						fpz=max(-mF[a2Ind]-mEpsilon,mF[a2Ind]-mEpsilon);
#else
						fpz=-mF[a2Ind]-mEpsilon;
#endif
						if(error<fpz){
							max_a2Ind=a2Ind;
							break;
						}
						continue;
					}else if(a2>0){
						if(a2s!=1)//same as a<mC
						{//a=  ]0..mC[
							fpz=max(-mF[a2Ind]-mEpsilon,mF[a2Ind]+mEpsilon);
							if(error<fpz){
								max_a2Ind=a2Ind;
								break;
							}
							continue;
						}
						//a= mC
						fpz=mF[a2Ind]+mEpsilon;
						if(error<fpz){
							max_a2Ind=a2Ind;
							break;
						}
						continue;
					}else{
						if(a2s!=-1)//same as a>-mC
						{
							fpz=max(mF[a2Ind]-mEpsilon,-mF[a2Ind]+mEpsilon);
							if(error<fpz){
								max_a2Ind=a2Ind;
								break;
							}
							continue;
						}
						//a=-mC
						fpz=-mF[a2Ind]+mEpsilon;
						if(error<fpz){
							max_a2Ind=a2Ind;
							break;
						}
						continue;
					}
				}
//-----------------------------------------------------------------
#elif WSS2==3
				{//search index leading to a maximum step
					ftype a2,d;
					signed char a2s;
					a2=mAlpha[a2Ind];
					a2s=mAsgn[a2Ind];
	//>0:
	// -f-epsi limit -a1 .. C-a1
	//<0:
	// -f+epsi limit -C-a1 .. -a1
					if(a2s==0)
					{
						d=-mF[a2Ind]-mEpsilon;
						if(d<0) d=0;
						if(d>mC) d=mC;
						d=fabs(d);
						if(d>max_da){max_da=d;max_a2Ind=a2Ind;}
						d=-mF[a2Ind]+mEpsilon;
						if(d<-mC) d=-mC;
						if(d>0) d=0;
						d=fabs(d);
						if(d>max_da){max_da=d;max_a2Ind=a2Ind;}
						if(max_da==mC) break;
					}else{
						if(a2s>0)
						{
							d=-mF[a2Ind]-mEpsilon;
							if(d<-a2) d=-a2;
							if(d>mC-a2) d=mC-a2;
							d=fabs(d);
							if(d>max_da){max_da=d;max_a2Ind=a2Ind;}
						}
						if(a2s<0)
						{
							d=-mF[a2Ind]+mEpsilon;
							if(d<-mC-a2) d=-mC-a2;
							if(d>-a2) d=-a2;
							d=fabs(d);
							if(d>max_da){max_da=d;max_a2Ind=a2Ind;}
						}
					}
				}
//-----------------------------------------------------------------
#elif WSS2==4
				{//search index leading to a maximum step
					ftype a2,d;
					signed char a2s;
					a2=mAlpha[a2Ind];
					a2s=mAsgn[a2Ind];
	//>0:
	// -f-epsi limit -a1 .. C-a1
	//<0:
	// -f+epsi limit -C-a1 .. -a1
					d=a2-mF[a2Ind]-mEpsilon;
					if(d>0){//positive update
						if(d>mC) d=mC;
						d=fabs(d-a2);
						if(d>max_da){max_da=d;max_a2Ind=a2Ind;}
					}else//neg update
					{
						d=-mF[a2Ind]+mEpsilon;
						if(d+a2<-mC) d=-mC-a2;
						if(d+a2>0) d=-a2;
						d=fabs(d);
						if(d>max_da){max_da=d;max_a2Ind=a2Ind;}
					}
					if(max_da==mC) break;
				}
#elif WSS2==5 //
//best(old) method:
//maximizes change of one of both alpha indices
//the four possibilities of alpha signs
//are regarded separate (in contrast to method 3 (useBox==3))
				ftype a2,f2;
				ftype u2,l2;
				double temp1,temp;
				ftype a1new,a2new;
				double g2,g1;
				double dam;
				temp1=mQ[a1Ind][a2Ind];
				temp=1.0/(1.0-temp1*temp1);
				da=0;//maximum change for a2Ind
				//get a2,f2,l2,u2
				a2=mAlpha[a2Ind];
#if useBothConst==1// uses all constraints and optimizes a1 and a2 exactly in both directions >0,<0
				//watch: the sign of alpha changes is ignored in all formulas
				//because we are only interested in the abolute of the change
				//using a subroutine for these calculations costs a huge overhead because of the large number of transfer variables
				if(a1==0){
					if(a2==0){
						//0000000000000000000000000000000000000000000000000000000000000000
						//0000000000000000000000000000000000000000000000000000000000000000
						//try all four configurations, a1+ a2+, a1- a2+, a1+ a2-, a1- a2-
						//a1=0 a2=0 >--optimize--> a1=0..C a2=0..C
						f2=mF[a2Ind]+mEpsilon;
						da1=(((double)f2*temp1-(double)f1)*temp);//mQ is normalized
						da2=(((double)f1*temp1-(double)f2)*temp);
						if (da1<0){//ADG a1 too low
							if (da2<0){//G a2 too low
								if(f2>0){
									if(f1>0) da=0;
									else if(mC+f1<0) da=mC;
									else da=-f1;
								}else if(mC+f2<0) da=mC;
								else da=-f2;
							}else{//D
								if (f2>0) da=0;
								else if (mC+f2<0) da=mC;
								else da=-f2;
							}
						}else if (da1>mC){//CFI a1 too high
							if (da2<0){//I a2 too low
								g2=-f2-mC*temp1;//optimal da2 for new da1
								if (g2>0) da=mC;
								else{
									if(f1>0) da=0;
									else if(mC+f1<0) da=mC;
									else da=-f1;
								}
							}else da=mC;
						}else{
							if(da2<0){
								if (f1>0) da=0;
								else if (mC+f1<0) da=mC;
								else da=-f1;
							}else if(da2>mC) da=mC;
							else da=max(da1,da2);
						}
						dam=da;
						//a1=0 a2=0 >--optimize--> a1=-C..0 a2=0..C
						da1=(((double)f2*temp1-(double)f11)*temp);//mQ is normalized
						da2=(((double)f11*temp1-(double)f2)*temp);
						if (da1+mC<0){//ADG 0 too low
							if (da2<0){//G 0 too low
								g2=-f2+mC*temp1;//optimal da2 for new da1
								if (g2>0) da=mC;
								else{//destination is on the border E|H
									if(f11>mC) da=mC;
									else if(f11<0) da=0;
									else da=f11;
								}
							}else da=mC;
						}else if (da1>0){//CFI 0 too high
							if (da2<0){//I 0 too low
								if(f2>0){
									if(f11>mC) da=mC;
									else if(f11<0) da=0;
									else da=f11;
								}else if(f2+mC<0) da=mC;
								else da=-f2;
							}else{//mF
								if(f2>0) da=0;
								else if(f2+mC<0) da=mC;
								else da=-f2;
							}
						}else{
							if(da2<0){//H
								if (f11>mC) da=mC;
								else if (f11<0) da=0;
								else da=f11;
							}else if(da2>mC) da=mC;
							else da=max(-da1,da2);
						}
						dam=max(dam,da);
						//a1=0 a2=0 >--optimize--> a1=0..C a2=-C..0
						f2=mF[a2Ind]-mEpsilon;
						da1=(((double)f2*temp1-(double)f1)*temp);//mQ is normalized
						da2=(((double)f1*temp1-(double)f2)*temp);
						if (da2+mC<0){//ADG 0 too low
							if (da1<0){//G 0 too low
								g1=-f1+mC*temp1;//optimal da1 for new da2
								if (g1>0) da=mC;
								else{//destination is on the border E|H
									if(f2>mC) da=mC;
									else if(f2<0) da=0;
									else da=f2;
								}
							}else da=mC;
						}else if (da2>0){//CFI 0 too high
							if (da1<0){//I 0 too low
								if(f1>0){
									if(f2>mC) da=mC;
									else if(f2<0) da=0;
									else da=f2;
								}else if(f1+mC<0) da=mC;
								else da=-f1;
							}else{//mF
								if(f1>0) da=0;
								else if(f1+mC<0) da=mC;
								else da=-f1;
							}
						}else{
							if(da1<0){//H
								if (f2>mC) da=mC;
								else if (f2<0) da=0;
								else da=f2;
							}else if(da1>mC) da=mC;
							else da=max(-da2,da1);
						}
						dam=max(dam,da);
						//a1=0 a2=0 >--optimize--> a1=-C..0 a2=-C..0
						da1=(((double)f2*temp1-(double)f11)*temp);//mQ is normalized
						da2=(((double)f11*temp1-(double)f2)*temp);
						if (da1<-mC){//ADG 0 too low
							if (da2>0){//A 0 too high
								g2=-f2+mC*temp1;//optimal da2 for new da1
								if (g2<0) da=mC;
								else{
									if(f11>mC) da=mC;
									else if(f11<0) da=0;
									else da=f11;
								}
							}else da=mC;
						}else if (da1>0){//CFI 0 too high
							if (da2<0){//I 0 too low
								if(f2>mC) da=mC;
								else if(f2<0) da=0;
								else da=f2;
							}else{//mC 0 too high
								if(f2<0){
									if(f11>mC) da=mC;
									else if(f11<0) da=0;
									else da=f11;
								}else if (f2>mC) da=mC;
								else da=f2;
							}
						}else {
							if(da2<-mC) da=mC;
							else if(da2>0){//B
								if(f11>mC) da=mC;
								else if(f11<0) da=0;
								else da=f11;
							}else da=max(-da1,-da2);
						}
						dam=max(dam,da);
						da=dam;
					}else if(a2>0){//try two configurations, a1->(+) a2->(+), a1->(-) a2->(+)
						//0000000000000000000000000000000000000000000000000000000000000000
						//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
						f2=mF[a2Ind]+mEpsilon;
						l2=0;
						u2=mC;
						//please replace a1=0, l2=0, u2=mC, l1=0, u1=mC and simplify
						da1=(((double)f2*temp1-(double)f1)*temp);//mQ is normalized
						da2=(((double)f1*temp1-(double)f2)*temp);
						a1new=(ftype)da1;
						a2new=a2+(ftype)da2;
						if (a1new<0)
						{//ADG a1 too low
							da1=0;
							if (a2new<0)
							{//G a2 too low
								da2=-a2;
								g2=-f2-da1*temp1;//optimal da2 for new da1
								if (da2<g2)
								{//destination is on the border D|E
									da2=g2;
									if(da2+a2>mC) {da2=mC-a2;}
								}else{//destination is on the border E|H
									g1=-f1-da2*temp1;//optimal da1 for new da2
									if (da1<g1)
									{
										da1=g1;
										if (da1>mC) {da1=mC;}
									}
								}
							}else if (a2new>u2)
							{//A a2 too high
								da2=u2-a2;
								g2=-f2-da1*temp1;//optimal da2 for new da1
								if (da2>g2)
								{
									da2=g2;
									if(da2+a2<l2) {da2=l2-a2;}
								}else{
									g1=-f1-da2*temp1;//optimal da1 for new da2
									if (da1<g1)
									{
										da1=g1;
										if (da1+a1>u1) {da1=u1-a1;}
									}
								}
							}else{//D
								da2=-f2-temp1*da1;//f2 will change by Q12*da1
								a2new=a2+(ftype)da2;
								if (a2new<l2) {da2=l2-a2;}
								else if (a2new>u2) {da2=u2-a2;}
							}
						}else if (a1new>u1)
						{//CFI a1 too high
							da1=u1-a1;
							if (a2new<l2)
							{//I a2 too low
								da2=l2-a2;
								g2=-f2-da1*temp1;//optimal da2 for new da1
								if (da2<g2)
								{
									da2=g2;
									if(da2+a2>u2) {da2=u2-a2;}
								}else{
									g1=-f1-da2*temp1;//optimal da1 for new da2
									if (da1>g1)
									{
										da1=g1;
										if (da1+a1<l1) {da1=l1-a1;}
									}
								}
							}else if (a2new>u2)
							{//mC a2 too high
								da2=u2-a2;
								g2=-f2-da1*temp1;//optimal da2 for new da1
								if (da2>g2)
								{
									da2=g2;
									if(da2+a2<l2) {da2=l2-a2;}
								}else{
									g1=-f1-da2*temp1;//optimal da1 for new da2
									if (da1>g1)
									{
										da1=g1;
										if (da1+a1<l1) {da1=l1-a1;}
									}
								}
							}else{//mF
								da2=-f2-temp1*da1;//f2 will change by Q12*da1
								a2new=a2+(ftype)da2;
								if (a2new<l2) {da2=l2-a2;}
								else if (a2new>u2) {da2=u2-a2;}
							}
						}else if(a2new<l2)
						{//H
							da2=l2-a2;
							da1=-f1-temp1*da2;//f1 will change by Q12*a2
							a1new=a1+(ftype)da1;
							if (a1new<l1) {da1=l1-a1;}
							else if (a1new>u1) {da1=u1-a1;}
						}else if(a2new>u2)
						{//B
							da2=u2-a2;
							da1=-f1-temp1*da2;//f1 will change by Q12*a2
							a1new=a1+(ftype)da1;
							if (a1new<l1) {da1=l1-a1;}
							else if (a1new>u1) {da1=u1-a1;}
						}
						da=max(da,max(fabs(da1),fabs(da2)));
						da1=(((double)f2*temp1-(double)f11)*temp);//mQ is normalized
						da2=(((double)f11*temp1-(double)f2)*temp);
						//please replace a1=0, l2=0, u2=mC, l11=-mC, u11=0 and simplify
						a1new=(ftype)da1;
						a2new=a2+(ftype)da2;
						if (a1new<l11)
						{//ADG a1 too low
							da1=l11;
							if (a2new<l2)
							{//G a2 too low
								da2=l2-a2;
								g2=-f2-da1*temp1;//optimal da2 for new da1
								if (da2<g2)
								{//destination is on the border D|E
									da2=g2;
									if(da2+a2>u2) {da2=u2-a2;}
								}else{//destination is on the border E|H
									g1=-f11-da2*temp1;//optimal da1 for new da2
									if (da1<g1)
									{
										da1=g1;
										if (da1>u11) {da1=u11;}
									}
								}
							}else if (a2new>u2)
							{//A a2 too high
								da2=u2-a2;
								g2=-f2-da1*temp1;//optimal da2 for new da1
								if (da2>g2)
								{
									da2=g2;
									if(da2+a2<l2) {da2=l2-a2;}
								}else{
									g1=-f11-da2*temp1;//optimal da1 for new da2
									if (da1<g1)
									{
										da1=g1;
										if (da1+a1>u11) {da1=u11-a1;}
									}
								}
							}else{//D
								da2=-f2-temp1*da1;//f2 will change by Q12*da1
								a2new=a2+(ftype)da2;
								if (a2new<l2) {da2=l2-a2;}
								else if (a2new>u2) {da2=u2-a2;}
							}
						}else if (a1new>u11)
						{//CFI a1 too high
							da1=u11-a1;
							if (a2new<l2)
							{//I a2 too low
								da2=l2-a2;
								g2=-f2-da1*temp1;//optimal da2 for new da1
								if (da2<g2)
								{
									da2=g2;
									if(da2+a2>u2) {da2=u2-a2;}
								}else{
									g1=-f11-da2*temp1;//optimal da1 for new da2
									if (da1>g1)
									{
										da1=g1;
										if (da1+a1<l11) {da1=l11-a1;}
									}
								}
							}else if (a2new>u2)
							{//mC a2 too high
								da2=u2-a2;
								g2=-f2-da1*temp1;//optimal da2 for new da1
								if (da2>g2)
								{
									da2=g2;
									if(da2+a2<l2) {da2=l2-a2;}
								}else{
									g1=-f11-da2*temp1;//optimal da1 for new da2
									if (da1>g1)
									{
										da1=g1;
										if (da1+a1<l11) {da1=l11-a1;}
									}
								}
							}else{//mF
								da2=-f2-temp1*da1;//f2 will change by Q12*da1
								a2new=a2+(ftype)da2;
								if (a2new<l2) {da2=l2-a2;}
								else if (a2new>u2) {da2=u2-a2;}
							}
						}else if(a2new<l2)
						{//H
							da2=l2-a2;
							da1=-f11-temp1*da2;//f11 will change by Q12*a2
							a1new=a1+(ftype)da1;
							if (a1new<l11) {da1=l11-a1;}
							else if (a1new>u11) {da1=u11-a1;}
						}else if(a2new>u2)
						{//B
							da2=u2-a2;
							da1=-f11-temp1*da2;//f11 will change by Q12*a2
							a1new=a1+(ftype)da1;
							if (a1new<l11) {da1=l11-a1;}
							else if (a1new>u11) {da1=u11-a1;}
						}
						da=max(da,max(fabs(da1),fabs(da2)));
					}else{//try two configurations, a1->(+) a2->(-), a1->(-) a2->(-)
						//0000000000000000000000000000000000000000000000000000000000000000
						//----------------------------------------------------------------
						f2=mF[a2Ind]-mEpsilon;
						l2=-mC;
						u2=0;
						//please replace a1=0, l2=-mC, u2=0, l11=0, u11=mC and simplify
						da1=(((double)f2*temp1-(double)f1)*temp);//mQ is normalized
						da2=(((double)f1*temp1-(double)f2)*temp);
						a1new=(ftype)da1;
						a2new=a2+(ftype)da2;
						if (a1new<l1)
						{//ADG a1 too low
							da1=l1-a1;
							if (a2new<l2)
							{//G a2 too low
								da2=l2-a2;
								g2=-f2-da1*temp1;//optimal da2 for new da1
								if (da2<g2)
								{//destination is on the border D|E
									da2=g2;
									if(da2+a2>u2) {da2=u2-a2;}
								}else{//destination is on the border E|H
									g1=-f1-da2*temp1;//optimal da1 for new da2
									if (da1<g1)
									{
										da1=g1;
										if (da1+a1>u1) {da1=u1;}
									}
								}
							}else if (a2new>u2)
							{//A a2 too high
								da2=u2-a2;
								g2=-f2-da1*temp1;//optimal da2 for new da1
								if (da2>g2)
								{
									da2=g2;
									if(da2+a2<l2) {da2=l2-a2;}
								}else{
									g1=-f1-da2*temp1;//optimal da1 for new da2
									if (da1<g1)
									{
										da1=g1;
										if (da1+a1>u1) {da1=u1;}
									}
								}
							}else{//D
								da2=-f2-temp1*da1;//f2 will change by Q12*da1
								a2new=a2+(ftype)da2;
								if (a2new<l2) {da2=l2-a2;}
								else if (a2new>u2) {da2=u2-a2;}
							}
						}else if (a1new>u1)
						{//CFI a1 too high
							da1=u1-a1;
							if (a2new<l2)
							{//I a2 too low
								da2=l2-a2;
								g2=-f2-da1*temp1;//optimal da2 for new da1
								if (da2<g2)
								{
									da2=g2;
									if(da2+a2>u2) {da2=u2-a2;}
								}else{
									g1=-f1-da2*temp1;//optimal da1 for new da2
									if (da1>g1)
									{
										da1=g1;
										if (da1+a1<l1) {da1=l1-a1;}
									}
								}
							}else if (a2new>u2)
							{//mC a2 too high
								da2=u2-a2;
								g2=-f2-da1*temp1;//optimal da2 for new da1
								if (da2>g2)
								{
									da2=g2;
									if(da2+a2<l2) {da2=l2-a2;}
								}else{
									g1=-f1-da2*temp1;//optimal da1 for new da2
									if (da1>g1)
									{
										da1=g1;
										if (da1+a1<l1) {da1=l1-a1;}
									}
								}
							}else{//mF
								da2=-f2-temp1*da1;//f2 will change by Q12*da1
								a2new=a2+(ftype)da2;
								if (a2new<l2) {da2=l2-a2;}
								else if (a2new>u2) {da2=u2-a2;}
							}
						}else if(a2new<l2)
						{//H
							da2=l2-a2;
							da1=-f1-temp1*da2;//f1 will change by Q12*a2
							a1new=a1+(ftype)da1;
							if (a1new<l1) {da1=l1-a1;}
							else if (a1new>u1) {da1=u1-a1;}
						}else if(a2new>u2)
						{//B
							da2=u2-a2;
							da1=-f1-temp1*da2;//f1 will change by Q12*a2
							a1new=a1+(ftype)da1;
							if (a1new<l1) {da1=l1-a1;}
							else if (a1new>u1) {da1=u1-a1;}
						}
						da=max(da,max(fabs(da1),fabs(da2)));

						//please replace a1=0, l2=-mC, u2=0, l11=-mC, u11=0 and simplify
						da1=(((double)f2*temp1-(double)f11)*temp);//mQ is normalized
						da2=(((double)f11*temp1-(double)f2)*temp);
						a1new=(ftype)da1;
						a2new=a2+(ftype)da2;
						if (a1new<l11)
						{//ADG a1 too low
							da1=l11-a1;
							if (a2new<l2)
							{//G a2 too low
								da2=l2-a2;
								g2=-f2-da1*temp1;//optimal da2 for new da1
								if (da2<g2)
								{//destination is on the border D|E
									da2=g2;
									if(da2+a2>u2) {da2=u2-a2;}
								}else{//destination is on the border E|H
									g1=-f11-da2*temp1;//optimal da1 for new da2
									if (da1<g1)
									{
										da1=g1;
										if (da1+a1>u11) {da1=u11;}
									}
								}
							}else if (a2new>u2)
							{//A a2 too high
								da2=u2-a2;
								g2=-f2-da1*temp1;//optimal da2 for new da1
								if (da2>g2)
								{
									da2=g2;
									if(da2+a2<l2) {da2=l2-a2;}
								}else{
									g1=-f11-da2*temp1;//optimal da1 for new da2
									if (da1<g1)
									{
										da1=g1;
										if (da1+a1>u11) {da1=u11-a1;}
									}
								}
							}else{//D
								da2=-f2-temp1*da1;//f2 will change by Q12*da1
								a2new=a2+(ftype)da2;
								if (a2new<l2) {da2=l2-a2;}
								else if (a2new>u2) {da2=u2-a2;}
							}
						}else if (a1new>u11)
						{//CFI a1 too high
							da1=u11-a1;
							if (a2new<l2)
							{//I a2 too low
								da2=l2-a2;
								g2=-f2-da1*temp1;//optimal da2 for new da1
								if (da2<g2)
								{
									da2=g2;
									if(da2+a2>u2) {da2=u2-a2;}
								}else{
									g1=-f11-da2*temp1;//optimal da1 for new da2
									if (da1>g1)
									{
										da1=g1;
										if (da1+a1<l11) {da1=l11-a1;}
									}
								}
							}else if (a2new>u2)
							{//mC a2 too high
								da2=u2-a2;
								g2=-f2-da1*temp1;//optimal da2 for new da1
								if (da2>g2)
								{
									da2=g2;
									if(da2+a2<l2) {da2=l2-a2;}
								}else{
									g1=-f11-da2*temp1;//optimal da1 for new da2
									if (da1>g1)
									{
										da1=g1;
										if (da1+a1<l11) {da1=l11-a1;}
									}
								}
							}else{//mF
								da2=-f2-temp1*da1;//f2 will change by Q12*da1
								a2new=a2+(ftype)da2;
								if (a2new<l2) {da2=l2-a2;}
								else if (a2new>u2) {da2=u2-a2;}
							}
						}else if(a2new<l2)
						{//H
							da2=l2-a2;
							da1=-f11-temp1*da2;//f11 will change by Q12*a2
							a1new=a1+(ftype)da1;
							if (a1new<l11) {da1=l11-a1;}
							else if (a1new>u11) {da1=u11-a1;}
						}else if(a2new>u2)
						{//B
							da2=u2-a2;
							da1=-f11-temp1*da2;//f11 will change by Q12*a2
							a1new=a1+(ftype)da1;
							if (a1new<l11) {da1=l11-a1;}
							else if (a1new>u11) {da1=u11-a1;}
						}
						da=max(da,max(fabs(da1),fabs(da2)));
					}
				}else{
                    if(a2==0){
						//+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
						//0000000000000000000000000000000000000000000000000000000000000000
						//replace a2=0, l2=0, u2=mC
						//try two configurations, a1->(+-) a2->(+), a1->(+-) a2->(-)
                        f2=mF[a2Ind]+mEpsilon;
                        l2=0;
                        u2=mC;
						da1=(((double)f2*temp1-(double)f1)*temp);//mQ is normalized
						da2=(((double)f1*temp1-(double)f2)*temp);
						a1new=a1+(ftype)da1;
						a2new=(ftype)da2;
						if (a1new<l1)
						{//ADG a1 too low
							da1=l1-a1;
							if (a2new<l2)
							{//G a2 too low
								da2=0;
								g2=-f2-da1*temp1;//optimal da2 for new da1
								if (da2<g2)
								{//destination is on the border D|E
									da2=g2;
									if(da2>u2) {da2=u2;}
								}else{//destination is on the border E|H
									g1=-f1-da2*temp1;//optimal da1 for new da2
									if (da1<g1)
									{
										da1=g1;
										if (da1+a1>u1) {da1=u1-a1;}
									}
								}
							}else if (a2new>u2)
							{//A a2 too high
								da2=u2-a2;
								g2=-f2-da1*temp1;//optimal da2 for new da1
								if (da2>g2)
								{
									da2=g2;
									if(da2+a2<l2) {da2=l2-a2;}
								}else{
									g1=-f1-da2*temp1;//optimal da1 for new da2
									if (da1<g1)
									{
										da1=g1;
										if (da1+a1>u1) {da1=u1-a1;}
									}
								}
							}else{//D
								da2=-f2-temp1*da1;//f2 will change by Q12*da1
								a2new=a2+(ftype)da2;
								if (a2new<l2) {da2=l2;}
								else if (a2new>u2) {da2=u2;}
							}
						}else if (a1new>u1)
						{//CFI a1 too high
							da1=u1-a1;
							if (a2new<l2)
							{//I a2 too low
								da2=l2-a2;
								g2=-f2-da1*temp1;//optimal da2 for new da1
								if (da2<g2)
								{
									da2=g2;
									if(da2+a2>u2) {da2=u2;}
								}else{
									g1=-f1-da2*temp1;//optimal da1 for new da2
									if (da1>g1)
									{
										da1=g1;
										if (da1+a1<l1) {da1=l1-a1;}
									}
								}
							}else if (a2new>u2)
							{//mC a2 too high
								da2=u2-a2;
								g2=-f2-da1*temp1;//optimal da2 for new da1
								if (da2>g2)
								{
									da2=g2;
									if(da2+a2<l2) {da2=l2;}
								}else{
									g1=-f1-da2*temp1;//optimal da1 for new da2
									if (da1>g1)
									{
										da1=g1;
										if (da1+a1<l1) {da1=l1-a1;}
									}
								}
							}else{//mF
								da2=-f2-temp1*da1;//f2 will change by Q12*da1
								a2new=a2+(ftype)da2;
								if (a2new<l2) {da2=l2-a2;}
								else if (a2new>u2) {da2=u2-a2;}
							}
						}else if(a2new<l2)
						{//H
							da2=l2-a2;
							da1=-f1-temp1*da2;//f1 will change by Q12*a2
							a1new=a1+(ftype)da1;
							if (a1new<l1) {da1=l1-a1;}
							else if (a1new>u1) {da1=u1-a1;}
						}else if(a2new>u2)
						{//B
							da2=u2-a2;
							da1=-f1-temp1*da2;//f1 will change by Q12*a2
							a1new=a1+(ftype)da1;
							if (a1new<l1) {da1=l1-a1;}
							else if (a1new>u1) {da1=u1-a1;}
						}
						da=max(da,max(fabs(da1),fabs(da2)));
						f2=mF[a2Ind]-mEpsilon;
						l2=-mC;
						u2=0;
						da1=(((double)f2*temp1-(double)f1)*temp);//mQ is normalized
						da2=(((double)f1*temp1-(double)f2)*temp);
						a1new=a1+(ftype)da1;
						a2new=a2+(ftype)da2;
						if (a1new<l1)
						{//ADG a1 too low
							da1=l1-a1;
							if (a2new<l2)
							{//G a2 too low
								da2=l2-a2;
								g2=-f2-da1*temp1;//optimal da2 for new da1
								if (da2<g2)
								{//destination is on the border D|E
									da2=g2;
									if(da2+a2>u2) {da2=u2-a2;}
								}else{//destination is on the border E|H
									g1=-f1-da2*temp1;//optimal da1 for new da2
									if (da1<g1)
									{
										da1=g1;
										if (da1+a1>u1) {da1=u1-a1;}
									}
								}
							}else if (a2new>u2)
							{//A a2 too high
								da2=u2-a2;
								g2=-f2-da1*temp1;//optimal da2 for new da1
								if (da2>g2)
								{
									da2=g2;
									if(da2+a2<l2) {da2=l2-a2;}
								}else{
									g1=-f1-da2*temp1;//optimal da1 for new da2
									if (da1<g1)
									{
										da1=g1;
										if (da1+a1>u1) {da1=u1-a1;}
									}
								}
							}else{//D
								da2=-f2-temp1*da1;//f2 will change by Q12*da1
								a2new=a2+(ftype)da2;
								if (a2new<l2) {da2=l2-a2;}
								else if (a2new>u2) {da2=u2-a2;}
							}
						}else if (a1new>u1)
						{//CFI a1 too high
							da1=u1-a1;
							if (a2new<l2)
							{//I a2 too low
								da2=l2-a2;
								g2=-f2-da1*temp1;//optimal da2 for new da1
								if (da2<g2)
								{
									da2=g2;
									if(da2+a2>u2) {da2=u2-a2;}
								}else{
									g1=-f1-da2*temp1;//optimal da1 for new da2
									if (da1>g1)
									{
										da1=g1;
										if (da1+a1<l1) {da1=l1-a1;}
									}
								}
							}else if (a2new>u2)
							{//mC a2 too high
								da2=u2-a2;
								g2=-f2-da1*temp1;//optimal da2 for new da1
								if (da2>g2)
								{
									da2=g2;
									if(da2+a2<l2) {da2=l2-a2;}
								}else{
									g1=-f1-da2*temp1;//optimal da1 for new da2
									if (da1>g1)
									{
										da1=g1;
										if (da1+a1<l1) {da1=l1-a1;}
									}
								}
							}else{//mF
								da2=-f2-temp1*da1;//f2 will change by Q12*da1
								a2new=a2+(ftype)da2;
								if (a2new<l2) {da2=l2-a2;}
								else if (a2new>u2) {da2=u2-a2;}
							}
						}else if(a2new<l2)
						{//H
							da2=l2-a2;
							da1=-f1-temp1*da2;//f1 will change by Q12*a2
							a1new=a1+(ftype)da1;
							if (a1new<l1) {da1=l1-a1;}
							else if (a1new>u1) {da1=u1-a1;}
						}else if(a2new>u2)
						{//B
							da2=u2-a2;
							da1=-f1-temp1*da2;//f1 will change by Q12*a2
							a1new=a1+(ftype)da1;
							if (a1new<l1) {da1=l1-a1;}
							else if (a1new>u1) {da1=u1-a1;}
						}
						da=max(da,max(fabs(da1),fabs(da2)));
					}else if(a2>0){
						//+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
						//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
						f2=mF[a2Ind]+mEpsilon;
						l2=0;
						u2=mC;
						da1=(((double)f2*temp1-(double)f1)*temp);//mQ is normalized
						da2=(((double)f1*temp1-(double)f2)*temp);
						a1new=a1+(ftype)da1;
						a2new=a2+(ftype)da2;
						if (a1new<l1)
						{//ADG a1 too low
							da1=l1-a1;
							if (a2new<l2)
							{//G a2 too low
								da2=-a2;
								g2=-f2-da1*temp1;//optimal da2 for new da1
								if (da2<g2)
								{//destination is on the border D|E
									da2=g2;
									if(da2+a2>u2) {da2=u2-a2;}
								}else{//destination is on the border E|H
									g1=-f1-da2*temp1;//optimal da1 for new da2
									if (da1<g1)
									{
										da1=g1;
										if (da1+a1>u1) {da1=u1-a1;}
									}
								}
							}else if (a2new>u2)
							{//A a2 too high
								da2=u2-a2;
								g2=-f2-da1*temp1;//optimal da2 for new da1
								if (da2>g2)
								{
									da2=g2;
									if(da2+a2<l2) {da2=l2-a2;}
								}else{
									g1=-f1-da2*temp1;//optimal da1 for new da2
									if (da1<g1)
									{
										da1=g1;
										if (da1+a1>u1) {da1=u1-a1;}
									}
								}
							}else{//D
								da2=-f2-temp1*da1;//f2 will change by Q12*da1
								a2new=a2+(ftype)da2;
								if (a2new<l2) {da2=l2-a2;}
								else if (a2new>u2) {da2=u2-a2;}
							}
						}else if (a1new>u1)
						{//CFI a1 too high
							da1=u1-a1;
							if (a2new<l2)
							{//I a2 too low
								da2=l2-a2;
								g2=-f2-da1*temp1;//optimal da2 for new da1
								if (da2<g2)
								{
									da2=g2;
									if(da2+a2>u2) {da2=u2-a2;}
								}else{
									g1=-f1-da2*temp1;//optimal da1 for new da2
									if (da1>g1)
									{
										da1=g1;
										if (da1+a1<l1) {da1=l1-a1;}
									}
								}
							}else if (a2new>u2)
							{//mC a2 too high
								da2=u2-a2;
								g2=-f2-da1*temp1;//optimal da2 for new da1
								if (da2>g2)
								{
									da2=g2;
									if(da2+a2<l2) {da2=l2-a2;}
								}else{
									g1=-f1-da2*temp1;//optimal da1 for new da2
									if (da1>g1)
									{
										da1=g1;
										if (da1+a1<l1) {da1=l1-a1;}
									}
								}
							}else{//mF
								da2=-f2-temp1*da1;//f2 will change by Q12*da1
								a2new=a2+(ftype)da2;
								if (a2new<l2) {da2=l2-a2;}
								else if (a2new>u2) {da2=u2-a2;}
							}
						}else if(a2new<l2)
						{//H
							da2=l2-a2;
							da1=-f1-temp1*da2;//f1 will change by Q12*a2
							a1new=a1+(ftype)da1;
							if (a1new<l1) {da1=l1-a1;}
							else if (a1new>u1) {da1=u1-a1;}
						}else if(a2new>u2)
						{//B
							da2=u2-a2;
							da1=-f1-temp1*da2;//f1 will change by Q12*a2
							a1new=a1+(ftype)da1;
							if (a1new<l1) {da1=l1-a1;}
							else if (a1new>u1) {da1=u1-a1;}
						}
						da=max(da,max(fabs(da1),fabs(da2)));
					}else{
						//+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
						//-----------------------------------------------------------------
						f2=mF[a2Ind]-mEpsilon;
						l2=-mC;
						u2=0;
						da1=(((double)f2*temp1-(double)f1)*temp);//mQ is normalized
						da2=(((double)f1*temp1-(double)f2)*temp);
						a1new=a1+(ftype)da1;
						a2new=a2+(ftype)da2;
						if (a1new<l1)
						{//ADG a1 too low
							da1=l1-a1;
							if (a2new<l2)
							{//G a2 too low
								da2=l2-a2;
								g2=-f2-da1*temp1;//optimal da2 for new da1
								if (da2<g2)
								{//destination is on the border D|E
									da2=g2;
									if(da2+a2>u2) {da2=u2-a2;}
								}else{//destination is on the border E|H
									g1=-f1-da2*temp1;//optimal da1 for new da2
									if (da1<g1)
									{
										da1=g1;
										if (da1+a1>u1) {da1=u1-a1;}
									}
								}
							}else if (a2new>u2)
							{//A a2 too high
								da2=u2-a2;
								g2=-f2-da1*temp1;//optimal da2 for new da1
								if (da2>g2)
								{
									da2=g2;
									if(da2+a2<l2) {da2=l2-a2;}
								}else{
									g1=-f1-da2*temp1;//optimal da1 for new da2
									if (da1<g1)
									{
										da1=g1;
										if (da1+a1>u1) {da1=u1-a1;}
									}
								}
							}else{//D
								da2=-f2-temp1*da1;//f2 will change by Q12*da1
								a2new=a2+(ftype)da2;
								if (a2new<l2) {da2=l2-a2;}
								else if (a2new>u2) {da2=u2-a2;}
							}
						}else if (a1new>u1)
						{//CFI a1 too high
							da1=u1-a1;
							if (a2new<l2)
							{//I a2 too low
								da2=l2-a2;
								g2=-f2-da1*temp1;//optimal da2 for new da1
								if (da2<g2)
								{
									da2=g2;
									if(da2+a2>u2) {da2=u2-a2;}
								}else{
									g1=-f1-da2*temp1;//optimal da1 for new da2
									if (da1>g1)
									{
										da1=g1;
										if (da1+a1<l1) {da1=l1-a1;}
									}
								}
							}else if (a2new>u2)
							{//mC a2 too high
								da2=u2-a2;
								g2=-f2-da1*temp1;//optimal da2 for new da1
								if (da2>g2)
								{
									da2=g2;
									if(da2+a2<l2) {da2=l2-a2;}
								}else{
									g1=-f1-da2*temp1;//optimal da1 for new da2
									if (da1>g1)
									{
										da1=g1;
										if (da1+a1<l1) {da1=l1-a1;}
									}
								}
							}else{//mF
								da2=-f2-temp1*da1;//f2 will change by Q12*da1
								a2new=a2+(ftype)da2;
								if (a2new<l2) {da2=l2-a2;}
								else if (a2new>u2) {da2=u2-a2;}
							}
						}else if(a2new<l2)
						{//H
							da2=l2-a2;
							da1=-f1-temp1*da2;//f1 will change by Q12*a2
							a1new=a1+(ftype)da1;
							if (a1new<l1) {da1=l1-a1;}
							else if (a1new>u1) {da1=u1-a1;}
						}else if(a2new>u2)
						{//B
							da2=u2-a2;
							da1=-f1-temp1*da2;//f1 will change by Q12*a2
							a1new=a1+(ftype)da1;
							if (a1new<l1) {da1=l1-a1;}
							else if (a1new>u1) {da1=u1-a1;}
						}
						da=max(da,max(fabs(da1),fabs(da2)));
					}
                }
#else
			//use only positive constraint for alpha -> all alphas remain positive
				f2=mF[a2Ind]+mEpsilon;
				l2=0;
				u2=mC;
				da1=(((double)f2*temp1-(double)f1)*temp);//mQ is normalized
				da2=(((double)f1*temp1-(double)f2)*temp);
				a1new=a1+(ftype)da1;
				a2new=a2+(ftype)da2;
				if (a1new<l1)
				{//ADG a1 too low
					da1=l1-a1;
					if (a2new<l2)
					{//G a2 too low
						da2=l2-a2;
						g2=-f2-da1*temp1;//optimal da2 for new da1
						if (da2<g2)
						{//destination is on the border D|E
							da2=g2;
							if(da2+a2>u2) {da2=u2-a2;}
						}else{//destination is on the border E|H
							g1=-f1-da2*temp1;//optimal da1 for new da2
							if (da1<g1)
							{
								da1=g1;
								if (da1+a1>u1) {da1=u1-a1;}
							}
						}
					}else if (a2new>u2)
					{//A a2 too high
						da2=u2-a2;
						g2=-f2-da1*temp1;//optimal da2 for new da1
						if (da2>g2)
						{
							da2=g2;
							if(da2+a2<l2) {da2=l2-a2;}
						}else{
							g1=-f1-da2*temp1;//optimal da1 for new da2
							if (da1<g1)
							{
								da1=g1;
								if (da1+a1>u1) {da1=u1-a1;}
							}
						}
					}else{//D
						da2=-f2-temp1*da1;//f2 will change by Q12*da1
						a2new=a2+(ftype)da2;
						if (a2new<l2) {da2=l2-a2;}
						else if (a2new>u2) {da2=u2-a2;}
					}
				}else if (a1new>u1)
				{//CFI a1 too high
					da1=u1-a1;
					if (a2new<l2)
					{//I a2 too low
						da2=l2-a2;
						g2=-f2-da1*temp1;//optimal da2 for new da1
						if (da2<g2)
						{
							da2=g2;
							if(da2+a2>u2) {da2=u2-a2;}
						}else{
							g1=-f1-da2*temp1;//optimal da1 for new da2
							if (da1>g1)
							{
								da1=g1;
								if (da1+a1<l1) {da1=l1-a1;}
							}
						}
					}else if (a2new>u2)
					{//mC a2 too high
						da2=u2-a2;
						g2=-f2-da1*temp1;//optimal da2 for new da1
						if (da2>g2)
						{
							da2=g2;
							if(da2+a2<l2) {da2=l2-a2;}
						}else{
							g1=-f1-da2*temp1;//optimal da1 for new da2
							if (da1>g1)
							{
								da1=g1;
								if (da1+a1<l1) {da1=l1-a1;}
							}
						}
					}else{//mF
						da2=-f2-temp1*da1;//f2 will change by Q12*da1
						a2new=a2+(ftype)da2;
						if (a2new<l2) {da2=l2-a2;}
						else if (a2new>u2) {da2=u2-a2;}
					}
				}else if(a2new<l2)
				{//H
					da2=l2-a2;
					da1=-f1-temp1*da2;//f1 will change by Q12*a2
					a1new=a1+(ftype)da1;
					if (a1new<l1) {da1=l1-a1;}
					else if (a1new>u1) {da1=u1-a1;}
				}else if(a2new>u2)
				{//B
					da2=u2-a2;
					da1=-f1-temp1*da2;//f1 will change by Q12*a2
					a1new=a1+(ftype)da1;
					if (a1new<l1) {da1=l1-a1;}
					else if (a1new>u1) {da1=u1-a1;}
				}
				da=max(da,max(fabs(da1),fabs(da2)));
#endif
	//			getAlphaUpdates(da1,da2,al1,al2,a1Ind,a2Ind,a1,f1,lc1,uc1,mEpsilon);
	//			da=max(fabs(da1),fabs(da2));

				if (da>max_da) {
					max_da=da;
					max_a2Ind=a2Ind;
					if(da==mC) break;//more change is not possible (the sign never changes)
				}
#elif WSS2==66 // that is nonsense!!!! - changes toward zero can become indefinitely large
//maximize unconstrained change of one alpha index
//this doesnt work very well!!
				ftype a2,f2;
				ftype u2,l2;
				double temp1,temp;
				ftype a1new,a2new;
				double g2,g1;
				double dam;
				temp1=mQ[a1Ind][a2Ind];
				temp=1.0/(1.0-temp1*temp1);
				da=0;//maximum change for a2Ind
				//get a2,f2,l2,u2
                f2=mF[a2Ind]-mEpsilon;
				da1=fabs(((double)f2*temp1-(double)f1)*temp);//mQ is normalized
				da2=fabs(((double)f1*temp1-(double)f2)*temp);
				if(da1>max_da){max_da=da1; max_a2Ind=a2Ind;}
				if(da2>max_da){max_da=da2; max_a2Ind=a2Ind;}
				da1=fabs(((double)f2*temp1-(double)f11)*temp);//mQ is normalized
				da2=fabs(((double)f11*temp1-(double)f2)*temp);
				if(da1>max_da){max_da=da1; max_a2Ind=a2Ind;}
				if(da2>max_da){max_da=da2; max_a2Ind=a2Ind;}
				f2=mF[a2Ind]+mEpsilon;
				da1=fabs(((double)f2*temp1-(double)f1)*temp);//mQ is normalized
				da2=fabs(((double)f1*temp1-(double)f2)*temp);
				if(da1>max_da){max_da=da1; max_a2Ind=a2Ind;}
				if(da2>max_da){max_da=da2; max_a2Ind=a2Ind;}
				da1=fabs(((double)f2*temp1-(double)f11)*temp);//mQ is normalized
				da2=fabs(((double)f11*temp1-(double)f2)*temp);
				if(da1>max_da){max_da=da1; max_a2Ind=a2Ind;}
				if(da2>max_da){max_da=da2; max_a2Ind=a2Ind;}
#elif WSS2==6
//uses exact box optimization, finds exact signs of both alphas
//and maximizes change of one alpha index
//doesnt work as good as method 1
//because it produces a1=6,a2=-0.05 instead of a1=6, a2=0
//although the first solution minimizes the primal constraint exactly
				getAlphaUpdatesnew(da1,da2,al1,al2,a1Ind,a2Ind,a1,f1,lc1,uc1);
/*				{
					double dda1,dda2;
					signed char aal1,aal2;
					getAlphaUpdates(dda1,dda2,aal1,aal2,a1Ind,a2Ind,a1,f1,lc1,uc1);
					dda1=dda1;
					dda2=dda2;
				}
*/
				da=max(fabs(da1),fabs(da2));
				if (da>max_da) {
					max_da=da;
					max_a2Ind=a2Ind;
				}
#endif
			}
#endif
#if timelog==1
			tk=clock();
			mTWSS2Count+=tk-tk0;
#endif
			if (max_a2Ind==-1)
			{
#if timelog==1//update single
				clock_t tk0,tk;
				tk0=clock();
#endif
				//#####################################################
				//#####################################################
				//#####################################################
				//#####################################################
				//mAlpha pair not found?
				//optimize single mAlpha:
				a2Ind=max_a2Ind;
#if SingleUpdate==1
				max_da1=-f1;//keep the sign of alpha
				max_al1=(signed char)5;
				if(max_da1<l1-a1) {max_da1=l1-a1;max_al1=lc1;}
				if(max_da1>u1-a1) {max_da1=u1-a1;max_al1=uc1;}

#if useBothConst==1
				if(a1==0)
				{//try both, mAlpha positive and mAlpha negative
					signed char max_al11;
					ftype max_da11;

					max_da11=-f11;
					max_al11=(signed char)5;
					if(max_da11<l11) {max_da11=l11;max_al11=-1;}
					if(max_da11>u11) {max_da11=u11;max_al11=0;}

					if(fabs(max_da11)>fabs(max_da1))
					{//chose the direction with most change
						max_da1=max_da11;
						max_al1=max_al11;
					}
				}
#endif
#elif SingleUpdate==2//exact signum
				//try positive
				ftype d;
					f1=mF[a1Ind]+mEpsilon;
					d=a1-f1;
					if(d<0)
					{//negative update necessary
						max_da1=-mF[a1Ind]+mEpsilon;
						max_al1=(signed char)5;
						if(max_da1<-mC-a1) {max_da1=-mC-a1;max_al1=-1;}
						if(max_da1>-a1) {max_da1=-a1;max_al1=0;}
					}else{//positive update necessary
						max_da1=-f1;
						max_al1=(signed char)5;
						if(max_da1<-a1) {max_da1=-a1;max_al1=0;}
						if(max_da1>mC-a1) {max_da1=mC-a1;max_al1=+1;}
					}
#endif
				//update mF for new mAlpha
				ftype* qa1;
				qa1=mQ[a1Ind];
				if(max_al1==5) //switch to free
				{
					if(mAsgn[a1Ind]==-1)mShc--;
					else if(mAsgn[a1Ind]==+1)mShc--;
					else if(mAsgn[a1Ind]==0)mShz--;
					mAlpha[a1Ind]+=(ftype)max_da1;
				}else if(max_al1==0)//switch to zero
				{
					if(mAsgn[a1Ind]==-1){mShz++;mShc--;}
					else if(mAsgn[a1Ind]==+1){mShz++;mShc--;}
					else if(mAsgn[a1Ind]==5)mShz++;
					mAlpha[a1Ind]=0.;
				}else if(max_al1==+1)//switch to +mC
				{
					if(mAsgn[a1Ind]==0){mShz--;mShc++;}
					else if(mAsgn[a1Ind]==5)mShc++;
					mAlpha[a1Ind]=mC;
				}else //switch to -mC
				{
					if(mAsgn[a1Ind]==0){mShz--;mShc++;}
					else if(mAsgn[a1Ind]==5)mShc++;
					mAlpha[a1Ind]=-mC;
				}
				mAsgn[a1Ind]=max_al1;
#if timelog==1
				tk=clock();
				mTU1Count+=tk-tk0;
//update F single
				tk0=tk;
#endif
				for (i=0;i<mNs;i++)
				{
					mF[i]+=(ftype)(max_da1*qa1[i]);
				}
				//optimization for single mAlpha is complete
#if logi
				pinfo("                                %d %d\n",a1Ind,max_al1);
#endif
#if timelog==1
				tk=clock();
				mTF1Count+=tk-tk0;
#endif
			}else{//two alpha indices found:
#if check==1
				//gives total error
				// 0.5*aQa-la+-ea, Qa-l=mF -> (0.5*mF-0.5*l+-e)*a
				{
					mGeerr=0;
					for(i=0;i<mNs;i++)
					{
						if(mAsgn[i]==0) continue;
						if(mAlpha[i]>0)
							mGeerr+=mAlpha[i]*(0.5*mF[i]+0.5*mmXY[i]+mEpsilon);//mPar->epsi);
						else
							mGeerr+=mAlpha[i]*(0.5*mF[i]+0.5*mmXY[i]-mEpsilon);//mPar->epsi);
					}
#if logi
					pinfo("step %d before dual opti diistr fct: %f change %f\n",mACount,mGeerr,mGeerr-mGeerr1);
#endif
				}
#endif
#if check==1
			//check consistency of mF and mAsgn
			//rebuilds mF to avoid rounding errors in mF
				{
				int p;
				for(p=0;p<mNs;p++)
				{
					double lsum;
					int k;
					lsum=mmXY[p];
					for(k=0;k<mNs;k++)
					{
						if(mAsgn[k]!=0)
						{
							lsum+=mQ[k][p]*mAlpha[k];
						}
					}
					if(fabs(mF[p]-lsum)>0.0000001) 
					{
						pinfo("error in mF-consistency before\n");
						exit(-1);
						lsum=lsum;
					}
					if((mAlpha[p]>=mC-0.000001 && mAsgn[p]!=1) || (mAlpha[p]<=-mC+0.000001 && mAsgn[p]!=-1) || (mAlpha[p]==0 && mAsgn[p]!=0))
					{
						pinfo("mAlpha and mAsgn error\n");
						exit(-1);
					}
				}
				}
#endif

				//#####################################################
				//#####################################################
				//#####################################################
				//#####################################################
				//optimize two alphas
				a2Ind=max_a2Ind;
				updateQ(a2Ind);
#if timelog==1//update double
				clock_t tk0,tk;
				tk0=clock();
#endif
#if DualUpdate==1
				getAlphaUpdates(da1,da2,al1,al2,a1Ind,a2Ind,a1,f1,lc1,uc1);
#elif DualUpdate==2
				getAlphaUpdatesnew(da1,da2,al1,al2,a1Ind,a2Ind,a1,f1,lc1,uc1);
#endif
				max_da1=da1;
				max_da2=da2;
				max_al1=al1;
				max_al2=al2;
				//update mF for new mAlpha
				ftype* qa1,*qa2;
				qa1=mQ[a1Ind];
				qa2=mQ[a2Ind];

				if(max_al1==5) //switch to free
				{
					if(mAsgn[a1Ind]==-1)mShc--;
					else if(mAsgn[a1Ind]==+1)mShc--;
					else if(mAsgn[a1Ind]==0)mShz--;
					mAlpha[a1Ind]+=(ftype)max_da1;
				}else if(max_al1==0)//switch to zero
				{
					if(mAsgn[a1Ind]==-1){mShz++;mShc--;}
					else if(mAsgn[a1Ind]==+1){mShz++;mShc--;}
					else if(mAsgn[a1Ind]==5)mShz++;
					mAlpha[a1Ind]=0.;
				}else if(max_al1==+1)//switch to +mC
				{
					if(mAsgn[a1Ind]==0){mShz--;mShc++;}
					else if(mAsgn[a1Ind]==5)mShc++;
					mAlpha[a1Ind]=mC;
				}else //switch to -mC
				{
					if(mAsgn[a1Ind]==0){mShz--;mShc++;}
					else if(mAsgn[a1Ind]==5)mShc++;
					mAlpha[a1Ind]=-mC;
				}
				if(max_al2==5) //switch to free
				{
					if(mAsgn[a2Ind]==-1)mShc--;
					else if(mAsgn[a2Ind]==+1)mShc--;
					else if(mAsgn[a2Ind]==0)mShz--;
					mAlpha[a2Ind]+=(ftype)max_da2;
				}else if(max_al2==0)//switch to zero
				{
					if(mAsgn[a2Ind]==-1){mShz++;mShc--;}
					else if(mAsgn[a2Ind]==+1){mShz++;mShc--;}
					else if(mAsgn[a2Ind]==5)mShz++;
					mAlpha[a2Ind]=0.;
				}else if(max_al2==+1)//switch to +mC
				{
					if(mAsgn[a2Ind]==0){mShz--;mShc++;}
					else if(mAsgn[a2Ind]==5)mShc++;
					mAlpha[a2Ind]=mC;
				}else //switch to -mC
				{
					if(mAsgn[a2Ind]==0){mShz--;mShc++;}
					else if(mAsgn[a2Ind]==5)mShc++;
					mAlpha[a2Ind]=-mC;
				}
				mAsgn[a1Ind]=max_al1;
				mAsgn[a2Ind]=max_al2;

#if timelog==1
				tk=clock();
				mTU2Count+=tk-tk0;
				tk0=tk;
#endif

				for (i=0;i<mNs;i++)
				{
					mF[i]+=(ftype)(max_da1*qa1[i] + max_da2*qa2[i]);
				}
#if logi
				pinfo("                 %f %f    %d %d %d %d %f %f\n",max_da1,max_da2,a1Ind,a2Ind,max_al1,max_al2,mAlpha[a1Ind],mAlpha[a2Ind]);
#endif
#if timelog==1
				tk=clock();
				mTF2Count+=tk-tk0;
#endif
			}//optimization of one or two alphas ready
#if dynamics==1
			if(minirun==0)
			{
				fprintf(fdyndi,"%d %d %g %g \n",a1Ind,a2Ind,max_da1,max_da2);
				tk=clock();
				fprintf(fdynt,"%f 0\n",timeDiff(t0,tk));
			}
#endif

#if check==1
			//check consistency of mF and mAsgn
			//rebuilds mF to avoid rounding errors in mF
			{
				int p;
				for(p=0;p<mNs;p++)
				{
					double lsum;
					int k;
					lsum=mmXY[p];
					for(k=0;k<mNs;k++)
					{
						if(mAsgn[k]!=0)
						{

							lsum+=mQ[k][p]*mAlpha[k];
						}
					}
					if(fabs(mF[p]-lsum)>0.0000001) 
					{
						pinfo("error in mF-consistency\n");
						exit(-1);
						lsum=lsum;
					}
					if((mAlpha[p]>=mC-0.000001 && mAsgn[p]!=1) || (mAlpha[p]<=-mC+0.000001 && mAsgn[p]!=-1) || (mAlpha[p]==0 && mAsgn[p]!=0))
					{
						pinfo("mAlpha and mAsgn error\n");
						exit(-1);
					}
				}
			}
#endif
#if check==1
				//gives total error
				// 0.5*aQa-la+-ea, Qa-l=mF -> (0.5*mF-0.5*l+-e)*a
			{
					mGeerr1=mGeerr;
					mGeerr=0;
					for(i=0;i<mNs;i++)
					{
						if(mAsgn[i]==0) continue;
						if(mAlpha[i]>0)
							mGeerr+=mAlpha[i]*(0.5*mF[i]+0.5*mmXY[i]+mEpsilon);//mPar->epsi);
						else
							mGeerr+=mAlpha[i]*(0.5*mF[i]+0.5*mmXY[i]-mEpsilon);//mPar->epsi);
					}
#if logi
					pinfo("step %d diistr fct: %f change %f\n",mACount,mGeerr,mGeerr-mGeerr1);
#endif
					if(mGeerr>mGeerr1 && mACount>1) {
						pinfo("ascending error pre!!\n");
						pinfo("mQ: %f %f %f %f mF: %f %f\n",mQ[a1Ind][a1Ind],mQ[a1Ind][a2Ind],mQ[a2Ind][a1Ind],mQ[a2Ind][a2Ind],mF[a1Ind],mF[a2Ind]);
						exit(-1);
					}
			}
#endif
//############################################################
//############################################################
//############################################################
//############################################################
//############################################################
			/*
			collect all, update state, delete zeros.
			stop if #c>1 or #all>10 or #all*2 < #cycles#
			*/
			if(mPar->doblock){
#if timelog==1//block total
				clock_t tk0,tk;
				tk0=clock();
#endif
				//uses mLine,mMat
				list<sline>::iterator linei;
				sline linet;
				int ind;
				ind=a1Ind;//collect a1Ind at first
				if(!mLine->empty())
					for(linei=mLine->begin();(*linei).ind!=ind && linei!=mLine->end();linei++);
				if(mLine->empty()|| linei==mLine->end())
				{//index is not in list
					if(mAsgn[ind]!=0)
					{//don't collect zeros
						linet.ind=ind;
						mLine->push_back(linet);
					}
				}
				else if(mAsgn[ind]==0){//delete indices, which changed to zero
					mLine->erase(linei);
				}
			//	if(mAsgn[ind]==1 || mAsgn[ind]==-1) {mLine.clear();mLineit=mACount;mLinecs=mShc;}
				if(a2Ind>=0)
				{
					ind=a2Ind;
					if(!mLine->empty())
						for(linei=mLine->begin();(*linei).ind!=ind && linei!=mLine->end();linei++);
					if(mLine->empty()|| linei==mLine->end())
					{//index is not in list
						if(mAsgn[ind]!=0)
						{//don't collect zeros
							linet.ind=ind;
							mLine->push_back(linet);
						}
					}
					else if(mAsgn[ind]==0){//delete indices, which changed to zero
						mLine->erase(linei);
					}
					//if(mAsgn[ind]==1 || mAsgn[ind]==-1 || mAsgn[ind]==0) {mLine.clear();mLineit=mACount;mLinecs=mShc;}
				}

				i=(int)mLine->size();
				if(i>0 && (mShc-mLinecs>=mPar->blockCmax || (double)i*mPar->blockOver<(double)(mACount-mLineit) || i>=mPar->blockNmax))
//its better to do more blockoptimizations than selecting only oszillating alphas (eg. Q-update saves)
/*
				if((double)i*mPar->blockOver>=(double)(mACount-mLineit))//block full after too less updates?
				{//thow out all non-oszillating alphas
					mLine->clear();
					mLineit=mACount;
					mLinecs=mShc;
				}else
*/
				if(mACount>mLineit+1)
				{
					for(linei=mLine->begin();linei!=mLine->end();linei++)
					{
						ind=(*linei).ind;
						if(mAlpha[ind]>0){
							(*linei).f=(ftype)(-1.)*(mF[ind]+mEpsilon);
							(*linei).high=mC;
							(*linei).low=0;
							(*linei).highsgn=1;
							(*linei).lowsgn=0;
                        }else{
                            (*linei).f=(ftype)(-1.)*(mF[ind]-mEpsilon);
							(*linei).high=0;
							(*linei).low=-mC;
							(*linei).highsgn=0;
							(*linei).lowsgn=-1;
						}
						(*linei).a=mAlpha[ind];
					}
					{
						int bback;
#if timelog==1//block routine
						clock_t tk0,tk;
						tk0=clock();
#endif
						dd1=0;dd2=0;
						bback=getdalist(mLine);
						if (bback==-1) mBCCount++;
						if (bback==-2) mBSCount++;
#if timelog==1
						tk=clock();
						mTBCountsmax=max(mTBCountsmax,timeDiff(tk0,tk));//max of single block opt in seconds(double)
						mTBCounts+=tk-tk0;//total getdalist (without overhead)
#endif
					}
					//daopt and dasgnopt contains bounded solution
					//update mF and QC and mAlpha
					for(linei=mLine->begin();linei!=mLine->end();linei++)
					{
						ftype* qind;
						signed char oldsgn,newsgn;
						ind=(*linei).ind;
						qind=mQ[ind];
						oldsgn=mAsgn[ind];
						newsgn=(*linei).asgnopt;
#if blocklogi
						pinfo("%d gauss update mAlpha %d from %f to %f\n",mACount,ind,mAlpha[ind],mAlpha[ind]+(*linei).daopt);
#endif
						//update mAlpha
						mAlpha[ind]+=(*linei).daopt;
						//update f
						for (j=0;j<mNs;j++)
							mF[j]+=(ftype)((*linei).daopt*qind[j]);
						//update QC and mShc,mShz
						if(oldsgn==5)//old sign is free
						{
	                        if(newsgn==1)mShc++;
							else if(newsgn==0)mShz++;
							else if(newsgn==-1)mShc++;
						}else if(oldsgn==1)//old sign is mC
						{
							if(newsgn==5)mShc--;
							else if(newsgn==0){mShz++;mShc--;}
						}else if(oldsgn==0)//old sign is free
						{
							if(newsgn==5)mShz--;
							else if(newsgn==1){mShz--;mShc++;}
							else if(newsgn==-1){mShc++;mShz--;}
						}else if(oldsgn==-1)//old sign is -mC
						{
							if(newsgn==5)mShc--;
							else if(newsgn==0){mShc--;mShz++;}
						}
						mAsgn[ind]=newsgn;
					}
					mLine->clear();
					mLineit=mACount;
					mLinecs=mShc;
					mBCount++;
#if dynamics
					if(minirun==0)
					{
						tk=clock();
						fprintf(fdynt,"%f 1\n",timeDiff(t0,tk));
					}
#endif
				}
#if timelog==1
				tk=clock();
				mTBCount+=tk-tk0;//total block time (with overhead)
#endif
			}
#if matlablog==1
			for(int p=0;p<mNs;p++)
			{
				signed char aaaa;
				aaaa=mAsgn[p];
				if(aaaa!=0) aaaa=1;
				azlog.push_back(aaaa);
				alog.push_back(mAlpha[p]);
			}
#endif
#if check==1
			//check consistency of mF and mAsgn
			//rebuilds mF to avoid rounding errors in mF
			{
				int p;
				for(p=0;p<mNs;p++)
				{
					double lsum;
					int k;
					lsum=mmXY[p];
					for(k=0;k<mNs;k++)
					{
						if(mAsgn[k]!=0)
						{
							lsum+=mQ[k][p]*mAlpha[k];
						}
					}
					if(fabs(mF[p]-lsum)>0.0000001) 
					{
						pinfo("error in mF-consistency\n");
						exit(-1);
						lsum=lsum;
					}
					if((mAlpha[p]>=mC-0.000001 && mAsgn[p]!=1) || (mAlpha[p]<=-mC+0.000001 && mAsgn[p]!=-1) || (mAlpha[p]==0 && mAsgn[p]!=0))
					{
						pinfo("mAlpha and mAsgn error\n");
						exit(-1);
					}
				}
			}
#endif
#if check==1
				//gives total error
				// 0.5*aQa-la+-ea, Qa-l=mF -> (0.5*mF-0.5*l+-e)*a
				{
					mGeerr1=mGeerr;
					mGeerr=0;
					for(i=0;i<mNs;i++)
					{
						if(mAsgn[i]==0) continue;
						if(mAlpha[i]>0)
							mGeerr+=mAlpha[i]*(0.5*mF[i]+0.5*mmXY[i]+mEpsilon);//mPar->epsi);
						else
							mGeerr+=mAlpha[i]*(0.5*mF[i]+0.5*mmXY[i]-mEpsilon);//mPar->epsi);
					}
#if logi
					pinfo("step %d diistr fct: %f change %f\n",mACount,mGeerr,mGeerr-mGeerr1);
#endif
					if(mGeerr>mGeerr1 && mACount>1) {
						pinfo("ascending errorpost!!\n");
//						exit(-1);
					}
				}
#endif
			mACount++;
			if(mPar->timeout>-1) if((mACount & 63)==0)
			{//don't know how to set up timer interrupt
                tk=clock();
				goon=timeDiff(t0,tk)<mPar->timeout;
			}
		}while(goon);
        tk=clock();
		mTCount=timeDiff(t0,tk);
		mTimeout = (mPar->timeout>=0 && mTCount>=mPar->timeout);
		if(mTimeout) 
		{
	        pinfo("          stopping smo-calculation after mTimeout\n");
		}
		if(minirun==0)
		{
			pinfo("    epsi:%.5f feat:%d/%d/%d #it:%d #blk:%d #Q:%d time:%.4f t:%f\n",mEpsilon,-mShz,mShc,mNs,mACount,mBCount,mQCount,mTCount,mTerm);
		}
		if(mTimeout) break;
#if matlablog==1
		ann_it.push_back(mACount);
#endif
    }while(!mLastepsi); //loop for doing annealing steps until neps=nepsilon
	fflush(flog);
	mFeatures=-mShz;

	if(reduce==1)
	{
		if(istep<isteps-1)
		{//before last minisubset
			j=0;
//			for(i=0;i<mNs;i++)if(mAsgn[i]!=0) {mSwp[iswp++]=istart+i;j++;}
			for(i=0;i<mNs;i++)if(mAsgn[i]!=0) {mMask[istart+i]=1;j++;}
			pinfo("    reduction step %d of %d ready, reduced %d-%d->%d\n",istep,isteps-1,mNs,mNs-j,j);
			if(mTimeout)
			{
				mmXY=originalXY;//xy is back at its base
				mPar->epsi=originalEpsi;//epsi is back
				mQ=originalQ;//Q is back
				goto ready;
			}
			istart=iend;
			iend+=isize;
			if(istep==isteps-2) iend=mDim;//the two last subsets will be combined
			istep++;
			if(istep==isteps-(mDim-isteps*isize)-1) isize++;
			//next set is from istart to iend
			goto runsubset;
		}
		else if(istep==isteps-1)
		{//next is whole run
			minirun=0;
			j=0;
//			for(i=0;i<mNs;i++)if(mAsgn[i]!=0) {mSwp[iswp++]=istart+i;j++;}
			for(i=0;i<mNs;i++)if(mAsgn[i]!=0) {mMask[istart+i]=1;j++;}
			pinfo("    reduction step %d of %d ready, reduced %d-%d->%d\n",istep,isteps-1,mNs,mNs-j,j);
			if(mTimeout)
			{
				mmXY=originalXY;//xy is back at its base
				mPar->epsi=originalEpsi;//epsi is back
				mQ=originalQ;//Q is back
				goto ready;
			}
			//mSwp[0..iswp-1] contains references to important support mFeatures
			goto runsubset;
		}
		goto ready;
	}//if reduce
ready:
	mTCount=timeDiff(t0,clock());
	pinfo("          smo termination error: %.6f\n",mVio);
	pinfo("          smo total running time: %.4f seconds\n",mTCount);
#if timelog==1
	pinfo(" Qupdate:%.5f block total:%.5f blocks:%.5f blocksmax:%.5f WSS1:%.5f WSS2:%.5f\n",
		(double)((double)mTQCount/(double)CLOCKS_PER_SEC),
		(double)((double)mTBCount/(double)CLOCKS_PER_SEC),//total with overhead
		(double)((double)mTBCounts/(double)CLOCKS_PER_SEC),//total without overhead
		mTBCountsmax,//max single block
		(double)((double)mTWSS1Count/(double)CLOCKS_PER_SEC),(double)((double)mTWSS2Count/(double)CLOCKS_PER_SEC));
	pinfo(" block cancellations: singular: %d, Block complex %d \n",mBSCount,mBCCount);
	pinfo(" double Update:%.5f + %.5f single update:%.5f + %.5f total overhead:%.5f\n",
		(double)((double)mTU2Count/(double)CLOCKS_PER_SEC),(double)((double)mTF2Count/(double)CLOCKS_PER_SEC),
		(double)((double)mTU1Count/(double)CLOCKS_PER_SEC),(double)((double)mTF1Count/(double)CLOCKS_PER_SEC),
		mTCount-(double)((mTQCount+mTBCount+mTWSS1Count+mTWSS2Count+mTU1Count+mTU2Count+mTF1Count+mTF2Count)/(double)CLOCKS_PER_SEC));
#endif
#if dynamics==1
	fclose(fdyna);
	fclose(fdynkkt);
	fclose(fdyndi);
	fclose(fdynt);
#endif
	if(reduce)
	{
		for(i=0;i<iswp;)
		{//reverse swapping of unimportant mFeatures 
			int l;
			int i1,i2;
			ftype *datai1,*datai2;
			i1=mSwp[i++];
			i2=mSwp[i++];
			datai1=&mSet->mData[i1*mSet->mN];
			datai2=&mSet->mData[i2*mSet->mN];
			for (l=0;l<mSet->mN;l++)
				swap(datai1[l],datai2[l]);
			swap(mAlpha[i1],mAlpha[i2]);
		}
	}//if reduce
}

//###################################################################################################
//###################################################################################################
//###################################################################################################
//###################################################################################################
//###################################################################################################
//###################################################################################################
//class Ranking stores one feature found by the SMO and offers sorting (std::vector) see class Rankingi in headerfile
bool Rankingi::operator<(const Rankingi &i2)
{//local comparison operator for sorting
	return fabs(a)>fabs(i2.a);
}
bool operator<(const Rankingi &a,const Rankingi &b)
{//global comparison operator for sorting
	return fabs(a.a)>fabs(b.a);
}

//###################################################################################################
//###################################################################################################
//###################################################################################################
//###################################################################################################
//###################################################################################################
//###################################################################################################
//class MeanRankingi
bool MeanRankingi::operator<(const MeanRankingi &b)
{//returns true for desired sequence (this - b)
	if(n>b.n) return true;//descending order feature frequency
	if(n<b.n) return false;
	if(dimRefSum<b.dimRefSum) return true;//ascending order mean ranking position
	if(dimRefSum>b.dimRefSum) return false;//ascending order mean ranking position
	return (a>b.a);//descending order feature intensity
}
bool operator<(const MeanRankingi &a,const MeanRankingi &b)
{//returns true for desired sequence (a - b)
	if(a.n>b.n) return true;//descending order feature frequency
	if(a.n<b.n) return false;
	if(a.dimRefSum<b.dimRefSum) return true;//ascending order mean ranking position
	if(a.dimRefSum>b.dimRefSum) return false;//ascending order mean ranking position
	return (a.a>b.a);//descending order feature intensity
}
//class MeanRanking offers functionality to average over more than one Ranking
MeanRanking::MeanRanking()
{//constructor: zero initialization
	mrFeat=NULL;
	free();
}
MeanRanking::~MeanRanking()
{//destructor: delete all reserved memory
	free();
}
void MeanRanking::free()
{//release memory and set all members to zero
	if (mrFeat) delete[] mrFeat;
	mrFeat=NULL;
	mDimMem=0;
	mCollectedFeatures=0;
	mCollectedSets=0;
	mDim=0;
	mEpsilon=0;
	mC=0;
	mDeletedByFlimit=false;
}
void MeanRanking::reserve(int dim)
{//allocate memory for a maximum number of dim features
	//in crossvalidation, the number of dimensions(possible features,column objects) remains constant
	if(mrFeat)
	{
		if(mDimMem<dim+1) {//not enough memory allocated
			delete[] mrFeat;
            mrFeat=new MeanRankingi[dim+1];
			mDimMem=dim+1;
		}
	}else{//no memory allocated
        mrFeat=new MeanRankingi[dim+1];
		mDimMem=dim+1;
	}
	clear(dim);
}
void MeanRanking::clear(int dim)
{//allocate memory for dim features, initialize with zero
	int j;
	if(mDimMem<dim+1) reserve(dim);
	for (j = 0; j < dim; j++)
	{
		mrFeat[j].a=0.;
		mrFeat[j].dimRef=j;
		mrFeat[j].dimRefSum=0;
		mrFeat[j].n=0;
	}
	mCollectedFeatures=0;
	mCollectedSets=0;
	mDeletedByFlimit=false;
//	mDim=dim;
	mDim=0;
}
void MeanRanking::collectFrom(Ranking *rank,int flimit)
{//adds one Ranking to MeanRanking for averaging
//to do: if kernel was used, ranking is not possible, but in dyadic mode (features are constant) ranking should be applied if aquired
	int i;
	i=0;
	vector<Rankingi>::iterator ri;
	if (mCollectedSets==0) mDim=rank->mDim;//for kernel dimensionality is not constant - feature collection works only for one set
//	else if(rank->mDim!=mDim) {xinfo("error in MeanRanking::collectFrom - argument does not match\n");exit(-1);}
	if(mCollectedSets==0)//this works currently only for the first set
	for (ri = rank->mFeat.begin(); ri!=rank->mFeat.end(); ri++)
	{//collect all features from all subsets into mrank 
		mrFeat[(*ri).dimRef].a+=(*ri).a;
		mrFeat[(*ri).dimRef].n++;
		mrFeat[(*ri).dimRef].dimRefSum+=i;//for building mean index
		//predefined: dimRef = i (rank.clear)
		i++;
		if(flimit>=0 && i==flimit) break;
	}else i=rank->mFeat.size();
	if(flimit>=0) i=min(flimit,i);
	mCollectedFeatures+=i;
	mCollectedSets++;
	if(rank->mDeletedByFlimit) mDeletedByFlimit=true;
}
void MeanRanking::extractFromTo(Dataset *from,Dataset* to,int feat)
{//extracts 'feat' features in 'this' from Dataset 'from' to a destination set 'to'
	//used if ranking is aquired (determine cv-averaged features instead of using retrained model)
	//toDo: if collected sets>1, determine optimum number of features from 'rank' for best generalization
	int i,j;
	int takes;
	if(from->mDim!=mDim) {xinfo("error in MeanRanking::extractFromTo - argument does not match\n");exit(-1);}
	sort(&mrFeat[0],&mrFeat[mDim]);//sort by nsum-,dimRefSum+
	//either take
//	for(i=0;i<mDim && mrFeat[i].n>0;i++);
	//or take
	i=mCollectedFeatures/=mCollectedSets;
	if(feat>0) i=feat;
	takes=i;//take so much features into "to"
	to->reserve(from->mN,takes,true);//holds new mDim-reduced set with y
	for(i=0;i<from->mN;i++) to->mY[i]=from->mY[i];
	feat=0;
	for(j=0;j<takes;j++)if(mrFeat[j].n>0){
		ftype* dataj;
		dataj=&(from->mData[from->mN*mrFeat[j].dimRef]);
		for(i=0;i<from->mN;i++) to->mData[j*from->mN+i]=dataj[i];
		feat++;
	}
	to->mN=from->mN;
	to->mDim=feat;
	to->mTrans=true;
}
void MeanRanking::transformBack(Ranking *rank)
{//transforms the rank.dimRefs by this.dimRef
	//used if ranking is aquired (determine cv-averaged features instead of using retrained model)
	//after retraining Dataset 'to' (see above) the feature indices must be retransformed to Dataset 'from'
	vector<Rankingi>::iterator ri;
	for(ri=rank->mFeat.begin();ri!=rank->mFeat.end();ri++)
	{//rebuild original indices in psvm model
		(*ri).dimRef=mrFeat[(*ri).dimRef].dimRef;
	}
}
void MeanRanking::putback(Ranking *rank)
{//stores averaged features of (meanranking)'this' into Ranking 'rank' for building psvm model
//'this' is possibly not sorted!, rank will be sorted
	if(mCollectedSets!=1) {xinfo("error in MeanRanking::putback - no data available\n");exit(-1);}
	int i;
	rank->mFeat.resize(mCollectedFeatures);
	rank->mDim=mDim;
	rank->mDeletedByFlimit=mDeletedByFlimit;
	for(i=0;i<mDim;i++)
	{
		if(mrFeat[i].n>0){
			rank->mFeat[mrFeat[i].dimRefSum].a=mrFeat[i].a;//reconstruction of original sequence by using dimRefSum
			rank->mFeat[mrFeat[i].dimRefSum].dimRef=mrFeat[i].dimRef;
		}
	}
	//	sort(rank->mFeat.begin(),rank->mFeat.end());
	//sort is not necessary because permutation sequence is restored from dimRefsum
}
void MeanRanking::copyFrom(MeanRanking* mrank)
{//copies a whole MeanRanking object from 'mrank' into 'this'
	reserve(mrank->mDim);
	mCollectedFeatures=mrank->mCollectedFeatures;
	mCollectedSets=mrank->mCollectedSets;
	mC=mrank->mC;
	mDeletedByFlimit=mrank->mDeletedByFlimit;
	mDim=mrank->mDim;
	mEpsilon=mrank->mEpsilon;
	memcpy(mrFeat,mrank->mrFeat,mDim*sizeof(MeanRankingi));
}

//class rs stores ROC points
bool rs::operator<(const rs &b)
{//local comparison operator for sorting
	return yt>b.yt;
}
bool operator<(const rs &a,const rs &b)//global comparator for sort function
{//global comp-operator
	return a.yt>b.yt;
}
//###################################################################################################
//###################################################################################################
//###################################################################################################
//###################################################################################################
//###################################################################################################
//###################################################################################################
//###################################################################################################
//class PSVMmodel offers functionality to determine psvm model components and to predict data
PSVMmodel::PSVMmodel()
{//constructor: clear all members
	svz=NULL;
	b=0;
}
PSVMmodel::~PSVMmodel()
{//destructor:free used memory
	if(svz) delete svz;//object memory cleared by delete
}
void PSVMmodel::free()
{//clear model components and release memory
	pinfo("   clearing model\n");
	if(svz) delete svz;svz=NULL;
	b=0;
	//the other components will be deleted autmatically in destructor, but nevertheless:
	np.free();
	rank.mDim=0;
	rank.mFeat.clear();
	smo.free();
}
bool PSVMmodel::getRankFromSet(Dataset* set,bool Qreset,bool Areset)
{//run smo and get feature ranking in "rank" (sparse vector alpha)
	//feature limitation has to be done by user (special case: Ccut)
	int j;
	Rankingi rr;
	if(!set)
	{
		xinfo("error in PSVMmodel::getRankFromSet: set not available\n");
		return false;
	}
	//run smo
	pinfo("   running SMO\n");
	smo.mPar=&par;
	smo.run(set,b,Qreset,Areset);
	//compress alpha and get rank
	if(par.flimit>0 && smo.mShc>par.flimit && par.allowCcut==0)
	{//if feature limitation applied -> reset model for too many bounded features
		//cause: bounded features are not sortable -> cutting feature vector would lead to non-deterministic behavier
		//this avoids obtaining models with too many features
		rank.mFeat.clear();
		rank.mDeletedByFlimit=true;
		minfo("model deleted because C= %g is too low for flimit %d !\n",par.C,par.flimit);
	}else{//copy used alpha indices into Ranking structure
		if(par.doretrain)
		{
			//select first flimit features
			//create new dataset
			//insert flimit feature dimensions
			//retrain
			//transform indices back
			Dataset extractionset;
			int i;
			//sort the features
			rank.mFeat.reserve(smo.mFeatures);//mFeat is vector
			rank.mFeat.clear();
			for (j = 0; j < set->mDim; j++)
			if (smo.mAlpha[j]!=0)
			{
				rr.dimRef=j;
				rr.a=smo.mAlpha[j];
				rank.mFeat.push_back(rr);
			}
			sort(rank.mFeat.begin(),rank.mFeat.end());

			{//insert flimit features into extractopmset
				int takes;
				double esave;
//				vector<Rankingi>::iterator ri;
				if(par.flimit>0) takes=min(par.flimit,(int)rank.mFeat.size());//take so much features
				else takes=rank.mFeat.size();
				extractionset.reserve(set->mN,takes,true);//holds new mDim-reduced set with y
				for(j=0;j<set->mN;j++) extractionset.mY[j]=set->mY[j];
				for(j=0;j<takes;j++){
					ftype* dataj;
					dataj=&(set->mData[set->mN*rank.mFeat[j].dimRef]);
					for(i=0;i<set->mN;i++) extractionset.mData[j*set->mN+i]=dataj[i];
				}
				extractionset.mN=set->mN;
				extractionset.mDim=takes;
				extractionset.mTrans=true;
				esave=par.epsi;
				par.epsi*=par.retrain_fe;
				smo.run(&extractionset,b,true,true);
				par.epsi=esave;
//				ri=rank.mFeat.begin();
				for(j=0;j<takes;j++)
				{
//					if(smo.mAlpha[j]==0)
//					{
//						ri++;
//						rank.mFeat.erase(ri);
//					}
					rank.mFeat[j].a=smo.mAlpha[j];
//					ri++;
				}
			}
			rank.mDeletedByFlimit=false;
			sort(rank.mFeat.begin(),rank.mFeat.end());
		}else{
			rank.mFeat.reserve(smo.mFeatures);//mFeat is vector
			rank.mFeat.clear();
			for (j = 0; j < set->mDim; j++)
			if (smo.mAlpha[j]!=0)
			{
				rr.dimRef=j;
				rr.a=smo.mAlpha[j];
				rank.mFeat.push_back(rr);
			}
			rank.mDeletedByFlimit=false;
			sort(rank.mFeat.begin(),rank.mFeat.end());
		}
	}

	rank.mDim=set->mDim;
	//print messages
	char* text;
	if(par.isrelation) text="feature";else text="support vector";
	int suz;
	suz=rank.mFeat.size();
	minfo("   found %d %ss for epsilon=%g+-%.2f%%, C=%g\n",rank.mFeat.size(),text,par.epsi,par.epsiTol*100,par.C);

	if(verbose)
	{
		if(suz>0)pinfo("   most important %s is at index %d (from 1) weight %g \n",text,rank.mFeat.front().dimRef+1,fabs((double)rank.mFeat.front().a));
		if(par.doannealing>0){
			pinfo("   epsilon annealing decrease factor = %.7f \n",par.edec);
		}
		if(par.doblock>0){
			pinfo("   block optimization Nmax=%d Cmax=%d max cycles=%f \n",par.blockNmax,par.blockCmax,par.blockOver);
		}
		if(par.doreduction>0){
			pinfo("   reduction with size %d and epsilon fact %f \n",par.redsize,par.redfact);
		}
		if(par.domaxselection==2){
			pinfo("   wss: %.2f%% of maximum violation\n",par.selectionfact);
		}else if(par.domaxselection==1){
			pinfo("   wss: maximum violation\n");
		}else if(par.domaxselection==0){
			pinfo("   wss:  first violation\n");
		}
	}
	return !smo.mTimeout;
}
bool PSVMmodel::getModel(Dataset* set,Trainres *res,bool Qreset,bool Areset)
{//gets model for kernelmatrix set, puts results into res, normalizes set!!!
	//attention: set must not be normalized
	//if set is already normalized, the psvm.np will be overwritten with wrong normalization parameters!!!!
	bool ret;
	np.getFromSet(set);
	np.putOnSet(set,false);//limit is irrelevant
	getB(set);//needed in getRankFromSet
	ret=getRankFromSet(set,Qreset,Areset);//get features into psvm.rank
	{//test with set
		ftype *yy;
		double mse,rate;
		int tp,tn,fp,fn;
//		double mseadd,rateadd;
		yy=new ftype[set->mN];
		getY(yy,set,par.flimit,false);//no flimit,no more normalization
//		getAddErrors(yy,set,&mseadd,&rateadd,&mse,&rate);
		getErrors(yy,set,&mse,&rate,&tp,&tn,&fp,&fn);
//		pinfo("epsilon: %f, C: %f, training regression MSE: %f \n",par.epsi,par.C,mse);
//		pinfo("epsilon: %f, C: %f, training classificn err: %f \n",par.epsi,par.C,rate);
		res->cla_TError=rate;
//		res->cla_TError=getCError(tp,tn,fp,fn);
		res->reg_TError=mse;
		res->cla_Ttp=tp;
		res->cla_Ttn=tn;
		res->cla_Tfp=fp;
		res->cla_Tfn=fn;
//		res->cla_misses=(int)rateadd;
		delete[] yy;
	}
	return ret;
}
double PSVMmodel::getCError(double tp,double tn,double fp,double fn)
{
	if(par.useBalancedErrors)
		return 1-(tp/(2*(fn+tp))+tn/(2*(fp+tn)));
	else
		return (fp+fn)/(tp+fn+tn+fp);
}
#if 0
bool PSVMmodel::getModelWithLoops(Dataset* set,Trainres *res,bool getmodel)
{//uses hyperparameter selection and or crossvalidation cycles for training a predictor for dataset 'set'
//attention!!!: in crossvalidation mode the final model is not valid because it belongs to the last subset training, a retraining over whole set is required
//returns true if at least one SMO training phase was stopped after timeout.
/*in: kernelmatrix set
	action: normalization, crossval,modelselection if required
	out in res: epsilon, C, GeneralizationError, Trainerror, modelselectiondata
	results for:
	modelsel+CV:	trainres:C,epsilon,G-error			psvm: undefined(not trained on whole set)
	CV:				trainres:G-error					psvm: undefined
	modelsel:		trainres:C,epsilon,T-error,rank		psvm: b,np,rank
	simple training:trainres:T-error,rank				psvm: b,np,rank
	attention: reserves memory for holding the modelselection data in structure Trainres (if the pointer is zero)
*/

    Dataset *intrainset,*intestset;//subsets of dataset
	
	MeanRanking *inrank;//stores one Ranking per hyperparameter, feature ranking average capabilities are not used in this routine
	double *inerror_d;//save errors for continuous prediction
	double *inerror_i;//save errors for class prediction
	int *inerror_n;//number of cumulated errors(testobjects)
	ftype ee1,ee2,eed,cc1,cc2,ccd;//hyperparameter ranges
	int nc,ne,nce;//number of hyperparameter tests
	int nci,nei;//iterators

	ftype *yy;//for calculating predictions
	int memYy;//used memory in yy for recycling
	int infold;//number of crossvalidation cycles
	int kin;//iterator
	int i;
	int hypnr;
	double mse,rate;//display results
	bool Qreset;//false:saves Q-recalculation in modelselection
	bool Areset;//false:saves smo-status in modelselection/crossvalidation, not testet!!!

	bool notimeout;//gets true, if SMO was stopped in one training cycle at least
	bool notimeoutall;
	notimeout=false;
	notimeoutall=false;

	if(!set)
	{
		xinfo("error in PSVMmodel::getModelWithLoops: set not available\n");
		return false;
	}

	res->maxFeat=0;//only for user information: maximum of lagrangian alpha, helpful to select C because C limits alpha. If maxFeat==C, increasing C can change the predictor, otherwise not
	ee1=par.ms_e1;
	ee2=par.ms_e2;
	eed=par.ms_ed;
	cc1=par.ms_c1;
	cc2=par.ms_c2;
	ccd=par.ms_cd;
	if(cc1<=0 || cc2<=0 || ccd<=0)
	{
		xinfo("error in GetModelWithLoops: illegal hyperparameters\n");exit(-1);
	}else if(ee1<=0 || ee2<=0 || eed<=0)
	{//use automatic selection of epsilon
//		xinfo("error in GetModelWithLoops: illegal hyperparameters\n");exit(-1);
		ne=1;
		ee1=-1;ee2=-1;eed=1;
	}else{
		//initialize grid search parameters, correct them
		if(eed<(ftype)1e-30) eed=(ftype)1e-30;
		ne=1+(int)(double)((ee2-ee1)/eed+0.0001);
		ee2=ee1+eed*(ne-1);
	}
	if(ccd<(ftype)1e-30) ccd=(ftype)1e-30;
	nc=1+(int)(double)((cc2-cc1)/ccd+0.0001);
	cc2=cc1+ccd*(nc-1);
	nce=ne*nc;
	inrank=new MeanRanking[nce];
	inerror_d=new double[nce];
	inerror_i=new double[nce];
	inerror_n=new int[nce];
	res->msnc=nc;
	res->msne=ne;
	if(res->ms==NULL) res->ms=new Msdata[nce];
	yy=NULL;
	memYy=0;

	infold=par.infold;
	if(infold==0) infold=1;
	if(infold!=1) {intrainset=new Dataset;intestset=new Dataset;}
	if(infold>set->mN||infold<0) infold=set->mN;
	//save one ranking per hyperparametercombi
	//save errors per hyperparametercombi
	//toDo: problem: saving all models during hyperparameterselection is very expensive in memory!!!  

	//clear rankings and errors
	for(i=0;i<nce;i++){
		inrank[i].clear(set->mDim);//holds features for all subsets
		inerror_d[i]=0;
		inerror_i[i]=0;
		inerror_n[i]=0;
	}
	//do combined crossvalidation with parameterselection
	for(kin=0;kin<infold;kin++)
	{
		if(infold==1) {
			intrainset=set;
			intestset=intrainset;
			np.getFromSet(set);
			np.putOnSet(set);
        }else {
			set->tileTo(intrainset,intestset,kin,infold);
            intrainset->normalize(intestset);
		}
		getB(intrainset);//needed in getRankFromSet
		hypnr=0;
		Qreset=true;
		Areset=true;
		for(nci=0;nci<nc;nci++)
		{
			par.C=cc2-ccd*nci;
			if(par.C<=0) par.C=(ftype)1e-30;
			Areset=true;//epsilon-decrease causes little changes of model
			for(nei=0;nei<ne;nei++)
			{
				par.epsi=ee2-eed*nei;
//				if(par.epsi<=0) par.epsi=(ftype)1e-30;
				//epsilon may be negative for resetting it in smo leading to 1 support vector
				inrank[hypnr].mEpsilon=par.epsi;
				inrank[hypnr].mC=par.C;
				if(!inrank[hypnr].mDeletedByFlimit)// && (hypnr<nc || !inrank[hypnr-nc].mDeletedByFlimit))
				{//last training was not deleted caused by too high C, so dont ignore all lower Cs
					notimeout=getRankFromSet(intrainset,Qreset,Areset);//get features into psvm.rank
					notimeoutall=notimeoutall || notimeout;
					inrank[hypnr].mEpsilon=par.epsi;//for epsi=-1 now epsi=maxepsi leading to >=1 supportvector
					if(rank.mDeletedByFlimit) inrank[hypnr].mDeletedByFlimit=true;
					if(rank.mFeat.size()>0) res->maxFeat=max(res->maxFeat,(double)fabs(rank.mFeat.front().a));
				}else{
					rank.mFeat.clear();
					rank.mDeletedByFlimit=true;
					rank.mDim=intrainset->mDim;
				}
				{//else
					{
						inrank[hypnr].collectFrom(&rank,par.flimit);//in crossvalidation mode, this works currently not correct
						{//test with intestset, which may be same as intrainset!
							if(memYy<intestset->mN){
								if(yy) delete[] yy;
								yy=new ftype[intestset->mN];
								memYy=intestset->mN;
							}
							getY(yy,intestset,par.flimit,false);//no flimit,no more normalization
							getAddErrors(yy,intestset,&inerror_d[hypnr],&inerror_i[hypnr],&mse,&rate);
							pinfo("epsilon: %g, C: %g, regression MSE: %g \n",par.epsi,par.C,mse);
							pinfo("epsilon: %g, C: %g, classificn err: %g \n",par.epsi,par.C,rate);
							inerror_n[hypnr]+=intestset->mN;
						}
					}
				}
				hypnr++;
				Qreset=false;//saves smocalculations for same data subset
//				Areset=false;smo.mMaxepsi=par.epsi;
				//Areset is not tested, remains true
			}
		}
		if(infold>1)pinfo("crossvalidation cycle %d of %d ready\n",kin+1,infold);
	}
	//determine best hyperparameters for best MSE(regression) and error rate(classification)
	double besterror_d,besterror_i;
	int bestparam_d,bestparam_i;
	bestparam_d=-1;//regression
	bestparam_i=-1;//classification
	besterror_d=0;//regression
	besterror_i=0;//classification
    for(i=0;i<nce;i++)
	{
		double ed,ei;
	//	if(!inrank[i].mDeletedByFlimit || bestparam_d<0||bestparam_i<0)
		{//problem: if bounded features > flimit, no model exists, so which is the best?
			ed=inerror_d[i]/inerror_n[i];//regression
			ei=inerror_i[i]/inerror_n[i];//classification
			if(bestparam_d<0 || besterror_d>ed){besterror_d=ed;bestparam_d=i;}
			if(bestparam_i<0 || besterror_i>ei){besterror_i=ei;bestparam_i=i;}
			if(besterror_d==ed && inrank[bestparam_d].mCollectedFeatures>inrank[i].mCollectedFeatures){besterror_d=ed;bestparam_d=i;}
			if(besterror_i==ei && inrank[bestparam_i].mCollectedFeatures>inrank[i].mCollectedFeatures){besterror_i=ei;bestparam_i=i;}
			res->ms[i].c=inrank[i].mC;
			res->ms[i].e=inrank[i].mEpsilon;
			res->ms[i].eec=ei;//classification
			res->ms[i].eer=ed;//regression
		}
/*	else		{//featurelimit violated ->use not these c,epsilon
			res->ms[i].c=inrank[i].mC;
			res->ms[i].e=inrank[i].mEpsilon;
			res->ms[i].eec=0.5;//classification
			res->ms[i].eer=-1;//regression
		}
		*/
	}
	//fill result structure
	res->reg_nf=(int)(inrank[bestparam_d].mCollectedFeatures/inrank[bestparam_d].mCollectedSets);//mistake:featurer limitation ignored!
	res->cla_nf=(int)(inrank[bestparam_i].mCollectedFeatures/inrank[bestparam_i].mCollectedSets);
	if(par.flimit>=0) res->reg_nf=min(par.flimit,res->reg_nf);//this is incorrect, mCollectedFeatures has to be limited instead of its mean
	if(par.flimit>=0) res->cla_nf=min(par.flimit,res->cla_nf);
	res->reg_c=inrank[bestparam_d].mC;
	res->cla_c=inrank[bestparam_i].mC;
	res->reg_e=inrank[bestparam_d].mEpsilon;
	res->cla_e=inrank[bestparam_i].mEpsilon;
	res->reg_mrank.copyFrom(&inrank[bestparam_d]);
	res->cla_mrank.copyFrom(&inrank[bestparam_i]);
	if(infold==1){//no crossval
		//if the final model over complete dataset is in inrank, copy it into final psvm model 
		if(par.doregress)//regression is default, if not explicitely turned off
			inrank[bestparam_d].putback(&rank);//give best model to psvm
		else
			inrank[bestparam_i].putback(&rank);//
	}
	res->reg_c1=cc1;
	res->reg_c2=cc2;
	res->reg_e1=ee1;
	res->reg_e2=ee2;
	res->cla_c1=cc1;
	res->cla_c2=cc2;
	res->cla_e1=ee1;
	res->cla_e2=ee2;

	//print messages
	if(nce!=1 && infold!=1)
	{//hyperparameterselection with crossval.
		res->reg_GError=besterror_d;
		res->cla_GError=besterror_i;
		res->cla_misses=(int)(inerror_i[bestparam_i]);//mean number of misses, natural, n datapoints are tested during crossvalidation, same as testing the whole set at once
		pinfo("Result after hyperparameterselection with crossvalidation:\n");
		pinfo("best reg: C: %g, epsi: %g, mean features: %d, G-MSE %g\n",res->reg_c,res->reg_e,res->reg_nf,res->reg_GError);
		pinfo("best cls: C: %g, epsi: %g, mean features: %d, G-error %g\n",res->cla_c,res->cla_e,res->cla_nf,res->cla_GError);
		pinfo("best cls: mean accuracy: %g, mean number of missclassified objects: %d of %d\n",1-res->cla_GError,res->cla_misses,set->mN);
		pinfo("attention: current model is built from last n-fold subset of the training data\n");
	}else if(nce!=1 && infold==1)
	{//hyperparameterselection over training error
		res->reg_TError=besterror_d;
		res->cla_TError=besterror_i;
		res->cla_misses=(int)inerror_i[bestparam_i];//number of misses
		pinfo("Result after hyperparameterselection over training error:\n");
		pinfo("best reg: C: %g, epsi: %g, features: %d, T-MSE %g\n",res->reg_c,res->reg_e,res->reg_nf,res->reg_TError);
		pinfo("best cls: C: %g, epsi: %g, features: %d, T-error %g\n",res->cla_c,res->cla_e,res->cla_nf,res->cla_TError);
		pinfo("best cls: accuracy: %g, number of missclassified objects: %d of %d\n",1-res->cla_TError,res->cla_misses,set->mN);
	}else if(nce==1 && infold!=1)
	{//crossval without modelselection
		res->reg_GError=besterror_d;
		res->cla_GError=besterror_i;
		res->cla_misses=(int)(inerror_i[bestparam_i]);//mean number of misses
		pinfo("Result after Crossvalidation:\n");
		pinfo("reg: C: %g, epsi: %g, mean features: %d, G-MSE %g\n",res->reg_c,res->reg_e,res->reg_nf,res->reg_GError);
		pinfo("cls: C: %g, epsi: %g, mean features: %d, G-error %g\n",res->cla_c,res->cla_e,res->cla_nf,res->cla_GError);
		pinfo("cls: mean accuracy: %g, mean number of missclassified objects: %d of %d\n",1-res->cla_GError,res->cla_misses,set->mN);
		pinfo("attention: current model is built from last n-fold subset of the training data\n");
	}else if(nce==1 && infold==1)
	{//training only
		res->reg_TError=besterror_d;
		res->cla_TError=besterror_i;
		res->cla_misses=(int)inerror_i[bestparam_i];//number of misses
		pinfo("Result of simple Training:\n");
		pinfo("reg: C: %g, epsi: %g, features: %d, T-MSE %g\n",res->reg_c,res->reg_e,res->reg_nf,res->reg_TError);
		pinfo("cls: C: %g, epsi: %g, features: %d, T-error %g\n",res->cla_c,res->cla_e,res->cla_nf,res->cla_TError);
		pinfo("cls: accuracy: %g, number of missclassified objects: %d of %d\n",1-res->cla_TError,res->cla_misses,set->mN);
	}

	if(yy) delete[]yy;
	delete[] inrank;
	delete[] inerror_d;
	delete[] inerror_i;
	delete[] inerror_n;
	if(intrainset!=set)
	{//if set was copied, delete the copies
        if(intrainset==intestset) delete intrainset;
		else {delete intrainset;delete intestset;}
	}
	return notimeoutall;//false if at least one SMO training phase was stopped after timeout.
	//the overall running time of this procedure may be a multiple of the desired time limit, because timeout limits every training phase
}
#else
bool PSVMmodel::getModelWithLoops(Dataset* set,Trainres *res,bool getmodel)
{//gets model for kernelmatrix set, puts results into res, normalizes set!!!
/*
outer cv-loop
model selection m
inner cv-loop ic( if not, testset=trainset)
 train ->model_ic, Terror_ic, Gerror_ic
inner cv-loop
	if(rank)
    	rmodel=rank(model_ic)
model selection
choose best model with respect to G or T from rmodel or retrained
outer cv-loop

cases: (c)lassification, (r)egression
cases: innercv(ic), outercv(oc), modelsel(ms)
cases: rank (requires innercv)

ic requires inner retraining
oc requires outer retraining
	based on original dataset
	based on reduced dataset (rankiong)


*/
    Dataset *intrainset,*intestset;//subsets of dataset
    Dataset *outtrainset,*outtestset;//subsets of dataset
	
	MeanRanking *inrank;//stores one Ranking per hyperparameter
	MeanRanking outrank_d,outrank_i;//collects all models during outer loop

	double *inerror_d;//errors for all hyperparameters regression
	double *inerror_i;//errors(number of false classified objects) for all hyperparams classification
	int *inerror_itp;//errors(number of tp classified objects) for all hyperparams classification
	int *inerror_itn;//errors(number of tn classified objects) for all hyperparams classification
	int *inerror_ifp;//errors(number of fp classified objects) for all hyperparams classification
	int *inerror_ifn;//errors(number of fn classified objects) for all hyperparams classification
	int *inerror_n;//number of cumulated testobjects within other inerror values
	double outerror_d;//cumulated errors during outer loop regression
	double outerror_i;//cumulated errors during outer loop classific
	int outerror_itp;//cumulated errors during outer loop classific
	int outerror_itn;//cumulated errors during outer loop classific
	int outerror_ifp;//cumulated errors during outer loop classific
	int outerror_ifn;//cumulated errors during outer loop classific
	int outerror_n;//number of cumulated errors(testobjects)

	int infold,outfold;//number of crossvalidation cycles
	int kin,kout;//iterator crossvalidation cycle

	ftype ee1,ee2,eed,cc1,cc2,ccd;//hyperparameter ranges
	int nc,ne,nce;//number of hyperparameter tests
	int nci,nei;//iterators

	double besterror_d,besterror_i;
	//int besterror_itp,besterror_itn,besterror_ifp,besterror_ifn;
	int bestparam_d,bestparam_i;

	ftype *yy;//for calculating predictions
	int memYy;//used memory in yy for recycling
	double dummy;
	double dummy1;
	int i;
	int hypnr;
	double mse,rate;//display results
	bool Qreset;//false:saves Q-recalculation in modelselection
	bool Areset;//false:saves smo-status in modelselection/crossvalidation, not testet!!!

	bool notimeout;//gets true, if SMO was stopped in one training cycle at least
	bool notimeoutall;

//	double rrr,rrc,rcr,rcc;


	notimeout=false;
	notimeoutall=false;

	if(!set)
	{
		xinfo("error in PSVMmodel::getModelWithLoops: set not available\n");
		return false;
	}

	res->maxFeat=0;//only for user information: maximum of lagrangian alpha, helpful to select C because C limits alpha. If maxFeat==C, increasing C can change the optimal predictor, otherwise not

	{	//calculate hyperparameter search region
		//modify ee2,cc2 to match eed,ccd
		ee1=par.ms_e1;
		ee2=par.ms_e2;
		eed=par.ms_ed;
		cc1=par.ms_c1;
		cc2=par.ms_c2;
		ccd=par.ms_cd;
		if(cc1<=0 || cc2<=0 || ccd<=0)
		{
			xinfo("error in GetModelWithLoops: illegal hyperparameters\n");exit(-1);
		}else if(ee1<=0 || ee2<=0 || eed<=0)
		{//use automatic selection of epsilon
	//		xinfo("error in GetModelWithLoops: illegal hyperparameters\n");exit(-1);
			ne=1;
			ee1=-1;ee2=-1;eed=1;
		}else{
			//initialize grid search parameters, correct them
			if(eed<(ftype)1e-30) eed=(ftype)1e-30;
			ne=1+(int)(double)((ee2-ee1)/eed+0.0001);
			ee2=ee1+eed*(ne-1);
		}
		if(ccd<(ftype)1e-30) ccd=(ftype)1e-30;
		nc=1+(int)(double)((cc2-cc1)/ccd+0.0001);
		cc2=cc1+ccd*(nc-1);
		nce=ne*nc;
		//result: cc1,cc2,ccd,ee1,ee2,eed, nc,ne,nce
	}

	{//initialize ranking arrays for inner loop over all hyperparameters
		//remember per hyperparameter: alpharanking(icv), reg_error, cla_error, number of icv test objects
		inrank=new MeanRanking[nce];
		inerror_d=new double[nce];
		inerror_i=new double[nce];
		inerror_itp=new int[nce];
		inerror_itn=new int[nce];
		inerror_ifp=new int[nce];
		inerror_ifn=new int[nce];
		inerror_n=new int[nce];
	}

	{//prepare pointer to predicted labels
		yy=NULL;
		memYy=0;
	}

	{//check minimum training size of two
		int minset;
		minset=set->mN;
		if(minset<2) {xinfo("More than two training objects required!\n");exit(-1);}
		if(par.outfold>minset||par.outfold<0) minset=minset-1;
		else if(par.outfold>1) minset=(par.outfold-1)*minset/par.outfold;//integer calculation!!
		if(minset<2) {xinfo("More than two training objects required! Watch outer crossvalidation!\n");exit(-1);}
		if(par.infold>minset||par.infold<0) minset=minset-1;
		else if(par.infold>1) minset=(par.infold-1)*minset/par.infold;//integer calculation!!
		if(minset<2) {xinfo("More than two training objects required! Watch crossvalidation!\n");exit(-1);}
	}

	{//prepare number of outfold cycles
		outfold=par.outfold;
		if(outfold>set->mN||outfold<0) outfold=set->mN;
		if(outfold==0) outfold=1;
	//prepare number of infold cycles later, when size of set is known
	//prepare memory of crossvalidation sets
		if(outfold!=1) {outtrainset=new Dataset;outtestset=new Dataset;}
		if(par.infold!=1) {intrainset=new Dataset;intestset=new Dataset;}
	}

	//if(outfold==1)
	{//prepare result structure
		res->msnc=nc;
		res->msne=ne;
		res->n_hyp=nce;
		if(res->ms==NULL) res->ms=new Msdata[nce];
	}


	//reset outer errors and ranking
	outrank_d.reserve(set->mDim);//regression(double) collect inner models, e,C from reg
	outrank_i.reserve(set->mDim);//classification(int) collect inner models, e,C from reg
//	outrank_di.reserve(set->mDim);//regression(double) ranking, e,C from cls
//	outrank_ii.reserve(set->mDim);//classification(int) ranking, e,C from cls
//	outerrorr_d=0;
//	outerrorr_i=0;
	outerror_d=0;
	outerror_i=0;
	outerror_itp=0;
	outerror_itn=0;
	outerror_ifp=0;
	outerror_ifn=0;
	outerror_n=0;
	outrank_d.mC=0;

	//do outer crossvalidation
	for(kout=0;kout<outfold;kout++)
	{
/*		if(outfold==1) {//no outer crossvalidation
			outtrainset=set;//only pointers
			outtestset=set;
//			outtrainset->normalize();
        }else {
			set->tileTo(outtrainset,outtestset,kout,outfold);
//			outtrainset->normalize(outtestset);
		}
		*/
		infold=par.infold;
		if(infold==0) infold=1;
		if(outfold==1) {//no outer crossvalidation
			if(infold==1)
			{
				outtrainset=set;//only pointers
				outtestset=set;
			}else{
				outtrainset=set;//only pointers
				outtestset=set;
			}
        }else {
			if(infold==1)
			{
				set->tileTo(outtrainset,outtestset,kout,outfold);
				outtrainset->normalize(outtestset,par.useNormLimit);
			}else{
				set->tileTo(outtrainset,outtestset,kout,outfold);
			}
		}
		infold=par.infold;
		if(infold>outtrainset->mN||infold<0) infold=outtrainset->mN;
		if(infold==0) infold=1;

		//out+in :: out_=tile(set)					in_=tile(out) norm int+inp, \in\ norm outt,outp \out\ \ret\ in_=tile(set), norm(int+inp) \in\ np=norm(set) \out\
		//out    :: out_=tile(set) norm outt+outp	in=outt						\in\				\out\ \ret\ in=set, np=norm(set) \in\
		//in     :: out=set							in_=tile(out) norm int+inp  \in\ np=norm(set)	\out\
		//       :: out=set							in=out	np=norm(in)         \in\ .
		//remember one ranking per hyperparametercombi
		//remember errors per hyperparametercombi
		//toDo: problem: saving all models during hyperparameterselection is very expensive in memory!!!  
		//clear rankings and errors
		for(i=0;i<nce;i++){
			inrank[i].clear(set->mDim);//holds features for all subsets
			inerror_d[i]=0;
			inerror_i[i]=0;
			inerror_itp[i]=0;
			inerror_itn[i]=0;
			inerror_ifp[i]=0;
			inerror_ifn[i]=0;
			inerror_n[i]=0;
		}
		//do combined crossvalidation with parameterselection
		for(kin=0;kin<infold;kin++)
		{
			if(infold==1) {
				if(outfold==1)
				{
					intrainset=outtrainset;
					intestset=outtrainset;
					np.getFromSet(set);
					np.putOnSet(set,false);
				}else 
				{
					intrainset=outtrainset;
					intestset=outtrainset;
//					intrainset->normalize(intestset);//modify data clones
				}
			}else {
				outtrainset->tileTo(intrainset,intestset,kin,infold);
				intrainset->normalize(intestset,par.useNormLimit);//modify data clones
			}
			getB(intrainset);//needed in getRankFromSet
			//todo: if bias removing is activated, this has to be done within each cv-training set
			hypnr=0;
			Qreset=true;
			Areset=true;
			for(nci=0;nci<nc;nci++)
			{//descending direction
				par.C=cc2-ccd*nci;
				if(par.C<=0) par.C=(ftype)1e-30;
				Areset=true;//epsilon-decrease causes little changes of model
				for(nei=0;nei<ne;nei++)
				{//descending direction
					int tc;
					par.epsi=ee2-eed*nei;
	//				if(par.epsi<=0) par.epsi=(ftype)1e-30;
					//epsilon may be negative for resetting it in smo leading to 1 support vector
					inrank[hypnr].mEpsilon=par.epsi;
					inrank[hypnr].mC=par.C;
					if(!inrank[hypnr].mDeletedByFlimit)// && (hypnr<nc || !inrank[hypnr-nc].mDeletedByFlimit))
					{//none of the last cv-trainings was deleted caused by too low C, so dont ignore current hyperparameter
						notimeout=getRankFromSet(intrainset,Qreset,Areset);//get features into psvm.rank
						notimeoutall=notimeoutall || notimeout;
						inrank[hypnr].mEpsilon=par.epsi;//for epsi=-1 psvm calculated new epsi=maxepsi leading to >=1 supportvector
						if(rank.mDeletedByFlimit) inrank[hypnr].mDeletedByFlimit=true;
						if(rank.mFeat.size()>0) res->maxFeat=max(res->maxFeat,(double)fabs(rank.mFeat.front().a));
					}else{//one of the last cv-trainings was deleted caused by too low C,
						//so ignore current hyperparameters and generate default model zero
						rank.mFeat.clear();
						rank.mDeletedByFlimit=true;
						rank.mDim=intrainset->mDim;
					}
					{//use calculated model to predict intest
						inrank[hypnr].collectFrom(&rank,par.flimit);//in crossvalidation mode(infold!=1), inrank.mrFeat content is not valid
						tc=rank.mFeat.size();
						if(par.flimit>=0) tc=min(tc,par.flimit);
						{//test with intestset, which may be same as intrainset!
							if(memYy<intestset->mN){
								if(yy) delete[] yy;
								yy=new ftype[intestset->mN];
								memYy=intestset->mN;
							}
							getY(yy,intestset,par.flimit,false);//no flimit,no more normalization
//							getAddErrors(yy,intestset,&inerror_d[hypnr],&inerror_i[hypnr],&mse,&rate);
							getAddErrors(yy,intestset,&inerror_d[hypnr]/*,&inerror_i[hypnr]*/,&mse,&rate,&inerror_itp[hypnr],&inerror_itn[hypnr],&inerror_ifp[hypnr],&inerror_ifn[hypnr]);
							if(outfold!=1) minfo("ocv %d/%d, ",kout+1,outfold);if(infold!=1) minfo("icv %d/%d, ",kin+1,infold);
							minfo("epsilon: %g, C: %g, feat: %d, regression MSE: %g \n",par.epsi,par.C,tc,mse);
							if(outfold!=1) minfo("ocv %d/%d, ",kout+1,outfold);if(infold!=1) minfo("icv %d/%d, ",kin+1,infold);
							minfo("epsilon: %g, C: %g, feat: %d, classificn err: %g \n",par.epsi,par.C,tc,rate);
							inerror_n[hypnr]+=intestset->mN;
							//todo: collect statistics of predictions (which objects get wrong predictions)
						}
					}
					hypnr++;
					if(!par.doretrain) Qreset=false;//saves smocalculations for same data subset
	//				Areset=false;smo.mMaxepsi=par.epsi;
					//Areset is not tested, remains true
				}//epsilon
			}//c
			if(infold>1)minfo("inner crossvalidation cycle %d of %d ready\n",kin+1,infold);
		}//inner cv
		//result:
		//inerror and inrank = errors and features for all hyperparameters
		//inerror is either from generalization(CV) or from training

		//determine best hyperparameters for best MSE(regression) and error rate(classification)
		bestparam_d=-1;//regression
		bestparam_i=-1;//classification
		besterror_d=0;//regression
		besterror_i=0;//classification
	    for(i=0;i<nce;i++)
		{
			double ed,ei;
			{//problem: if bounded features > flimit, no model exists, so which is the best?
				//here: if no model exists, use the 'nomodel-error' (alpha=0)
				inerror_i[i]=getCError(inerror_itp[i],inerror_itn[i],inerror_ifp[i],inerror_ifn[i]);
				ed=inerror_d[i]/inerror_n[i];//regression
				ei=inerror_i[i];///inerror_n[i];//classification
				//use the better error
				if(bestparam_d<0 || besterror_d>ed){besterror_d=ed;bestparam_d=i;}
				if(bestparam_i<0 || besterror_i>ei){besterror_i=ei;bestparam_i=i;}
				//equal errors: use less features
				if(besterror_d==ed && inrank[bestparam_d].mCollectedFeatures>inrank[i].mCollectedFeatures){besterror_d=ed;bestparam_d=i;}
				if(besterror_i==ei && inrank[bestparam_i].mCollectedFeatures>inrank[i].mCollectedFeatures){besterror_i=ei;bestparam_i=i;}//????collectedfeatures are limited flimit
				//equal error, equal features: use better regression error (for classification)
				if(besterror_i==ei && inrank[bestparam_i].mCollectedFeatures==inrank[i].mCollectedFeatures && ed<=inerror_d[bestparam_i]/inerror_n[bestparam_i]){besterror_i=ei;bestparam_i=i;}
				//equal error, equal features: use lowest Hyperparameters (otherwise C sometimes rises to infinity), for classification see last line
				if(besterror_d==ed && inrank[bestparam_d].mCollectedFeatures==inrank[i].mCollectedFeatures && !inrank[i].mDeletedByFlimit){besterror_d=ed;bestparam_d=i;}
				if(outfold==1){//without outer loop: give hyperparameter vs error graph
					res->ms[i].c=inrank[i].mC;
					res->ms[i].e=inrank[i].mEpsilon;
					res->ms[i].eec=ei;//classification
					res->ms[i].eer=ed;//regression
				}
			}
		}
		//result:
		//besterror,bestparam = indices for inerror,inrank
		minfo("outer loop %d of %d: regression: epsi %g C: %g feat: %d error %g\n",kout+1,outfold,inrank[bestparam_d].mEpsilon,inrank[bestparam_d].mC,inrank[bestparam_d].mCollectedFeatures/inrank[bestparam_d].mCollectedSets,besterror_d);
		minfo("outer loop %d of %d: classifica: epsi %g C: %g feat: %d error %g\n",kout+1,outfold,inrank[bestparam_i].mEpsilon,inrank[bestparam_i].mC,inrank[bestparam_i].mCollectedFeatures/inrank[bestparam_i].mCollectedSets,besterror_i);

		//out+in :: out_=tile(set)					in_=tile(out) norm int+inp, \in\ norm outt,outp \out\ \ret\ in_=tile(set), norm(int+inp) \in\ np=norm(set) \out\
		//out    :: out_=tile(set) norm outt+outp	in=outt						\in\				\out\ \ret\ in=set, np=norm(set) \in\
		//in     :: out=set							in_=tile(out) norm int+inp  \in\ np=norm(set)	\out\
		//       :: out=set							in=out	np=norm(in)         \in\ .
		if(infold==1){
			if(outfold==1)
			{//outtest==outtrain, outfold==1, infold==1
				//no more testing, model complete
				//no retraining necessary

				if(getmodel && nce>1)
				{//copy model with best training error into this.rank
					if(par.doregress)//regression is default, if not explicitely turned off
						inrank[bestparam_d].putback(&rank);//give best model to psvm
					else
						inrank[bestparam_i].putback(&rank);//
				}
/*
				int kkk;
				kkk=(int)rank.mFeat.size();
				if(par.flimit>0) kkk=min(kkk,par.flimit);
				minfo("feature ranking\n");
				for(i=0;i<kkk;i++)
				{	//printf("feature index %d : %d\n",i+1,rank.mFeat[i].dimRef);
					//minfo("feature index %d : %d\n",i+1,rank.mFeat[i].dimRef);
	//				xinfo("%d\n",rank.mFeat[i].dimRef+1);
				}
				*/
				minfo("no crossvalidation, all calculations accomplished!\n");
				if(par.doregress)
				{
					//Training error only - no Generalization results!
					res->reg_Tnf=inrank[bestparam_d].mCollectedFeatures;
					res->reg_TError=besterror_d;
					res->reg_Te=inrank[bestparam_d].mEpsilon;
					res->reg_Tc=inrank[bestparam_d].mC;
//					if(par.flimit>=0) res->reg_Tnf=min(par.flimit,res->reg_Tnf);
				}
				if(par.doclass)
				{
					//Training error only - no Generalization results!
					res->cla_Tnf=inrank[bestparam_i].mCollectedFeatures;
					res->cla_TError=besterror_i;
					res->cla_Ttp=inerror_itp[bestparam_i];
					res->cla_Ttn=inerror_itn[bestparam_i];
					res->cla_Tfp=inerror_ifp[bestparam_i];
					res->cla_Tfn=inerror_ifn[bestparam_i];
					res->cla_Te=inrank[bestparam_i].mEpsilon;
					res->cla_Tc=inrank[bestparam_i].mC;
//					if(par.flimit>=0) res->cla_Tnf=min(par.flimit,res->cla_Tnf);
				}
				//all final model components ready!
				//o0i0 result: b,np, inrank, inerror, besterror, bestparam, outerror -> Training error only
			}else{//outfold only
		//out+in :: out_=tile(set)					in_=tile(out) norm int+inp, \in\ norm outt,outp \out\ \ret\ in_=tile(set), norm(int+inp) \in\ np=norm(set) \out\
		//out    :: out_=tile(set) norm outt+outp	in=outt						\in\				\out\ \ret\ in=set, np=norm(set) \in\
		//in     :: out=set							in_=tile(out) norm int+inp  \in\ np=norm(set)	\out\
		//       :: out=set							in=out	np=norm(in)         \in\ .
				//mode -crossval 1 -1
				//makes sense, if number of features is restricted (-flimit 5)
				//for each outer step: use best hyperparameters for training, test the generalization
				//outrank_d, outerror_d
				//outrank_i, outerror_i
				//outer retraining necessary
				if(par.doregress){
//					double tc;
					minfo("outer crossval - regression: using model with best training error for testing\n");
					inrank[bestparam_d].putback(&rank);//no inner cv: inrank contains inner model
					outrank_d.collectFrom(&rank,par.flimit);//collect best models does not work
					outrank_d.mEpsilon+=inrank[bestparam_d].mEpsilon;//mean
					outrank_d.mC+=inrank[bestparam_d].mC;//mean
//					tc=inrank[bestparam_d].mCollectedFeatures;
//					if(par.flimit>=0) tc=min(par.flimit,tc);
//					outrank_d.mCollectedFeatures+=tc;//mean
					{//test outer testset 
						if(memYy<outtestset->mN){
							if(yy) delete[] yy;
							yy=new ftype[outtestset->mN];
							memYy=outtestset->mN;
						}
						getY(yy,outtestset,par.flimit,false);//no flimit,no more normalization
						dummy=0;
						getAddErrors(yy,outtestset,&outerror_d,/*&dummy,*/&mse,&rate,&outerror_itp,&outerror_itn,&outerror_ifp,&outerror_ifn);
					}
					if(bestparam_d==bestparam_i && par.doclass)
					{
						minfo("outer crossval - classifica: using same model for testing\n");
//						outerror_i+=dummy;
						outrank_i.collectFrom(&rank,par.flimit);
						outrank_i.mEpsilon+=inrank[bestparam_d].mEpsilon;
						outrank_i.mC+=inrank[bestparam_d].mC;
//						outrank_i.mCollectedFeatures+=tc;
/*						outrank_itp+=inerror_itp;
						outrank_itn+=inerror_ifp;
						outrank_ifp+=inerror_itn;
						outrank_ifn+=inerror_ifn;
*/
					}
				}
				if(par.doclass && !(bestparam_d==bestparam_i && par.doregress))
				{
//					double tc;
					minfo("outer crossval - classifica: using model with best training error for testing\n");
					inrank[bestparam_i].putback(&rank);
					outrank_i.collectFrom(&rank,par.flimit);
					outrank_i.mEpsilon+=inrank[bestparam_i].mEpsilon;
					outrank_i.mC+=inrank[bestparam_i].mC;
//					tc=inrank[bestparam_i].mCollectedFeatures;
//					if(par.flimit>=0) tc=min(par.flimit,tc);
//					outrank_i.mCollectedFeatures+=tc;//mean
					{//test outer testset
						if(memYy<outtestset->mN){
							if(yy) delete[] yy;
							yy=new ftype[outtestset->mN];
							memYy=outtestset->mN;
						}
						getY(yy,outtestset,par.flimit,false);//no flimit,no more normalization
						getAddErrors(yy,outtestset,&dummy,/*&outerror_i,*/&mse,&rate,&outerror_itp,&outerror_itn,&outerror_ifp,&outerror_ifn);
/*						printf("outer loop cycle %d error mse %f rate %f\n",kout,mse,rate);
						int kkk;
						kkk=(int)rank.mFeat.size();
						if(par.flimit>0) kkk=min(kkk,par.flimit);
						minfo("feature ranking\n");
						for(i=0;i<kkk;i++)
						{	//printf("feature index %d : %d\n",i+1,rank.mFeat[i].dimRef);
							//minfo("feature index %d : %d\n",i+1,rank.mFeat[i].dimRef);
			//				xinfo("%d\n",rank.mFeat[i].dimRef+1);
						}
						*/
					}
				}
				//result: outerror -> Generalization error only, without bias!!
				//model and training error can be determined after outer retraining
			}
		}else{//infold used : inner crossvalidation accomplished - retraining necessary
		//out+in :: out_=tile(set)					in_=tile(out) norm int+inp, \in\ norm outt,outp \out\ \ret\ in_=tile(set), norm(int+inp) \in\ np=norm(set) \out\
		//out    :: out_=tile(set) norm outt+outp	in=outt						\in\				\out\ \ret\ in=set, np=norm(set) \in\
		//in     :: out=set							in_=tile(out) norm int+inp  \in\ np=norm(set)	\out\
		//       :: out=set							in=out	np=norm(in)         \in\ .
			//we have :inrank models, best hyperparameters, best errors
			//we need : model based on outtrainset
			if(outfold==1)//infold only
			{
				if(par.doregress)
				{
					//results from inner crossvalidation, Generalization error error!
					res->reg_GError=besterror_d;
					res->reg_Ge=inrank[bestparam_d].mEpsilon;//mean value!!
					res->reg_Gc=inrank[bestparam_d].mC;//mean value!!
					res->reg_Gnf=inrank[bestparam_d].mCollectedFeatures/inrank[bestparam_d].mCollectedSets;
//					if(par.flimit>=0) res->reg_Gnf=min(par.flimit,res->reg_Gnf);
				}
				if(par.doclass)
				{
					//results from outer crossvalidation, Generalization error error!
					res->cla_GError=besterror_i;
					res->cla_Gtp=inerror_itp[bestparam_i];
					res->cla_Gtn=inerror_itn[bestparam_i];
					res->cla_Gfp=inerror_ifp[bestparam_i];
					res->cla_Gfn=inerror_ifn[bestparam_i];
					res->cla_Ge=inrank[bestparam_i].mEpsilon;//mean value!!
					res->cla_Gc=inrank[bestparam_i].mC;//mean value!!
					res->cla_Gnf=inrank[bestparam_i].mCollectedFeatures/inrank[bestparam_i].mCollectedSets;
//					if(par.flimit>=0) res->cla_Gnf=min(par.flimit,res->cla_Gnf);
				}
				if(getmodel)
				{//use selected hyperparameters for retraining the complete set
					//result: inrank, inerror, besterror, bestparam -> Generalization error, biased by hxperparameterselection!!
					//result: outerror, outrank -> Retraining error, with selected hyperparameters!!
					np.getFromSet(set);
					np.putOnSet(set,false);
					getB(set);//part of retraining, new dataset outtrainset
					if(par.doregress)
					{
						minfo("regression: retraining with determined hyperparameters\n");
						par.epsi=inrank[bestparam_d].mEpsilon;
						par.C=inrank[bestparam_d].mC;
						getRankFromSet(set);
						notimeoutall=notimeoutall || notimeout;
                        if(rank.mFeat.size()>0) res->maxFeat=max(res->maxFeat,(double)fabs(rank.mFeat.front().a));
						res->reg_Tnf=rank.mFeat.size();
						if(par.flimit>=0) res->reg_Tnf=min(par.flimit,res->reg_Tnf);
						res->reg_Te=par.epsi;
						res->reg_Tc=par.C;

						{//test with outtestset, which may be same as outtrainset!
							if(memYy<set->mN){
								if(yy) delete[] yy;
								yy=new ftype[set->mN];
								memYy=set->mN;
							}
							getY(yy,set,par.flimit,false);//no flimit,no more normalization
							dummy=0;
							getAddErrors(yy,set,&dummy1,/*&dummy,*/&mse,&rate,&res->cla_Ttp,&res->cla_Ttn,&res->cla_Tfp,&res->cla_Tfn);
							minfo("retrained reg: epsi: %f, C: %f, features: %d, error %g\n",par.epsi,par.C,res->reg_Tnf,mse);
						}
						//results from retraining, Training error!
						res->reg_TError=mse;
						if(par.doclass && bestparam_d==bestparam_i){
							minfo("classification: same hyperparameters");
							minfo("retrained cls: epsi: %f, C: %f, features: %d, error %g\n",par.epsi,par.C,res->reg_Tnf,rate);
							res->cla_Tnf=res->reg_Tnf;
							res->cla_TError=rate;
							res->cla_Te=par.epsi;
							res->cla_Tc=par.C;
						}
					}
					if(par.doclass && !(bestparam_d==bestparam_i && par.doregress))
					{
						minfo("classification: retraining with determined hyperparameters\n");
						par.epsi=inrank[bestparam_i].mEpsilon;
						par.C=inrank[bestparam_i].mC;
						getRankFromSet(set);
						notimeoutall=notimeoutall || notimeout;
						if(rank.mFeat.size()>0) res->maxFeat=max(res->maxFeat,(double)fabs(rank.mFeat.front().a));
						res->cla_Tnf=rank.mFeat.size();
						if(par.flimit>=0) res->cla_Tnf=min(par.flimit,res->cla_Tnf);
						res->cla_Te=par.epsi;
						res->cla_Tc=par.C;
						{//test with outtestset, which may be same as outtrainset!
							if(memYy<set->mN){
								if(yy) delete[] yy;
								yy=new ftype[set->mN];
								memYy=set->mN;
							}
							getY(yy,set,par.flimit,false);//no flimit,no more normalization
							getAddErrors(yy,set,&dummy,/*&dummy1,*/&mse,&rate,&res->cla_Ttp,&res->cla_Ttn,&res->cla_Tfp,&res->cla_Tfn);
							minfo("retrained cls: epsi: %f, C: %f, features: %d, error %g\n",par.epsi,par.C,res->cla_Tnf,rate);
						}
						//results from retraining, Training error!
						res->cla_TError=rate;
					}//if classify
					//all final model components ready!
				}//if getmodel
			}else//infold+outfold
		//out+in :: out_=tile(set)					in_=tile(out) norm int+inp, \in\ norm outt,outp \out\ \ret\ in_=tile(set), norm(int+inp) \in\ np=norm(set) \out\
		//out    :: out_=tile(set) norm outt+outp	in=outt						\in\				\out\ \ret\ in=set, np=norm(set) \in\
		//in     :: out=set							in_=tile(out) norm int+inp  \in\ np=norm(set)	\out\
		//       :: out=set							in=out	np=norm(in)         \in\ .
			{//for every(this) outfold step, we got now hyperparameters and use them now for retraining and testing the outer testset (measures the generalization of psvm&hyperparameterselection)
				//collect results of outer cycles into
				//outrank_d, outerror_d
				//outrank_i, outerror_i
				//result: outerror -> Generalization error only, without bias!!
				//outer retraining necessary
				int tc;
				outtrainset->normalize(outtestset,par.useNormLimit);//modify data clones
				getB(outtrainset);//part of retraining, new dataset outtrainset
				if(par.doregress)
				{
					minfo("outer crossvalidation - regression: retraining with determined hyperparameters\n");
					par.epsi=inrank[bestparam_d].mEpsilon;
					par.C=inrank[bestparam_d].mC;
					getRankFromSet(outtrainset);
					notimeoutall=notimeoutall || notimeout;
					if(rank.mDeletedByFlimit) outrank_d.mDeletedByFlimit=true;
					if(rank.mFeat.size()>0) res->maxFeat=max(res->maxFeat,(double)fabs(rank.mFeat.front().a));
					outrank_d.collectFrom(&rank,par.flimit);
					outrank_d.mEpsilon+=par.epsi;
					outrank_d.mC+=par.C;
					tc=rank.mFeat.size();
					if(par.flimit>=0) tc=min(par.flimit,tc);
//					outrank_d.mCollectedFeatures+=tc;//mean
					{//test with outtestset
						if(memYy<outtestset->mN){
							if(yy) delete[] yy;
							yy=new ftype[outtestset->mN];
							memYy=outtestset->mN;
						}
						getY(yy,outtestset,par.flimit,false);//no flimit,no more normalization
						dummy=0;
						getAddErrors(yy,outtestset,&outerror_d/*,&dummy*/,&mse,&rate,&outerror_itp,&outerror_itn,&outerror_ifp,&outerror_ifn);
						minfo("retrained reg: epsi: %f, C: %f, features: %d, error %g\n",inrank[bestparam_d].mEpsilon,inrank[bestparam_d].mC,tc,mse);
					}
					if(par.doclass && bestparam_d==bestparam_i){
						minfo("outer crossvalidation - classifica: same hyperparameters");
//						outerror_i+=dummy;
						outrank_i.mEpsilon+=par.epsi;
						outrank_i.mC+=par.C;
						outrank_i.collectFrom(&rank,par.flimit);
						minfo("retrained cls: epsi: %f, C: %f, features: %d, error %g\n",inrank[bestparam_i].mEpsilon,inrank[bestparam_i].mC,tc,rate);
					}
				}
				if(par.doclass && !(bestparam_d==bestparam_i && par.doregress))
				{
					minfo("outer crossvalidation - classifica: retraining with determined hyperparameters\n");
					par.epsi=inrank[bestparam_i].mEpsilon;
					par.C=inrank[bestparam_i].mC;
					getRankFromSet(outtrainset);
					notimeoutall=notimeoutall || notimeout;
					if(rank.mDeletedByFlimit) outrank_d.mDeletedByFlimit=true;
					if(rank.mFeat.size()>0) res->maxFeat=max(res->maxFeat,(double)fabs(rank.mFeat.front().a));
					outrank_i.collectFrom(&rank,par.flimit);
					outrank_i.mEpsilon+=par.epsi;
					outrank_i.mC+=par.C;
					tc=rank.mFeat.size();
					if(par.flimit>=0) tc=min(par.flimit,tc);
//					outrank_i.mCollectedFeatures+=tc;//mean
					{//test with outtestset, which may be same as outtrainset!
						if(memYy<outtestset->mN){
							if(yy) delete[] yy;
							yy=new ftype[outtestset->mN];
							memYy=outtestset->mN;
						}
						getY(yy,outtestset,par.flimit,false);//no flimit,no more normalization
						dummy=0;
						getAddErrors(yy,outtestset,&dummy/*,&outerror_i*/,&mse,&rate,&outerror_itp,&outerror_itn,&outerror_ifp,&outerror_ifn);
						minfo("retrained cls: epsi: %f, C: %f, features: %d, error %g\n",inrank[bestparam_i].mEpsilon,inrank[bestparam_i].mC,tc,rate);
					}
				}//classifi
			}//if outfold
#if 0
			if(par.doregress)
			{//1.rankretrain regression!!!
				minfo("outer crossvalidation - regression: retraining with determined hyperparameters\n");
				par.epsi=inrank[bestparam_d].mEpsilon;
				par.C=inrank[bestparam_d].mC;
				inrank[bestparam_d].extractFromTo(outtrainset,extractionset,par.flimit);//collect new dataset selection based on feature ranking
				getRankFromSet(extractionset);
				inrank[bestparam_d].transformBack(&rank);//transforms features in rank according to unreduced dataset
				outrankr_d.collectFrom(&rank,par.flimit);
				outrankr_d.mEpsilon+=par.epsi;
				outrankr_d.mC+=par.C;
				{//test with outtestset, which may be same as outtrainset!
					if(memYy<outtestset->mN){
						if(yy) delete[] yy;
						yy=new ftype[outtestset->mN];
						memYy=outtestset->mN;
					}
					getY(yy,outtestset,par.flimit,false);//no flimit,no more normalization
					dummy=0;
					getAddErrors(yy,outtestset,&outerrorr_d,&dummy,&mse,&rate);
					minfo("retrained rank reg: epsi: %f, C: %f, features: %d, error %g\n",inrank[bestparam_d].mEpsilon,inrank[bestparam_d].mC,rank.mFeat.size(),mse);
				}
				if(par.doclass && bestparam_d==bestparam_i)
				{//regression and classification is the same
					minfo("outer crossvalidation - classifica: same hyperparameters");
					outrankr_i.collectFrom(&rank,par.flimit);
					outrankr_i.mEpsilon+=par.epsi;
					outrankr_i.mC+=par.C;
					outerrorr_i+=dummy;
					minfo("retrained rank cls: epsi: %f, C: %f, features: %d, error %g\n",inrank[bestparam_i].mEpsilon,inrank[bestparam_i].mC,rank.mFeat.size(),rate);
				}
			}
			if(par.doclass && !(bestparam_d==bestparam_i && par.doregress))
			{//reduced retraining for classification
				//2.rankretrain classification!!!
				minfo("outer crossvalidation - classifica: retraining with determined hyperparameters\n");
				par.epsi=inrank[bestparam_i].mEpsilon;
				par.C=inrank[bestparam_i].mC;
				inrank[bestparam_i].extractFromTo(outtrainset,extractionset,par.flimit);
				getRankFromSet(extractionset);
				inrank[bestparam_i].transformBack(&rank);//transforms features in rank according to unreduced dataset
				outrankr_i.collectFrom(&rank,par.flimit);
				outrankr_i.mEpsilon+=par.epsi;
				outrankr_i.mC+=par.C;
				{//test with outtestset, which may be same as outtrainset!
					if(memYy<outtestset->mN){
						if(yy) delete[] yy;
						yy=new ftype[outtestset->mN];
						memYy=outtestset->mN;
					}
					getY(yy,outtestset,par.flimit,false);//no flimit,no more normalization
					getAddErrors(yy,outtestset,&dummy,&outerrorr_i,&mse,&rate);
					minfo("retrained rank cls: epsi: %f, C: %f, features: %d, error %g\n",inrank[bestparam_i].mEpsilon,inrank[bestparam_i].mC,rank.mFeat.size(),rate);
				}
			}
#endif
		}//if infold
		outerror_n+=outtestset->mN;
		if(outfold!=1)minfo("outer crossvalidation cycle %d of %d ready\n",kout+1,outfold);
	}//outer crossval
	if(intestset!=intrainset) if(intestset) delete intestset; intestset=NULL;
	if(intrainset!=outtrainset) if(intrainset) delete intrainset; intrainset=NULL;
	if(outtestset!=outtrainset) if(outtestset) delete outtestset; outtestset=NULL;
	if(outtrainset!=set) if(outtrainset) delete outtrainset; outtrainset=NULL;
	if(outfold!=1){
//		outerrorr_d/=outerror_n;
//		outerrorr_i/=outerror_n;
		outerror_i=getCError(outerror_itp,outerror_itn,outerror_ifp,outerror_ifn);
		outerror_d/=outerror_n;//from now, outerror is relative
		//outerror_i/=outerror_n;
//		minfo("ranking regression: predicted generalization error %f \nranking classification: predicted generalization error %f \nregression: predicted generalization error %f \nclassification: predicted generalization error %f \n",outerrorr_d,outerrorr_i,outerror_d,outerror_i);
		minfo("result outer loop:\nregression: error %f \nclassification: error %f \n",outerror_d,outerror_i);
	}
	//RETRAINING1
	if(outfold!=1 && getmodel)
	{//outfold requires a complete retraining with modelselection phase
		infold=par.infold;
		if(infold>set->mN||infold<0) infold=set->mN;
		if(infold==0) infold=1;
		if(par.infold!=1) {intrainset=new Dataset;intestset=new Dataset;}
		//remember one ranking per hyperparametercombi
		//remember errors per hyperparametercombi
		//toDo: problem: saving all models during hyperparameterselection is very expensive in memory!!!  
		//clear rankings and errors
		for(i=0;i<nce;i++){
			inrank[i].clear(set->mDim);//holds features for all subsets
			inerror_d[i]=0;
			inerror_i[i]=0;
			inerror_n[i]=0;
			inerror_itp[i]=0;
			inerror_itn[i]=0;
			inerror_ifp[i]=0;
			inerror_ifn[i]=0;
		}
		//do combined crossvalidation with parameterselection
		for(kin=0;kin<infold;kin++)
		{
		//out+in :: out_=tile(set)					in_=tile(out) norm int+inp, \in\ norm outt,outp \out\ \ret\ in_=tile(set), norm(int+inp) \in\ np=norm(set) \out\
		//out    :: out_=tile(set) norm outt+outp	in=outt						\in\				\out\ \ret\ in=set, np=norm(set) \in\
		//in     :: out=set							in_=tile(out) norm int+inp  \in\ np=norm(set)	\out\
		//       :: out=set							in=out	np=norm(in)         \in\ .
			if(infold==1) {
				intrainset=set;
				intestset=set;
				np.getFromSet(set);
				np.putOnSet(set,false);
			}else {
				set->tileTo(intrainset,intestset,kin,infold);
				intrainset->normalize(intestset,par.useNormLimit);//modify data clones
			}
			getB(intrainset);//needed in getRankFromSet
			//todo: if bias removing is activated, this has to be done within each cv-training set
			hypnr=0;
			Qreset=true;
			Areset=true;
			for(nci=0;nci<nc;nci++)
			{//descending direction
				par.C=cc2-ccd*nci;
				if(par.C<=0) par.C=(ftype)1e-30;
				Areset=true;//epsilon-decrease causes little changes of model
				for(nei=0;nei<ne;nei++)
				{//descending direction
					par.epsi=ee2-eed*nei;
	//				if(par.epsi<=0) par.epsi=(ftype)1e-30;
					//epsilon may be negative for resetting it in smo leading to 1 support vector
					inrank[hypnr].mEpsilon=par.epsi;
					inrank[hypnr].mC=par.C;
					if(!inrank[hypnr].mDeletedByFlimit)// && (hypnr<nc || !inrank[hypnr-nc].mDeletedByFlimit))
					{//last training was not deleted caused by too high C, so dont ignore all lower Cs
						notimeout=getRankFromSet(intrainset,Qreset,Areset);//get features into psvm.rank
						notimeoutall=notimeoutall || notimeout;
						inrank[hypnr].mEpsilon=par.epsi;//for epsi=-1 psvm calculated new epsi=maxepsi leading to >=1 supportvector
						if(rank.mDeletedByFlimit) inrank[hypnr].mDeletedByFlimit=true;
						if(rank.mFeat.size()>0) res->maxFeat=max(res->maxFeat,(double)fabs(rank.mFeat.front().a));
					}else{//ignore hyperparameter nei and generate default model zero
						rank.mFeat.clear();
						rank.mDeletedByFlimit=true;
						rank.mDim=intrainset->mDim;
					}
					{//use calculated model to predict intest
						inrank[hypnr].collectFrom(&rank,par.flimit);//in crossvalidation mode, this works currently not correct
						{//test with intestset, which may be same as intrainset!
							if(memYy<intestset->mN){
								if(yy) delete[] yy;
								yy=new ftype[intestset->mN];
								memYy=intestset->mN;
							}
							getY(yy,intestset,par.flimit,false);//no flimit,no more normalization
							getAddErrors(yy,intestset,&inerror_d[hypnr]/*,&inerror_i[hypnr]*/,&mse,&rate,&inerror_itp[hypnr],&inerror_itn[hypnr],&inerror_ifp[hypnr],&inerror_ifn[hypnr]);
							if(infold!=1) minfo("retraining1 icv %d/%d, ",kin+1,infold);
							minfo("epsilon: %g, C: %g, regression MSE: %g \n",par.epsi,par.C,mse);
							if(infold!=1) minfo("retraining1 icv %d/%d, ",kin+1,infold);
							minfo("epsilon: %g, C: %g, classificn err: %g \n",par.epsi,par.C,rate);
							inerror_n[hypnr]+=intestset->mN;
							//todo: collect statistics of predictions (which objects get wrong predictions)
						}
					}
					hypnr++;
					if(!par.doretrain) Qreset=false;//saves smocalculations for same data subset
	//				Areset=false;smo.mMaxepsi=par.epsi;
					//reset is not tested, remains true
				}//epsilon
			}//c
			if(infold>1)minfo("inner crossvalidation cycle %d of %d ready\n",kin+1,infold);
		}//inner cv
		//result:
		//inerror and inrank = errors and features for all hyperparameters
		//inerror is either from generalization(CV) or from training

		//determine best hyperparameters for best MSE(regression) and error rate(classification)
		bestparam_d=-1;//regression
		bestparam_i=-1;//classification
		besterror_d=0;//regression
		besterror_i=0;//classification
	    for(i=0;i<nce;i++)
		{
			double ed,ei;
			{//problem: if bounded features > flimit, no model exists, so which is the best?
				//here: if no model exists, use the 'nomodel-error' (alpha=0)
				inerror_i[i]=getCError(inerror_itp[i],inerror_itn[i],inerror_ifp[i],inerror_ifn[i]);
				ed=inerror_d[i]/inerror_n[i];//regression
				ei=inerror_i[i];///inerror_n[i];//classification
				//use the better error
				if(bestparam_d<0 || besterror_d>ed){besterror_d=ed;bestparam_d=i;}
				if(bestparam_i<0 || besterror_i>ei){besterror_i=ei;bestparam_i=i;}
				//equal errors: use less features
				if(besterror_d==ed && inrank[bestparam_d].mCollectedFeatures>inrank[i].mCollectedFeatures){besterror_d=ed;bestparam_d=i;}
				if(besterror_i==ei && inrank[bestparam_i].mCollectedFeatures>inrank[i].mCollectedFeatures){besterror_i=ei;bestparam_i=i;}
				//equal error, equal features: use better regression error (for classification)
				if(besterror_i==ei && inrank[bestparam_i].mCollectedFeatures==inrank[i].mCollectedFeatures && ed<=inerror_d[bestparam_i]/inerror_n[bestparam_i]){besterror_i=ei;bestparam_i=i;}
				//equal error, equal features: use lowest Hyperparameters (otherwise C sometimes rises to infinity), for classification see last line
				if(besterror_d==ed && inrank[bestparam_d].mCollectedFeatures==inrank[i].mCollectedFeatures && !inrank[i].mDeletedByFlimit){besterror_d=ed;bestparam_d=i;}
				//give hyperparameter vs error graph
					res->ms[i].c=inrank[i].mC;
					res->ms[i].e=inrank[i].mEpsilon;
					res->ms[i].eec=ei;//classification
					res->ms[i].eer=ed;//regression
			}
		}
		//result:
		//besterror,bestparam = indices for inerror,inrank
		minfo("retraining1 : regression: epsi %g C: %g error %g\n",inrank[bestparam_d].mEpsilon,inrank[bestparam_d].mC,besterror_d);
		minfo("retraining1 : classifica: epsi %g C: %g error %g\n",inrank[bestparam_i].mEpsilon,inrank[bestparam_i].mC,besterror_i);

		if(infold==1){
		//out+in :: out_=tile(set)					in_=tile(out) norm int+inp, \in\ norm outt,outp \out\ \ret\ in_=tile(set), norm(int+inp) \in\ np=norm(set) \out\
		//out    :: out_=tile(set) norm outt+outp	in=outt						\in\				\out\ \ret\ in=set, np=norm(set) \in\
		//in     :: out=set							in_=tile(out) norm int+inp  \in\ np=norm(set)	\out\
		//       :: out=set							in=out	np=norm(in)         \in\ .
			//no more testing, model complete
			//no retraining necessary

			//inrank[bestparam_d]
			//inrank[bestparam_r]
			//inerror_d[bestparam_d]/inerror_n[]
			//inerror_i[bestparam_i]/inerror_n[]
			if(nce>1)
			{//copy model with best training error into this.rank
//		        res->reg_mrank.copyFrom(&inrank[bestparam_d]);
//				res->cla_mrank.copyFrom(&inrank[bestparam_i]);
				if(par.doregress)//regression is default, if not explicitely turned off
					inrank[bestparam_d].putback(&rank);//give best model to psvm
				else
					inrank[bestparam_i].putback(&rank);//
			}
/*			int kkk;
			kkk=(int)rank.mFeat.size();
			if(par.flimit>0) kkk=min(kkk,par.flimit);
			minfo("feature ranking\n");
			for(i=0;i<kkk;i++)
			{	//printf("feature index %d : %d\n",i+1,rank.mFeat[i].dimRef);
				//minfo("feature index %d : %d\n",i+1,rank.mFeat[i].dimRef);
//				xinfo("%d\n",rank.mFeat[i].dimRef+1);
			}
*/
			minfo("Final retraining after outer loop ready.");
			//o0i0 result: b,np, inrank, inerror, besterror, bestparam, outerror -> Training error only
		}else{//infold used : inner crossvalidation accomplished - retraining necessary
		//out+in :: out_=tile(set)					in_=tile(out) norm int+inp, \in\ norm outt,outp \out\ \ret\ in_=tile(set), norm(int+inp) \in\ np=norm(set) \out\
		//out    :: out_=tile(set) norm outt+outp	in=outt						\in\				\out\ \ret\ in=set, np=norm(set) \in\
		//in     :: out=set							in_=tile(out) norm int+inp  \in\ np=norm(set)	\out\
		//       :: out=set							in=out	np=norm(in)         \in\ .
			//we have :inrank models, best hyperparameters, best errors
			//we need : model based on outtrainset
			//use selected hyperparameters for retraining the complete set
			int tc;
			np.getFromSet(set);
			np.putOnSet(set,false);
			getB(set);//part of retraining, new dataset outtrainset

			if(par.doregress)
			{
				minfo("retraining2 regression: use determined hyperparameters");
				par.epsi=inrank[bestparam_d].mEpsilon;
				par.C=inrank[bestparam_d].mC;
				getRankFromSet(set);
				notimeoutall=notimeoutall || notimeout;
				if(rank.mFeat.size()>0) res->maxFeat=max(res->maxFeat,(double)fabs(rank.mFeat.front().a));
				tc=rank.mFeat.size();
				if(par.flimit>=0) tc=min(par.flimit,tc);
				res->reg_Tnf=tc;
				res->reg_Te=inrank[bestparam_d].mEpsilon;
				res->reg_Tc=inrank[bestparam_d].mC;
				{//test with outtestset, which may be same as outtrainset!
					if(memYy<set->mN){
						if(yy) delete[] yy;
						yy=new ftype[set->mN];
						memYy=set->mN;
					}
					getY(yy,set,par.flimit,false);//no flimit,no more normalization
					dummy=0;
					getAddErrors(yy,set,&dummy1,/*&dummy,*/&mse,&rate,&res->cla_Ttp,&res->cla_Ttn,&res->cla_Tfp,&res->cla_Tfn);
					minfo("retrained reg: epsi: %f, C: %f, features: %d, error %g\n",par.epsi,par.C,tc,mse);
				}
				res->reg_TError=mse;
				if(par.doclass && bestparam_d==bestparam_i){
					minfo("classification: same hyperparameters");
					minfo("retrained cls: epsi: %f, C: %f, features: %d, error %g\n",par.epsi,par.C,tc,rate);
					res->cla_TError=rate;
					res->cla_Tnf=tc;
					res->cla_Te=inrank[bestparam_i].mEpsilon;
					res->cla_Tc=inrank[bestparam_i].mC;
				}
			}
			if(par.doclass && !(bestparam_d==bestparam_i && par.doregress))
			{
				minfo("retraining2 classification: use determined hyperparameters");
				par.epsi=inrank[bestparam_i].mEpsilon;
				par.C=inrank[bestparam_i].mC;
				getRankFromSet(set);
				notimeoutall=notimeoutall || notimeout;
				if(rank.mFeat.size()>0) res->maxFeat=max(res->maxFeat,(double)fabs(rank.mFeat.front().a));
				tc=rank.mFeat.size();
				if(par.flimit>=0) tc=min(par.flimit,tc);
				{//test with outtestset, which may be same as outtrainset!
					if(memYy<set->mN){
						if(yy) delete[] yy;
						yy=new ftype[set->mN];
						memYy=set->mN;
					}
					getY(yy,set,par.flimit,false);//no flimit,no more normalization
					getAddErrors(yy,set,&dummy/*,&dummy1*/,&mse,&rate,&res->cla_Ttp,&res->cla_Ttn,&res->cla_Tfp,&res->cla_Tfn);
					minfo("retrained cls: epsi: %f, C: %f, features: %d, error %g\n",par.epsi,par.C,tc,rate);
				}
				res->cla_Tnf=tc;
				res->cla_Te=inrank[bestparam_i].mEpsilon;
				res->cla_Tc=inrank[bestparam_i].mC;
				res->cla_TError=rate;
			}//if classify
#if 0
			if(par.doregress)
			{//1.rankretrain regression!!!
				minfo("outer crossvalidation - regression: retraining with determined hyperparameters");
				par.epsi=inrank[bestparam_d].mEpsilon;
				par.C=inrank[bestparam_d].mC;
				inrank[bestparam_d].extractFromTo(outtrainset,extractionset,par.flimit);//collect new dataset selection based on feature ranking
				getRankFromSet(extractionset);
				inrank[bestparam_d].transformBack(&rank);//transforms features in rank according to unreduced dataset
				outrankr_d.collectFrom(&rank,par.flimit);
				outrankr_d.mEpsilon+=par.epsi;
				outrankr_d.mC+=par.C;
				{//test with outtestset, which may be same as outtrainset!
					if(memYy<outtestset->mN){
						if(yy) delete[] yy;
						yy=new ftype[outtestset->mN];
						memYy=outtestset->mN;
					}
					getY(yy,outtestset,par.flimit,false);//no flimit,no more normalization
					dummy=0;
					getAddErrors(yy,outtestset,&outerrorr_d,&dummy,&mse,&rate);
					minfo("retrained rank reg: epsi: %f, C: %f, features: %d, error %g\n",inrank[bestparam_d].mEpsilon,inrank[bestparam_d].mC,rank.mFeat.size(),mse);
				}
				if(par.doclass && bestparam_d==bestparam_i)
				{//regression and classification is the same
					minfo("outer crossvalidation - classification: same hyperparameters");
					outrankr_i.collectFrom(&rank,par.flimit);
					outrankr_i.mEpsilon+=par.epsi;
					outrankr_i.mC+=par.C;
					outerrorr_i+=dummy;
					minfo("retrained rank cls: epsi: %f, C: %f, features: %d, error %g\n",inrank[bestparam_i].mEpsilon,inrank[bestparam_i].mC,rank.mFeat.size(),rate);
				}
			}
			if(par.doclass && !(bestparam_d==bestparam_i && par.doregress))
			{//reduced retraining for classification
				//2.rankretrain classification!!!
				minfo("outer crossvalidation - classification: retraining with determined hyperparameters");
				par.epsi=inrank[bestparam_i].mEpsilon;
				par.C=inrank[bestparam_i].mC;
				inrank[bestparam_i].extractFromTo(outtrainset,extractionset,par.flimit);
				getRankFromSet(extractionset);
				inrank[bestparam_i].transformBack(&rank);//transforms features in rank according to unreduced dataset
				outrankr_i.collectFrom(&rank,par.flimit);
				outrankr_i.mEpsilon+=par.epsi;
				outrankr_i.mC+=par.C;
				{//test with outtestset, which may be same as outtrainset!
					if(memYy<outtestset->mN){
						if(yy) delete[] yy;
						yy=new ftype[outtestset->mN];
						memYy=outtestset->mN;
					}
					getY(yy,outtestset,par.flimit,false);//no flimit,no more normalization
					getAddErrors(yy,outtestset,&dummy,&outerrorr_i,&mse,&rate);
					minfo("retrained rank cls: epsi: %f, C: %f, features: %d, error %g\n",inrank[bestparam_i].mEpsilon,inrank[bestparam_i].mC,rank.mFeat.size(),rate);
				}
			}
#endif
		}//if infold
	}
	//o0i0 result: b,np, inrank, inerror, besterror, bestparam, outerror -> Training error only
	//o1i0 result: outerror -> Generalization error only, without bias!!
	//o1i1 result: outerror -> Generalization error only, without bias!!
	//o0i1 result: inrank, inerror, besterror, bestparam -> Generalization error, biased by hxperparameterselection!!
		 //result: np,b,rank, mse,rate -> Retraining error, with selected hyperparameters!!
	if(outfold!=1){
		if(infold==1){
			//outfold only
			if(par.doregress)
			{
				if(getmodel)
				{
					//results from retraining, Training error!
					res->reg_Tnf=inrank[bestparam_d].mCollectedFeatures;
					res->reg_TError=besterror_d;
					res->reg_Te=inrank[bestparam_d].mEpsilon;
					res->reg_Tc=inrank[bestparam_d].mC;
					if(par.flimit>=0) res->reg_Tnf=min(par.flimit,res->reg_Tnf);
				}
				//results from outer crossvalidation, Generalization error error!
				res->reg_GError=outerror_d;
				res->reg_Ge=outrank_d.mEpsilon/outrank_d.mCollectedSets;//mean value!!
				res->reg_Gc=outrank_d.mC/outrank_d.mCollectedSets;//mean value!!
				res->reg_Gnf=outrank_d.mCollectedFeatures/outrank_d.mCollectedSets;//mean
			}
			if(par.doclass)
			{
				if(getmodel)
				{
					//results from retraining, Training error!
					res->cla_Tnf=inrank[bestparam_i].mCollectedFeatures;
					res->cla_TError=besterror_i;
					res->cla_Ttp=inerror_itp[bestparam_i];
					res->cla_Ttn=inerror_itn[bestparam_i];
					res->cla_Tfp=inerror_ifp[bestparam_i];
					res->cla_Tfn=inerror_ifn[bestparam_i];
					res->cla_Te=inrank[bestparam_i].mEpsilon;
					res->cla_Tc=inrank[bestparam_i].mC;
					if(par.flimit>=0) res->cla_Tnf=min(par.flimit,res->cla_Tnf);
				}
				//results from outer crossvalidation, Generalization error error!
				res->cla_GError=outerror_i;
				res->cla_Gtp=outerror_itp;
				res->cla_Gtn=outerror_itn;
				res->cla_Gfp=outerror_ifp;
				res->cla_Gfn=outerror_ifn;
				res->cla_Ge=outrank_i.mEpsilon/outrank_i.mCollectedSets;//mean value!!
				res->cla_Gc=outrank_i.mC/outrank_i.mCollectedSets;//mean value!!
				res->cla_Gnf=outrank_i.mCollectedFeatures/outrank_i.mCollectedSets;//mean
			}
		}else{
			//infold +outfold
			if(par.doregress)
			{
				if(getmodel)
				{
					//results from retraining, Training error!
//					res->reg_Tnf=inrank[bestparam_d].mCollectedFeatures/inrank[bestparam_d].mCollectedSets;
//					res->reg_TError=besterror_d;
//					res->reg_Te=inrank[bestparam_d].mEpsilon/inrank[bestparam_d].mCollectedSets;
//					res->reg_Tc=inrank[bestparam_d].mC/inrank[bestparam_d].mCollectedSets;
//					if(par.flimit>=0) res->reg_Tnf=min(par.flimit,res->reg_Tnf);
				}
				//results from outer crossvalidation, Generalization error error!
				res->reg_GError=outerror_d;
				res->reg_Ge=outrank_d.mEpsilon/outrank_d.mCollectedSets;//mean value!!
				res->reg_Gc=outrank_d.mC/outrank_d.mCollectedSets;//mean value!!
				res->reg_Gnf=outrank_d.mCollectedFeatures/outrank_d.mCollectedSets;//mean
			}
			if(par.doclass)
			{
				if(getmodel)
				{
					//results from retraining, Training error!
//					res->cla_Tnf=inrank[bestparam_i].mCollectedFeatures/inrank[bestparam_i].mCollectedSets;
//					res->cla_TError=besterror_i;
//					res->cla_Te=inrank[bestparam_i].mEpsilon/inrank[bestparam_i].mCollectedSets;
//					res->cla_Tc=inrank[bestparam_i].mC/inrank[bestparam_i].mCollectedSets;
//					if(par.flimit>=0) res->cla_Tnf=min(par.flimit,res->cla_Tnf);
				}
				//results from outer crossvalidation, Generalization error error!
				res->cla_GError=outerror_i;
				res->cla_Gtp=outerror_itp;
				res->cla_Gtn=outerror_itn;
				res->cla_Gfp=outerror_ifp;
				res->cla_Gfn=outerror_ifn;
				res->cla_Ge=outrank_i.mEpsilon/outrank_i.mCollectedSets;//mean value!!
				res->cla_Gc=outrank_i.mC/outrank_i.mCollectedSets;//mean value!!
				res->cla_Gnf=outrank_i.mCollectedFeatures/outrank_i.mCollectedSets;//mean
			}
		}
	}

	//calculate hyperparameter ranges
	res->c_min=cc1;
	res->c_max=cc2;
	res->e_min=ee1;
	res->e_max=ee2;
/*
	//print messages
	if(nce!=1 && infold!=1)
	{//hyperparameterselection with crossval.
		res->reg_GError=besterror_d;
		res->cla_GError=besterror_i;
		res->cla_misses=(int)(inerror_i[bestparam_i]);//mean number of misses, natural, n datapoints are tested during crossvalidation, same as testing the whole set at once
		minfo("Result after hyperparameterselection with crossvalidation:\n");
		minfo("best reg: C: %g, epsi: %g, mean features: %d, G-MSE %g\n",res->reg_c,res->reg_e,res->reg_nf,res->reg_GError);
		minfo("best cls: C: %g, epsi: %g, mean features: %d, G-error %g\n",res->cla_c,res->cla_e,res->cla_nf,res->cla_GError);
		minfo("best cls: mean accuracy: %g, mean number of missclassified objects: %d of %d\n",1-res->cla_GError,res->cla_misses,set->mN);
		minfo("attention: current model is built from last n-fold subset of the training data\n");
	}else if(nce!=1 && infold==1)
	{//hyperparameterselection over training error
		res->reg_TError=besterror_d;
		res->cla_TError=besterror_i;
		res->cla_misses=(int)inerror_i[bestparam_i];//number of misses
		minfo("Result after hyperparameterselection over training error:\n");
		minfo("best reg: C: %g, epsi: %g, features: %d, T-MSE %g\n",res->reg_c,res->reg_e,res->reg_nf,res->reg_TError);
		minfo("best cls: C: %g, epsi: %g, features: %d, T-error %g\n",res->cla_c,res->cla_e,res->cla_nf,res->cla_TError);
		minfo("best cls: accuracy: %g, number of missclassified objects: %d of %d\n",1-res->cla_TError,res->cla_misses,set->mN);
	}else if(nce==1 && infold!=1)
	{//crossval without modelselection
		res->reg_GError=besterror_d;
		res->cla_GError=besterror_i;
		res->cla_misses=(int)(inerror_i[bestparam_i]);//mean number of misses
		minfo("Result after Crossvalidation:\n");
		minfo("reg: C: %g, epsi: %g, mean features: %d, G-MSE %g\n",res->reg_c,res->reg_e,res->reg_nf,res->reg_GError);
		minfo("cls: C: %g, epsi: %g, mean features: %d, G-error %g\n",res->cla_c,res->cla_e,res->cla_nf,res->cla_GError);
		minfo("cls: mean accuracy: %g, mean number of missclassified objects: %d of %d\n",1-res->cla_GError,res->cla_misses,set->mN);
		minfo("attention: current model is built from last n-fold subset of the training data\n");
	}else if(nce==1 && infold==1)
	{//training only
		res->reg_TError=besterror_d;
		res->cla_TError=besterror_i;
		res->cla_misses=(int)inerror_i[bestparam_i];//number of misses
		minfo("Result of simple Training:\n");
		minfo("reg: C: %g, epsi: %g, features: %d, T-MSE %g\n",res->reg_c,res->reg_e,res->reg_nf,res->reg_TError);
		minfo("cls: C: %g, epsi: %g, features: %d, T-error %g\n",res->cla_c,res->cla_e,res->cla_nf,res->cla_TError);
		minfo("cls: accuracy: %g, number of missclassified objects: %d of %d\n",1-res->cla_TError,res->cla_misses,set->mN);
	}
	*/
	if(yy) delete[]yy;
	delete[] inrank;
	delete[] inerror_d;
	delete[] inerror_i;
	delete[] inerror_itp;
	delete[] inerror_itn;
	delete[] inerror_ifp;
	delete[] inerror_ifn;
	delete[] inerror_n;
	if(intestset!=intrainset) if(intestset) delete intestset;
	if(intrainset!=set) if(intrainset) delete intrainset;
#if 0
	if(par.doclass)
	{
		int a1,a2,a3,a4,a5;
		a1=res->cla_Gtp+res->cla_Gtn+res->cla_Gfp+res->cla_Gfn;//nr of objects
		a2=res->cla_Gtp+res->cla_Gfn;//nr of +1 objects
		a3=res->cla_Gtn+res->cla_Gfp;// nr of -1 objects
		a4=res->cla_Gtp+res->cla_Gtn;// nr of correct objects
		a5=res->cla_Gfn+res->cla_Gfp;// nr of wrong classified objects
		if(a1>0){
            res->cla_Gacc=(double)a4/(double)a1;
            res->cla_Gerr=(double)a5/(double)a1;
		}else{
			res->cla_Gacc=-1;
			res->cla_Gerr=-1;
		}
		if(a2>0 && a3>0){
			res->cla_Gaccb=0.5*((double)res->cla_Gtp/(double)a2+(double)res->cla_Gtn/(double)a3);
			res->cla_Gerrb=0.5*((double)res->cla_Gfn/(double)a2+(double)res->cla_Gfp/(double)a3);
		}else{
			res->cla_Gaccb=-1;
			res->cla_Gerrb=-1;
		}
		a1=res->cla_Ttp+res->cla_Ttn+res->cla_Tfp+res->cla_Tfn;//total
		a2=res->cla_Ttp+res->cla_Tfn;//+1
		a3=res->cla_Ttn+res->cla_Tfp;//-1
		a4=res->cla_Ttp+res->cla_Ttn;//correct
		a5=res->cla_Tfn+res->cla_Tfp;//wrong
		if(a1>0){
            res->cla_Tacc=(double)a4/(double)a1;
            res->cla_Terr=(double)a5/(double)a1;
		}else{
			res->cla_Tacc=-1;
			res->cla_Terr=-1;
		}
		if(a2>0 && a3>0){
			res->cla_Taccb=0.5*((double)res->cla_Ttp/(double)a2+(double)res->cla_Ttn/(double)a3);
			res->cla_Terrb=0.5*((double)res->cla_Tfn/(double)a2+(double)(res->cla_Tfp)/(double)a3);
		}else{
			res->cla_Taccb=-1;
			res->cla_Terrb=-1;
		}
	}
#endif
	return notimeoutall;//false if at least one SMO training phase was stopped after timeout.
	//the overall running time of this procedure may be a multiple of the desired time limit, because timeout limits every training phase
}
#endif
void PSVMmodel::getB(Dataset* set)
{//get model parameter "b" for "set"
	int i;
	if(!set)
	{
		xinfo("error in getB: set not valid\n");
		exit(-1);
	}
	b=0;
	for(i=0;i<set->mN;i++)
	{
		b=b+set->mY[i];
	}
	if(set->mN==0) b=FLT_MAX;else b/=set->mN;
	pinfo("   model parameter b = %.3f\n",b);
}
void PSVMmodel::getY(ftype* yy,Dataset* set,int limitfeat,bool full)
{/*	computes Y for existing model using 
	- rank[0..limitfeat-1]; if limitfeat ==-1 use all rank-elements
	- b (level shift)
	- np (normalization) if full==true
	- svz (build relations with support features) if full == true && !isrelation
	caller is responsible for allocating memory y
*/
	int i,j;
	if(!set || !yy)
	{
		xinfo("error in getY: set or prediction vector not valid!\n");
		exit(-1);
	}
	pinfo("   calculating predictions for dataset dim(%d,%d)...\n",set->mN,set->mDim);
	if(limitfeat>=0) limitfeat=min(limitfeat,(int)rank.mFeat.size());
	else limitfeat=rank.mFeat.size();
	if(full)
	{//normalize, support vector(relational), apply kernelfkt
		//isrelation: mtrans=true, set.mdim=np.mdim=rank.mdim
		//else: mtrals=false, set.dim=svz.dim and np.dim=rank.dim
		if(par.isrelation)//do normalization, no svz needed
		{//element-wise kernelfunction + normalization
			if(!set->mTrans || set->mDim!=np.mDim || set->mDim!=rank.mDim)
			{
				xinfo("error in getY: relational: set is not transposed or set does not match model!\n");
				exit(-1);
			}
			for(j=0;j<set->mN;j++) yy[j]=b;
			for(i=0;i<limitfeat;i++)
			{
				ftype* data;
				NpEntry* tnp;
				ftype al;
				ftype x;
				data=&set->mData[set->mN*rank.mFeat[i].dimRef];
				tnp=&np.mX[rank.mFeat[i].dimRef];
				al=rank.mFeat[i].a;
				for(j=0;j<set->mN;j++)
				{//inefficient
                    if(par.kernel==LINEAR) x=kf_LINEAR(data[j]);
					if(par.kernel==POLY) x=kf_POLY(data[j]);
					if(par.kernel==RBF) x=kf_RBF(data[j]);
					if(par.kernel==SIGMOID) x=kf_SIGMOID(data[j]);
					if(par.kernel==SIN) x=kf_SIN(data[j]);
					if(par.kernel==PLUMMER) x=kf_PLUMMER(data[j]);
					if(par.kernel==USER) x=kf_USER(data[j]);
					if(par.useNormLimit)
					{
						x=min(x,tnp->max);
						x=max(x,tnp->min);
					}
                    x=(x-tnp->mean)* tnp->scale;
					yy[j]+=al*x;
				}
			}//for i
		}else{//complete kernel + normalization, not transposed!!
			if(!svz)
			{
				xinfo("error in getY: kernel: support vectors not valid!\n");
				exit(-1);
			}
			if(set->mTrans || set->mDim!=svz->mDim || np.mDim!=rank.mDim)
			{
				xinfo("error in getY: kernel: set is transposed or set does not match model!\n");
				exit(-1);
			}
			for(j=0;j<set->mN;j++) yy[j]=b;
			for(i=0;i<limitfeat;i++)
			{
				ftype al;
				NpEntry* tnp;
				ftype* sdata;
				al=rank.mFeat[i].a;
				tnp=&np.mX[rank.mFeat[i].dimRef];
				sdata=&svz->mData[i*svz->mDim];
				for(j=0;j<set->mN;j++){
					ftype* data;
					ftype x;
					data=&set->mData[j*set->mDim];
					if(par.kernel==LINEAR) x=kf_LINEAR(data,sdata,svz->mDim);
					if(par.kernel==POLY) x=kf_POLY(data,sdata,svz->mDim);
					if(par.kernel==RBF) x=kf_RBF(data,sdata,svz->mDim);
					if(par.kernel==SIGMOID) x=kf_SIGMOID(data,sdata,svz->mDim);
					if(par.kernel==SIN) x=kf_SIN(data,sdata,svz->mDim);
					if(par.kernel==PLUMMER) x=kf_PLUMMER(data,sdata,svz->mDim);
					if(par.kernel==USER) x=kf_USER(data,sdata,svz->mDim);
					if(par.useNormLimit)
					{
						x=min(x,tnp->max);
						x=max(x,tnp->min);
					}
					x=(x-tnp->mean)* tnp->scale;
					x*= al;
					yy[j]+=x;
				}
			}
		}
	}else
	{//inputset is normalized kernel matrix (transposed)
		//y is <K,alpha>+b
		if(!set->mTrans)
		{
			xinfo("error in getY: transposed kernel matrix required\n");
			exit(-1);
		}
		for(j=0;j<set->mN;j++) yy[j]=b;
		for(i=0;i<limitfeat;i++)
		{
			ftype* data;
			ftype al;
			data=&set->mData[set->mN*rank.mFeat[i].dimRef];
			al=rank.mFeat[i].a;
			for(j=0;j<set->mN;j++) yy[j]+=al*data[j];
		}
	}
}
#if 0
//outdated
ftype PSVMmodel::getSingleY(ftype* kvector,int limitfeat)
{/*	compute Y for one vector with existing model using 
	- rank[0..limitfeat-1]; if limitfeat ==-1 use all rank-elements
	- b (level shift)
	- np (normalization)
	- svz (build relations with support features) if !isrelation
*/
	int i,f;
	ftype x,y;
	y=0;
	if(limitfeat>=0) limitfeat=min(limitfeat,(int)rank->feat.size());
	else limitfeat=rank->feat.size();
	for(i=0;i<limitfeat;i++)
	{//normalize before scaling with alpha
		f=rank->feat[i].dimRef;//get feature reference
		if(!par.isrelation)
		{
			if(par.kernel==LINEAR) x=kf_LINEAR(kvector,svz->mData[i],svz->mDim);
			if(par.kernel==POLY) x=kf_POLY(kvector,svz->mData[i],svz->mDim);
			if(par.kernel==RBF) x=kf_RBF(kvector,svz->mData[i],svz->mDim);
			if(par.kernel==SIGMOID) x=kf_SIGMOID(kvector,svz->mData[i],svz->mDim);
			if(par.kernel==SIN) x=kf_SIN(kvector,svz->mData[i],svz->mDim);
			if(par.kernel==PLUMMER) x=kf_PLUMMER(kvector,svz->mData[i],svz->mDim);
		}else
		{
			if(par.kernel==LINEAR) x=(ftype)kf_LINEAR(kvector[f]);
			if(par.kernel==POLY) x=(ftype)kf_POLY(kvector[f]);
			if(par.kernel==RBF) x=(ftype)kf_RBF(kvector[f]);
			if(par.kernel==SIGMOID) x=(ftype)kf_SIGMOID(kvector[f]);
			if(par.kernel==SIN) x=(ftype)kf_SIN(kvector[f]);
			if(par.kernel==PLUMMER) x=(ftype)kf_PLUMMER(kvector[f]);
		}
		{
			x=min(x,np[f].max);
			x=max(x,np[f].min);
			x=(x-np[f].mean)* np[f].scale;
		}
		x*= rank->feat[i].a;
		y+=x;
	}

	return y+b;

}
ftype PSVMmodel::getSingleY_direct(ftype* kvector,int limitfeat)
{/*	compute Y for one vector with existing model using 
	- rank[0..limitfeat-1]; if limitfeat ==-1 use all rank-elements
	- b (level shift)
*/
	int i,f;
	ftype x,y;
	y=0;
	if(limitfeat>=0) limitfeat=min(limitfeat,(int)rank->feat.size());
	else limitfeat=rank->feat.size();
	for(i=0;i<limitfeat;i++)
	{
		f=rank->feat[i].dimRef;//get feature reference
		x=kvector[f];//get data
		x*= rank->feat[i].a;//skalarproduct with alpha
		y+=x;
	}

	return y+b;
}
void PSVMmodel::addB(ftype* kvector)
{//adds b to "vector" with "dim" dimensions
	int i;
	for(i=0;i<dim;i++)
		kvector[i]+=b;
}
#endif

bool PSVMmodel::save(char* fn)
{//saves b, np, ranking to a file with name 'fn'
	FILE* f;
	int i,j;
	pinfo("   saving model file %s \n",fn);
	f=fopen(fn,"w");
	if(!f) return false;
	fprintf(f,"b = %e\n",b);
	fprintf(f,"dim = %d\n",rank.mDim);
	fprintf(f,"features = %d\n",(int)rank.mFeat.size());
	fprintf(f,"isrelation = %d\n",(int)par.isrelation);
	fprintf(f,"kernel = %d %e %e %e\n",par.kernel,par.k1,par.k2,par.k3);
	fprintf(f,"model data: feature index, feature expression, normalization min, norm max, norm mean, norm scale\n");
	for(i=0;i<(int)rank.mFeat.size();i++)
	{
		j=rank.mFeat[i].dimRef;
		fprintf(f,"%d %.9e %.9e %.9e %.9e %.9e\n",j,rank.mFeat[i].a,np.mX[j].min,np.mX[j].max,np.mX[j].mean,np.mX[j].scale);
	}
	if(!par.isrelation)
	{
		if(!svz) return false;
		if(svz->mDim<0) return false;
		fprintf(f,"svzdim = %d\n",svz->mDim);
		fprintf(f,"model data: supportvector expressions 1..N\n");
		for(i=0;i<(int)rank.mFeat.size();i++)
		{
			for(int k=0;k<svz->mDim;k++)
				fprintf(f,"%.9e ",svz->mData[i*svz->mDim+k]);
			fprintf(f,"\n");
		}
	}
	fclose(f);
	return true;
}
bool PSVMmodel::load(char* fn)
{//loads b, np, ranking from a file with name 'fn'
	FILE* f;
	int i,j,nf;
	int dim;
	f=fopen(fn,"r");
	double scan_b,scan_a,scan1,scan2,scan3,scan4;
	if(!f) return false;
	pinfo("   loading model file %s \n",fn);
	fscanf(f,"b = %le\n",&scan_b);b=(ftype)scan_b;
	fscanf(f,"dim = %d\n",&dim);
	fscanf(f,"features = %d\n",&nf);
	int temp;
	fscanf(f,"isrelation = %d\n",&temp);par.isrelation=(temp==1);
	fscanf(f,"kernel = %d %le %le %le\n",&par.kernel,&par.k1,&par.k2,&par.k3);
	np.reserve(dim);
	rank.mFeat.reserve(nf);
	rank.mDim=dim;
	Rankingi rr;
	fscanf(f,"model data: feature index, feature expression, normalization min, norm max, norm mean, norm scale\n");
	for(i=0;i<nf;i++)
	{
		fscanf(f,"%d %le ",&rr.dimRef,&scan_a);rr.a=(ftype)scan_a;
		j=rr.dimRef;
		fscanf(f,"%le %le %le %le\n",&scan1,&scan2,&scan3,&scan4);np.mX[j].min=(ftype)scan1;np.mX[j].max=(ftype)scan2;np.mX[j].mean=(ftype)scan3;np.mX[j].scale=(ftype)scan4;
		rank.mFeat.push_back(rr);
	}
	np.mDim=dim;
	if(!par.isrelation)
	{
		int svzdim;
		fscanf(f,"svzdim = %d\n",&svzdim);
		fscanf(f,"model data: supportvector expressions 1..N\n");
		if(!svz) svz=new Dataset;
		svz->free();
		svz->reserve(nf,svzdim,false);
		for(i=0;i<nf;i++)
		{
			for(int k=0;k<svzdim;k++)
				{fscanf(f,"%le ",&scan1);svz->mData[i*svzdim+k]=(ftype)scan1;}
			fscanf(f,"\n");
		}
		svz->mN=nf;
		svz->mDim=svzdim;
	}
	fclose(f);
	pinfo("   model dimensions %d\n",dim);
	pinfo("   model features %d\n",nf);
//	if(alpha) delete[] alpha;
//	alpha=NULL;
	return true;
}
void PSVMmodel::kernel(Dataset* xset)
{//writes kernel matrix k(x,x) into xset
	int L,P,N,l,p,n;
	if(par.isrelation) //applying a kernel-like function for relational data is just for fun
	{//sorry for not using a dynamic kernel function pointer, therefore it is an inline function
		//xset is transposed
		L=xset->mN;
		P=xset->mDim;
		N=L*P;
		if(par.kernel==LINEAR)
		{
			pinfo("   get input data as kernel matrix dim(%d,%d)\n",L,P);
		}else if(par.kernel==POLY)
		{
			pinfo("   get input data with POLYNOM function as kernel matrix dim(%d,%d)\n",L,P);
			pinfo("   using kernel parameters k1=%g k2=%g k3=%g\n",par.k1,par.k2,par.k3);
			for(n=0;n<N;n++)
				xset->mData[n]=(ftype)kf_POLY(xset->mData[n]);
		}else if(par.kernel==RBF)
		{
			pinfo("   get input data with RBF function as kernel matrix dim(%d,%d)\n",L,P);
			pinfo("   using kernel parameter k1=%g\n",par.k1);
			for(n=0;n<N;n++)
				xset->mData[n]=(ftype)kf_RBF(xset->mData[n]);
		}else if(par.kernel==SIGMOID)
		{
			pinfo("   get input data with SIGMOID function as kernel matrix dim(%d,%d)\n",L,P);
			pinfo("   using kernel parameters k1=%g k2=%g\n",par.k1,par.k2);
			for(n=0;n<N;n++)
				xset->mData[n]=(ftype)kf_SIGMOID(xset->mData[n]);
		}else if(par.kernel==SIN)
		{
			pinfo("   get input data with SIN function as kernel matrix dim(%d,%d)\n",L,P);
			pinfo("   using kernel parameters k1=%g k2=%g\n",par.k1,par.k2);
			for(n=0;n<N;n++)
				xset->mData[n]=(ftype)kf_SIN(xset->mData[n]);
		}else if(par.kernel==PLUMMER)
		{
			pinfo("   get input data with PLUMMER function as kernel matrix dim(%d,%d)\n",L,P);
			pinfo("   using kernel parameters k1=%g k2=%g k3=%g\n",par.k1,par.k2,par.k3);
			for(n=0;n<N;n++)
				xset->mData[n]=(ftype)kf_PLUMMER(xset->mData[n]);
		}else if(par.kernel==USER)
		{
			pinfo("   get input data with USER function as kernel matrix dim(%d,%d)\n",L,P);
			pinfo("   using kernel parameters k1=%g k2=%g k3=%g\n",par.k1,par.k2,par.k3);
			for(n=0;n<N;n++)
				xset->mData[n]=(ftype)kf_USER(xset->mData[n]);
		}
		return;//relational kernel ready
	}//par.isrelation==false

	//standard kernel without complex feature vectors Z
	Dataset kset;//destination dataset transposed, but symetric
	P=L=xset->mN;
	N=xset->mDim;
	int pL,pN,lN,lL;
	ftype* data;
	kset.reserve(L,P,false);
	kset.mTrans=true;
	kset.mStdKernel=true;//crossvalidation: splitting (tileTo) with rows AND cols
	//kernel matrix is symetric for implemented kernel functions
	//x is not transposed! 
	if(par.kernel==LINEAR)
	{
		pinfo("   building up the LINEAR kernel matrix dim(%d,%d)\n",L,L);
		pinfo("   using no kernel parameters\n");
		pL=0;pN=0;
		for(p=0;p<P;p++){
			data=&xset->mData[pN];
			lN=pN;lL=pL;//because we start at l=p
			for(l=p;l<L;l++)
            {//compute only the upper right triangular, K is symmetric, L=P
				kset.mData[pL+l]=kset.mData[lL+p]=kf_LINEAR(&xset->mData[lN],data,N);
				lN+=N;lL+=L;
			}
			pL+=L;pN+=N;
		}
	}else if(par.kernel==POLY)
	{
		pinfo("   building up the POLYNOM kernel matrix dim(%d,%d)\n",L,L);
		pinfo("   using kernel parameters k1=%g k2=%g k3=%g\n",par.k1,par.k2,par.k3);
		pL=0;pN=0;
		for(p=0;p<P;p++){
			data=&xset->mData[pN];
			lN=pN;lL=pL;
			for(l=p;l<L;l++)
            {
				kset.mData[pL+l]=kset.mData[lL+p]=kf_POLY(&xset->mData[lN],data,N);
				lN+=N;lL+=L;
			}
			pL+=L;pN+=N;
		}
	}else if(par.kernel==RBF)
	{
		pinfo("   building up the RBF kernel matrix dim(%d,%d)\n",L,L);
		pinfo("   using kernel parameter k1=%g\n",par.k1);
		pL=0;pN=0;
		for(p=0;p<P;p++){
			data=&xset->mData[pN];
			lN=pN;lL=pL;
			for(l=p;l<L;l++)
            {
				kset.mData[pL+l]=kset.mData[lL+p]=kf_RBF(&xset->mData[lN],data,N);
				lN+=N;lL+=L;
			}
			pL+=L;pN+=N;
		}
	}else if(par.kernel==SIGMOID)
	{
		pinfo("   building up the SIGMOID kernel matrix dim(%d,%d)\n",L,L);
		pinfo("   using kernel parameters k1=%g k2=%g\n",par.k1,par.k2);
		pL=0;pN=0;
		for(p=0;p<P;p++){
			data=&xset->mData[pN];
			lN=pN;lL=pL;
			for(l=p;l<L;l++)
            {
				kset.mData[pL+l]=kset.mData[lL+p]=kf_SIGMOID(&xset->mData[lN],data,N);
				lN+=N;lL+=L;
			}
			pL+=L;pN+=N;
		}
	}else if(par.kernel==SIN)
	{
		pinfo("   building up the SIN kernel matrix dim(%d,%d)\n",L,L);
		pinfo("   using kernel parameters k1=%g k2=%g\n",par.k1,par.k2);
		pL=0;pN=0;
		for(p=0;p<P;p++){
			data=&xset->mData[pN];
			lN=pN;lL=pL;
			for(l=p;l<L;l++)
            {
				kset.mData[pL+l]=kset.mData[lL+p]=kf_SIN(&xset->mData[lN],data,N);
				lN+=N;lL+=L;
			}
			pL+=L;pN+=N;
		}
	}else if(par.kernel==PLUMMER)
	{
		pinfo("   building up the PLUMMER kernel matrix dim(%d,%d)\n",L,L);
		pinfo("   using kernel parameters k1=%g k2=%g k3=%g\n",par.k1,par.k2,par.k3);
		pL=0;pN=0;
		for(p=0;p<P;p++){
			data=&xset->mData[pN];
			lN=pN;lL=pL;
			for(l=p;l<L;l++)
            {
				kset.mData[pL+l]=kset.mData[lL+p]=kf_PLUMMER(&xset->mData[lN],data,N);
				lN+=N;lL+=L;
			}
			pL+=L;pN+=N;
		}
	}else if(par.kernel==USER)
	{
		pinfo("   building up the USER-defined kernel matrix dim(%d,%d)\n",L,L);
		pinfo("   using no kernel parameters\n");
		pL=0;pN=0;
		for(p=0;p<P;p++){
			data=&xset->mData[pN];
			lN=pN;lL=pL;
			for(l=p;l<L;l++)
            {
				kset.mData[pL+l]=kset.mData[lL+p]=kf_USER(&xset->mData[lN],data,N);
				lN+=N;lL+=L;
			}
			pL+=L;pN+=N;
		}
	}
	kset.mN=L;kset.mDim=P;
	//replace xset with computed kset data, but keep labels mY
	delete[] xset->mData;
	xset->mData=kset.mData;
	xset->mDim=P;
	xset->mN=L;
	xset->mDatamem=kset.mDatamem;
	xset->mTrans=kset.mTrans;
	xset->mStdKernel=kset.mStdKernel;
	kset.mData=NULL;//for destructor of kset
	kset.mDatamem=0;
}
void PSVMmodel::kernel(Dataset* xset,Dataset* zset)
{//gets kernel matrix k(x,z) into xset
	int L,P,N,l,p;
	if(par.isrelation || xset->mTrans || zset->mTrans)
	{//does not make sense!!!!
		xinfo("error in kernel: could not build kernel matrix with complex feature vectors in relational mode!\n");
		exit(-1);
	}
	Dataset kset;
	if(xset->mDim!=zset->mDim) 
	{
		xinfo("error in kernel: X does not match Z, could not build kernel matrix!\n");
		exit(-1);
	}
	L=xset->mN;
	P=zset->mN;
	N=xset->mDim;
	int pL,pN,lN;
	ftype* data;
	kset.reserve(L,P,false);
	kset.mTrans=true;
	kset.mStdKernel=false;
	//kernel matrix is symetric, for implementet kernel functions
	//x is not transposed! 
	if(par.kernel==LINEAR)
	{
		pinfo("   building up the LINEAR kernel matrix dim(%d,%d)\n",L,L);
		pinfo("   using no kernel parameters\n");
		pL=0;pN=0;
		for(p=0;p<P;p++){
			data=&zset->mData[pN];
			lN=0;
			for(l=0;l<L;l++)
            {
				kset.mData[pL+l]=kf_LINEAR(&xset->mData[lN],data,N);
				lN+=N;
			}
			pL+=L;pN+=N;
		}
	}else if(par.kernel==POLY)
	{
		pinfo("   building up the POLYNOM kernel matrix dim(%d,%d)\n",L,P);
		pinfo("   using kernel parameters k1=%g k2=%g k3=%g\n",par.k1,par.k2,par.k3);
		pL=0;pN=0;
		for(p=0;p<P;p++){
			data=&zset->mData[pN];
			lN=0;
			for(l=0;l<L;l++)
            {
				kset.mData[pL+l]=kf_POLY(&xset->mData[lN],data,N);
				lN+=N;
			}
			pL+=L;pN+=N;
		}
	}else if(par.kernel==RBF)
	{
		pinfo("   building up the RBF kernel matrix dim(%d,%d)\n",L,P);
		pinfo("   using kernel parameter k1=%g\n",par.k1);
		pL=0;pN=0;
		for(p=0;p<P;p++){
			data=&zset->mData[pN];
			lN=0;
			for(l=0;l<L;l++)
            {
				kset.mData[pL+l]=kf_RBF(&xset->mData[lN],data,N);
				lN+=N;
			}
			pL+=L;pN+=N;
		}
	}else if(par.kernel==SIGMOID)
	{
		pinfo("   building up the SIGMOID kernel matrix dim(%d,%d)\n",L,P);
		pinfo("   using kernel parameters k1=%g k2=%g\n",par.k1,par.k2);
		pL=0;pN=0;
		for(p=0;p<P;p++){
			data=&zset->mData[pN];
			lN=0;
			for(l=0;l<L;l++)
            {
				kset.mData[pL+l]=kf_SIGMOID(&xset->mData[lN],data,N);
				lN+=N;
			}
			pL+=L;pN+=N;
		}
	}else if(par.kernel==SIN)
	{
		pinfo("   building up the SIN kernel matrix dim(%d,%d)\n",L,P);
		pinfo("   using kernel parameters k1=%g k2=%g\n",par.k1,par.k2);
		pL=0;pN=0;
		for(p=0;p<P;p++){
			data=&zset->mData[pN];
			lN=0;
			for(l=0;l<L;l++)
            {
				kset.mData[pL+l]=kf_SIN(&xset->mData[lN],data,N);
				lN+=N;
			}
			pL+=L;pN+=N;
		}
	}else if(par.kernel==PLUMMER)
	{
		pinfo("   building up the PLUMMER kernel matrix dim(%d,%d)\n",L,P);
		pinfo("   using kernel parameters k1=%g k2=%g k3=%g\n",par.k1,par.k2,par.k3);
		pL=0;pN=0;
		for(p=0;p<P;p++){
			data=&zset->mData[pN];
			lN=0;
			for(l=0;l<L;l++)
            {
				kset.mData[pL+l]=kf_PLUMMER(&xset->mData[lN],data,N);
				lN+=N;
			}
			pL+=L;pN+=N;
		}
	}else if(par.kernel==USER)
	{
		pinfo("   building up the USER-defined kernel matrix dim(%d,%d)\n",L,L);
		pinfo("   using no kernel parameters\n");
		pL=0;pN=0;
		for(p=0;p<P;p++){
			data=&zset->mData[pN];
			lN=0;
			for(l=0;l<L;l++)
            {
				kset.mData[pL+l]=kf_USER(&xset->mData[lN],data,N);
				lN+=N;
			}
			pL+=L;pN+=N;
		}
	}
	kset.mN=L;kset.mDim=P;
	//replace xset with computed kset data, but keep labels mY
	delete[] xset->mData;
	xset->mData=kset.mData;
	xset->mDim=P;
	xset->mN=L;
	xset->mDatamem=kset.mDatamem;
	xset->mTrans=kset.mTrans;
	xset->mStdKernel=kset.mStdKernel;
	kset.mData=NULL;//for destructor of kset
	kset.mDatamem=0;
}
void PSVMmodel::getSVZ(Dataset* zset)
{//requires valid 'rank' and saves selected support vectors into svz
	int n,d;
	if(zset->mN!=rank.mDim)
	{
		xinfo("error in getSVZ: no match between set of complex feature vectors and model !\n");
		exit(-1);
	}
	if(svz==NULL) svz=new Dataset;
	svz->reserve(rank.mFeat.size(),zset->mDim,false);
	svz->mDim=zset->mDim;
	svz->mN=rank.mFeat.size();
	for(n=0;n<svz->mN;n++)
	{
		ftype* zdata,*sdata;
		zdata=&zset->mData[svz->mDim*rank.mFeat[n].dimRef];
		sdata=&svz->mData[svz->mDim*n];
		for(d=0;d<svz->mDim;d++)
			sdata[d]=zdata[d];
	}
	svz->mTrans=false;
}
#if 0
void PSVMmodel::getErrors(ftype* y,Dataset* ms,double *mse,double* rate)
{//gets MSE and classification error of y-vector vs. dataset.mY
	int i;
	*mse=0;
	*rate=0;
	for(i=0;i<ms->mN;i++)
	{
		if((ms->mY[i]>=0 && y[i]<0) ||(ms->mY[i]<0 && y[i]>=0)) (*rate)++;
		*mse+=sqr(ms->mY[i]-y[i]);
	}
	*rate=(*rate/(double)ms->mN);
	*mse=(*mse/(double)ms->mN);
}
void PSVMmodel::getAddErrors(ftype* y,Dataset* ms,double *mseadd, double *rateadd,double *mse,double* rate)
{//gets MSE and classification error of y-vector vs. dataset.mY, cumulates mseadd and rateadd
	int i;
	*mse=0;
	*rate=0;
	for(i=0;i<ms->mN;i++)
	{
		if((ms->mY[i]>=0 && y[i]<0) ||(ms->mY[i]<0 && y[i]>=0)) {(*rate)++;(*rateadd)++;}
		*mse+=sqr(ms->mY[i]-y[i]);
		*mseadd+=sqr(ms->mY[i]-y[i]);
	}
	*rate=(*rate/(double)ms->mN);
	*mse=(*mse/(double)ms->mN);
}
#endif
void PSVMmodel::getErrors(ftype* y,Dataset* ms,double *mse,double* rate,int* tp, int* tn, int* fp, int* fn)
{//gets MSE and classification error of y-vector vs. dataset.mY
	int i;
	*mse=0;
	*rate=0;
	*tp=0;
	*tn=0;
	*fp=0;
	*fn=0;
	for(i=0;i<ms->mN;i++)
	{
		if((ms->mY[i]>=0 && y[i]<0) ||(ms->mY[i]<0 && y[i]>=0)) (*rate)++;
		if(ms->mY[i]>=0 && y[i]>=0) (*tp)++;
		if(ms->mY[i]>=0 && y[i]<0) (*fn)++;
		if(ms->mY[i]<0 && y[i]>=0) (*fp)++;
		if(ms->mY[i]<0 && y[i]<0) (*tn)++;
		*mse+=sqr(ms->mY[i]-y[i]);
	}
	*rate=(*rate/(double)ms->mN);
	*mse=(*mse/(double)ms->mN);
}
//void PSVMmodel::getAddErrors(ftype* y,Dataset* ms,double *mseadd, double *rateadd,double *mse,double* rate,int* tpa,int* tna,int* fpa,int* fna,int* tp,int* tn,int* fp,int* fn)
void PSVMmodel::getAddErrors(ftype* y,Dataset* ms,double *mseadd, /*double *rateadd,*/double *mse,double* rate,int* tpa,int* tna,int* fpa,int* fna)
{//gets MSE and classification error of y-vector vs. dataset.mY, cumulates mseadd and rateadd
	int i;
	*mse=0;
	*rate=0;
/*	*tp=0;
	*tn=0;
	*fp=0;
	*fn=0;
*/	for(i=0;i<ms->mN;i++)
	{
		if((ms->mY[i]>=0 && y[i]<0) ||(ms->mY[i]<0 && y[i]>=0)) {(*rate)++;}
		if(ms->mY[i]>=0 && y[i]>=0) {(*tpa)++;}
		if(ms->mY[i]>=0 && y[i]<0){(*fna)++;}
		if(ms->mY[i]<0 && y[i]>=0){(*fpa)++;}
		if(ms->mY[i]<0 && y[i]<0){(*tna)++;}
/*		if(ms->mY[i]>=0 && y[i]>=0) {(*tp)++;(*tpa)++;}
		if(ms->mY[i]>=0 && y[i]<0){(*fn)++;(*fna)++;}
		if(ms->mY[i]<0 && y[i]>=0){(*fp)++;(*fpa)++;}
		if(ms->mY[i]<0 && y[i]<0){(*tn)++;(*tna)++;}
		*/
		*mse+=sqr(ms->mY[i]-y[i]);
		*mseadd+=sqr(ms->mY[i]-y[i]);
	}
	*rate=(*rate/(double)ms->mN);
	*mse=(*mse/(double)ms->mN);
	//rate=(tp+tn)/mN
}
inline double PSVMmodel::kf_LINEAR(double x)
	{return x;}
inline double PSVMmodel::kf_POLY(double x)
	{return pow(par.k1*x+par.k2,par.k3);}
inline double PSVMmodel::kf_RBF(double x)
	{return exp(-par.k1*x);}
inline double PSVMmodel::kf_SIGMOID(double x)
	{return tanh(par.k1*x+par.k2);}
inline double PSVMmodel::kf_SIN(double x)
	{return sin(par.k1*x+par.k2)*exp(-par.k3*(x));}
inline double PSVMmodel::kf_PLUMMER(double x)
	{return pow(par.k1*x+par.k2,-par.k3);}
inline double PSVMmodel::kf_USER(double x)//enter here your kernel function
	{return x;}

inline ftype PSVMmodel::kf_LINEAR(ftype* x,ftype* z,int n)
{
	double d=0;
	for(int i=0;i<n;i++) d+=(double)x[i]*(double)z[i];
	return (ftype)kf_LINEAR(d);
}
inline ftype PSVMmodel::kf_POLY(ftype* x,ftype* z,int n)
{
	double d=0;
	for(int i=0;i<n;i++) d+=(double)x[i]*(double)z[i];
	return (ftype)kf_POLY(d);
}
inline ftype PSVMmodel::kf_RBF(ftype* x,ftype* z,int n)
{
	double d=0;
	for(int i=0;i<n;i++) d+=sqr((double)x[i]-(double)z[i]);
	return (ftype)kf_RBF(d);
}
inline ftype PSVMmodel::kf_SIGMOID(ftype* x,ftype* z,int n)
{
	double d=0;
	for(int i=0;i<n;i++) d+=(double)x[i]*(double)z[i];
	return (ftype)kf_SIGMOID(d);
}
inline ftype PSVMmodel::kf_SIN(ftype* x,ftype* z,int n)
{
	double d=0;
	for(int i=0;i<n;i++) d+=sqr((double)x[i]-(double)z[i]);
	return (ftype)kf_SIN(sqrt(d));
}
inline ftype PSVMmodel::kf_PLUMMER(ftype* x,ftype* z,int n)
{
	double d=0;
	for(int i=0;i<n;i++) d+=sqr((double)x[i]-(double)z[i]);
	return (ftype)kf_PLUMMER(d);
}
inline ftype PSVMmodel::kf_USER(ftype* x,ftype* z,int n)
{//enter here the formula to get the argument for your kernel function
	double d=0;
	for(int i=0;i<n;i++) d+=(double)x[i]*(double)z[i];
	return (ftype)kf_USER(d);
}
